/**
* Implementiert methoden für das navigieren innerhalb der App reagiert auf eingabe einer Route
*
* @class appController.js
*/
AppRouter = Backbone.Router.extend({

    routes: {
        "": "moduleView",
        "configuration": "configuration",
        "infos": "infos",
        "bosoinfos": "bosoInfos",
        "bosoDatenschutz": "bosoDatenschutz",
        "moduleView": "moduleView",
        "register": "register",
        "syncStatus": "syncStatus",
        "navigate/:viewId/:dataId/:refDataId": "navigateTo",
    },

    initialize: function () {
        $('.back').on('click', 'a', function () {
            window.history.back();
            return false;
        });

        this.firstPage = true;
        // configure jQuery ajax default settings
        $.ajaxSetup({
            type: 'POST',
            //contentType: 'application/json; charset=utf-8',
            //dataType: 'json',
            cache: false,
            //async: false,
            timeout: 120000, //1 min before timeout
        });

        // initialize window.currentUser
        this.initCurrentUser();
        this.initCurrentDevice();
        this.initWsUrls();

        this.initLanguage();

        this.initData();
        this.modifySignatureStorage();
    },

    initCurrentUser: function () {
        window.currentUser = JSON.parse(window.localStorage.getItem("currentUser"));
        if (!window.currentUser) {
            setCurrentUser(new window.UserRawElement());
        }
    },

    initCurrentDevice: function () {
        window.currentDevice = JSON.parse(window.localStorage.getItem("currentDevice"));
        if (!window.currentDevice) {
            setCurrentDevice(new window.DeviceRawElement());
        }
    },

    initWsUrls: function () {
        window.appVersion = new window.AppVersionRawElement();
    },

    initData: function () {
        window.initUserRawElementData();
        window.initFeedbacksData();
        window.initGenericViewsData();
        window.initGenericViewFieldsData();
        window.initGenericEntitiesData();
        window.initGenericPropertiesData();
        //window.initGenericDataTmp();
        window.initGenericDataData();
        window.initGenericValidationData();
        window.initGenericViewActionsData();
        window.initGenericViewFiltersData();
    },

    modifySignatureStorage: function () {
        if (window.navigator.notification) {
            var models = _.filter(window.genericData.models, function (model) {
                return model.get('EntityId') === CONSTS.ATTACHEMENTS_ENTITY_ID && model.get('CustomMax_001');
            });
            if (models.length > 0) {
                var count = models.length;
                _.each(models, function (model) {
                    var thatModel = model;
                    //FileHelper.persistFile(model.get('CustomMax_001'), model.get('Custom_001'), model.get('DataId'), function ()
                    FileHelper.saveFile(model.get('CustomMax_001'), model.get('DataId'), model.get('Custom_001'), function()
                    {
                        thatModel.PersistToLocalStorageImmediately = false;
                        thatModel.unset('CustomMax_001');
                        thatModel.saveItem();
                        count--;
                        if (count <= 0) {
                            window.persistGenericData();
                        }
                    });
                });
            }
        }
    },

    initLanguage: function () {
        var userLang = (navigator.language) ? navigator.language : navigator.userLanguage;
    },

    /* Generic View Routes */
    navigateTo: function (viewId, dataId, refObjId) {
        var view = window.genericViews.get(viewId);
        
        if (!view) {
            alert('Die Ansicht existiert nicht oder Sie besitzen nicht die benötigten Zugriffsrechte!');
            return;
        } 
        
        var functionName = "this." + view.get('ViewType');
        if (typeof eval(functionName) === 'function') {
            eval(functionName + "(view, dataId, refObjId)");
        }
    },

    List: function (view, modelId, refModelId) {
        var newView = jQuery.extend(true, {}, new window.ListView({ view: view, modelId: modelId, refModelId: refModelId }));
        this.changePage(view.attributes.ViewType, newView);
    },

    Insert: function (view, modelId, refModelId) {
        var newView = jQuery.extend(true, {}, new window.InsertView({ view: view, modelId: null, refModelId: refModelId }));
        this.changePage(view.attributes.ViewType, newView);
    },

    Edit: function (view, modelId, refModelId) {
        var newView = jQuery.extend(true, {}, new window.EditView({ view: view, modelId: modelId, refModelId: refModelId }));
        this.changePage(view.attributes.ViewType, newView);
    },

    Detail: function (view, model, refModel) {
        var newView = jQuery.extend(true, {}, new window.DetailView({ view: view, modelId: model, refModelId: refModel }));
        this.changePage(view.attributes.ViewType, newView);
    },

    Report: function (view, model, refModel) {
        var newView = jQuery.extend(true, {}, new window.ReportView({ view: view, modelId: model, refModelId: refModel }));
        this.changePage(view.attributes.ViewType, newView);
    },

    Custom: function (view, model, refModel) {
        var newView = jQuery.extend(true, {}, new window.CustomView({ view: view, modelId: model, refModelId: refModel }));
        this.changePage(view.attributes.ViewType, newView);
    },

    moduleView: function () {
        var newView = jQuery.extend(true, {}, new ModuleView({}));
        this.changePage('ModuleView', newView);
    },

    configuration: function () {
        var newView = jQuery.extend(true, {}, new ConfigurationFormView({}));
        this.changePage('ConfigurationFormView', new ConfigurationFormView({}));

    },
    infos: function () {
        var newView = jQuery.extend(true, {}, new InfosView({}));
        this.changePage('InfosView', newView);
    },

    bosoInfos: function () {
        var newView = jQuery.extend(true, {}, new BosoInfosView({}));
        this.changePage('BosoInfosView', newView);
    },

    bosoDatenschutz: function () {
        var newView = jQuery.extend(true, {}, new BosoDatenschutzView({}));
        this.changePage('BosoDatenschutzView', newView);
    },


    register: function () {
        var newView = jQuery.extend(true, {}, new RegisterView({}));
        this.changePage('RegisterView', newView);

    },
    syncStatus: function () {
        var newView = jQuery.extend(true, {}, new SyncStatusView({}));
        if (!SyncController.SyncIsRunning)
            SyncController.syncData(false);
        this.changePage('SyncStatusView', newView);
    },

    addSubTitleIfBosoViews: function(viewId) {
        if (!GlobalSettings.IsBoso()) {
            return;
        }

        var ENTITY_ID_USER = '612d526a-a19c-d78a-f5b7-e96e002999cb';

        var VIEWID_MEASUREMENTS = 'ae1bdcc7-ef9c-277c-2db2-fd2efc8b1790';
        var VIEWID_STATISTICS = '7ca0a619-8598-11ab-1f39-2c744e6f0bfa';
        var VIEWID_GRAFICS = 'a46d54f2-47df-3a93-3f0b-51d48ee7bc6f';
        var VIEWID_REPORT = '74a9585c-98b0-864e-a3cd-ba15b932efb9'

        if(viewId === VIEWID_MEASUREMENTS || viewId === VIEWID_STATISTICS || viewId === VIEWID_GRAFICS || viewId === VIEWID_REPORT)
        {
            var currentBosoUser = _.find(window.genericData.models, function (model) {
                return model.get('EntityId') === ENTITY_ID_USER
                    && model.get('Custom_005') === 1
                    && model.get('IsActive') === true;
            });

            if(!!currentBosoUser) {
                var userFirstName = !!currentBosoUser.get('Custom_006') ? currentBosoUser.get('Custom_006') : '';
                var userLastName = !!currentBosoUser.get('Custom_002') ? currentBosoUser.get('Custom_002') : '';
                $('#subtitle').text(userFirstName + ' ' + userLastName);
            }

            var titleContainer = $('#titleContainer');
            if(!!titleContainer && titleContainer.length > 0) {
                titleContainer.css('padding-top', '6px');
            }
        }
    },

    changePage: function (viewId, newView) {
        $('#pagecontent').children().empty();
        Utils.Swipe.reset();
        $('#' + viewId).unbind();
        $('#' + viewId).empty();
        $('#' + viewId).append($(newView.el));



        /*Default Header herstellen und dann pro View anpassen */
        var backbutton = $('#backButton')
        backbutton.off();

        backbutton.css('display', '');
        backbutton.click(function () {
            window.history.back();
        });

        var homeButton = $('#homeButton');
        homeButton.off();

        homeButton.css('display', '');
        homeButton.click(function () {
            window.app.navigate('', { trigger: true });
        });

        var title = $('#title');
        if(!!title && title.length > 0) {
            title.text('');
        }

        if (GlobalSettings.IsBoso()) {
            var subtitle = $('#subtitle');
            if (!!subtitle && subtitle.length > 0) {
                subtitle.text('');
            }
      
            var titleContainer = $('#titleContainer');
            if(!!titleContainer && titleContainer.length > 0) {
                titleContainer.css('padding-top', '12px');
            }

            if(!!newView && !!newView.options && !!newView.options.view) {
                $('#title').text(newView.options.view.get('ViewName'));
            
                var viewGuid = newView.options.view.get('ViewId');
                if(!!viewGuid) {
                    this.addSubTitleIfBosoViews(viewGuid);
                }
            }
        }

        var syncStatusButton = $('#sync_status_button');
        if (syncStatusButton.length > 0) {
            syncStatusButton.hide();
        }
        if (viewId == 'ModuleView') {
            backbutton.hide();//css('display', 'none');
            var syncOn = JSON.parse(window.localStorage.getItem('synchronizeData'));
            // var syncOn = !GlobalSettings.SyncDeaktivated();
            if (syncOn == true) {
                var syncStatusButton = $('#sync_status_button');
                if (syncStatusButton.length <= 0) {
                    var syncStatusButton = $('<a/>', {
                        id: "sync_status_button",
                        text: '0',
                        href: '#syncStatus',
                        class: 'ui-link ui-btn-left ui-btn ui-icon-refresh ui-btn-icon-left'
                    }).appendTo($('#header'));

                } else {
                    syncStatusButton.show();
                }
                SyncController.checkSync();
            }
        } else if (viewId == 'Edit' || viewId == 'Insert') {
            homeButton.off();

            homeButton.click(function () {
                if (ActionHelper.ViewDeactivated) {
                    ActionHelper.doNavigate(new Object(), null, null, null, null, null, null, null, null, '');
                } else {
                    ActionHelper.navigate(new Object(), null, null, null, null, null, null, null, null, null, null, '');
                }

            });

            backbutton.off();

            backbutton.click(function () {
                if (ActionHelper.ViewDeactivated) {
                    ActionHelper.doNavigate(new Object(), null, 1, null, null, true);
                } else {
                    ActionHelper.navigate(new Object(), null, 1, null, null, true);
                }

            });
        }
        $.mobile.changePage('#' + viewId, { changeHash: false });
        newView.render();
    }

});

function setCurrentUser(currentUser) {
    window.localStorage.setItem("currentUser", JSON.stringify(currentUser));
    window.currentUser = JSON.parse(window.localStorage.getItem("currentUser"));
};

function setCurrentDevice(currentDevice) {
    window.localStorage.setItem("currentDevice", JSON.stringify(currentDevice));
    window.currentDevice = JSON.parse(window.localStorage.getItem("currentDevice"));
};

function globalStatus(nr) {
    var globalStatus = localStorage.getItem(nr);
    if (globalStatus == null) {
        setGlobalStatus(nr, '');
        globalStatus = '';
    }
    return globalStatus;
};


function setGlobalStatus(key, value) {
    localStorage.setItem(key, value);
};


function hideloader() {
    $.mobile.loading('hide');
    $('#loader_page').hide();
};

function showLoader(loaderText) {
    $('#loader_page').show();
    $.mobile.loading('show', {
        text: loaderText,
        textVisible: true,
        theme: 'a',
    });
};