﻿/**
* Implementiert alle Methoden der MF ModuleBundles wie z.B. PZE, HW
* PZE: Implementiert alle Methoden welche speziell für das ModuleBundle Projektzeiterfassung(PZE)
* HW (hier als Electrics definiert): Implementiert alle Methoden welche speziell für das ModuleBundle Handwerk(HW)
*
* Desweiteren werden alle beforeSave und onSave Actions von hier ausgelöst (inkl. Kundenspezifische Aktionen - LMS, BOSO)
*
* @class MFCustom.js
*/
MFCustom = {
    /*
    * PZE static variables
    */
    PZE_TIME_ENTITYID: '58c6e8c4-abee-f68b-b0bf-ce9eeccf81d1',

    /*
    * HW static variables
    */
    HW_ORDER_ENTITYID: '62c2a5be-5698-4e06-8d70-8fd34d96ba5e',

    /*
    * LMS static variables
    */
    LMS_AUFTRAG_ENTITYID: '29d63800-43aa-d907-3f84-e987fedcd678',
    LMS_AUFNAHMEBOGEN_ENTITYID: '2f411e72-345f-397c-3a96-dcdd0b126d67',
    LMS_TEILE_ENTITYID: '351a99ac-fcfa-5b92-76db-df8326c6d958',
    LMS_LEISTUNG_ENTITYID: '9f3ff230-4096-565d-963f-7ef3a8f7a9fc',
    LMS_ZEIT_ENTITYID: '14683dfd-97e6-e3ab-af57-8df3f6e5e50c',

    /*
    * BOSO static variables
    */
    BOSO_USER_ENTITYID: '612d526a-a19c-d78a-f5b7-e96e002999cb',
    BOSO_MEASUREMENT_ENTITYID: '7c2f883a-d243-785f-b4d9-a6c705661022',

    /*
    * PZE Projekt
    */
    PZE_PROJEKT_ENTITYID:'beb71ded-cac1-6843-025c-bc039d244ec2',

    beforeSave: function (model) {
        var self = this;
        switch (model.attributes.EntityId) {
            case self.PZE_TIME_ENTITYID:
                PZE.beforeSaveTime(model);
                break;
            case self.LMS_AUFTRAG_ENTITYID:
                Loedige.beforeSaveOrder(model);
                break;
            case self.LMS_TEILE_ENTITYID:
                Loedige.beforeSaveMaterialItem(model);
                break;
            case self.LMS_LEISTUNG_ENTITYID:
                Loedige.beforeSaveLeistung(model);
                break;
            case self.LMS_AUFNAHMEBOGEN_ENTITYID:
                Loedige.beforeSaveAufnahmebogen(model);
                break;
            case self.BOSO_USER_ENTITYID:
                Boso.beforeSaveUser(model);
                break;
            default:
                break;
        }
        self = null;
    },

    onSave: function (model) {
        var self = this;
        switch (model.attributes.EntityId) {
            case self.HW_ORDER_ENTITYID:
                Electrics.onSaveOrder(model);
                break;
            case self.LMS_ZEIT_ENTITYID:
                Loedige.onSaveTime(model);
                break;
            case self.LMS_AUFTRAG_ENTITYID:
                Loedige.onSaveOrder(model);
                break;
            case self.PZE_PROJEKT_ENTITYID:
                PZE.addEmployeeOnProjectCreation(model);
            case self.BOSO_MEASUREMENT_ENTITYID:
                if(!Boso.onSaveTimeout){
                    Boso.onSaveTimeout = new Utils.ExplicitTimeout(function () {
                        Boso.onSaveActions(model);
                    }, 0, 'onSaveTimeout');

                    if (Boso.onSaveTimeout.cleared) {
                        Boso.onSaveTimeout.restart();
                    }
                }
            default:
                break;
        }
        self = null;
    },

    prepareFilePreview: function () {
        var actionHelper = ActionHelper;
        var model = actionHelper.CurrentView.model;
        var fileType = model.attributes.Custom_001;
        var fileHelper = FileHelper;
        var constants = CONSTS;

        root = $(actionHelper.CurrentView.el).find('#' + actionHelper.CurrentView.Type + 'Container');

        var fs = $('<div/>', {
            class: 'ui-field-contain',
            id: 'attachment-preview-container'
        }).appendTo(root);
        fs.css("overflow", "hidden");

        var lable = $('<label/>', {
            class: 'select',
            text: "Datei: ",
            for: 'attachment-content',
        }).appendTo(fs);

        var URL = model.attributes.Custom_006;
        var FileType = model.attributes.Custom_001;

        if (URL != null && URL != '') {
            fileHelper.loadFile(URL, function (base64) {
                switch (FileType) {
                    case 'jpg':
                        var img = $('<img/>', {
                            id: 'attachment-content',
                            src: 'data:image/jpeg;base64,' + base64
                        }).appendTo(fs);
                        img.attr('height', '300px');
                        break;
                    case 'png':
                        var img = $('<img/>', {
                            id: 'attachment-content',
                            src: 'data:image/png;base64,' + base64
                        }).appendTo(fs);
                        img.attr('height', '300px');
                        break;
                    default:

                }
            }, function (msg) {
                console.log('Image konnte nicht geladen werden: ' + msg);
                var img = $('<img/>', {
                    id: 'attachment-content',
                    src: ''
                }).appendTo(fs);
                img.attr('height', '300px');
            });
        }

        constants = null;
        actionHelper = null;
        fileHelper = null;
    },

    /**
    * createPhotoAttachement - 
    */
    createPhotoAttachement: function () {
        FileHelper.getPhoto(MFCustom.createPictureAttachementCallback, ActionHelper.CurrentView.model.id);
    },

    /**
    * createPhotoAttachement - 
    */
    createGalleryAttachement: function () {
        FileHelper.getGalleryPhoto(MFCustom.createPictureAttachementCallback, ActionHelper.CurrentView.model.id);
    },

    createPictureAttachementCallback: function (base64, fileName) {
        var actionHelper = ActionHelper;
        var refModelId = actionHelper.CurrentView.options.refModelId;
        if (refModelId != null && refModelId != '') {
            var parent = window.genericData.get(refModelId);

            actionHelper.CurrentView.model.attributes.Custom_004 = parent.attributes.EntityId;
            actionHelper.CurrentView.model.attributes.Custom_005 = parent.id;
            actionHelper.CurrentView.model.attributes.Custom_001 = CONSTS.FILETYPE_JPG;
            actionHelper.CurrentView.model.attributes.Custom_006 = fileName;

            var filePreviewFieldContainer = $('#attachment-preview-container');

            if ($('#attachment-content').length > 0) {
                $('#attachment-content').remove();
            }
            var img = $('<img/>', {
                id: 'attachment-content',
                src: 'data:image/jpeg;base64,' + base64
            }).appendTo(filePreviewFieldContainer);
            img.attr('height', '300px');

            parent = null;
        }
        actionHelper = null;
    },

    createDocumentAttachment: function () {
        MFCustom.createFileChooser();
    },

    /*
    * Creates File Chooser for Attachements
    */
    createFileChooser: function () {
        //var urlParams = Utils.getUrlParams();
        //var view = ActionHelper.Views[urlParams.viewId];
        //var model = view.model;

        var view = ActionHelper.CurrentView;
        var model = view.model;
        var container = $(view.el).find('actionPanel');

        var fs = $('<div/>', {
            class: 'ui-btn ui-btn-icon-right ui-icon-carat-r'
        }).appendTo(container);

        var p = $('<p/>', {
            text: 'Datei: ',
        }).appendTo(fs);

        var fileInput = $('<input/>', {
            type: 'file',
            name: 'filechooser',
            id: 'file-chooser',
            class: 'form-control'
        }).appendTo(fs);

        //var bt = $(view.el).find('#14b112a7-af42-be61-275b-7bc79fd7f5f9');
        //bt.attr('type', 'file');
        //bt.attr('accept', CONSTS.FILE_CHOOSER_ACCEPTS);

        //// Check for the various File API support.
        //if (!window.File || !window.FileReader || !window.FileList || !window.Blob) {
        //    return;
        //} 

        //function handleFileSelect(evt) {
        //    var files = evt.target.files; // FileList object

        //    // files is a FileList of File objects. List some properties.
        //    var output = [];
        //    for (var i = 0, f; f = files[i]; i++) {
        //        output.push('<li><strong>', escape(f.name), '</strong> (', f.type || 'n/a', ') - ',
        //                    f.size, ' bytes, last modified: ',
        //                    f.lastModifiedDate.toLocaleDateString(), '</li>');
        //    }
        //    document.getElementById('list').innerHTML = '<ul>' + output.join('') + '</ul>';
        //}

        //bt.addEventListener('change', handleFileSelect, false);

        //var fileInput = $('<input/>', {
        //    type: 'file',
        //    name: 'filechooser',
        //    id: 'file-chooser',
        //    class: 'form-control'
        //}).appendTo(fs);
        //fileInput.attr('accept', CONSTS.FILE_CHOOSER_ACCEPTS);

        //bt.change(function readSingleFile(evt) {
        //    //Retrieve the first (and only!) File from the FileList object
        //    var f = evt.target.files[0];
        //    if (f) {
        //        var r = new FileReader();
        //        r.onload = function (e) {
        //            if (CONSTS.FILE_CHOOSER_ACCEPTS.indexOf(f.type) > -1) {
        //                if (f.size <= CONSTS.FILE_CHOOSER_MAX_UPLOAD_SIZE) {
        //                    var urlParams = Utils.getUrlParams();
        //                    var view = ActionHelper.Views[urlParams.viewId];
        //                    var model = view.model
        //                    var parentModel = view.parentModel;
        //                    var contents = e.target.result;
        //                    var type = f.type;
        //                    var fileName = f.name;
        //                    var BASE64_TAG = 'base64,';

        //                    var pos = contents.indexOf(BASE64_TAG);
        //                    var content = contents.substring(pos + BASE64_TAG.length);
        //                    var type = '';
        //                    if (f.type.indexOf('image') > -1) {
        //                        type = f.type.substring(6);
        //                    }
        //                    if (f.type.indexOf('application') > -1) {
        //                        type = f.type.substring(12);
        //                    }

        //                    model.attributes.CustomMax_001 = content;
        //                    model.attributes.Custom_001 = type;
        //                    model.attributes.Custom_006 = fileName;

        //                    model.attributes.Custom_004 = parentModel.attributes.EntityId;
        //                    model.attributes.Custom_005 = parentModel.id;

        //                    var container = $('#attachment-content-container');

        //                    MFCustom.prepareFilePreview(container, content, type);
        //                } else {
        //                    var maxFileSizeMB = CONSTS.FILE_CHOOSER_MAX_UPLOAD_SIZE / 1024 / 1024;
        //                    alert('Die Größe der ausgewählten Datei übersteigt leider die Maximalgröße von ' + maxFileSizeMB + 'MB');
        //                    $('#file-chooser').val('');
        //                }
        //            } else {
        //                alert('Dieser Dateityp ist leider nicht erlaubt.');
        //                $('#file-chooser').val('');
        //            }
        //        }
        //        r.readAsDataURL(f);
        //    } else {
        //        alert("Datei konnte nicht geladen werden!");
        //    }
        //});

        var br = $('<br/>', {}).appendTo(container);

        var div = $('<div/>', {
            id: 'attachment-content-container'
        }).appendTo(container);

        if (model != null &&
            model.attributes.CustomMax_001 != null &&
            model.attributes.CustomMax_001 != '') {
            this.prepareFilePreview(div, model.attributes.CustomMax_001, model.attributes.Custom_001);
        }

        view = null;
        model = null;
    },

    deleteSelectedAttachements: function () {
        var dataRawElement;
        var files = [];
        showConfirmMessage('Markierte Einträge wirklich löschen?', function (buttonIndex) {
            if (buttonIndex !== 1) {
                return;
            }

            $(ActionHelper.CurrentView.el).find('#listView .checkBox').each(function () {
                if ($(this).hasClass("ui-icon-check") || $(this.parentNode.firstChild).hasClass("ui-checkbox-on")) {
                    dataRawElement = window.genericData.get(this.id);
                    var oldModel = jQuery.extend(true, {}, dataRawElement);
                    ActionHelper.OldModels.push(oldModel);
                    dataRawElement.set("IsActive", false);
                    dataRawElement.attributes.SyncTime = CONSTS.DATE_DEFAULT;
                    dataRawElement.saveItem();
                    window.genericData.add(dataRawElement);
                    if (ActionHelper.IsValid == true) {
                        files.push(dataRawElement.attributes.Custom_006);
                        var element = $(this.parentNode.firstChild);
                        var checked = element.hasClass("ui-checkbox-on")
                        if (checked) {
                            $(this).closest('tr').remove();
                        } else {
                            $(this.parentNode).remove();
                        }
                    }
                }
            });
            if (dataRawElement != null) {
                dataRawElement.syncWithServer();
            }
        }, 'Sicher?', 'OK, Abbrechen');

        this.count = files.length;
        this.fileHelper = FileHelper;

        function doNext() {
            if (this.count <= 0) {
                //DELETE FILES DONE!
                this.fileHelper = null;
                this.count = null;
            } else {
                this.currentFileName = files[this.count - 1];
                if (navigator.notification) {
                    if (this.currentFileName != null) {
                        this.fileHelper.deleteFile(this.currentFileName, $.proxy(function () {
                            this.count--;
                            $.proxy(doNext, this)();
                        }, this));
                    }
                } else {
                    this.count--;
                    $.proxy(doNext, this)();
                }
            }
        } $.proxy(doNext, this)();
    },

    createContactsEmailView: function () {
        var page = $('#Custom');
        page.removeClass('custom_view_padding');
        var container = $('#CustomViewContent');

        var list = $('<ul/>', {
            id: 'listview'
        }).appendTo(container);
        list.css('margin', '0px !important');
        list.attr('data-role', 'listview');
        list.attr('data-filter', 'true');
        ActionHelper.ReturnValue = "";

        var contacts = ContactsHelper.getContacts('email', function (contacts) {
            _.each(contacts, function (contact) {
                var listItem = $('<li/>', {}).appendTo(list);
                var listItemText = '';
                if (contact.Name != null) {
                    listItemText += contact.Name + ' ';
                }
                listItemText += contact.Email;

                var a = $('<a/>', {
                    text: listItemText,
                    class: 'list_item',
                }).appendTo(listItem);
                a.data('Email', contact.Email);
            });
            $('.list_item').on('click', function (ev) {
                var element = ev.currentTarget;
                ActionHelper.ReturnValue = $(element).data('Email');
                ActionHelper.doNavigate(null, null, 1, null, null, null, null, null, null, null);
            });
            if (navigator.notification) {
                list.listview("refresh");
            }
        });

        setTimeout(function () {
            //RECONFIGURE JQM LIST AFTER IT WAS RENDERED
            $('.ui-input-clear').removeAttr('href');
            var inputSearch = $('.ui-input-search');
            var placeholder = $(inputSearch[0].childNodes[0]);
            placeholder.attr('placeholder', 'Liste durchsuchen...');
        }, 0);
    },

    createContactsPhoneNumberView: function () {
        var page = $('#Custom');
        page.removeClass('custom_view_padding');
        var container = $('#CustomViewContent');

        var list = $('<ul/>', {
            id: 'listview'
        }).appendTo(container);
        list.css('margin', '0px !important');
        list.attr('data-role', 'listview');
        list.attr('data-filter', 'true');

        var contacts = ContactsHelper.getContacts('phone', function (contacts) {
            _.each(contacts, function (contact) {
                var listItem = $('<li/>', {}).appendTo(list);
                var listItemText = '';
                if (contact.Name != null) {
                    listItemText += contact.Name + ' ';
                }
                listItemText += contact.PhoneNumber;

                var a = $('<a/>', {
                    text: listItemText,
                    class: 'list_item',
                }).appendTo(listItem);
                a.data('PhoneNumber', contact.PhoneNumber);
            });
            $('.list_item').on('click', function (ev) {
                var element = ev.currentTarget;
                ActionHelper.ReturnValue = $(element).data('PhoneNumber');
                ActionHelper.doNavigate(null, null, 1, null, null, null, null, null, null, null);
            });
            if (navigator.notification) {
                list.listview("refresh");
            }
        });

        setTimeout(function () {
            //RECONFIGURE JQM LIST AFTER IT WAS RENDERED
            $('.ui-input-clear').removeAttr('href');
            var inputSearch = $('.ui-input-search');
            var placeholder = $(inputSearch[0].childNodes[0]);
            placeholder.attr('placeholder', 'Liste durchsuchen...');
        }, 0);
    },

};

PZE = {

    PORJECTWORKER_ENTITYID: '8dae00a3-c6fe-28bf-fd55-4930e13bb43a',
    WORKER_ENTITYID: '0d204c69-ad36-e5ab-f566-4f7136fa5931',

    beforeSaveTime: function (model) {
        if (model.IsSync) return;
        var TIME_ENTITYID = '58c6e8c4-abee-f68b-b0bf-ce9eeccf81d1';
        var TIME_STATUS_OPEN = '1b588858-3aa3-bd49-b94b-eb15d2138d1f';
        if (model.attributes.Custom_002 == TIME_STATUS_OPEN) {
            var timeEntries = _.filter(window.genericData.models, function (time) {
                return time.attributes.EntityId == TIME_ENTITYID &&
                    time.attributes.IsActive == true &&
                    time.attributes.Custom_002 == TIME_STATUS_OPEN &&
                    model.attributes.ObjectId != time.attributes.ObjectId &&
                    time.attributes.Custom_001 == model.attributes.Custom_001;
            });

            if (timeEntries.length > 0) {
                alert('Es gibt schon einen offenen Zeiteintrag für diesen Mitarbeiter!');
                ActionHelper.IsValid = false;
            }
        }
    },

    onChangeUhrzeitBisMobil_old: function () {
        var TIME_ENTITYID = '58c6e8c4-abee-f68b-b0bf-ce9eeccf81d1';

        var FIELD_ID_ZeitVon = 'd0ee1c7e-eb9f-c028-c202-1db990defb91';
        var FIELD_ID_ZeitBis = 'e3a959ef-00ba-3e19-7f4b-8ff9062f862d';
        var FIELD_ID_AnzahlStunden = 'd73a15b5-b837-9711-dbb5-52d5deb9fa66'
        var ZeitVonField = $('#' + FIELD_ID_ZeitVon);
        if ($('#' + FIELD_ID_ZeitVon).length <= 0) {
            var FIELD_ID_ZeitVon = '95689426-d962-b4df-278c-5259b658c7bf';
            var FIELD_ID_ZeitBis = 'd111f2a9-3cfd-fc0b-bcfb-1a0ceed8684d';
            var FIELD_ID_AnzahlStunden = '4b1299a0-479e-c876-8d94-2c98a7d9985b';
        }
        if ($('#' + FIELD_ID_ZeitVon).length <= 0) {
            var FIELD_ID_ZeitVon = '144a215f-5667-5370-91e5-655a50cf6ed4';
            var FIELD_ID_ZeitBis = 'ad534a27-b671-5ef4-b709-35fd9311e628';
            var FIELD_ID_AnzahlStunden = 'fda55a37-136a-6c5e-49bb-ee2ae654319d';
        }
        var ZeitVon = $('#' + FIELD_ID_ZeitVon).val();
        var ZeitBis = $('#' + FIELD_ID_ZeitBis).val();

        if (ZeitVon == undefined || ZeitVon == '' || ZeitBis == undefined || ZeitBis == '') {
            return false;
        }

        ZeitVon = moment(ZeitVon, CONSTS.TIME_PERSIST_FORMAT);
        ZeitBis = moment(ZeitBis, CONSTS.TIME_PERSIST_FORMAT);
        var minutes = parseInt((ZeitBis - ZeitVon) / 1000 / 60);
        var hours = minutes / 60;

        var power = Math.pow(10, 2 || 0);
        var hours = String(Math.round(hours * power) / power);


        var pause = false;
        var self = this;
        var model = ActionHelper.CurrentView.model;
        var mitarbeiter = _.first(_.filter(window.genericData.models, function (pw) {
            return pw.attributes.EntityId == self.WORKER_ENTITYID &&
            (model == undefined || pw.attributes.ObjectId == model.attributes.Custom_001) &&
                pw.attributes.Custom_008 == 1;
        }));
        if (mitarbeiter != undefined) {
            pause = true;
        }
        else {
            _.each(ActionHelper.Models, function (modelID) {
                var projektmitarbeiter = _.first(_.filter(window.genericData.models, function (model) {
                    return model.attributes.ObjectId == modelID;
                }));
                var refModel = _.first(_.filter(window.genericData.models, function (model) {
                    return model.attributes.ObjectId == projektmitarbeiter.attributes.Custom_002 &&
                            model.attributes.Custom_008 == 1;
                }));

                if (pause != false || refModel != null)
                    pause = true;
            });
        }






        if (pause) {
            if (hours > 9) {
                hours = hours - 1;
            }
            else if (hours > 6) {
                hours = hours - 0.5;
            }

        }

        $('#' + FIELD_ID_AnzahlStunden).val(hours.toString().replace('.', ','));
    },

    onChangeUhrzeitBisMobil: function () {
        var that = this;
        var TIME_ENTITYID = '58c6e8c4-abee-f68b-b0bf-ce9eeccf81d1';
        var FIELD_ID_ZeitVon = 'bb70de3f-ee1b-4145-a8a8-82ed674584ef';
        var FIELD_ID_ZeitBis = '27e9e9fa-d789-e978-6ff6-cbc79239678c';
        var FIELD_ID_AnzahlStunden = '65471e6c-c792-1473-aa7d-d6ae40a855f4';
        var MitarbeiterEntityID = '0d204c69-ad36-e5ab-f566-4f7136fa5931';

        var PROPERTY_ID_ZeitVon = 'b99d5030-c7fe-0a03-6486-ad176a1eda87';
        var PROPERTY_ID_ZeitBis = 'a3ab9fc4-5560-a0e5-b422-414596935b40';
        var PROPERTY_ID_AnzahlStunden = '707b2254-d40b-f106-5bc5-16015d2df8ba';
        var PROPERTY_ID_Mitarbeiter = '21b4de90-911c-da4b-238d-f1170e091b4c';


        var genericView = ActionHelper.CurrentView.options.view;
        var genericFieldsSorted = GenericHelper.getViewFields(genericView);
        var selectedMitarbeiter = new Array();

        var FIELD_ZeitVon = _.first(_.filter(genericFieldsSorted, function (field) {
            return field.attributes.PropertyId == PROPERTY_ID_ZeitVon;
        }));
        if (FIELD_ZeitVon != undefined)
            FIELD_ID_ZeitVon = FIELD_ZeitVon.id;

        var FIELD_ZeitBis = _.first(_.filter(genericFieldsSorted, function (field) {
            return field.attributes.PropertyId == PROPERTY_ID_ZeitBis;
        }));
        if (FIELD_ZeitBis != undefined)
            FIELD_ID_ZeitBis = FIELD_ZeitBis.id;

        var FIELD_AnzahlStunden = _.first(_.filter(genericFieldsSorted, function (field) {
            return field.attributes.PropertyId == PROPERTY_ID_AnzahlStunden;
        }));
        if (FIELD_AnzahlStunden != undefined)
            FIELD_ID_AnzahlStunden = FIELD_AnzahlStunden.id;


        var field = _.first(_.filter(genericFieldsSorted, function (field) {
            return field.attributes.PropertyId == PROPERTY_ID_Mitarbeiter;
        }));


        var dropdown;
        if (field!=undefined && $('#' + field.attributes.FieldId) != undefined)
            dropdown = $('#' + field.attributes.FieldId).val();

        if (dropdown != undefined && dropdown.length > 0) {
            selectedMitarbeiter.push(dropdown);

        }

        //TODO
        if (ActionHelper.Models != undefined && ActionHelper.Models.length > 0) {
            var selList = ActionHelper.Models;
            _.each(selList, function (mitarbeiterid) {
                var mitarbeiter = window.genericData.get(mitarbeiterid);
                if (mitarbeiter.attributes.EntityId == '58c6e8c4-abee-f68b-b0bf-ce9eeccf81d1') {
                    selectedMitarbeiter.push(mitarbeiter.attributes.Custom_001);
                }
                else {
                    selectedMitarbeiter.push(mitarbeiter.attributes.Custom_002);
                }
            });
            selList = null;
        }


        var ZeitVon = $('#' + FIELD_ID_ZeitVon).val();
        var ZeitBis = $('#' + FIELD_ID_ZeitBis).val();

        if (ZeitVon == undefined || ZeitVon == '' || ZeitBis == undefined || ZeitBis == '') {
            return false;
        }

        ZeitVon = moment(ZeitVon, CONSTS.TIME_PERSIST_FORMAT);
        ZeitBis = moment(ZeitBis, CONSTS.TIME_PERSIST_FORMAT);
        var minutes = parseInt((ZeitBis - ZeitVon) / 1000 / 60);
        var hours = minutes / 60;

        var power = Math.pow(10, 2 || 0);
        var hours = String(Math.round(hours * power) / power);




        var MitarbeiterList = _.filter(window.genericData.models, function (pw) {
            return pw.attributes.EntityId == MitarbeiterEntityID && pw.attributes.Custom_008 == 1;
        });

        var Mitarbeiter = new Array();

        _.each(MitarbeiterList, function (mitarbeiter) {
            Mitarbeiter.push(mitarbeiter.attributes.DataId);
        });
        MitarbeiterList = null;



        //TODO: FJ. Mitarbeiter für InsertView aus DropDown auslesen
        var pause = false;



        _.each(selectedMitarbeiter, function (projektmiatrbeiter) {
            var index = Mitarbeiter.indexOf(projektmiatrbeiter);
            if (!(index < 0)) {
                pause = true;
                return true;
            }
        })


        if (pause) {
            if (hours > 9) {
                hours = hours - 1;
            }
            else if (hours > 6) {
                hours = hours - 0.5;
            }
        }

        Mitarbeiter = null;

        $('#' + FIELD_ID_AnzahlStunden).val(FormatHelper.getDecimalString(hours));
        $('#' + FIELD_ID_AnzahlStunden).val(FormatHelper.getDecimalString(hours));
    },

    isProjectLeader: function () {
        var self = this;
        var model = ActionHelper.CurrentView.model;
        
        if (Utils.isNull(model)) {
            return;
        }

        var projectLeader = _.filter(window.genericData.models, function (pw) {
            return pw.get('EntityId') == self.PORJECTWORKER_ENTITYID &&
            pw.get('Custom_001') == model.attributes.ObjectId &&
            pw.get('Custom_003') == 1;
        });
        self = null;

        if (projectLeader.length > 0) {
            return true;
        }
        return false;
    },

    getProjectLeader: function () {
        var self = this;

        var projectLeader = _.filter(window.genericData.models, function (model) {
            return model.get('EntityId') == self.PORJECTWORKER_ENTITYID &&
            model.get('Custom_001') == ActionHelper.CurrentView.options.modelId && //model.attributes.ObjectId &&
            model.get('Custom_003') == 1;
            model.get('UserId') == window.currentUser.UserId;
        });

        if (projectLeader.length > 0) {
            return true;
        }
        else {
            return false;
        }
    },


    getStandardProjekt: function () {
        var self = this;
        var genericData = window.genericData;
        var userId = window.currentUser.UserId;

        var myWorker = _.first(genericData.where({ Custom_005: userId, IsActive: true, EntityId: self.WORKER_ENTITYID }));
        if (myWorker != null) {
            var standardProjektWorker = _.first(genericData.where({ Custom_002: myWorker.id, Custom_004: '1', IsActive: true, EntityId: self.PORJECTWORKER_ENTITYID }));
            if (standardProjektWorker != null) {
                //Returns Projekt Id where ME is "IstStandard"
                return standardProjektWorker.attributes.Custom_001;
            }
        }

        standardProjektWorker = null;
        myWorker = null;
        genericData = null;
        self = null;
        userId = null;
        return null;
    },

    getProjektRetention: function (model, entity) {
        var ret = false;
        if (model.attributes.Custom_006 == "0") {
            ret = (moment().toDate().getTime() - model.attributes.SyncTime.getTime()) / 86400000 > entity.attributes.RetentionDays
        }

        return ret;
    },

    navigateToGehenbuchung: function () {
        var selectedModelsElements = $(ActionHelper.CurrentView.el).find('#listView .checkBox');
        var modelIds = [];
        var selectedModels = [];
        var GehenBuchungViewId = '7c2bd151-d718-fa3b-46cb-5fa60ac48a16';
        selectedModelsElements.each(function () {
            if ($(this).hasClass("ui-icon-check") || $(this.parentNode.firstChild).hasClass("ui-checkbox-on")) {
                var dataRawElement = window.genericData.get(this.id);
                modelIds.push(this.id);
                selectedModels.push(dataRawElement);
            }
        });
        if (selectedModels.length > 0) {
            var date = null;
            var time = null;
            var sameDateTime = true;
            _.each(selectedModels, function (model) {
                var currentDate = model.attributes.Custom_009;
                var currentTime = model.attributes.Custom_005;
                if (date == null) {
                    date = currentDate;
                    time = currentTime;
                } else {
                    if (date != currentDate || time != currentTime) {
                        sameDateTime = false;
                    }
                }
                currentDate = null;
                currentTime = null;
            });
            if (sameDateTime) {
                ActionHelper.Models = modelIds;
                ActionHelper.doNavigate(null, GehenBuchungViewId, null, null, null, null, null, null, null, null);
            } else {
                alert('Massenverarbeitung nur möglich bei gleicher Kommenbuchung.');
            }
        } else {
            alert('Für diese Aktion muss mindestens eine Zeit markiert werden.');
        }
        modelIds = null;
        selectedModels = null;
        selectedModelsElements = null;
    },

    setGehenbuchungDefaultValues: function () {
        var DateField = $('#557cb181-db65-ff25-3804-d5f244f4eadd');
        var VonField = $('#d0ee1c7e-eb9f-c028-c202-1db990defb91');

        var dataRawElement = window.genericData.get(ActionHelper.Models[0]);
        if (dataRawElement != null) {
            var date = moment(dataRawElement.attributes.Custom_009, CONSTS.DATE_PERSIST_FORMAT).format(FormatHelper.get_DATETIME_FORMAT());
            DateField.val(date);
            VonField.val(dataRawElement.attributes.Custom_005);
            PZE.onChangeUhrzeitBisMobil();
        }
    },

    addEmployeeOnProjectCreation: function (modeltmp) {
        var PZE_PORJECTEMPLOYEE_ENTITY_ID = '8dae00a3-c6fe-28bf-fd55-4930e13bb43a';
        var EMPLOYEES_ENTITYID =  '0d204c69-ad36-e5ab-f566-4f7136fa5931';

        var model = ActionHelper.CurrentView.model; //Projekt

        if (Utils.isNull(model)) {
            return;
        }

        var projectEmployees = _.filter(window.genericData.models, function (pw) {
            return pw.get('EntityId') == PZE_PORJECTEMPLOYEE_ENTITY_ID
                && pw.get('UserId') == window.currentUser.UserId
                && pw.get('Custom_001') == model.get('ObjectId');
        });

        if (projectEmployees.length > 0) {
            return;
        }

        var employees = _.filter(window.genericData.models, function (pw) {
            return pw.get('EntityId') == EMPLOYEES_ENTITYID && pw.get('UserId') == window.currentUser.UserId;
        });

        if (Utils.isNull(employees)) {
            return;
        }

        var employee = _.first(employees);

        var newModel = new window.GenericDataRawElement();
        newModel.attributes.Custom_001 = model.get('ObjectId'); //Projekt
        newModel.attributes.Custom_002 = employee.get('ObjectId'); //Mitarbeiter
        newModel.attributes.Custom_003 = true; //IstProjektleiter
        newModel.attributes.Custom_004 = true; //IstStandard

        newModel.attributes.EntityId = PZE_PORJECTEMPLOYEE_ENTITY_ID;

        GenericHelper.addModel(newModel,false,true);

        employee = null;
        employees = null;
        projectEmployees = null;
        model = null;
        EMPLOYEES_ENTITYID = null;
        PZE_PORJECTEMPLOYEE_ENTITY_ID = null;
    },
};

Electrics = {

    onSaveOrder: function (model) {
        var STATUS_SEND = '0c803b67-7999-e311-87ca-00155d36f205';
        var STATUS_RECEIVED = '7976c3ae-7a99-e311-87ca-00155d36f206';
        if (model.attributes.Custom_002 == STATUS_SEND) {
            model.attributes.Custom_002 = STATUS_RECEIVED;
            model.attributes.SyncTime = CONSTS.DATE_DEFAULT;
            model.saveItem();
        }
    },

    getAuftragRetentionDeleteAuftrag: function (model, entity) {
        var ret = false;
        if (model.attributes.SyncTime != null && model.attributes.SyncTime.getTime() != CONSTS.DATE_DEFAULT.getTime()) {
            if (model.attributes.Custom_002 == "7eb40d4f-1134-40a1-9918-c42e7f11ea92") {
                ret = (moment().toDate().getTime() - model.attributes.SyncTime.getTime()) / 86400000 > entity.attributes.RetentionDays
            }
            else if (model.attributes.Custom_002 == "dd224e4e-ec83-4606-b04b-ae3577ed6e86") {
                ret = true;
            }
        }
        return ret;
    },

    onChangeLeistungAnlegen: function () {

        var LA_Fahrzeit = '92c7805c-82d2-441c-9577-2589e5ef8d93';
        var LA_Fahrtpauschale = '019fc03c-3d3e-416a-95c6-2a00bb8c0340';
        var LA_Pauschale_VDE_Pruefung = 'ecd773fc-3399-4be0-9876-4eb81fa3681e';
        var leistungsArt = $('#f9c39273-0485-d416-cfa0-9ec7a93a6455').val();

        var von = $('#4dfb24f1-92d9-8765-2313-9143e1cbb4aa').closest('fieldset');
        var bis = $('#03cc18ef-8716-9ac2-9a46-5a521dcc56e2').closest('fieldset');
        var anzH = $('#0e3bec22-7867-0e2f-c619-8ffd774806c4').closest('fieldset');
        var anzKM = $('#a1001263-1c7e-69a5-809f-197ca119ce22').closest('fieldset');


        switch (leistungsArt) {
            case LA_Pauschale_VDE_Pruefung:
                von.hide();
                bis.hide();
                anzH.hide();
                anzKM.hide();
                break;
            case LA_Fahrtpauschale:
                von.hide();
                bis.hide();
                anzH.hide();
                anzKM.hide();
                break;
                //case LA_Fahrzeit:
                //    von.hide();
                //    bis.hide();
                //    anzH.hide();
                //    anzKM.hide();
                //    break;
            default:
                von.show();
                bis.show();
                anzH.show();
                anzKM.show();
                break;
        }
    },

    onChangeLeistungBearbeiten: function () {

        var LA_Fahrzeit = '92c7805c-82d2-441c-9577-2589e5ef8d93';
        var LA_Fahrtpauschale = '019fc03c-3d3e-416a-95c6-2a00bb8c0340';
        var LA_Pauschale_VDE_Pruefung = 'ecd773fc-3399-4be0-9876-4eb81fa3681e';
        var leistungsArt = $('#b9b4c585-69e2-231a-7cb0-7f40c82e28c8').val();

        var von = $('#45304a2f-21f8-3ebf-1667-6d143bc225da').closest('fieldset');
        var bis = $('#186c1665-a9c5-5936-bcfa-fad7eba08cf8').closest('fieldset');
        var anzH = $('#6f25234c-92cc-f8d9-ac9e-a6086618fe05').closest('fieldset');
        var anzKM = $('#606b32c1-3c6c-10f1-c467-40c07824b319').closest('fieldset');


        switch (leistungsArt) {
            case LA_Pauschale_VDE_Pruefung:
                von.hide();
                bis.hide();
                anzH.hide();
                anzKM.hide();
                break;
            case LA_Fahrtpauschale:
                von.hide();
                bis.hide();
                anzH.hide();
                anzKM.hide();
                break;
                //case LA_Fahrzeit:
                //    von.hide();
                //    bis.hide();
                //    anzH.hide();
                //    anzKM.hide();
                //    break;
            default:
                von.show();
                bis.show();
                anzH.show();
                anzKM.show();
                break;
        }
    },
};

SystemCommands = {
    SystemCommandsEntityID: '7b6ca89a-2a54-d9ab-a686-319cc377c862',

    Execute: function (execAsync, manually, callback) {
        var genericData = window.genericData;
        var self = this;

        //var SystemCommands = genericData.where({ IsActive: true, EntityId: self.SystemCommandsEntityID })
        var SystemCommands = genericData.where({ IsActive: true, EntityId: self.SystemCommandsEntityID, Custom_007: undefined })

        if (SystemCommands == null || SystemCommands.length == 0)
            return;

        if (execAsync == null) {
            execAsync = true;
        }

        _.each(
        SystemCommands,
        function (obj) {
            var updateValue = obj.attributes.CustomMax_002
            if (updateValue == undefined || updateValue.length == 0)
                updateValue = obj.attributes.Custom_005;
            switch (obj.attributes.Custom_002) {
                case 'JavaScript':
                    obj.attributes.CustomMax_001 = eval(obj.attributes.Custom_006);
                    break;
                case 'RecordUpdate':
                    var Record = _.first(genericData.where({ IsActive: true, ObjectId: obj.attributes.Custom_003.toLowerCase() }));
                    if (Record != undefined) {
                        Record.attributes.EntryTime = moment().format(CONSTS.DATETIME_PERSIST_FORMAT);
                        Record.attributes.SyncTime = CONSTS.DATE_DEFAULT;
                        Record.set(obj.attributes.Custom_004, updateValue);
                        Record.saveItem();
                    }
                    else {
                        obj.attributes.CustomMax_001 = 'Record not found';
                    }
                    break;
                case 'RecordDelete':
                    var Record = _.first(genericData.where({ IsActive: true, ObjectId: obj.attributes.Custom_003.toLowerCase() }));
                    if (Record != undefined) {
                        genericData.remove(Record);
                        Record.destroy();
                    }
                    break;
                default:
            }

            obj.attributes.EntryTime = moment().format(CONSTS.DATETIME_PERSIST_FORMAT);
            obj.attributes.SyncTime = CONSTS.DATE_DEFAULT;
            obj.set('Custom_007', moment().format(CONSTS.DATETIME_PERSIST_FORMAT));
            obj.saveItem();
        }
        );
        window.persistGenericData();

        //_.each(
        //JavaScriptCommands,
        //function (obj) {
        //    obj.attributes.Custom_001 = obj.attributes.Custom_001;
        //    obj.attributes.CustomMax_001; // Return Value
        //    //obj.attributes.Custom_001 //DataID
        //    //obj.attributes.Custom_002 //Type
        //    //obj.attributes.Custom_003 //Attribute
        //    //obj.attributes.Custom_004 //AttributeValue
        //    //obj.attributes.Custom_006 //JavaCode

        //}
        //);

        if (callback) {
            callback();
        }
        //// check, if synchronization is enabled in settings
        //if (window.localStorage.getItem('synchronizeData') == null || !window.localStorage.getItem('synchronizeData').bool()) {
        //    SyncController.SyncQueue.removeValue('Type', 'SystemCommands');
        //    SyncController.SyncIsRunning = false;
        //    SyncController.syncNext();
        //    return;
        //}
    },

    //synchronizeGenericData: function ( callback) {
    //    if (manually) {
    //        SyncController.synchronizeGenericDataWorker(execAsync, manually, callback);
    //    }
    //    else {
    //        setTimeout(function ()
    //        { SyncController.synchronizeGenericDataWorker(execAsync, manually, callback); }
    //        , 3000);
    //    }
    //},


};