﻿/**
* Addon um Kommunikation mit Bluetooth Geräten zu ermöglichen
*
* @class BluetoothHelper.js
*/
BluetoothHelper = {
	SERVICE_IDS: {
		bloodPressure: '1810',
		heartRate: '180D',
		weightScale: '181D',
		customService: '233bf000-5a34-1b6d-975c-000d5690abe4',
        deviceInformation: '180A',
        battery: '180F'
	},

	CHARACTERISTIC_IDS: {
		bloodPressureMeasurement: '2A35',
		weightMeasurement: '2A9D',
		weightScaleFeature: '2A9E',
		dateTime: '2A08',
		customCharacteristic: '233bf001-5a34-1b6d-975c-000d5690abe4',
        serialNumber: '2A25',
        batteryLevel: '2A19'
	},

	DESCRIPTOR_IDS: {
		clientCharacteristicConfiguration: '2902'
	},

	INIT_STATES: {
		disabled: 'disabled',
		enabled: 'enabled'
	},

	BONDING_STATES: {
		bonding: 'bonding',
		bonded: 'bonded',
		unbonded: 'unbonded'
	},

	CONNECTION_STATES: {
		connected: 'connected',
		disconnected: 'disconnected'
	},

	SCAN_STATES: {
		started: 'scanStarted',
		result: 'scanResult'
	},

	DISCOVER_STATES: {
		discovered: 'discovered'
	},

	SUBSCRIPTION_STATES: {
		subscribed: 'subscribed',
		subscribedResult: 'subscribedResult'
	},

	WRITE_STATES: {
		written: 'written'
	},

    Devices: [],

	stopScanTimer: null,

    onError: function (reason) {
        alert(reason);
    },

    initialize: function (success, error) {
    	var self = this;
    	var doInit = function () {
    		var initSuccess = function (initResponse) {
            	if(initResponse && initResponse.status && initResponse.status === self.INIT_STATES.enabled && _.isFunction(success)) {
            		success();
            	}
            }

    		//if "request" is set to "true" user is prompted to enable bluetooth
            bluetoothle.initialize(initSuccess, {"request": true});
    	}

        try { doInit(); }
        catch ( ex ) {
        	if(_.isFunction(error)) {
        		error(ex);
        	}
        	console.log('BluetoothHelper: ', 'initialize: ', ex.toString());
        };
    },

    hasPermission: function ( success ) {
    	try {
    		bluetoothle.hasPermission( success );
    	} catch (ex) {
    		console.log(JSON.stringify(ex))
    	}
    },

    requestPermission: function (success, error) {
    	try {
        	BluetoothHelper.hasPermission(function ( result ) {
        		if(!!result && !!result.hasPermission) { success(true); }
        		else { bluetoothle.requestPermission(success, error); }
        	} );
    	}
    	catch (ex) {
    		console.log('requestPermissionError: ' + JSON.stringify(ex));
    		error('Request Permission wird nicht unterstützt');
    	}
    },

	/// params = {
	///	scanSuccess: function,
	///	scanError: function,
	///  scanDuration: milliseconds,
	///	deviceFilter: {
    ///		authorizedDevices: ["gerät1","gerät2","gerät3"],
    ///  	services: ["service1", "service2", "service3"]
	///	}
	///}
    startScan: function (params) {
    	this.stopScanTimer = new Utils.ExplicitTimeout(function () {
    		BluetoothHelper.stopScan(params.scanSuccess, params.scanError);
    	}, params.scanDuration);

    	var start = function () {
        	try {
                BluetoothHelper.Devices = [];

                var filter = {
                    "allowDuplicates": false,
                    "scanMode": bluetoothle.SCAN_MODE_LOW_LATENCY,
                    "matchMode": bluetoothle.MATCH_MODE_AGGRESSIVE,
                    "matchNum": bluetoothle.MATCH_NUM_MAX_ADVERTISEMENT,
                    "callbackType": bluetoothle.CALLBACK_TYPE_ALL_MATCHES,
                }

                if(device.platform === 'iOS' || (device.platform === 'Android' && device.version[0] >=6 )) {
                	if(params && params.deviceFilter && params.deviceFilter.services) {
                		filter.services = params.deviceFilter.services
                	}
                }

                bluetoothle.startScan(function (result) {
                	var device = null;
                	if(result.status === BluetoothHelper.SCAN_STATES.started) {
                		BluetoothHelper.stopScanTimer.restart();
                	}

                	if(result.status === BluetoothHelper.SCAN_STATES.result) {
                		device = result;
                        console.log('device found:', device);
                	}

            		if(!!device && !!device.name && !!device.address) {
            			var isAuthorized = _.some(params.deviceFilter.authorizedDevices, function (devicename) {
            				return device.name.indexOf(devicename) > -1
            			});

            			var deviceExisting = _.some(BluetoothHelper.Devices, function (existingDevice) {
                        	return existingDevice.address === device.address
                        });

                        if(isAuthorized && !deviceExisting) {
                        	BluetoothHelper.Devices.push(device);
                        }
            		}
                }, function (error) {
                    console.log('BluetoothHelper:  BLE: scan failed:', error);
                }, filter);
            } catch (ex) {
            	console.log('BluetoothHelper: ', ex);
            }
        }

    	//GetPermission
    	var permissionResult = function (result) {
    		console.log('PermissionSuccess: ' + JSON.stringify(result));
    		start();
    	}

    	if(!!device && !!device.platform && device.platform === 'iOS') {
    		start();
    	} else {
    		BluetoothHelper.requestPermission(permissionResult, permissionResult);
    	}
    },

    stopScan: function (onSuccess, onError) {
        try {
            bluetoothle.stopScan(function () {
                console.log('BluetoothHelper:  BLE: scan stopped');

                if(BluetoothHelper.stopScanTimer) {
                	BluetoothHelper.stopScanTimer.clear();
                }

                if (!!onSuccess && _.isFunction( onSuccess )) {
                	var devices = BluetoothHelper.Devices.slice(0);
                	onSuccess( devices );
                	BluetoothHelper.Devices.length = 0;
                }
            }, function (error) {
            	if(!!onError && _.isFunction(onError)) {
            		onError(error);
            	}
                if(this.stopScanTimer) {
                	this.stopScanTimer.clear();
                }
                console.log('BluetoothHelper:  BLE: stop scan failed');
            });
        } catch (ex) {
        	if(!!onError && _.isFunction(onError)) {
        		onError(ex);
        	}
        	stopScanTimer.clear();
            console.log('BluetoothHelper: ', ex.toString());
        };
    },

    isScanning: function (result) {
    	try {
    		bluetoothle.isScanning(result);
    	} catch (ex) {
    		console.log('BluetoothHelper: ', ex.toString());
    	}
    },

    bond: function (address, success, error) {
        console.log('BluetoothHelper: Try to bond to' + address)

        if(!error) {
        	error = function (error) {
        		console.log('Error while bondig to device: ', JSON.stringify(error, null, 4));
        	}
        }

        try {
        	bluetoothle.bond(success, error, { 'address': address });
        } catch (ex) {
            console.log('BluetoothHelper: ', ex.toString());
        }
    },

    unbond: function (address, success, error) {
        console.log('BluetoothHelper: Try to bond to' + address)

        if(!error) {
        	error = function (error) {
        		console.log('Error while unbonding to device: ', error);
        	}
        }

        try {
            BluetoothHelper.initialize(function(){
                bluetoothle.unbond(success, error, { 'address': address });
            });
        } catch (ex) {
            console.log('BluetoothHelper: ', ex.toString());
        }
    },

    isBonded: function (address, success, error) {
    	if(!error) {
        	error = function (error) {
        		console.log('BluetoothHelper: Error on executing isBonded: ', JSON.stringify(error, null, 4));
        	}
        }

    	try {
    		bluetoothle.isBonded(success, error, {'address':address});
    	} catch (ex) {
    		console.log('BluetoothHelper: Error while bonding: ', ex.toString());
    	}
    },

    connect: function (address, success, error) {
        console.log('BluetoothHelper: Try to connect to' + address)
        try {
        	bluetoothle.connect(success, error, { 'address': address, 'autoConnect': false});
        } catch (ex) {
            console.log('BluetoothHelper: ', ex.toString());
        }
    },

    reconnect: function (address, success, error) {
        try {
        console.log('BluetoothHelper: Try to reconnect to: ', address);
            bluetoothle.reconnect(success, function(reconnectError) {
                if(_.isFunction(error)) {
                    console.log('reconnect error occured: ', JSON.stringify(reconnectError, null, 4));
                    error();
                }
            }, {'address': address});
        } catch (ex) {
            console.log('BluetoothHelper: ' + ex.toString());
        }
    },

    isConnected: function (address, success, error) {
        try {
            bluetoothle.isConnected(success, error, {'address': address});
        } catch (ex) {
            console.log(ex.toString());
        }
    },

    wasConnected: function (address, success, error) {
        try {
            bluetoothle.wasConnected(success, error, {'address': address});
        } catch (ex) {
            console.log(ex.toString());
        }
    },

    quirkConnect: function (address, success, error) {
        BluetoothHelper.wasConnected(address, function (wasConnectedResult) {
            if(wasConnectedResult && wasConnectedResult.wasConnected === false && _.isFunction(success)) {
                BluetoothHelper.connect(address, success, error);
            }

            if(wasConnectedResult && wasConnectedResult.wasConnected === true && _.isFunction(success)) {
                BluetoothHelper.reconnect(address, success, error);
            }
        }, error);
    },

    isPairedAndConnected: function (address, success, error) {
        var params = {
            "services": [
	            BluetoothHelper.SERVICE_IDS.bloodPressure,
	        	BluetoothHelper.SERVICE_IDS.heartRate,
	        	BluetoothHelper.SERVICE_IDS.weightScale
        	]
        }

        var onIsConnectedResult = function(result) {
            console.log('onIsConnectedResult: ', JSON.stringify(result));

            if(result && result.isConnected === true && _.isFunction(success)) {
                console.log('onIsConnectedResult status connected: ', JSON.stringify(result, null, 4));
                success(true);
            } else {
                success(false);
            }
        }

        try {
            console.log('Try retrieving pairedDevice devices');
        	bluetoothle.retrieveConnected(function (pairedDevices) {
                console.log('BluetoothHelper: Paired devices retrieve success: ', JSON.stringify(pairedDevices, null, 4));
                var isPaired = _.some(pairedDevices, function (pairedDevice) {
                    return pairedDevice.address === address;
                });

                console.log('BluetoothHelper: isPaired: ', isPaired);
                if(isPaired) {
                    BluetoothHelper.isConnected(address, onIsConnectedResult, error);
                } else if(!isPaired && _.isFunction(success)) {
                    success(false);
                }
            }, error, params);
        } catch (ex) {
        	console.log('BLE: RetrieveConnected: ' + JSON.stringify(ex));
        }
    },

    isPaired: function(address, success, error) {
        try {
            var params = {
                "services": [
                    BluetoothHelper.SERVICE_IDS.bloodPressure,
                    BluetoothHelper.SERVICE_IDS.heartRate,
                    BluetoothHelper.SERVICE_IDS.weightScale
                ]
            }

            bluetoothle.retrieveConnected(function (pairedDevices) {
                console.log('BluetoothHelper: Paired devices retrieve success: ', JSON.stringify(pairedDevices, null, 4));
                if(success && _.isFunction(success)) {
                    var isPaired = _.some(pairedDevices, function (pairedDevice) {
                        return pairedDevice.address === address;
                    });

                    !!isPaired ? success(true) : success(false);
                }
            }, error, params);
        } catch (ex) {
        	console.log('BLE: RetrieveConnected: ' + ex.toString());
        }
    },

    disconnect: function (address, success, error) {
        console.log('BluetoothHelper: Try to disconnect');

        var fnSuccess = function (result) {
            console.log('Successully disconnected: ', JSON.stringify(result, null, 4));
            if(!!success && _.isFunction(success)) {
                success();
            }
        }

        var fnError = function (result) {
            console.log('Error while trying to disconnect: ', JSON.stringify(error, null, 4));
            if(!!error && _.isFunction(error)) {
                error();
            }
        }

        bluetoothle.disconnect(success, error, { 'address': address });
    },

    close: function (address, success, error) {
        console.log('BluetoothHelper: Try to close connection ' + address);

        if(!success) {
        	success = function (success) {
        		console.log('Successully closed Connection: ', JSON.stringify(success, null, 4));
        	}
        }

        if(!error) {
        	error = function (error) {
        		console.log('Error on closing connection: ', JSON.stringify(error, null, 4));
        	}
        }

        try {
        	bluetoothle.close(success, error, { 'address': address });
        } catch (ex) {
        	console.log(ex.toString());
        }

    },

    isDiscovered: function (address, success, error) {
        // bluetoothle.isDiscovered(function(result) {
        //     console.log('isDiscovered: ', result);
        // }, function(error) {
        //     console.log('isDiscovered: ', error);
        // }, {"address": address});

        console.log("TEST:1 isDiscovered is Callled");
        var fnSuccess = function(result) {
            console.log('isDiscovered success:', result);
            if(_.isFunction(success)) {
                success();
            }
        }

        var fnError = function(result) {
            console.log('isDiscovered error:', result);
            if(_.isFunction(error)) {
                error(result);
            }
        }

        console.log("TEST: 2 isDiscovered is Callled", console.log(address));
        bluetoothle.isDiscovered(fnSuccess, fnError, {"address": address});
    },

    discover: function (address, success, error) {
        try {
            if (window.device.platform == 'iOS') {
                var getDescriptors = function (service, onDescriptorsDiscovered) {
                    console.log('BluetoothHelper: available characteristics', JSON.stringify(service, null, 4));
                    var characteristicsIndex = 0;
                    var discoverDescriptors = function() {
                        if(characteristicsIndex < service.characteristics.length) {
                            var params = {
                                "address": address,
                                "service": service.serviceId,
                                "characteristic": service.characteristics[characteristicsIndex].characteristicId
                            }

                            console.log('getDescriptors parameter: ', JSON.stringify(params, null, 4));

                            bluetoothle.descriptors(function (result) {
                                console.log('BluetoothHelper: Success discovering descriptors: ', JSON.stringify(result, null, 4));

                                if(result && result.descriptors) {
                                    service.characteristics[characteristicsIndex].descriptors.push(result.descriptors);
                                    characteristicsIndex++;
                                    discoverDescriptors();
                                }
                            }, error, params);
                        } else {
                            if(_.isFunction(onDescriptorsDiscovered)) {
                                onDescriptorsDiscovered();
                            }
                        }
                    }

                    discoverDescriptors();
                }

                var getCharacteristics = function(services, onCharacteristicsDiscovered) {
                    console.log('BluetoothHelper: discovering characteristics...');

                    var serviceIndex = 0;
                    var discoverCharacteristics = function () {
                        if(serviceIndex < services.length) {
                            var service = services[serviceIndex];
                            var params = {
                                "address": address,
                                "service": services[serviceIndex].serviceId
                            };

                            bluetoothle.characteristics(function (result) {
                                console.log('BluetoothHelper: Success discovering characteristics: ', JSON.stringify(result, null, 4));
                                if(result && result.characteristics) {
                                    _.each(result.characteristics, function(characteristic) {
                                        service.characteristics.push({
                                            "characteristicId": characteristic.uuid,
                                            "descriptors": []
                                       })
                                    })

                                    getDescriptors(service, function() {
                                        serviceIndex++;
                                        discoverCharacteristics();
                                    });
                                }
                            }, function (error) {
                                console.log('readCharacteristicsError: ', JSON.stringify(error, null, 4));
                            }, params);
                        } else {
                            if(_.isFunction(onCharacteristicsDiscovered)) {
                                onCharacteristicsDiscovered();
                            }
                        }
                    }

                    discoverCharacteristics();
                }

                var getServices = function () {
                    var params = {
                        "address": address
                    }

                    console.log('BluetoothHelper: discovering servcies...');
                    bluetoothle.services(function (result) {
                        console.log('BluetoothHelper: Success discovering servcies', JSON.stringify(result, null, 4));
                        var services = [];

                        _.each(result.services, function(service) {
                            services.push({
                               "serviceId": service,
                               "characteristics": []
                            });
                        });

                        getCharacteristics(services, function() {
                           success({
                              "address": address,
                              "status": BluetoothHelper.DISCOVER_STATES.discovered,
                              "name": services.name,
                              "services": services
                           });
                        });
                    }, function (error) {
                    	console.log(JSON.stringify(error, null, 4));
                    }, params);
                }

            	getServices();
            } else {
            	bluetoothle.discover(success, error, {
                    'address': address, 
                    "clearCache": true
                });
            }
        } catch (ex) {
            console.log('BluetoothHelper: ' + ex.toString());
        }
    },

    write: function (success, error, params) {
        //params in Form: {"value":"","serviceUuid":"180F","characteristicUuid":"2A19","type":"Response"}
        //if params.type == 'noResponse' no Callback will be fired
        console.log('BluetoothHelper: Try to write:\n', JSON.stringify(params, null, 4));
        try {
            bluetoothle.write(success, error, params);
        } catch (ex) {
            console.log('BluetoothHelper: ' + ex.toString());
        }

    },

    writeDescriptor: function (success, error, params) {
        console.log('BluetoothHelper: Try to write descriptor:\n' + (JSON.stringify(params, null, 4)))
        try {
            bluetoothle.writeDescriptor(success, error, params);
        } catch (ex) {
            console.log('BluetoothHelper: ' + ex.toString());
        }
    },

    read: function (success, error, params) {
        //Params in Form {"serviceUuid":"180F","characteristicUuid":"2A19"}
        console.log('BluetoothHelper: Try to read:\n' + (JSON.stringify(params, null, 4)))

        try {
        	bluetoothle.read(success, error, params);
        } catch (ex) {
            alert('BluetoothHelper: ' + ex.toString());
        };
    },

    readDescriptor: function (success, error, params) {
        console.log('BluetoothHelper: Try to read:\n' + (JSON.stringify(params, null, 4)))

        try {
        	bluetoothle.readDescriptor(success, error, params);
        } catch (ex) {
            alert('BluetoothHelper: ' + ex.toString());
        };
    },

    subscribe: function (success, error, params) {
        //Params in Form {"serviceUuid":"180D","characteristicUuid":"2A37","isNotification":true}
        //isNotification false means use indication. True means use Notification
        //Timeout darf nicht weniger als eine Sekunde sein sonst streiken Samsung-Geräte.
        console.log('BluetoothHelper: Try to subscribe:\n' + (JSON.stringify(params, null, 4)));
        try {
        	//funktioniert ohne Timeout nicht!!!
            bluetoothle.subscribe(success, error, params);
        } catch (ex) {
            console.log('BluetoothHelper: ' + ex.toString());
        }
    },

    unsubscribe: function (success, error, params) {
        console.log('BluetoothHelper: Try to unsubscribe:\n' + (JSON.stringify(params, null, 4)));
        try {
        	bluetoothle.unsubscribe(success, error, params);
        } catch (ex) {
        	console.log('BluetoothHelper:', ex.toString());
        }
    }
}
