﻿/**
* Addon um verschiedene Aktionen im Zusammenhang mit den Kontakten des ausführenden Gerätes zu ermöglichen
* Ermöglicht: Zugriff auf Email-Adressen, Telefonnummern, Namen, etc... der auf dem Gerät gespeicherten Kontakten.
* 
* @class ContactsHelper.js
*/
ContactsHelper = {

    /**
    * Ruft eine Liste von allen Kontakten mit Namen, Email Adressen und Telefonnummern ab
    *
    * @method getContacts
    * @param {FUNCTION} callback - Die Callback-Funktion wird von der übergeordneten Funktion aufgerufen sobald sie mit der Verarbeitung fertig ist. 
    * @param {STRING} type -
    */
    getContacts: function (type, callback) {
        var self = this;
        self.callback = callback;
        if (navigator.notification) {
            var options = new ContactFindOptions();
            options.multiple = true;
            var fields = ['displayName', 'name', 'emails', 'phoneNumbers'];
            navigator.contacts.find(fields,
                function (contacts) {
                    var returnContacts = [];
                    for (var i = 0; i < contacts.length; i++) {

                        switch (type) {
                            case 'email':
                                var contactsElement = new Object();
                                _.each(contacts[i].emails, function (mail) {
                                    if (mail.value != null) {
                                        contactsElement.Email = mail.value;
                                        return;
                                    }
                                });
                                if (contactsElement.Email != null) {
                                    var name = '';
                                    if (contacts[i].displayName == null || contacts[i].displayName == '') {
                                        if (contacts[i].name.formatted != null && contacts[i].name.formatted != '') {
                                            name += contacts[i].name.formatted + ' ';
                                        }
                                        if (contacts[i].name.familyName != null && contacts[i].name.familyName != '') {
                                            name += contacts[i].name.familyName + ' ';
                                        }
                                        if (contacts[i].name.givenName != null && contacts[i].name.givenName != '') {
                                            name += contacts[i].name.givenName + ' ';
                                        }
                                        if (contacts[i].name.middleName != null && contacts[i].name.middleName != '') {
                                            name += contacts[i].name.middleName + ' ';
                                        }
                                        if (name != '') {
                                            contactsElement.Name = name;
                                        }
                                    } else {
                                        contactsElement.Name = contacts[i].displayName;
                                    }
                                    returnContacts.push(contactsElement);
                                }
                                break;
                            case 'phone':
                                var contactsElement = new Object();
                                _.each(contacts[i].phoneNumbers, function (number) {
                                    if (!Utils.isNull(number.value)) {
                                        contactsElement.PhoneNumber = number.value;
                                        return;
                                    }
                                });
                                if (contactsElement.PhoneNumber != null) {
                                    var name = '';
                                    if (contacts[i].displayName == null || contacts[i].displayName == '') {
                                        if (contacts[i].name.formatted != null && contacts[i].name.formatted != '') {
                                            name += contacts[i].name.formatted + ' ';
                                        }
                                        if (contacts[i].name.familyName != null && contacts[i].name.familyName != '') {
                                            name += contacts[i].name.familyName + ' ';
                                        }
                                        if (contacts[i].name.givenName != null && contacts[i].name.givenName != '') {
                                            name += contacts[i].name.givenName + ' ';
                                        }
                                        if (contacts[i].name.middleName != null && contacts[i].name.middleName != '') {
                                            name += contacts[i].name.middleName + ' ';
                                        }
                                        if (name != '') {
                                            contactsElement.Name = name;
                                        }
                                    } else {
                                        contactsElement.Name = contacts[i].displayName;
                                    }
                                    returnContacts.push(contactsElement);
                                }
                                break;
                            default:
                                break;
                        }
                    }
                    self.callback(returnContacts);
                },
                function (err) {
                    alert('Fehler beim Abrufen der Kontakte: ' + err);
                }, options);
        }
        else
        {
            callback(alert('Im Browser ist kein Zugriff auf das Telefonbuch möglich'));
        }
    },
};