﻿/**
* Addon um verschiedene Aktionen im Zusammenhang mit Dateien zu ermöglichen
* Ermöglicht: Zugriff auf Dateien, Speicherung von Dateien, Auswahl eines Fotos aus der Gallery, Erstellen eines neuen Fotos.
* 
* @class FileHelper.js
*/
FileHelper = {
    saving: false,

    /* Open any document in File Explorer */
    getDocument: function (callback, fileName) {
        var input = $('<input />', {
            name: "filePicker",
            type: "file",
            class: 'hide',
            accept: 'image/*,video/*,.pdf,.doc,.docx'
        });

        input.click();
        input.change(function (ev) {
            alert($(this).val());
            alert(JSON.stringify(ev, null, 4));

            var directory;
            var fileName;

            var gotFileEntry = function (fileEntry) {
                fileEntry.file(function (file) {
                    var reader = new FileReader();
                    reader.onloadend = function (evt) {
                        if ($.isFunction(callback)) {
                            callback(evt.target.result);
                        }
                    };
                    reader.readAsDataURL(file);
                }, null);
            };


            FileHelper.getFileEntry(null, directory, fileName, gotFileEntry);

        });

    },


    /**
    * Dient dazu Fotos über die Kamera des mobilen Gerätes zu schießen
    *
    * @method getPhoto
    * @param {FUNCTION} callback - Die Callback-Funktion wird von der übergeordneten Funktion aufgerufen sobald sie mit der Verarbeitung fertig ist. 
    * @param {STRING} fileName - Gibt den Dateinamen des Fotos an.
    */
    getPhoto: function (callback, fileName) {
        if (!navigator.notification) {
            alert('Die Fotofunktion ist im Browser nicht verfügbar.');
            return;
        }
        this.getPicture(callback, Camera.PictureSourceType.CAMERA, fileName);
    },

    /**
    * Dient dazu Fotos aus der Gallerie des mobilen Gerätes zu öffnen
    *
    * @method getGalleryPhoto
    * @param {FUNCTION} callback - Die Callback-Funktion wird von der übergeordneten Funktion aufgerufen sobald sie mit der Verarbeitung fertig ist.
    * @param {STRING} fileName - Gibt den Dateinamen des Fotos an.
    */
    getGalleryPhoto: function (callback, fileName) {
        if (!navigator.notification) {
            alert('Die Fotofunktion ist im Browser nicht verfügbar.');
            return;
        }
        this.getPicture(callback, Camera.PictureSourceType.PHOTOLIBRARY, fileName);
    },

    /**
    * Dient dazu auf die Foto-Funktionen des Mobilen Gerätes zuzugreifen. Wird für die Methoden getPhoto und getGalleryPhoto verwendet
    *
    * @method getPicture
    * @param {FUNCTION} callback - Die Callback-Funktion wird von der übergeordneten Funktion aufgerufen sobald sie mit der Verarbeitung fertig ist.
    * @param {INT} sourceType - Gibt an, ob das Foto von der Kamera oder aus der Gallerie kommt.
    * @param {STRING} fileName - Gibt den Dateinamen des Fotos an.
    */
    getPicture: function (callback, sourceType, fileName) {
        if (!navigator.notification) {
            alert('Die Fotofunktion ist im Browser nicht verfügbar.');
            return;
        }
        var self = this;
        if (callback == null) {
            alert('kein callback');
            return;
        }
        navigator.camera.getPicture(
			//On success
			function (base64) {
			    self.saveFile(base64, fileName, "jpg", "MF_DATA", LocalFileSystem.PERSISTENT, false, callback);
			},
			//On error
			function () {
			    alert('Bild konnte nicht verarbeitet werden - Grund: ' + message);
			},
			{
			    //quality: 50,
			    //sourceType: sourceType,
			    //destinationType: Camera.DestinationType.DATA_URL,
			    //encodingType: Camera.EncodingType.JPEG,
			    //saveToPhotoAlbum: true,
			    //targetWidth: 200,
			    //targetHeight: 200,
			    quality: 40,
			    sourceType: sourceType,
			    destinationType: Camera.DestinationType.DATA_URL,
			    //encodingType: Camera.EncodingType.PNG,
			    //saveToPhotoAlbum: true,
			    //targetWidth: 200,
			    //targetHeight: 200,
			}
		);
    },

    /**
    * Dient dazu gesamte Ordner zu löschen
    * @method deleteDirectory
    * @param {STRING} dirName - gibt den Namen des zu löschenden Ordners an.
    * @param {FUNCTION} callback - Die Callback-Funktion wird von der übergeordneten Funktion aufgerufen sobald sie mit der Verarbeitung fertig ist.
    */
    deleteDirectory: function (dirName, storageType, callback) {
        if (!navigator.notification || !window.requestFileSystem) {
            return;
        }

        if (Utils.isNull(storageType)) { storageType = LocalFileSystem.PERSISTENT; }

        var failCallback = function (message) {
            showMessage('Fehler beim löschen des Verzeichnisses ' + dirName + '\n ErrorCode: ' + message.code);
        }

        var callbackDir = function (dataDir) {
            dataDir.removeRecursively(callback, failCallback);
        }

        this.getDirectory(LocalFileSystem.PERSISTENT, dirName, callbackDir);
    },


    /**
    * Dient dazu Files zu löschen
    * @method deleteFile
    * @param {STRING} fileName - gibt den Dateinamen der zu löschenden Datei an.
    * @param {FUNCTION} callback - Die Callback-Funktion wird von der übergeordneten Funktion aufgerufen sobald sie mit der Verarbeitung fertig ist.
    */
    deleteFile: function (fileName, dirName, storageType, callback) {
        if (!navigator.notification || !window.requestFileSystem) {
            return;
        }

        if (Utils.isNull(storageType)) { storageType = LocalFileSystem.PERSISTENT; }

        if (Utils.isNull(dirName)) { dirName = "MF_DATA"; }

        var failCallback = function (message) {
            console.log('Fehler beim Löschen der Datei:' + fileName + '\nError code: ' + message.code);
        }

        var callbackFile = function (fileEntry) {
            fileEntry.remove(function (entry) {
                console.log('file deleted ' + fileName);
                if ($.isFunction(callback)) {
                    callback();
                }
            }, failCallback);
        }

        this.getFileEntry(storageType, dirName, fileName, callbackFile);
    },

    /**
    * Dient dazu Dateien aus dem Dateisystem zu laden
    *
    * @method loadFile
    * @param {STRING} fileName - gibt den Dateinamen der zu ladenden Datei an.
    * @param {FUNCTION} callback - Die Callback-Funktion wird von der übergeordneten Funktion aufgerufen sobald sie mit der Verarbeitung fertig ist.
    * @param {FUNCTION} failCallback - Die Callback-Funktion wird von der übergeordneten Funktion aufgerufen, wenn während der Verarbeitung ein Fehler aufgetretten ist.
    */
    loadFile: function (fileName, fileTyp, callback, failCallback, binary) {
        if (!navigator.notification || !window.requestFileSystem) {
            $('#loader_page').hide();
            $.mobile.loading('hide');
            var genericDataElement = _.first(_.filter(genericData.models, function (model) {
                return model.attributes.ObjectId == fileName;
            }));
            var fileContent = null;
            if (genericDataElement != undefined && genericDataElement.attributes.CustomMax_001 != undefined && genericDataElement.attributes.CustomMax_001 != '')
                fileContent = genericDataElement.attributes.CustomMax_001;

            if ($.isFunction(callback)) {
                callback(fileContent);
            }
            fileContent = null;
            return;
        }

        if (!Utils.isNull(fileTyp)) { fileName = fileName + "." + fileTyp; }

        binary = !!binary;

        var gotFileEntry = function (fileEntry) {
            fileEntry.file(function (file) {
                var reader = new FileReader();
                reader.onloadend = function (evt) {

                    if ($.isFunction(callback)) {
                        callback(binary ? reader.result : evt.target.result);
                    }
                };

                if (binary) {
                    reader.readAsArrayBuffer(file);
                } else {
                    reader.readAsText(file);
                }
            });
        };

        this.getFileEntry(LocalFileSystem.PERSISTENT, "MF_DATA", fileName, gotFileEntry);

    },


    /**
  * Dient dazu Dateien im File-System abzuspeichern.
  *
  * @method persistFile
  * @param {STRING} fileContent - gibt den File-Content als Base64-Code an.
  * @param {STRING} fileName - gibt den Dateinamen der zu speichernden Datei an.
  * @param {STRING} storageType - LocalFileSystem.PERSISTENT, LocalFileSystem.TEMPORARY
  * @param {BIT} isBinary - gibt an ob die Datei Binär oder als Blob (Datei) vorliegt.
  * @param {FUNCTION} callback - Die Callback-Funktion wird von der übergeordneten Funktion aufgerufen sobald sie mit der Verarbeitung fertig ist.
  */
  saveFile: function (fileContent, fileName, fileType, directory, storageType, isBinary, callback) {

        var shortfileName = fileName;
        //If Browser return
        if (!navigator.notification || !window.requestFileSystem) {
            return;
        }
        if (Utils.isNull(fileContent)) { return; }
        if (Utils.isNull(fileName)) { return; }

        if (!Utils.isNull(fileType)) { fileName = fileName + "." + fileType; }

        // Options
        if (Utils.isNull(storageType)) { storageType = LocalFileSystem.PERSISTENT; }
        if (Utils.isNull(isBinary)) { isBinary = false; }

        var writeFile = function (writer) {
            if (isBinary) {
                var type = CONSTS.FILE_OPENER_TYPES[fileType.replace('.', '')];
                type = !!type ? type : fileType;
                fileContent = new Blob([fileContent], { type: type });
            }
            writer.write(fileContent);
            if ($.isFunction(callback)) {
                callback(fileContent, fileName, shortfileName);
            }
        };

        var defer = $.when(this.getFileWriter(storageType, directory, fileName, writeFile))

        defer.done(function () {
            //alert('Saved');
        });
    },

    getFileWriter: function (fileSystem, dirName, fileName, callback) {
        var gotFile = function (fileEntry) {
            fileEntry.createWriter(callback, function (message) {
                showMessage("getFileWriter->createWriter: could not write file > error code " + message.code);
            });
        }
        this.getFileEntry(fileSystem, dirName, fileName, gotFile);
    },

    getFileEntry: function (fileSystem, dirName, fileName, callback) {
        var gotDir = function (dataDir) {
            dataDir.getFile(fileName, { create: true, exclusive: false }, callback, function (message) {
                $('#loader_page').hide();
                $.mobile.loading('hide');
                alert("getFileEntry/getFile->dirName:" + dirName + "|fileName:" + fileName + " > error code " + message.code);

            });
        }
        this.getDirectory(fileSystem, dirName, gotDir)
    },

    getDirectory: function (fileSystem, dirName, callback) {
        var gotFSCallback = function (fileSystem) {
            if (Utils.isNull(dirName)) {
                callback(fileSystem.root);
            }
            else {
                fileSystem.root.getDirectory(dirName, { create: true }, callback, function (message) {
                    $('#loader_page').hide();
                    $.mobile.loading('hide');
                    alert("getDirectory: could not create directory > error code " + message.code);
                });
            }
        }

        this.getFileSystem(fileSystem, gotFSCallback);
    },

    getFileSystem: function (fileSystem, callback) {
        requestFileSystem(LocalFileSystem.PERSISTENT, 0, callback, function (message) {
            $('#loader_page').hide();
            $.mobile.loading('hide');
            alert("failed to request file system > error code " + message.code);
        });
    },

    openFile: function (fileSystem, dirName, fileName, fileType) {
        if (!navigator.notification) {
            return;
        }

        if (Utils.isNull(fileSystem)) { fileSystem = LocalFileSystem.PERSISTENT; }

        var mimeType = CONSTS.FILE_OPENER_TYPES[fileType];
        mimeType = !!mimeType ? mimeType : 'application/' + fileType;

        var open = function(filePath, mimeType) {
            cordova.plugins.fileOpener2.open(filePath, mimeType, {
                error: function (err) {
                    $('#loader_page').hide();
                    $.mobile.loading('hide');
                    alert("Datei konnte nicht geöffnet werden (Fehler:" + JSON.stringify(err, null, 4) + "). File:" + fileName + " Type:" + mimeType + " -- Bitte prüfen Sie, ob ein Dateiexplorer installiert ist.");
                },
                success: function (result) {
                    console.log('FileHelper - openFile: fileIsOpen:', result);
                }
            });
        }

        var gotFileEntry = function (fileEntry) {
            var filePath = fileEntry.toURL();
            fileEntry.file(function(file) {
                console.log('filePath:', filePath, ' - mimeType:', file.type);

                open(filePath, file.type);
            }, function(error) {
                console.log('fileEntry: File error:', error);
                open(filePath, mimeType);
            });
        };

        this.getFileEntry(fileSystem, dirName, fileName + '.' + fileType, gotFileEntry);
    },

    pickFile: function () 
    {
        var SUPPORTED_TYPES = 'application/octet-stream,application/json';
        return chooser.getFile(SUPPORTED_TYPES); 
    }
};