﻿/**
* Modul zum Ausführen der GenericValidation implementiert das Verhalten der verschiedenen Validatoren wie z.B. RequiredFieldValidator
*
* @class ValidationHelper.js
*/
ValidationHelper = {

    /**
     * Validates generic data depending on the validation type (e.g. RequiredFieldValidator)
     */
    validate: function (view) {
        $('.error').each(function () {
            $(this).hide();
        });
        var isValid = true;

        var validation = GenericHelper.getValidation(view);

        _.each(validation, function (validator) {
            var result = 1;
            var functionName = "ValidationHelper.validateBy_" + validator.attributes.ValidationType;

            var functionCall = functionName + "(" +
                "validator.attributes.Value_01," +
                "validator.attributes.Value_02," +
                "validator.attributes.Value_03," +
                "validator.attributes.Value_04," +
                "validator.attributes.Value_05" +
                ")";

            if (typeof eval(functionName) == 'function') {
                result = eval(functionCall);
            }

            if (result != 1) {
                isValid = false;
            }
        });
        if (!isValid) {
            if(!!GlobalSettings && GlobalSettings.IsBoso()) {
                alert(Utils.getText("UNGUELTIGE_EINGABE"));
            } else {
                alert("Ungültige Eingabe!");
            }
            
        }
        return isValid;
    },

    /**
     * Validates a field by RequiredFieldValidator and adds a ErrorMessage
     */
    validateBy_RequiredFieldValidator: function (id, id2, comparevalue) {
        //compare value entspricht Param_03
        if(!id) {
            return 1;
        }

        id = id.toUpperCase();
        var field = $('#' + id);
        if(!field || field.length === 0) {
            id = id.toLowerCase();
            field = $('#' + id);
        }

        if (field.is(":visible")||field.closest('fieldset').is(":visible")) {
            var value = field.val();
            //$(element).is(":visible")
            //Fälle:
            //2. Feld besitzt Text der ungleich Text 3.Feld ist -> Feld 1 nicht required
            //2. Feld besitzt Text der gleich Text 3.Feld ist -> Feld 1 required
            //2. Feld oder 3. Feld besitzt keinen Wert -> Feld 1 required

            /*
            Value_01 = das Feld das Pflichtfeld ist
            Optional Value_02 = Das Feld überprüft wird ob es content aus value_03 hat
            z.B. Auftragssstatus ändern --> 
            value_01 = Kommentarfeld, 
            value_02 = Statusfeld, 
            value_03 = kommaseparierte Liste von Stati die dazu führen das das Feld aus value_01 --> Kommentar zum Pflichtfeld wird
            */

            if (id2 != "" && id2 != null && comparevalue != "" && comparevalue != null) {
                var feld2value = $('#' + id2).val();

                var arr = comparevalue.split(",");
                var required = false;
                for (var i = 0; i < arr.length; i++) {
                    if (feld2value == arr[i]) {
                        required = true;
                    }
                }

                if (!required) { return 1; }

            }

            if (value == "" || value == null) {
                var errorMessage = $('#error_' + id);
                if(!!GlobalSettings && GlobalSettings.IsBoso()) {
                    errorMessage.text(Utils.getText("PFLICHTFELD"));
                } else {
                    errorMessage.text('Feld darf nicht leer sein!');
                }
                
                errorMessage.show();
                return 0;
            } else {
                return 1;
            }
        }
        return 1;
    },

    /**
     * Validates a field by RequiredFieldValidator and adds a ErrorMessage
     */
    validateBy_RegExpValidator: function (id, regexp, errorText) {
        var value = $('#' + id).val();
        var RegEx = new RegExp(regexp);
        
        var result = RegEx.test(value);
        if (!result) {
            var errorMessage = $('#error_' + id);
            errorMessage.text(errorText);
            errorMessage.show();
            return 0;
        } else {
            if (!Utils.isNull($('#error_' + id))); {
                var errorMessage = $('#error_' + id);
                errorMessage.hide();
            }
            return 1;
        }
    },

    /**
     * Validates a field by RequiredFieldValidator and adds a ErrorMessage
     */
    validateBy_LengthValidator: function (id, min, max) {
        var value = $('#' + id).val();
        if (value.length < min || value.length > max) {
            var errorMessage = $('#error_' + id);
            errorMessage.text('Wert muss eine länge zwischen ' + min + ' und ' + max + 'haben');
            errorMessage.show();
            return 0;
        } else {
            return 1;
        }
    },

    /**
     * Validates a field by CompareValidator and adds a ErrorMessage
     */
    validateBy_CompareValidator: function (type, id, operator, compareId) {
        var value = $('#' + id).val();
        var compareValue = $('#' + compareId).val();
        switch (type) {
            case ('number'):
                if (!eval(value + operator + compareValue)) {
                    errorMessage.text('Wert verstößt gegen folgende Regel: ' + value + operator + compareValue);
                    errorMessage.show();
                    return 0;
                } else {
                    return 1;
                }
                break;
            case ('nvarchar'):
                if (!eval(value + operator + compareValue)) {
                    errorMessage.text('Wert verstößt gegen folgende Regel: ' + value + operator + compareValue);
                    errorMessage.show();
                    return 0;
                } else {
                    return 1;
                }
                break;
            default:
                return 1;
                break;
        }
    },

    /**
     * Validates a field by CustomValidators and adds a ErrorMessage
     */
    validateBy_CustomValidator: function (methodname, param1, param2, param3, param4) {
        var functionName = methodname;
        var functionCall = functionName + "(" + param1 + "," + param2 + "," + param3 + "," + param4 + ")";
        if (typeof eval(functionName) == 'function') {
            result = eval(functionCall);
        }

        return result;
    },

    /**
     * Validates a field by Consistency Validator and adds a ErrorMessage
     */
    validateBy_ConsistencyValidator: function (id) {
        var value = $('#' + id).val();
        var viewField = window.genericViewFields.get(id);
        var view = window.genericViews.get(viewField.attributes.ViewId);
        var entitieModels = _.filter(window.genericData.models, function (model) {
            return model.attributes.EntityId == view.attributes.EntityId && model.get(viewField.attributes.GenericColumnName) == value;
        });
        if (entitieModels.length > 0) {
            errorMessage.text('Dieser Wert wird bereits verwendet! Bitte wählen Sie einen anderen Wert.');
            errorMessage.show();
            return 0;
        } else {
            return 1;
        }
    },
};