﻿/**
 * Definiert wie ViewFields aufgebaut werden müssen nach Ihrem DataType - Bestimmt demnach Aussehen und Aufbau der GenericViewFields
 *
 * @class ViewBuilder.js
 */
ViewBuilder = {

    /**
     * Creates fields of diffrent types for a given property
     */
    createField: function (view, field, type) {
        if (field.attributes.AssociatedViewField == undefined || field.attributes.AssociatedViewField == CONSTS.GUID_DEFAULT) {
            var property = window.genericProperties.get(field.attributes.PropertyId);
            var functionName = "ViewBuilder.create_" + property.attributes.DataType;
            var functionCall = functionName + "(view, property, field, type)";

            if (typeof eval(functionName) == 'function') {
                eval(functionCall);
            } else {
                ViewBuilder.createStandardField(view, property, field, type);
            }

            var inputSelector = '#' + field.attributes.FieldId;
            $(view.el).delegate(inputSelector, 'change', function () {
                eval(field.attributes.OnChangeJS);
            });

            if (field.attributes.VisibleJS != '' && field.attributes.VisibleJS != null) {
                var isVisible = eval(field.attributes.VisibleJS);
                if (!isVisible) {
                    var fieldContain = $(inputSelector).closest('.ui-field-contain');
                    fieldContain.hide();
                }
            }

            //Scroll to Element if clicked in
            if (window.devicePlatform != 'BROWSER') {
                $(view.el).find(inputSelector).focus(function () {
                    $('<div>', {
                        id: 'scrollpufferdiv',
                        style: 'height: 400px;'
                    }).appendTo($(view.el).find('#' + type + 'Container'));

                    $('html, body').animate({
                        scrollTop: $(this).offset().top - 250
                    }, 0);

                });

                $(view.el).find(inputSelector).blur(function () {
                    $(view.el).find('#scrollpufferdiv').remove();
                });
            }
        } else {
            this.create_subView(view, field, type);
        }
    },

    createFieldForReport: function (view, field, type, property) {
        if (field.attributes.AssociatedViewField == undefined || field.attributes.AssociatedViewField == CONSTS.GUID_DEFAULT) {
            //var property = window.genericProperties.get(field.attributes.PropertyId);
            var functionName = "ViewBuilder.create_" + property.attributes.DataType;
            var functionCall = functionName + "(view, property, field, type)";

            if (typeof eval(functionName) == 'function') {
                eval(functionCall);
            } else {
                ViewBuilder.createStandardField(view, property, field, type);
            }

            $(ActionHelper.CurrentView.el).delegate('#' + field.attributes.FieldId, 'change', function () {
                eval(field.attributes.OnChangeJS);
            });

            if (field.attributes.VisibleJS != '' && field.attributes.VisibleJS != null) {
                var isVisible = eval(field.attributes.VisibleJS);
                if (!isVisible) {
                    var fieldContain = $('#' + field.attributes.FieldId).closest('.ui-field-contain');
                    fieldContain.hide();
                }
            }
        } else {
            this.create_subView(view, field, type);
        }
    },
    /**
     * Creates a signature field for a given property of type 'date'
     */
    create_signature: function (view, property, field, type) {
        var model = view.model;

        root = $(view.el).find('#' + type + 'Container');

        var fs = $('<div/>', {
            class: 'ui-field-contain'
        }).appendTo(root);
        fs.css("overflow", "hidden");

        var label = property.attributes.PropertyName;
        if (field.attributes.ViewFieldLabel) {
            label = field.attributes.ViewFieldLabel;
        }

        if (field.attributes.RequiredField)
            label = label + '*';

        $('<label/>', {
            class: 'select',
            text: label + ": ",
            for: field.attributes.FieldId,
        }).appendTo(fs);

        var value = null;
        if (model != null) {
            value = model.get(property.attributes.GenericColumnName);
        }

        //div.jSignature('importData', model.get(property.attributes.GenericColumnName));

        if (value == '' || value == null) {
            value = DefaultValueHelper.getDefaultValue(field);
        }

        var img = $('<img/>', {
            id: field.attributes.FieldId,
            name: property.attributes.GenericColumnName,
            class: 'signature-image'
        }).appendTo(fs);
        img.data('params', {
            hasChanged: false
        });

        var hasContent = false;
        if (value != '' && value != null) {
            var attachment = window.genericData.get(value);
            if (attachment) {
                hasContent = true;
                var data = attachment.get('CustomMax_001');
                if (data) {
                    img.attr('src', 'data:image/' + attachment.get('Custom_001') + ';base64,' + data);
                } else if (window.navigator.notification) { // window.navigator.notification ist nur auf mobilen Geräten present
                    FileHelper.loadFile(attachment.get('DataId'), null, function (data) {
                        if (data) {
                            img.attr('src', 'data:image/' + attachment.get('Custom_001') + ';base64,' + data);
                        } else {
                            img.attr('src', 'data:image/gif;base64,R0lGODlhAQABAIAAAP7//wAAACH5BAAAAAAALAAAAAABAAEAAAICRAEAOw==');
                        }
                    }, function (msg) {
                        console.log('Image ' + attachment.get('DataId') + ' konnte nicht geladen werden : ' + msg);
                        img.attr('src', 'data:image/gif;base64,R0lGODlhAQABAIAAAP7//wAAACH5BAAAAAAALAAAAAABAAEAAAICRAEAOw==');
                    });
                }
            }
        }

        if (!hasContent) {
            img.css('min-height', '300px');
            img.css('width', '95%');
        }

        if (field.attributes.IsReadonly || (model != null && !model.attributes.IsEditableForCu)) {

        } else {
            $('#' + field.attributes.FieldId).click(function () {
                $.mobile.changePage('#overlay_signature', {
                    changeHash: false
                });

                var img = $('#' + field.attributes.FieldId);
                img.data('params', {
                    hasChanged: false
                });

                var sigContent = $('#sig_content');
                sigContent.empty();

                //RESET SIGNATURE BUTTON
                var resetButton = $('<button/>', {
                    text: 'Zurücksetzen',
                    class: 'ui-btn'
                }).appendTo(sigContent).click(function () {
                    signature.jSignature("clear");
                });

                $('<br/>', {}).appendTo(sigContent);

                //SAVE BUTTON
                var saveButton = $('<button/>', {
                    text: 'OK',
                    class: 'ui-btn'
                }).appendTo(sigContent).click(function () {
                    if (ActionHelper.CurrentView.model != null) {
                        ActionHelper.CurrentView.model.attributes.hasChanged = true;
                    }
                    var value = signature.jSignature("getData");
                    if (value && value.length > 22) {
                        var type = value.substring(11, 14);
                        var data = value.substring(22, value.length);

                        var img = $('#' + field.attributes.FieldId);

                        value = 'data:image/' + type + ';base64,' + data;
                        img.attr('src', value);

                        var hasChanged = signature.jSignature('getData', 'base30')[1].length > 1 ? true : false;
                        var isEmpty = signature.jSignature('getData', 'native').length == 0 ? true : false;

                        if (hasContent && !hasChanged) {
                            hasChanged = true;
                        }

                        img.data('params', {
                            hasChanged: hasChanged,
                            isEmpty: isEmpty
                        });
                    }

                    /* _.each(ActionHelper.Models, function (modelId) {
                         var model = window.genericData.get(modelId);
                         if (model != null) {
                             ActionHelper.setModelAttributes(model, ActionHelper.CurrentView.options.refModelId);
                         }
                     });*/
                    /*if (ActionHelper.CurrentView.model != null) {
                        ActionHelper.saveItem();
                    }*/

                    $.mobile.changePage('#main_container', {
                        changeHash: false
                    });
                });

                //SIGNATURE FIELD
                var signature = $('<div/>', {
                    id: field.attributes.FieldId,
                    name: property.attributes.GenericColumnName,
                    class: 'custom_signature'
                }).appendTo(sigContent);

                var w = Math.max(document.documentElement.clientWidth, window.innerWidth || 0)
                var h = Math.max(document.documentElement.clientHeight, window.innerHeight || 0)

                if (w != 0 && h != 0) {
                    if (h < w) {
                        signature.css('width', h);
                    } else {
                        signature.css('width', w);
                    }
                }

                signature.jSignature({
                    'background-color': 'transparent',
                    'decor-color': 'transparent',
                });

                var value = img.attr('src');
                if (value) {
                    signature.jSignature('setData', value);
                }
                signature.css('zoom', (1 / GlobalSettings.ZoomFaktor()));
                signature.css('overflow', 'hidden');

                window.onresize = function () {
                    setTimeout(function () {
                        if (value) {
                            signature.jSignature('setData', value);
                        }
                    }, 1000);
                }
            });
        }

        var tooltip = null;
        if (field.attributes.ToolTip != null && field.attributes.ToolTip != '') {
            tooltip = field.attributes.ToolTip;
        } else if (property.attributes.ToolTip != null && property.attributes.ToolTip != '') {
            tooltip = property.attributes.ToolTip;
        }
        if (tooltip != null) {
            $('<label/>', {
                class: 'tooltip',
                text: tooltip,
            }).appendTo(fs);
        }

        $('<label/>', {
            style: 'display: none',
            class: 'error',
            id: 'error_' + field.attributes.FieldId,
        }).appendTo(fs);

        //Styles
        //var height = 470 * zoom; //funktioniert leider nicht so einfach
        //$(div).css('height', height);
        //$(lable).css('width', '150px');
    },

    /**
     * Creates a gps field for a given property of type 'date'
     */
    create_gps: function (view, property, field, type) {
        //NOT IMPLEMENTED YET
    },

    /**
     * Creates a picture field for a given property of type 'date'
     */
    create_picture: function (view, property, field, type) {
        //NOT IMPLEMENTED YET
    },

    /**
     * Creates a date field for a given property of type 'date'
     */
    create_date: function (view, property, field, type) {
        var model = view.model;
        root = $(view.el).find('#' + type + 'Container');
        var fs = $('<div/>', {
            class: 'ui-field-contain'
        }).appendTo(root);

        var label = property.get('PropertyName');
        if (!!field.get('ViewFieldLabel')) {
            label = field.get('ViewFieldLabel');
        }

        if (field.get('RequiredField'))
            label = label + '*';

        var lable = $('<label/>', {
            class: 'select',
            text: label + ": ",
            for: field.get('FieldId'),
        }).appendTo(fs);

        var cntrlContainer = $('<div/>', {
            class: 'mf-controlgroup',
        }).appendTo(fs);
        cntrlContainer.attr('data-role', 'controlgroup');
        cntrlContainer.attr('data-type', 'horizontal');

        var showRemoveButton = property.get('Param_01') !== 'false';
        var input = $('<input/>', {
            id: field.get('FieldId'),
            name: property.get('GenericColumnName'),
        }).appendTo(cntrlContainer);

        if (showRemoveButton) {
            input.attr('data-wrapper-class', 'controlgroup-textinput ui-btn');

            var removeButton = $('<button/>', {
                text: "X",
                class: 'ui-btn ui-btn-inline mf-date-removebutton'
            }).appendTo(cntrlContainer);
            removeButton.click(function () {
                input.val('');
            });
        } else {
            input.attr('data-wrapper-class', 'controlgroup-textinput-full ui-btn');
        }

        input.attr('class', 'date');
        input.click(function () {
            if (ViewBuilder.DateClick != undefined) {
                ViewBuilder.DateClick();
            }
        });

        input.scroller({
            preset: 'date',
            theme: 'android-holo',
            display: 'modal',
            mode: 'scroller',
            dateOrder: FormatHelper.get_SCROLLER_DATEORDER(), //'ddmmy',
            dateFormat: FormatHelper.get_DATE_FORMAT().toLowerCase(), //FormatHelper.get_DATE_FORMAT(),
            monthNames: FormatHelper.get_MONTH_NAMES(), //['Januar', 'Februar', 'März', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember'],
            monthNamesShort: FormatHelper.get_MONTH_NAMES_SHORT(), //['Jan', 'Feb', 'Mär', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez'],
            dayNames: FormatHelper.get_DAY_NAMES(), //['Januar', 'Februar', 'März', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember'],
            dayNamesShort: FormatHelper.get_DAY_NAMES_SHORT(), //['Jan', 'Feb', 'Mär', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez'],
            monthText: Utils.getText('MONAT'),
            minuteText: Utils.getText('MINUTEN'),
            hourText: Utils.getText('STUNDEN'),
            dayText: Utils.getText('TAG'),
            yearText: Utils.getText('JAHR'),
            nowText: Utils.getText('JETZT'),
            cancelText: Utils.getText('BEENDEN'),
            setText: Utils.getText('OK'),
        });

        var setDateValue = function() {
            if (!model || !property) {
                return;
            }

            var dateModelValue = model.get(property.get('GenericColumnName'));
            if (!dateModelValue) {
                input.val('');
                return;
            }

            input.scroller('setDate', moment(dateModelValue, CONSTS.DATE_PERSIST_FORMAT).toDate(), true);
        }

        setDateValue();

        if (!!input.val()) {
            var value = DefaultValueHelper.getDefaultValue(field);
            if (value != '' && value != null) {
                input.val(value);
            }
        }

        var isReadonly = field.get('IsReadonly') || Boso.isBosoExampleUser(model);
        if (isReadonly || (model && !model.get('IsEditableForCu'))) {
            input.attr('readonly', true);
            input.attr('disabled', true);
            input.attr('class', 'readonly');
            removeButton.attr('disabled', true);
        }

        var tooltip = !!field.get('ToolTip') ? field.get('ToolTip') : property.get('ToolTip');
        if (!!tooltip) {
            $('<label />', {
                class: 'tooltip',
                text: tooltip,
            }).appendTo(fs);
        }

        var error = $('<label />', {
            style: 'display: none',
            class: 'error',
            id: 'error_' + field.get('FieldId'),
        }).appendTo(fs);
    },

    /**
     * Creates a time field for a given property of type 'time'
     */
    create_time: function (view, property, field, type) {

        var model = view.model;

        root = $(view.el).find('#' + type + 'Container');


        var fs = $('<div/>', {
            class: 'ui-field-contain'
        }).appendTo(root);

        var label = property.attributes.PropertyName;
        if (field.attributes.ViewFieldLabel != null && field.attributes.ViewFieldLabel != "") {
            label = field.attributes.ViewFieldLabel;
        }

        if (field.attributes.RequiredField)
            label = label + '*';

        var lable = $('<label/>', {
            class: 'select',
            text: label + ": ",
            for: field.attributes.FieldId,
            fieldtype: "time",
        }).appendTo(fs);

        var cntrlContainer = $('<div/>', {
            class: 'mf-controlgroup',
        }).appendTo(fs);
        cntrlContainer.attr('data-role', 'controlgroup');
        cntrlContainer.attr('data-type', 'horizontal');

        var input = $('<input/>', {
            id: field.attributes.FieldId,
            name: property.attributes.GenericColumnName,
        }).appendTo(cntrlContainer);
        input.attr('data-wrapper-class', 'controlgroup-textinput ui-btn');

        var removeButton = $('<button/>', {
            text: "X",
            class: 'ui-btn ui-btn-inline mf-date-removebutton'
        }).appendTo(cntrlContainer);
        removeButton.click(function () {
            input.val('');
        })

        input.attr('class', 'time');
        input.click(function () {
            ViewBuilder.TimeClick();
        });

        var timestep = property.attributes.Scale;
        if (timestep <= 0) {
            timestep = 1;
        }

        if (model != null) {
            var dateModelValue = model.get(property.attributes.GenericColumnName);
            if (dateModelValue != null && dateModelValue != '') {
                var dateValue = moment(dateModelValue, CONSTS.TIME_PERSIST_FORMAT).format(FormatHelper.get_TIME_FORMAT());
            }

            input.val(dateValue);
        }

        if (input.val() == '' || input.val() == null) {
            var value = DefaultValueHelper.getDefaultValue(field);
            if (value != '' && value != null) {
                input.val(value);
            }
        }

        if (input.val() != null && input.val() != '') {

            var timeFormat = FormatHelper.get_SHORT_TIME_FORMAT();
            if (property.attributes.Precision != undefined && property.attributes.Precision == "1") {
                timeFormat = FormatHelper.get_TIME_FORMAT();
            }

            var dateValue = new Date(moment(input.val(), timeFormat));

            if (property.attributes.Scale != null && property.attributes.Scale != '' && property.attributes.Scale > 0 && property.attributes.Scale < 31) {
                var minutes = dateValue.getMinutes();
                var hours = dateValue.getHours();
                var scale = property.attributes.Scale;
                if ((((minutes + 7.5) / scale | 0) * scale) >= 60)
                    hours++;
                minutes = (((minutes + 7.5) / scale | 0) * scale) % 60;
                hours = ((((minutes / 105) + .5) | 0) + hours) % 24;

                dateValue.setHours(hours, minutes);
            }
            var dateValue = moment(dateValue).format(timeFormat);
            input.val(dateValue);
        }

        if (window.devicePlatform != "WinCE" && window.devicePlatform != "Win32NT") {
            input.scroller({
                preset: 'time',
                theme: 'android-holo',
                display: 'modal',
                mode: 'scroller',
                timeFormat: 'HH:ii',
                timeWheels: 'HHii',
                hourText: Utils.getText('STUNDEN'),
                minuteText: Utils.getText('MINUTEN'),
                //Intervall
                stepMinute: timestep,
                monthNames: FormatHelper.get_MONTH_NAMES(), //['Januar', 'Februar', 'März', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember'],
                monthNamesShort: FormatHelper.get_MONTH_NAMES_SHORT(), //['Jan', 'Feb', 'Mär', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez'],
                monthText: Utils.getText('MONAT'),
                dayText: Utils.getText('TAG'),
                yearText: Utils.getText('JAHR'),
                nowText: Utils.getText('JETZT'),
                cancelText: Utils.getText('BEENDEN'),
                setText: Utils.getText('OK'),
            });

        } else {
            input.attr('readonly', true);
            input.find('input[readonly]').focus(function () {
                this.select();
            });
        }

        var isReadonly = field.get('IsReadonly') || Boso.isBosoExampleUser(model);
        if (isReadonly || (model != null && (model != null && !model.attributes.IsEditableForCu))) {
            input.attr('readonly', true);
            input.attr('disabled', true);
            input.attr('class', 'readonly');
            removeButton.attr('disabled', true);
        }

        var tooltip = null;
        if (field.attributes.ToolTip != null && field.attributes.ToolTip != '') {
            tooltip = field.attributes.ToolTip;
        } else if (property.attributes.ToolTip != null && property.attributes.ToolTip != '') {
            tooltip = property.attributes.ToolTip;
        }
        if (tooltip != null) {
            var tooltip = $('<label/>', {
                class: 'tooltip',
                text: tooltip,
            }).appendTo(fs);
        }

        var error = $('<label/>', {
            style: 'display: none',
            class: 'error',
            id: 'error_' + field.attributes.FieldId,
        }).appendTo(fs);

    },

    /**
     * Creates a datetime field for a given property of type 'datetime'
     */
    create_datetime: function (view, property, field, type) {
        var model = view.model;
        root = $(view.el).find('#' + type + 'Container');

        var fs = $('<div/>', {
            class: 'ui-field-contain'
        }).appendTo(root);

        var labeltext = field.get('ViewFieldLabel') ? field.get('ViewFieldLabel') : property.get('PropertyName');
        if (field.get('RequiredField')) {
            labeltext = labeltext + '*';
        }

        var lable = $('<label />', {
            class: 'select',
            text: labeltext + ": ",
            for: field.attributes.FieldId,
            fieldtype: "datetime",
        });
        lable.appendTo(fs);

        var cntrlContainer = $('<div/>', {
            class: 'mf-controlgroup',
        }).appendTo(fs);
        cntrlContainer.attr('data-role', 'controlgroup');
        cntrlContainer.attr('data-type', 'horizontal');

        var input = $('<input/>', {
            id: field.get('FieldId'),
            name: property.get('GenericColumnName'),
        }).appendTo(cntrlContainer);
        input.attr('data-wrapper-class', 'controlgroup-textinput ui-btn');

        var removeButton = $('<button/>', {
            text: "X",
            class: 'ui-btn ui-btn-inline mf-date-removebutton'
        }).appendTo(cntrlContainer);
        removeButton.click(function () {
            input.val('');
        })

        input.attr('class', 'date');
        input.click(function () {
            ViewBuilder.DateClick();
        });

        var timestep = property.get('Scale');
        if (timestep <= 0) {
            timestep = 1;
        }

        if (window.devicePlatform != "WinCE" && window.devicePlatform != "Win32NT") {
            input.scroller({
                preset: 'datetime',
                theme: 'android-holo',
                display: 'modal',
                mode: 'scroller',
                dateOrder: FormatHelper.get_SCROLLER_DATEORDER(),
                timeFormat: 'HH:ii',
                timeWheels: 'HHii',
                stepMinute: timestep,
                dateFormat: FormatHelper.get_DATE_DISPLAY_FORMAT(), //FormatHelper.get_DATE_FORMAT(),
                monthNames: FormatHelper.get_MONTH_NAMES(), //['Januar', 'Februar', 'März', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember'],
                monthNamesShort: FormatHelper.get_MONTH_NAMES_SHORT(), //['Jan', 'Feb', 'Mär', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez'],
                dayNames: FormatHelper.get_DAY_NAMES(), //['Januar', 'Februar', 'März', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember'],
                dayNamesShort: FormatHelper.get_DAY_NAMES_SHORT(), //['Jan', 'Feb', 'Mär', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez'],
                monthText: Utils.getText('MONAT'),
                dayText: Utils.getText('TAG'),
                yearText: Utils.getText('JAHR'),
                nowText: Utils.getText('JETZT'),
                cancelText: Utils.getText('BEENDEN'),
                setText: Utils.getText('OK'),
                hourText: Utils.getText('STUNDEN'),
                minuteText: Utils.getText('MINUTEN'),
            });
        } else {
            input.attr('readonly', true);
            input.find('input[readonly]').focus(function () {
                this.select();
            });
        }

        if (model != null) {
            var dateModelValue = model.get(property.get('GenericColumnName'));
            if (dateModelValue) {
                var dateValue = moment(dateModelValue, CONSTS.DATETIME_PERSIST_FORMAT).format(FormatHelper.get_SHORT_DATETIME_FORMAT()); //CONSTS.DATETIME_FORMAT);
            }
            input.val(dateValue);
        }

        if (input.val() === '' || input.val() === null) {
            var value = DefaultValueHelper.getDefaultValue(field);
            if (value !== '' && value !== null) {
                input.val(value);
            }
        }

        if (input.val() !== null && input.val() !== '') {
            var timeFormat = FormatHelper.get_SHORT_DATETIME_FORMAT();
            if (property.get('Precision') !== undefined && property.get('Precision') === "1") {
                timeFormat = FormatHelper.get_SHORT_DATETIME_FORMAT();
            }

            var dateValue = new Date(moment(input.val(), timeFormat));
            if (property.get('Scale') !== null && property.get('Scale') !== '' && property.get('Scale') > 0 && property.get('Scale') < 31) {
                var minutes = dateValue.getMinutes();
                var hours = dateValue.getHours();
                var scale = property.get('Scale');
                if ((((minutes + 7.5) / scale | 0) * scale) >= 60)
                    hours++;
                minutes = (((minutes + 7.5) / scale | 0) * scale) % 60;
                hours = ((((minutes / 105) + .5) | 0) + hours) % 24;

                dateValue.setHours(hours, minutes);
            }
            var dateValue = moment(dateValue).format(timeFormat);
            input.val(dateValue);
        }

        var isReadonly = field.get('IsReadonly') || Boso.isBosoExampleUser(model);
        if (isReadonly || (model !== null && !model.get('IsEditableForCu'))) {
            input.attr('readonly', true);
            input.attr('disabled', true);
            input.attr('class', 'readonly');
            removeButton.attr('disabled', true);
        }

        var tooltip = !!field.get('ToolTip') ? field.get('ToolTip') : property.get('ToolTip');
        if (tooltip) {
            var tooltip = $('<label />', {
                class: 'tooltip',
                text: tooltip,
            }).appendTo(fs);
        }

        var error = $('<label />', {
            style: 'display: none',
            class: 'error',
            id: 'error_' + field.get('FieldId'),
        });
        error.appendTo(fs);
    },

    getDisplayValue: function (model, displayProperty, field, displayProperty1, displayProperty2, displayProperty3) {
        var displayText1 = '';
        var displayText2 = '';
        var displayText3 = '';

        if (displayProperty1 == undefined && field.attributes.DisplayProperty1 != undefined && field.attributes.DisplayProperty1 != "00000000-0000-0000-0000-000000000000") {
            displayProperty1 = window.genericProperties.get(field.attributes.DisplayProperty1);
        }

        if (displayProperty2 == undefined && field.attributes.DisplayProperty2 != undefined && field.attributes.DisplayProperty2 != "00000000-0000-0000-0000-000000000000") {
            displayProperty2 = window.genericProperties.get(field.attributes.DisplayProperty2);
        }

        if (displayProperty3 == undefined && field.attributes.DisplayProperty3 != undefined && field.attributes.DisplayProperty3 != "00000000-0000-0000-0000-000000000000") {
            displayProperty3 = window.genericProperties.get(field.attributes.DisplayProperty3);
        }

        var dp = displayProperty;
        var refModel = model;

        while (refModel != undefined) {
            if (displayProperty1 != undefined && displayProperty1.attributes.EntityId == refModel.attributes.EntityId) {
                displayText1 = refModel.get(displayProperty1.attributes.GenericColumnName);
            }
            if (displayProperty2 != undefined && displayProperty2.attributes.EntityId == refModel.attributes.EntityId) {
                displayText2 = refModel.get(displayProperty2.attributes.GenericColumnName);
            }
            if (displayProperty3 != undefined && displayProperty3.attributes.EntityId == refModel.attributes.EntityId) {
                displayText3 = refModel.get(displayProperty3.attributes.GenericColumnName);
            }

            if (dp.attributes.LookupEntityId == undefined || dp.attributes.LookupEntityId == "00000000-0000-0000-0000-000000000000") {
                if (displayProperty1 == undefined && displayProperty2 == undefined && displayProperty3 == undefined)
                    displayText1 = refModel.get(dp.attributes.GenericColumnName);
                refModel = undefined;
            } else {
                var leID = dp.attributes.LookupEntityId;
                var refID = refModel.get(dp.attributes.GenericColumnName);
                var dpID = window.genericEntities.get(leID).attributes.DisplayPropertyId;
                dp = window.genericProperties.get(dpID);
                refModel = _.first(window.genericData.where({
                    IsActive: true,
                    EntityId: leID,
                    ObjectId: refID
                }));
                //if (refModel != undefined)
                //    displayText = refModel.get(dp.attributes.GenericColumnName);
            }
        }
        if (displayText1 == undefined)
            displayText1 = '';

        if (displayText2 == undefined)
            displayText2 = '';

        if (displayText3 == undefined)
            displayText3 = '';

        if (displayText1.length > 0 && displayText2.length > 0)
            displayText2 = ', ' + displayText2;

        if ((displayText1.length > 0 || displayText2.length > 0) && displayText3.length > 0)
            displayText3 = ', ' + displayText3;

        displayText1 = displayText1 + displayText2 + displayText3;
        return displayText1;
    },

    /**
     * Creates a dropdown for a given property of type 'dropdown'
     */
    create_dropdown: function (view, property, field, type) {

        var model = view.model;
        if (model == null && ActionHelper.Models != null) {
            if (ActionHelper.Models.length > 0) {
                model = _.first(_.filter(window.genericData.models, function (model) {
                    return model.attributes.ObjectId == ActionHelper.Models[0];
                }));
                ActionHelper.CurrentView.model = model;
            }
        }
        var refModelId = view.options.refModelId

        root = $(view.el).find('#' + type + 'Container');

        var fss = $('<div/>', {
            //id: field.attributes.FieldId,
            class: 'ui-field-contain',
            name: property.get('GenericColumnName'),
        }).appendTo(root);
        //fss.attr('data-role', 'fieldcontain');
        //fss.attr('data-type', 'horizontal');

        var label = property.get('PropertyName');
        if (field.attributes.ViewFieldLabel != null && field.attributes.ViewFieldLabel != "") {
            label = field.attributes.ViewFieldLabel;
        }

        if (field.attributes.RequiredField)
            label = label + '*';

        var lable = $('<label/>', {
            class: 'select',
            text: label + ": ",
            for: field.attributes.FieldId,
        }).appendTo(fss);


        var fs = $('<div/>', {
            //id: field.attributes.FieldId,
            class: 'ui-controlgroup-controls',
            name: property.attributes.GenericColumnName,
        }).appendTo(fss);
        fs.attr('data-role', 'controlgroup');
        fs.attr('data-type', 'horizontal');
        fs.attr('fieldtype', 'dropdown')


        var lookupEntries = window.genericData.where({
            IsActive: true,
            EntityId: property.get('LookupEntityId')
        });

        var lookUpEntity = window.genericEntities.get(property.get('LookupEntityId'));

        if (lookUpEntity == undefined) {
            alert('Lookupentität nicht vorhanden:' + property.attributes.PropertyName);
            return 0;
        }

        var displayProperty = window.genericProperties.get(lookUpEntity.get('DisplayPropertyId'));

        if (displayProperty == undefined) {
            alert('DisplayPropertyId nicht vorhanden:' + lookUpEntity.attributes.EntityName);
            return 0;
        }

        var sortProperty = window.genericProperties.get(lookUpEntity.get('SortPropertyId'));

        var lookupEntriesSorted = _.sortBy(lookupEntries, function (entry) {
            return entry.get(sortProperty.get('GenericColumnName'));
        });

        var dropdown = $('<select/>', {
            id: field.attributes.FieldId,
            name: property.attributes.GenericColumnName,
        }).appendTo(fs);


        var dropdown_null_item = $('<option/>', {
            value: '',
            text: ''
        }).appendTo(dropdown);

        var index = 0;
        var selectedIndex = 0;
        var defaultIndex = 0;
        var defaultValue = DefaultValueHelper.getDefaultValue(field);
        var useDefault = false;
        if (!Utils.isNull(defaultValue) && model != undefined && (Utils.isNull(model.get(property.attributes.GenericColumnName)) || view.Type != 'Insert')) {
            useDefault = true;
        }

        var returnValue = undefined;
        if (field == ActionHelper.ReturnField) {
            returnValue = ActionHelper.ReturnValue;
            ActionHelper.ReturnValue = undefined;
            ActionHelper.ReturnField = undefined;
            ActionHelper.ReturnID = undefined;
        }

        ////////////////////////
        var isReadonly = field.get('IsReadonly') || Boso.isBosoExampleUser(model);
        if (isReadonly || (view.model != undefined && !view.model.get('IsEditableForCu'))) {
            var value;

            if (returnValue) {
                value = returnValue;
            }


            if (value == undefined && useDefault == true && defaultValue && view.model != undefined && view.model.attributes.IsEditableForCu) {
                value = defaultValue;
            }


            if (value == undefined && view.model != undefined) {
                value = view.model.get(property.attributes.GenericColumnName);
            }

            if (value == undefined && refModelId != 'null') {
                value = refModelId;
            }

            if (value != undefined) {
                lookupEntriesSorted = _.filter(lookupEntriesSorted, function (entity) {
                    return entity.attributes.ObjectId == value;
                });
            }
        }

        var displayProperty1;
        var displayProperty2;
        var displayProperty3;

        if (field.attributes.DisplayProperty1 != undefined && field.attributes.DisplayProperty1 != "00000000-0000-0000-0000-000000000000") {
            displayProperty1 = window.genericProperties.get(field.attributes.DisplayProperty1);
        }

        if (field.attributes.DisplayProperty2 != undefined && field.attributes.DisplayProperty2 != "00000000-0000-0000-0000-000000000000") {
            displayProperty2 = window.genericProperties.get(field.attributes.DisplayProperty2);
        }

        if (field.attributes.DisplayProperty3 != undefined && field.attributes.DisplayProperty3 != "00000000-0000-0000-0000-000000000000") {
            displayProperty3 = window.genericProperties.get(field.attributes.DisplayProperty3);
        }


        _.each(lookupEntriesSorted, function (model) {
            var filter = true;
            if (field.attributes.DropDownFilterJS != null && field.attributes.DropDownFilterJS != '') {
                filter = eval(field.attributes.DropDownFilterJS);
            }

            if (filter) {
                index++
                var displayText1 = ViewBuilder.getDisplayValue(model, displayProperty, field, displayProperty1, displayProperty2, displayProperty3);

                var dropdown_item = $('<option/>', {
                    value: model.attributes.ObjectId,
                    text: displayText1 //model.get(displayProperty.attributes.GenericColumnName)
                }).appendTo(dropdown);

                if (view.model != undefined && model.attributes.ObjectId == view.model.get(property.attributes.GenericColumnName)) {
                    selectedIndex = index;
                } else if (refModelId != 'null' && model.attributes.ObjectId == refModelId) {
                    selectedIndex = index;
                }

                if (model.attributes.ObjectId == defaultValue || model.attributes.ObjectId == returnValue) {
                    defaultIndex = index;
                }
            }
        });

        dropdown.selectmenu(); // Initializes
        dropdown.selectmenu('refresh', true);

        if (dropdown[0] != undefined) {
            dropdown[0].selectedIndex = selectedIndex;
            dropdown.selectmenu("refresh", true);

            if ((returnValue || dropdown[0].selectedIndex == 0 || useDefault) && defaultIndex != 0) {
                dropdown[0].selectedIndex = defaultIndex;
                dropdown.selectmenu("refresh", true);
                //dropdown.change();
            }
        }
        
        var isReadonly = field.get('IsReadonly') || Boso.isBosoExampleUser(model);
        if (isReadonly || (view.model != undefined && !view.model.attributes.IsEditableForCu)) {
            dropdown.selectmenu('disable');
        }

        var actions = _.filter(window.genericViewActions.models, function (action) {
            return field.attributes.FieldId == action.attributes.Selector.toLowerCase() && action.attributes.ActionType == 'lookupbutton' && action.attributes.IsActive == true;
        });
        //'ActionHelper.' + viewAction.attributes.CommandName + ActionHelper.prepareParams(viewAction);
        if (actions != undefined && actions.length > 0) {

            _.each(actions, function (action) {
                var clss = 'btn btn-primary';
                if (action.attributes.CssClass != undefined && action.attributes.CssClass.length > 0) {
                    CssClass = action.attributes.CssClass;
                    var dataid = '';
                    var value = '';
                    CssClass = CssClass.replace(/\./g, ' ');
                    clss += ' ' + CssClass;
                }

                var isVisible = true;
                if (action.attributes.IsVisible != '' && action.attributes.IsVisible != null) {
                    isVisible = eval(action.attributes.IsVisible);
                    //if (!isVisible) $('#' + field.attributes.FieldId).closest('fieldset').hide();
                }
                if (isVisible) {
                    //var button = $('<button/>', {
                    //    type: 'button',
                    //    class: 'btn btn-primary',
                    //    id: action.attributes.ViewActionId,
                    //    text: action.attributes.ActionName,
                    //}).appendTo(td3);

                    var button = $('<button/>', {
                        type: 'button',
                        class: clss,
                        id: action.attributes.ViewActionId,
                        text: action.attributes.ActionName,
                    }).appendTo(fs);

                    button.attr('model', field.attributes.FieldId);

                    var Command = 'ActionHelper.' + action.attributes.CommandName + ActionHelper.prepareParams(action);
                    $(ActionHelper.CurrentView.el).delegate("#" + action.attributes.ViewActionId, action.attributes.Event, function () {
                        eval(Command);
                    });
                }
            });
        }

        var tooltip = null;
        if (field.attributes.ToolTip != null && field.attributes.ToolTip != '') {
            tooltip = field.attributes.ToolTip;
        } else if (property.attributes.ToolTip != null && property.attributes.ToolTip != '') {
            tooltip = property.attributes.ToolTip;
        }
        if (tooltip != null) {
            var tooltip = $('<label/>', {
                class: 'tooltip',
                text: tooltip,
            }).appendTo(fs);
        }

        var error = $('<label/>', {
            style: 'display: none',
            class: 'error',
            id: 'error_' + field.attributes.FieldId,
        }).appendTo(fss);
    },

    /**
     * Adds a list with suggestions to a nvarchar-field.
     */
    create_autocomplete: function (view, property, field, type, autocompleteGroup) {
        var EntityId_BASE_AutoCompleteHelper = 'eb57edf5-3d65-e3c9-d7f9-d68d2943f900';
        var lookupEntries = window.genericData.where({
            IsActive: true,
            EntityId: EntityId_BASE_AutoCompleteHelper,
            Custom_001: autocompleteGroup
        });
        var lookupEntriesSorted = _.sortBy(lookupEntries, function (entry) {
            return entry.get('Custom_002');
        });

        root = $(view.el).find('#' + type + 'Container');

        var fc = $('<div/>', {
            class: 'ui-field-contain'
        }).appendTo(root);

        // Field-Label hinzufügen
        var label = property.get('PropertyName');
        if (field.get('ViewFieldLabel') != null && field.get('ViewFieldLabel') != "") {
            label = field.get('ViewFieldLabel');
        }
        var lable = $('<label/>', {
            text: label + ": ",
            for: field.get('FieldId'),
        }).appendTo(fc);

        // Autocomplete-Textfield erzeugen
        var suggestionContainer = $('<div />', {
            class: 'suggestionContainer',
            id: 'tmp' + field.get('FieldId')
        }).appendTo(fc);

        var ul = $('<ul/>', {
            //class: 'ui-listview ui-listview-inset ui-corner-all ui-shadow',
        }).appendTo(suggestionContainer);

        ul.attr('data-role', 'listview');
        ul.attr('data-inset', true);
        ul.attr('data-filter', true);
        ul.attr('data-filter-reveal', true);
        ul.attr('data-filter-placeholder', '');

        // Vorschlag-Liste erzeugen
        for (var i = 0; i < lookupEntriesSorted.length; i++) {
            var li = $('<li/>', {
                //class: 'ui-btn ui-btn-up-c ui-btn-icon-right ui-li-has-arrow ui-li ui-first-child ui-screen-hidden'
            }).appendTo(ul);

            var a = $('<a />', {
                text: lookupEntriesSorted[i].get('Custom_002')
            }).appendTo(li);
        }

        //Eventhandler für die Listelemente. Bei Klick wird der Text des Listitems in das Suchfeld eingetragen
        $('.suggestionContainer > ul > li > a').click(function () {
            var target = $(this).closest('.suggestionContainer').find('input[data-type="search"]');
            target.val($(this).text());
            target.change();
        });

        $('.suggestionContainer').on('click', 'a[title="Clear text"]', function () {
            $(this).removeAttr('href');
        })

        // Setze die ids für die Input-Felder
        setTimeout(function () {
            $('#tmp' + field.get('FieldId')).find('input[data-type="search"]').attr('id', field.get('FieldId'));
        }, 0);
    },

    /**
     * Creates a textfield for a given property of type 'nvarchar'
     */
    create_nvarchar: function (view, property, field, type) {

        if (!Utils.isNull(property.get('AutoCompleteGroup'))) {
            this.create_autocomplete(view, property, field, type, property.get('AutoCompleteGroup'))
            return;
        }

        var model = view.model;

        root = $(view.el).find('#' + type + 'Container');

        var actions = _.filter(window.genericViewActions.models, function (action) {
            return field.attributes.FieldId == action.attributes.Selector.toLowerCase() && action.attributes.ActionType == 'lookupbutton' && action.attributes.IsActive == true;
        });

        var fc = $('<div/>', {
            class: 'ui-field-contain'
        }).appendTo(root);

        var label = property.attributes.PropertyName;
        if (field.attributes.ViewFieldLabel != null && field.attributes.ViewFieldLabel != "") {
            label = field.attributes.ViewFieldLabel;
        }

        if (field.attributes.RequiredField)
            label = label + '*';

        var lable = $('<label/>', {
            text: label + ": ",
            for: field.attributes.FieldId,
        }).appendTo(fc);

        var clss = 'textarea';
        if (field.attributes.CssClass != undefined && field.attributes.CssClass.length > 0) {
            CssClass = field.attributes.CssClass;
            var dataid = '';
            var value = '';
            CssClass = CssClass.replace(/\./g, ' ');
            clss += ' ' + CssClass;
        }
        if (property.attributes.Size == CONSTS.INTEGER_DEFAULT) {
            var input = $('<textarea/>', {
                id: field.attributes.FieldId,
                name: property.attributes.GenericColumnName,
                class: clss
            }).appendTo(fc);
            input.attr('data-role', 'none');

            fc.addClass('textareacontainer');
            fc.attr('id', 'field-' + field.attributes.FieldId);

            _.each(actions, function (action) {
                var clss = 'ui-btn ui-btn-inline';
                if (action.attributes.CssClass != undefined && action.attributes.CssClass.length > 0) {
                    CssClass = action.attributes.CssClass;
                    var dataid = '';
                    var value = '';
                    CssClass = CssClass.replace(/\./g, ' ');
                    clss += ' ' + CssClass;
                }



                var button = $('<button/>', {
                    id: action.attributes.ViewActionId,
                    text: action.attributes.ActionName,
                    model: field.attributes.FieldId,
                    class: clss
                }).appendTo(fc);

                var Command = 'ActionHelper.' + action.attributes.CommandName + ActionHelper.prepareParams(action);
                $(ActionHelper.CurrentView.el).delegate("#" + action.attributes.ViewActionId, action.attributes.Event, function () {
                    eval(Command);
                });
            });

            $('<div class="listcontentclear">').appendTo(fc);

        } else {
            if (actions != undefined && actions.length > 0) {
                var cntrlContainer = $('<div/>', {
                    class: 'mf-controlgroup',
                }).appendTo(fc);

                cntrlContainer.attr('data-role', 'controlgroup');
                cntrlContainer.attr('data-type', 'horizontal');

                var input = $('<input/>', {
                    id: field.attributes.FieldId,
                    name: property.attributes.GenericColumnName,
                    //maxlength: property.attributes.Size,
                    type: 'text',
                    class: 'mf-button-field'
                }).appendTo(cntrlContainer);

                input.attr('data-wrapper-class', 'controlgroup-textinput ui-btn');

                _.each(actions, function (action) {
                    var clss = '';
                    if (action.attributes.CssClass != undefined && action.attributes.CssClass.length > 0) {
                        CssClass = action.attributes.CssClass;
                        var dataid = '';
                        var value = '';
                        CssClass = CssClass.replace(/\./g, ' ');
                        clss += ' ' + CssClass;
                    }

                    var button = $('<button/>', {
                        id: action.attributes.ViewActionId,
                        text: action.attributes.ActionName,
                        model: field.attributes.FieldId,
                        class: clss,
                    }).appendTo(cntrlContainer);

                    var Command = 'ActionHelper.' + action.attributes.CommandName + ActionHelper.prepareParams(action);
                    $(ActionHelper.CurrentView.el).delegate("#" + action.attributes.ViewActionId, action.attributes.Event, function () {
                        eval(Command);
                    });
                });
            } else {
                var input = $('<input/>', {
                    id: field.attributes.FieldId,
                    name: property.attributes.GenericColumnName,
                    maxlength: property.attributes.Size,
                }).appendTo(fc);
            }
        }

        if (model != undefined)
            input.val(model.get(property.attributes.GenericColumnName));

        if (input.val() == '' || input.val() == null) {
            var value = DefaultValueHelper.getDefaultValue(field);
            input.val(value);
        }

        if (field == ActionHelper.ReturnField && ActionHelper.ReturnValue != null) {
            var val = input.val();
            if (val == null || val == ' ') {
                val = '';
            }
            if (val.length != 0) {
                val = val.trim();
                val = val + ' ';
            }
            defaultValue = ActionHelper.ReturnValue;
            ActionHelper.ReturnValue = undefined;
            ActionHelper.ReturnField = undefined;
            ActionHelper.ReturnID = undefined;
            input.val(val + defaultValue);
        }

        var isReadonly = field.get('IsReadonly') || Boso.isBosoExampleUser(model);
        if (isReadonly || (model != undefined && (model != null && !model.attributes.IsEditableForCu))) {
            input.attr('readonly', true);
            input.attr('disabled', 'disabled');
            input.addClass('readonly');
        }

        //'ActionHelper.' + viewAction.attributes.CommandName + ActionHelper.prepareParams(viewAction);
        var tooltip = null;
        if (field.attributes.ToolTip != null && field.attributes.ToolTip != '') {
            tooltip = field.attributes.ToolTip;
        } else if (property.attributes.ToolTip != null && property.attributes.ToolTip != '') {
            tooltip = property.attributes.ToolTip;
        }
        if (tooltip != null) {
            var tooltip = $('<label/>', {
                class: 'tooltip',
                text: tooltip,
            }).appendTo(fc);
        }

        var error = $('<label/>', {
            style: 'display: none',
            class: 'error',
            id: 'error_' + field.get('FieldId'),
        }).appendTo(fc);
    },


    create_int: function (view, property, field, type) {
        ViewBuilder.create_decimal(view, property, field, type);
    },
    /**
     * Creates a standard input field for a given property without a type
     */
    create_decimal: function (view, property, field, type) {

        var model = view.model;

        root = $(view.el).find('#' + type + 'Container');

        var fc = $('<div/>', {
            class: 'ui-field-contain'
        }).appendTo(root);

        var label = property.attributes.PropertyName;
        if (field.attributes.ViewFieldLabel != null && field.attributes.ViewFieldLabel != "") {
            label = field.attributes.ViewFieldLabel;
        }

        if (field.attributes.RequiredField)
            label = label + '*';

        var lable = $('<label/>', {
            class: 'select',
            text: label + ": ",
            for: field.attributes.FieldId,
        }).appendTo(fc);

        var input = $('<input/>', {
            id: field.attributes.FieldId,
            name: property.attributes.GenericColumnName,
            type: 'number',
        }).appendTo(fc);

        if (model != null) {
            var value = model.get(property.attributes.GenericColumnName);

            if (value != null && value !== '' && value != undefined) {
                var scale = property.get('Scale');
                if (scale == undefined || scale == '' || scale < 0)
                    scale = '0';
                if (property.attributes.DataType == 'decimal')
                    value = parseFloat(value).toFixed(scale);
                else
                    value = parseFloat(value).toFixed();
                value = FormatHelper.getDecimalString(value); // value.toString().replace('.', ',');
            }

            input.val(value);
        }
        if (input.val() == '' || input.val() == null) {
            var value = DefaultValueHelper.getDefaultValue(field);
            input.val(value);
        }

        var isReadonly = field.get('IsReadonly') || Boso.isBosoExampleUser(model);
        if (isReadonly || (model != null && !model.attributes.IsEditableForCu)) {
            input.attr('readonly', true);
            input.attr('disabled', true);
            input.attr('class', 'readonly');
        }

        var tooltip = null;
        if (field.attributes.ToolTip != null && field.attributes.ToolTip != '') {
            tooltip = field.attributes.ToolTip;
        } else if (property.attributes.ToolTip != null && property.attributes.ToolTip != '') {
            tooltip = property.attributes.ToolTip;
        }
        if (tooltip != null) {
            var tooltip = $('<label/>', {
                class: 'tooltip',
                text: tooltip,
            }).appendTo(fc);
        }

        var error = $('<label/>', {
            style: 'display: none',
            class: 'error',
            id: 'error_' + field.attributes.FieldId,
        }).appendTo(fc);

    },

    /**
     * Creates a standard input field for a given property without a type
     */
    createStandardField: function (view, property, field, type) {

        var model = view.model;

        root = $(view.el).find('#' + type + 'Container');

        var fs = $('<fieldset/>', {}).appendTo(root);
        fs.attr('data-role', 'fieldcontain');

        var lable = $('<label/>', {
            class: 'select',
            text: property.attributes.PropertyName + ": ",
            for: field.attributes.FieldId,
        }).appendTo(fs);

        var input = $('<input/>', {
            id: field.attributes.FieldId,
            name: property.attributes.GenericColumnName,
        }).appendTo(fs);

        if (model != null) {
            if (model != undefined)
                input.val(model.get(property.attributes.GenericColumnName));

            if (input.val() == '' || input.val() == null) {
                var value = DefaultValueHelper.getDefaultValue(field);
                input.val(value);
            }
        }

        var isReadonly = field.get('IsReadonly') || Boso.isBosoExampleUser(model);
        if (isReadonly || (model != null && !model.attributes.IsEditableForCu)) {
            input.attr('readonly', true);
            input.attr('disabled', true);
            input.attr('class', 'readonly');
        }

        var tooltip = null;
        if (field.attributes.ToolTip != null && field.attributes.ToolTip != '') {
            tooltip = field.attributes.ToolTip;
        } else if (property.attributes.ToolTip != null && property.attributes.ToolTip != '') {
            tooltip = property.attributes.ToolTip;
        }
        if (tooltip != null) {
            var tooltip = $('<label/>', {
                class: 'tooltip',
                text: tooltip,
            }).appendTo(fs);
        }

        var error = $('<label/>', {
            style: 'display: none',
            class: 'error',
            id: 'error_' + field.attributes.FieldId,
        }).appendTo(fs);

        //Styles

        $(lable).css('width', '150px');

        $(input).css('width', '55%');




    },

    /**
     * Creates a standard input field for a given property without a type
     */
    create_identity: function (view, property, field, type) {

        var model = view.model;

        root = $(view.el).find('#' + type + 'Container');

        var fs = $('<fieldset/>', {}).appendTo(root);
        fs.attr('data-role', 'fieldcontain');
        fs.attr('fieldtype', 'identity');

        var label = property.attributes.PropertyName;
        if (field.attributes.ViewFieldLabel != null && field.attributes.ViewFieldLabel != "") {
            label = field.attributes.ViewFieldLabel;
        }

        if (field.attributes.RequiredField)
            label = label + '*';

        var lable = $('<label/>', {
            class: 'select',
            text: label + ": ",
            for: field.attributes.FieldId,
        }).appendTo(fs);

        var input = $('<input/>', {
            id: field.attributes.FieldId,
            name: property.attributes.GenericColumnName,
        }).appendTo(fs);

        if (model != undefined)
            input.val(model.get(property.attributes.GenericColumnName));

        if (input.val() == '' || input.val() == null) {
            var value = DefaultValueHelper.getDefaultValue(field);
            input.val(value);
        }

        var tooltip = null;
        if (field.attributes.ToolTip != null && field.attributes.ToolTip != '') {
            tooltip = field.attributes.ToolTip;
        } else if (property.attributes.ToolTip != null && property.attributes.ToolTip != '') {
            tooltip = property.attributes.ToolTip;
        }
        if (tooltip != null) {
            var tooltip = $('<label/>', {
                class: 'tooltip',
                text: tooltip,
            }).appendTo(fs);
        }

        input.attr('readonly', true);
        input.attr('disabled', true);
        input.attr('class', 'readonly');

        //Styles
        //$(lable).css('width', '150px');
    },

    /**
     * Creates a dropdown for a given property of type 'user'
     */
    create_user: function (view, property, field, type) {
        var model = view.model;
        var refModelId = view.options.refModelId

        root = $(view.el).find('#' + type + 'Container');

        var fs = $('<div/>', {
            class: 'ui-field-contain'
        }).appendTo(root);

        var label = property.attributes.PropertyName;
        if (field.attributes.ViewFieldLabel != null && field.attributes.ViewFieldLabel != "") {
            label = field.attributes.ViewFieldLabel;
        }

        if (field.attributes.RequiredField)
            label = label + '*';

        var lable = $('<label/>', {
            text: label + ": ",
            for: field.attributes.FieldId,
        }).appendTo(fs);

        var dropdown = $('<select/>', {
            id: field.attributes.FieldId,
            name: property.attributes.GenericColumnName,
        }).appendTo(fs);

        var dropdown_null_item = $('<option/>', {
            value: '',
            text: ''
        }).appendTo(dropdown);

        var index = 0;
        var selectedIndex = 0;
        var defaultIndex = 0;
        var defaultValue = DefaultValueHelper.getDefaultValue(field);
        _.each(window.userRawElementEntries.models, function (entry) {
            index++
            var dropdown_item = $('<option/>', {
                value: entry.attributes.UserId,
                text: entry.attributes.DisplayName,
            }).appendTo(dropdown);

            if (model != undefined && entry.attributes.UserId == model.get(property.attributes.GenericColumnName)) {
                selectedIndex = index;
            }
            if (entry.attributes.UserId == defaultValue) {
                defaultIndex = index;
            }
        });
        dropdown.selectmenu();
        dropdown[0].selectedIndex = selectedIndex;
        dropdown.selectmenu("refresh", true);

        if (dropdown[0].selectedIndex == 0 && defaultIndex != 0) {
            dropdown[0].selectedIndex = defaultIndex;
            dropdown.selectmenu("refresh", true);
        }

        var isReadonly = field.get('IsReadonly') || Boso.isBosoExampleUser(model);
        if (isReadonly || (model != null && !model.attributes.IsEditableForCu)) {
            dropdown.selectmenu('disable');
        }

        var tooltip = null;
        if (field.attributes.ToolTip != null && field.attributes.ToolTip != '') {
            tooltip = field.attributes.ToolTip;
        } else if (property.attributes.ToolTip != null && property.attributes.ToolTip != '') {
            tooltip = property.attributes.ToolTip;
        }
        if (tooltip != null) {
            var tooltip = $('<label/>', {
                class: 'tooltip',
                text: tooltip,
            }).appendTo(fs);
        }

        var error = $('<label/>', {
            style: 'display: none',
            class: 'error',
            id: 'error_' + field.attributes.FieldId,
        }).appendTo(fs);

    },

    /**
     * Creates a checkbox for a given property of type 'bit'
     */
    create_bit: function (view, property, field, type) {

        var model = view.model;

        root = $(view.el).find('#' + type + 'Container');

        var fs = $('<fieldset/>', {}).appendTo(root);
        fs.attr('data-role', 'controlgroup');

        var label = property.attributes.PropertyName;
        if (field.attributes.ViewFieldLabel != null && field.attributes.ViewFieldLabel != "") {
            label = field.attributes.ViewFieldLabel;
        }

        if (field.attributes.RequiredField)
            label = label + '*';

        var lable = $('<label/>', {
            class: 'select',
            text: label,
            for: field.attributes.FieldId,
        }).appendTo(fs);

        var input = $('<input/>', {
            id: field.attributes.FieldId,
            name: property.attributes.GenericColumnName,
            type: 'checkbox',
        }).appendTo(fs);

        if (model != undefined)
            var bit = model.get(property.attributes.GenericColumnName);

        if (bit != 1) {
            input.removeAttr('checked');
        } else {
            input.attr('checked', 'true');
        }

        if (input.val() == '' || input.val() == null) {
            var value = DefaultValueHelper.getDefaultValue(field);
            if (value != '' && value != null) {
                if (value != 1) {
                    input.removeAttr('checked');
                } else {
                    input.attr('checked', 'true');
                }
            }
        }

        var isReadonly = field.get('IsReadonly') || Boso.isBosoExampleUser(model);
        if (isReadonly || (model != null && !model.attributes.IsEditableForCu)) {
            input.attr('readonly', true);
            input.attr('disabled', true);
            input.attr('class', 'readonly');
        }

        var error = $('<label/>', {
            style: 'display: none',
            class: 'error',
            id: 'error_' + field.attributes.FieldId,
        }).appendTo(fs);

        //Styles

        $(lable).css('min-width', '150px');
    },

    create_decision: function (view, property, field, type) {
        var BOSO_fieldId_ConnectGoogleInsertView = '5433d09d-ac9c-4c5c-bba6-71689f676880'
        var BOSO_fieldId_ConnectGoogleEditView = '14e7d09d-ac9c-4b35-a979-d1689f662d75'
        var BOSO_fieldId_ConnectGoogleListView = '16aed09d-ac9c-493d-a9f4-71689f5d2067'

        if (!!field && !!field.id &&
            (field.id === BOSO_fieldId_ConnectGoogleInsertView 
            || field.id === BOSO_fieldId_ConnectGoogleEditView
            || field.id === BOSO_fieldId_ConnectGoogleListView )) 
        {
            return;
        }

        var model = view.model;
        root = $(view.el).find('#' + type + 'Container');

        var container = $('<div />', {
            class: 'ui-field-contain'
        }).appendTo(root);

        var fs = $('<fieldset />', {}).appendTo(container);
        fs.attr('data-role', 'controlgroup');
        fs.attr('data-type', 'horizontal');
        //fs.attr('data-mini', 'true');
        fs.attr('fieldtype', "decision");

        var labelText = property.get('PropertyName');
        if (field.get('ViewFieldLabel')) {
            labelText = field.get('ViewFieldLabel');
        }

        if (field.get('RequiredField')) {
            labelText = labelText + '*';
        }

        var label = $('<legend />', {
            text: labelText + ':',
        });
        label.appendTo(fs);

        var hiddenField = $('<input />', {
            id: field.get('FieldId'),
            type: 'hidden'
        }).appendTo(fs);

        var checked = null;
        if (model && property && property.get('GenericColumnName')) {
            checked = model.get(property.get('GenericColumnName'));
        }

        var inputYesChecked = false;
        var inputNoChecked = false;
        if (checked !== null) {
            inputYesChecked = checked === '1';
            inputNoChecked = checked !== '1';
        } else {
            inputYesChecked = field.get('DefaultValue') === '0'
            inputNoChecked = field.get('DefaultValue') === '1'
        }

        var defaultFields = Utils.getText("JA") + "|" + Utils.getText("NEIN");
        var configFieldList = property.get('Param_01');
        var fieldList = defaultFields

        if (configFieldList && configFieldList.split("|").length > 1) {
            fieldList = configFieldList;
        }

        var fieldsArray = fieldList.split("|");

        var input_item = $('<input />', {
            name: field.get('FieldId'),
            type: 'radio',
            value: 'true',
            id: 'radioTrue' + field.get('FieldId'),
        }).appendTo(fs);

        var labelYes = $('<label />', {
            class: 'decision',
            text: fieldsArray[0],
            for: 'radioTrue' + field.get('FieldId'),
            id: 'lblradioTrue' + field.get('FieldId'),
        }).appendTo(fs);

        if (inputYesChecked) {
            $(input_item).prop('checked', 'checked');
            $(labelYes).addClass('ui-btn-active');
        }

        var input_item2 = $('<input />', {
            //id: field.attributes.FieldId,
            name: field.get('FieldId'),
            type: 'radio',
            value: 'false',
            id: 'radioFalse' + field.get('FieldId'),
        }).appendTo(fs);

        var labelNo = $('<label/>', {
            class: 'decision',
            text: fieldsArray[1],
            for: 'radioFalse' + field.get('FieldId'),
            id: 'lblradioFalse' + field.get('FieldId'),
        }).appendTo(fs);

        if (inputNoChecked) {
            $(input_item2).prop('checked', 'checked');
            $(labelNo).addClass('ui-btn-active');
        }

        if (inputYesChecked) {
            hiddenField.val('true');
        } else if (inputNoChecked) {
            hiddenField.val('false');
        }

        var tooltip = field.get('ToolTip') ? field.get('ToolTip') : property.get('ToolTip');
        if (tooltip) {
            $('<label />', {
                class: 'tooltip',
                text: tooltip,
            }).appendTo(container);
        }

        //hidden Error Field
        $('<label />', {
            style: 'display: none',
            class: 'error',
            id: 'error_' + field.get('FieldId'),
        }).appendTo(container);

        var setSciTimActivationDate = function(reset) {
            var fieldIdInsert = '6d84d09d-ac9c-4e0d-a9b6-c1699b0639c4';
            var fieldIdEdit = '9c00d09d-ac9c-4c83-812c-31699b08744b'

            var sciTimActivatedOnFieldId = null;
            switch (view.Type) {
                case "Insert":
                    sciTimActivatedOnFieldId = fieldIdInsert;
                    break;
                case "Edit":
                    sciTimActivatedOnFieldId = fieldIdEdit;
                    break;
                default:
                    sciTimActivatedOnFieldId = fieldIdEdit;
            }

            var activationDateField = $('#' + sciTimActivatedOnFieldId);
            if (!!activationDateField && activationDateField.length === 1) {
                if (!!reset && reset === true) {
                    activationDateField.val(null);
                    return;
                }

                var currentDateTime = moment();
                activationDateField.val(currentDateTime.format(FormatHelper.get_DATE_FORMAT()));
                activationDateField.scroller('setDate', currentDateTime.toDate(), true);
            }
        }

        var getSciTIMActivationDate = function() {
            var returnValue = '';
            var fieldIdInsert = '6d84d09d-ac9c-4e0d-a9b6-c1699b0639c4';
            var fieldIdEdit = '9c00d09d-ac9c-4c83-812c-31699b08744b'

            var sciTimActivatedOnFieldId = null;
            switch (view.Type) {
                case "Insert":
                    sciTimActivatedOnFieldId = fieldIdInsert;
                    break;
                case "Edit":
                    sciTimActivatedOnFieldId = fieldIdEdit;
                    break;
                default:
                    sciTimActivatedOnFieldId = fieldIdEdit;
            }

            var activationDateField = $('#' + sciTimActivatedOnFieldId);
            if (!!activationDateField && activationDateField.length === 1) { 
                returnValue = activationDateField.val();
            }

            return returnValue;
        }

        var setCasProZdraviActivationDate = function(reset) {
            var fieldIdInsert = 'd3dc3987-0c1a-4757-95ca-716e17efb36f';
            var fieldIdEdit = '26443987-0c1a-49dd-8596-b16e17ee635a';

            var casProZdraviActivatedOnFieldId = null;
            switch (view.Type) {
                case "Insert":
                    casProZdraviActivatedOnFieldId = fieldIdInsert;
                    break;
                case "Edit":
                    casProZdraviActivatedOnFieldId = fieldIdEdit;
                    break;
                default:
                    casProZdraviActivatedOnFieldId = fieldIdEdit;
            }

            var activationDateField = $('#' + casProZdraviActivatedOnFieldId);
            if (!!activationDateField && activationDateField.length === 1) {
                if (!!reset && reset === true) {
                    activationDateField.val(null);
                    return;
                }

                var currentDateTime = moment();
                activationDateField.val(currentDateTime.format(FormatHelper.get_DATE_FORMAT()));
                activationDateField.scroller('setDate', currentDateTime.toDate(), true);
            }
        }

        var getCasProZdraviActivationDate = function() {
            var returnValue = '';
            var fieldIdInsert = 'd3dc3987-0c1a-4757-95ca-716e17efb36f';
            var fieldIdEdit = '26443987-0c1a-49dd-8596-b16e17ee635a'

            var casProZdraviActivatedOnFieldId = null;
            switch (view.Type) {
                case "Insert":
                    casProZdraviActivatedOnFieldId = fieldIdInsert;
                    break;
                case "Edit":
                    casProZdraviActivatedOnFieldId = fieldIdEdit;
                    break;
                default:
                    casProZdraviActivatedOnFieldId = fieldIdEdit;
            }

            var activationDateField = $('#' + casProZdraviActivatedOnFieldId);
            if (!!activationDateField && activationDateField.length === 1) { 
                returnValue = activationDateField.val();
            }

            return returnValue;
        }

        var fnInputItemClick = function () {
            var label = $('#lblradioTrue' + field.get('FieldId'));
            var label2 = $('#lblradioFalse' + field.get('FieldId'));

            var moveSwitch = function () {
                $(label).addClass('ui-btn-active');
                $(input_item).attr('checked', true);
                
                $(label2).removeClass('ui-btn-active');
                $(input_item2).attr('checked', false);
                hiddenField.val('true');

                var focusedElement = $(':focus');
                focusedElement.blur();
            }

            if (!$(label).hasClass('ui-btn-active')) {
                var PROPERTYID_CONNECT_TO_SCITIM = '47093987-0C1A-4128-98F4-D168A37B1B58';
                var PROPERTYID_CONNECT_TO_CASPROZDRAVI = 'd1023987-0c1a-4820-9a53-716e17e7642c';
                if (GlobalSettings.IsBoso() && field.get('PropertyId') && field.get('PropertyId').toLowerCase() === PROPERTYID_CONNECT_TO_SCITIM.toLowerCase()) {
                    if (!!getCasProZdraviActivationDate()) {
                        var msgString = Utils.getText('BOSO_OTHER_SERVICE_ALREADY_ACTIVE');
                        msgString = msgString.replace('{SERVICE1}', 'SciTIM');
                        msgString = msgString.replace('{SERVICE2}', 'Čas pro zdraví');

                        alert(msgString);
                        $(input_item).attr('checked', false);
                    } else {
                        Boso.requestSciTimPassword(function (result) {
                            if (result === true) {
                                setSciTimActivationDate();
                                moveSwitch();
                            } else {
                                $(input_item).attr('checked', false);
                            }
                        });
                    }
                } else if (GlobalSettings.IsBoso() && field.get('PropertyId') && field.get('PropertyId').toLowerCase() === PROPERTYID_CONNECT_TO_CASPROZDRAVI.toLowerCase()) {
                    if (!!getSciTIMActivationDate()) {
                        var msgString = Utils.getText('BOSO_OTHER_SERVICE_ALREADY_ACTIVE');
                        msgString = msgString.replace('{SERVICE1}', 'Čas pro zdraví');
                        msgString = msgString.replace('{SERVICE2}', 'SciTIM');

                        alert(msgString);
                        $(input_item).attr('checked', false);
                    } else {
                        Boso.requestCasProZdraviId(function (result) {
                            if (result === true) {
                                setCasProZdraviActivationDate();
                                moveSwitch();
                            } else {
                                $(input_item).attr('checked', false);
                            }
                        });
                    }
                } else {
                    moveSwitch();
                }
            }
            return false;
        };

        var fnInputItem2Click = function () {
            var label = $('#lblradioFalse' + field.get('FieldId'));
            var label2 = $('#lblradioTrue' + field.get('FieldId'));

            var moveSwitch = function () {
                $(label).addClass('ui-btn-active');
                $(label2).removeClass('ui-btn-active');

                $(input_item2).attr('checked', true);
                $(input_item).attr('checked', false);
                hiddenField.val('false');

                var focusedElement = $(':focus');
                focusedElement.blur();
            }

            if (!$(label).hasClass('ui-btn-active')) {
                var PROPERTYID_CONNECT_TO_SCITIM = '47093987-0C1A-4128-98F4-D168A37B1B58';
                var PROPERTYID_CONNECT_TO_CASPROZDRAVI = 'd1023987-0c1a-4820-9a53-716e17e7642c';
                if (GlobalSettings.IsBoso() && field.get('PropertyId') && field.get('PropertyId').toLowerCase() === PROPERTYID_CONNECT_TO_SCITIM.toLowerCase()) {
                    setSciTimActivationDate(true);
                    moveSwitch();
                } else if (GlobalSettings.IsBoso() && field.get('PropertyId') && field.get('PropertyId').toLowerCase() === PROPERTYID_CONNECT_TO_CASPROZDRAVI.toLowerCase()) {
                    setCasProZdraviActivationDate(true);
                    moveSwitch();
                } else {
                    moveSwitch();
                }
            }
        };

        //Events
        var isReadonly = field.get('IsReadonly') || Boso.isBosoExampleUser(model);
        if (!isReadonly || (model != undefined && model.get('IsEditableForCu'))) {
            $(input_item).off('click', fnInputItemClick).on('click', fnInputItemClick);
            $(input_item2).off('click', fnInputItem2Click).on('click', fnInputItem2Click);
        }

        if (isReadonly || (model != undefined && !model.get('IsEditableForCu'))) {
            $(input_item).prop("disabled", true);
            $(input_item2).prop("disabled", true);
        }
    },


    /**
     * Creates a link with href mobilenummer or if not readonly a text field
     */
    create_tel: function (view, property, field, type) {

        var model = view.model;
        root = $(view.el).find('#' + type + 'Container');

        var fs = $('<div/>', {
            class: 'ui-field-contain'
        }).appendTo(root);

        var value = "";
        if (model != undefined) {
            value = model.get(property.attributes.GenericColumnName);
        }

        if (value == '' || value == null) {
            value = DefaultValueHelper.getDefaultValue(field);
        }

        var label = property.attributes.PropertyName;
        if (field.attributes.ViewFieldLabel != null && field.attributes.ViewFieldLabel != "") {
            label = field.attributes.ViewFieldLabel;
        }

        if (field.attributes.RequiredField)
            label = label + '*';

        var lable = $('<label />', {
            text: label + ": ",
            for: field.attributes.FieldId,
        }).appendTo(fs);

        var isReadonly = field.get('IsReadonly') || Boso.isBosoExampleUser(model);
        if (isReadonly || (model != undefined && !model.attributes.IsEditableForCu)) { //Nur Telefonnummer anzeigen

            var fieldElem = $('<a/>', {
                id: field.attributes.FieldId,
                name: property.attributes.GenericColumnName,
                href: 'tel:' + value,
                text: value
            }).appendTo(fs);
        } else {
            var input = $('<input/>', {
                id: field.attributes.FieldId,
                type: 'text',
                name: property.attributes.GenericColumnName,
            }).appendTo(fs);
            $(input).val(value);
        }

        var tooltip = null;
        if (field.attributes.ToolTip != null && field.attributes.ToolTip != '') {
            tooltip = field.attributes.ToolTip;
        } else if (property.attributes.ToolTip != null && property.attributes.ToolTip != '') {
            tooltip = property.attributes.ToolTip;
        }
        if (tooltip != null) {
            var tooltip = $('<label/>', {
                class: 'tooltip',
                text: tooltip,
            }).appendTo(fs);
        }

        var error = $('<label/>', {
            style: 'display: none',
            class: 'error',
            id: 'error_' + field.attributes.FieldId,
        }).appendTo(fs);
    },

    /**
     * Creates a AssociatedView infield
     */
    create_subView: function (view, field, type) {
        var associatedViewFieldId = field.attributes.AssociatedViewField;

        var associatedViewField = window.genericViewFields.get(associatedViewFieldId);
        if (associatedViewField == undefined)
            return true;

        var associatedView = window.genericViews.get(associatedViewField.attributes.ViewId);

        if (field.attributes.PropertyId == CONSTS.GUID_DEFAULT) {
            if (associatedView.attributes.ViewType == "List") {

                var refModelId = view.model.id;
                var modelId = null;
                var subView = jQuery.extend(true, {}, new window.ListView({
                    view: associatedView,
                    modelId: modelId,
                    refModelId: refModelId
                }));

                root = $(view.el).find('#' + type + 'Container');
                var subViewContainer = $('<div/>', {
                    class: 'suviewContainer' + refModelId,
                }).appendTo(root);

                subView.isSubView = true;
                subView.render();

                $(subViewContainer).append($(subView.el));
            }
        }
    }
};