﻿/**
* Backbone Model für die Entität GenericData
*
* @class GenericDataRawElement.js
*/
function generateModelGenericDataRawElement() {
    window.GenericDataRawElement = Backbone.Model.extend({
        idAttribute: 'DataId',
        PersistToLocalStorageImmediately: true,
        defaults: {
            SyncTime: CONSTS.DATE_DEFAULT,
            IsEditableForCu: true,
            IsBlocked: false,

            DataId: CONSTS.GUID_DEFAULT,
            ObjectId: CONSTS.GUID_DEFAULT,
            MandatorId: CONSTS.GUID_DEFAULT,
            OrgUnitId: CONSTS.GUID_DEFAULT,
            UserId: CONSTS.GUID_DEFAULT,
            EntityId: CONSTS.GUID_DEFAULT,
            IsActive: false,

            CreatedBy: CONSTS.GUID_DEFAULT,
            CreatedDate: CONSTS.DATE_DEFAULT,
            ModifiedBy: CONSTS.GUID_DEFAULT,
            ModifiedDate: CONSTS.DATE_DEFAULT,
            TriggerStamp: CONSTS.DATE_DEFAULT,
            hasChanged: false,
        },
        localStorage: new Store("GenericData"),
        initialize: function () {
            //this.bind("saved", this.syncWithServer, this);
        },
        syncWithServer: function () {
            if (this.toBeSynchronized()) {
                SyncController.syncData(false);
            }
        },
        toBeSynchronized: function () {
            return (this.attributes.SyncTime == null || this.attributes.SyncTime.getTime() === CONSTS.DATE_DEFAULT.getTime());
        },
        set: function (attributes, options) {
            if (!this.id) this.id = attributes.DataId;

            if (attributes.SyncTime) {
                attributes.SyncTime = moment(attributes.SyncTime).toDate();
            }
            var attrString = attributes.toString();
            if (attrString != null &&
                (attrString.match(/Custom*/) ||
                attrString.match(/IsActive*/))) {
                var value = this.attributes[attrString];
                if (this.attributes.hasChanged == false && value != options) {
                    if (!(options == '' && value == null) && !(options == null && value == '')) {
                        this.attributes.hasChanged = true;
                    }
                }
            }

            Backbone.Model.prototype.set.call(this, attributes, options);
        },

        toJSON: function () {
            var json = Backbone.Model.prototype.toJSON.call(this);
            return _.extend(json);
        },

        saveItem: function () {     	
            var success = true;
            var changed = this.attributes.hasChanged;
            var self = this;
            var actionHelper = ActionHelper;
            if (changed || self.IsSync == true || self.IsNew) {
                if (actionHelper.CurrentView.options.view != null && self.IsSync != true) {
                    actionHelper.IsValid = ValidationHelper.validate(actionHelper.CurrentView);
                }

                if (actionHelper.IsValid) {
                    actionHelper.globalBeforeSave(self);
                    if (actionHelper.IsValid) {
                        actionHelper.globalOnSave(self);
                        self.attributes.ModifiedBy = window.currentUser.UserId;;
                        self.attributes.ModifiedDate = moment().format(CONSTS.DATETIME_PERSIST_FORMAT, 'de');
                        self.IsSync = false;
                        var persistToLocalStorageImmediately = self.PersistToLocalStorageImmediately;
                        if (self.IsNew) {
                            //self.save();
                            window.genericData.add(self);
                            self.IsNew = false;
                            if (persistToLocalStorageImmediately != undefined) {
                                self.PersistToLocalStorageImmediately = persistToLocalStorageImmediately;
                            }
                        }
                        
                        self.save();
                        self.attributes.hasChanged = false;
                        self.syncWithServer();
                        //var dataRawElement = new window.GenericDataRawElement();
                        //dataRawElement.syncWithServer();
                        //dataRawElement = null;
                    }
                }

                //ROLLBACK
                if (actionHelper.IsValid == false) {
                    success = false;
                    self.attributes.hasChanged = false;
                    var oldModel = actionHelper.OldModels.findByValue('id', self.id)[0];
                    if (oldModel != null && !self.IsNew) {
                        self.attributes = jQuery.extend(true, {}, oldModel.attributes);
                        self.save();
                    }
                }
            }
            actionHelper.OldModels.removeElementByValue('id', self.id);
            self = null;
            actionHelper = null;
            return success;
        },

        csvExportColumns: function () { return ""; },
        csvIdValueMapping: function () {
            var arr = new Array();
            return arr;
        }
    });

    window.GenericData = Backbone.Collection.extend({
        model: window.GenericDataRawElement,
        localStorage: new Store("GenericData"),
    });

    window.persistGenericData = function () {
        var myData = JSON.stringify(window.genericData.models);
        window.localStorage.setItem(window.genericData.localStorage.name, myData);
        myData = null;
    };

    window.initGenericDataData = function () {
        window.genericData = new window.GenericData();
        window.genericData.fetch({ async: false });

        window.genericData.bind("add", function (obj) {
            obj.attributes.CreatedBy = window.currentUser.UserId;
            obj.attributes.CreatedDate = moment().format(CONSTS.DATETIME_PERSIST_FORMAT, 'de');
            obj.trigger("saved");
        });
    };
}