﻿/**
* Backbone Model für die Entität User
*
* @class UserRawElement.js
*/
function generateModelUserRawElement() {
    window.UserRawElement = Backbone.Model.extend({
        defaults: {
            UserId: CONSTS.GUID_DEFAULT,
            MandatorId: CONSTS.GUID_DEFAULT,
            OrgUnitId: CONSTS.GUID_DEFAULT,
            PhoneId: null,
            DevicePlatform: null,
            LiveId: null,
            SalutationId: CONSTS.INTEGER_DEFAULT,
            Lastname: null,
            Firstname: null,
            Company: null,
            Street: null,
            HouseNo: null,
            ZipCode: null,
            City: null,
            CountryCode: null,
            Phone: null,
            Fax: null,
            Email: null,
            Password: null,
            LanguageCode: null,
            StatusId: RECORD_STATUS.CANCELED,
            ActivationKey: null,
            RegisterDate: CONSTS.DATE_DEFAULT,
            ActivationDate: CONSTS.DATE_DEFAULT,
            DeactivationDate: CONSTS.DATE_DEFAULT,
            RetentionOrderManagement: CONSTS.INTEGER_DEFAULT,
            RetentionLogbook: CONSTS.INTEGER_DEFAULT,
            RetentionTimeRecordings: CONSTS.INTEGER_DEFAULT,
            RetentionReceipts: CONSTS.INTEGER_DEFAULT,
            UserElementAuth: {
                MandatorId: CONSTS.GUID_DEFAULT,
                OrgUnitId: CONSTS.GUID_DEFAULT,
                UserId: CONSTS.GUID_DEFAULT,
                PhoneId: null,
                LanguageCode: null
            }
        },
        localStorage: new Store("UserRawElementEntries"),
        PersistToLocalStorageImmediately: true,
        set: function (attributes, options) {
            if (!this.id) this.id = attributes.UserId;

            if (attributes.UserId) {
                attributes.UserElementAuth = {};
                attributes.UserElementAuth.MandatorId = attributes.MandatorId;
                attributes.UserElementAuth.OrgUnitId = attributes.OrgUnitId;
                attributes.UserElementAuth.UserId = attributes.UserId;
                attributes.UserElementAuth.PhoneId = attributes.PhoneId;
                attributes.UserElementAuth.LanguageCode = attributes.LanguageCode;
            }

            if (attributes.RegisterDate) {
                attributes.RegisterDate = moment(attributes.RegisterDate).toDate();
            }
            if (attributes.ActivationDate) {
                attributes.ActivationDate = moment(attributes.ActivationDate).toDate();
            }
            if (attributes.DeactivationDate) {
                attributes.DeactivationDate = moment(attributes.DeactivationDate).toDate();
            }

            if (attributes.Modules && attributes.Modules.length > 0) {
                for (var module in attributes.Modules) {
                    attributes.Modules[module].StartDate = moment(attributes.Modules[module].StartDate).toDate();
                    attributes.Modules[module].CancelDate = moment(attributes.Modules[module].CancelDate).toDate();
                    attributes.Modules[module].EndDate = moment(attributes.Modules[module].EndDate).toDate();
                }
            }

            Backbone.Model.prototype.set.call(this, attributes, options);
        },

        toJSON: function () {
            var json = Backbone.Model.prototype.toJSON.call(this);
            return _.extend(json);
        }

        //toASP: function () {
        //    this.attributes.RegisterDate = moment(this.attributes.RegisterDate).toASPDateTimeOffset();
        //    this.attributes.ActivationDate = moment(this.attributes.ActivationDate).toASPDateTimeOffset();
        //    this.attributes.DeactivationDate = moment(this.attributes.DeactivationDate).toASPDateTimeOffset();

        //    if (this.attributes.Modules && this.attributes.Modules.length > 0) {
        //        for (var module in this.attributes.Modules) {
        //            this.attributes.Modules[module].StartDate = moment(this.attributes.Modules[module].StartDate).toASPDateTimeOffset();
        //            this.attributes.Modules[module].CancelDate = moment(this.attributes.Modules[module].CancelDate).toASPDateTimeOffset();
        //            this.attributes.Modules[module].EndDate = moment(this.attributes.Modules[module].EndDate).toASPDateTimeOffset();
        //        }
        //    }

        //    return this;
        //}
    });

    window.UserRawElementEntries = Backbone.Collection.extend({
        model: window.UserRawElement,
        localStorage: new Store("UserRawElementEntries")
    });

    window.persistUserRawElementData = function () {
        var store = window.userRawElementEntries.localStorage;
        var myData = JSON.stringify(store.data);
        window.localStorage.setItem(store.name, myData);
    };


    window.initUserRawElementData = function () {
        window.userRawElementEntries = new window.UserRawElementEntries();
        window.userRawElementEntries.fetch({ async: false });
    };
}