﻿/**
* Implementiert eine Ansicht welche den Synchronisationsstatus der Entitäten amzeigt inklusive informationen wann die entität zuletzt synchronisiert wurde und wieviele Datensätze dieser Entität synchronisiert wurden
*
* @class SyncStatusView.js
*/
window.SyncStatusView = Backbone.View.extend({

    events: {
        "click #backButton": "backButton",
    },

    backButton: function () {
        window.history.back();
    },

    buildView: function () {
        var notSynced = SyncController.getNotSyncedItemsCount();



        var that = this;
        var tableBody = $(that.el).find('#syncStatus');
        var entities = [];
        _.each(window.currentDevice.Entities, function (devEntity) {
            var entity = window.genericEntities.get(devEntity.EntityId);
            if (entity != undefined) {
                entity.LastSyncTime = devEntity.SyncTime;
                entities.push(window.genericEntities.get(devEntity.EntityId));
            }
        });
        var entities = _.filter(entities, function (entity) {
            return entity.attributes.IsSystemEntity == false;
        });

        entities = _.sortBy(entities, function (entity) {
            return entity.attributes.EntityName;
        });


        var syncStatus = $('<div/>', {
            class: 'sync-status'
        }).appendTo(tableBody);

        var span = $('<p/>', {
            class: 'header-sync',
            text: 'Allgemeines',
        }).appendTo(syncStatus);


        var syncisrunning = SyncController.SyncIsRunning;
        var connection = SyncController.checkConnection(false, false);

        if (syncisrunning) {
            var pSync = $('<p/>', {
            }).appendTo(syncStatus);

            var spanDateTitle = $('<span/>', {
                class: 'sync-title',
                text: 'Synchronisation gestartet.',
            }).appendTo(pSync);

        }

        if (!connection) {
            var pSync = $('<p/>', {
            }).appendTo(syncStatus);

            var spanDateTitle = $('<span/>', {
                class: 'sync-title error',
                text: 'keine Internetverbindung',
            }).appendTo(pSync);
        }

        var pCount = $('<p/>', {
        }).appendTo(syncStatus);

        var spanCountTitle = $('<span/>', {
            class: 'sync-title',
            text: 'Anzahl nicht synchronisierter Records: ',
        }).appendTo(pCount);

        var spanCount = $('<span/>', {
            text: notSynced,
        }).appendTo(pCount);

        if (notSynced > 0) {
            spanCountTitle.addClass('error');
            spanCount.addClass('error');
        }

        //var pNotSyncedCount = $('<p/>', {
        //}).appendTo(syncStatus);

        //var spanCountTitle = $('<span/>', {
        //    class: 'sync-title',
        //    text: 'Anzahl nicht synchronisierter Records: ',
        //}).appendTo(pNotSyncedCount);

        //var spanCount = $('<span/>', {
        //    text: notSynced,
        //}).appendTo(pNotSyncedCount);

        _.each(entities, function (entity) {
            var genericData = _.filter(window.genericData.models, function (data) {
                return (data.attributes.EntityId == entity.attributes.EntityId && data.attributes.IsActive == true);
            });
            var notSynced = SyncController.getNotSyncedItemsCount(entity.attributes.EntityId);

            var syncStatus = $('<div/>', {
                class: 'sync-status'
            }).appendTo(tableBody);

            var span = $('<p/>', {
                class: 'header-sync',
                text: entity.attributes.EntityName,
            }).appendTo(syncStatus);

            var date = moment(entity.LastSyncTime).format(FormatHelper.get_DATETIME_FORMAT());
            var pDate = $('<p/>', {
            }).appendTo(syncStatus);

            var spanDateTitle = $('<span/>', {
                class: 'sync-title',
                text: 'Letzte Synchronisation: ',
            }).appendTo(pDate);

            var spanCount = $('<span/>', {
                text: date,
            }).appendTo(pDate);

            var pCount = $('<p/>', {
            }).appendTo(syncStatus);

            var spanCountTitle = $('<span/>', {
                class: 'sync-title',
                text: 'Anzahl lokal gespeicherter Records: ',
            }).appendTo(pCount);

            var spanCount = $('<span/>', {
                text: genericData.length,
            }).appendTo(pCount);

            var pNotSyncedCount = $('<p/>', {
            }).appendTo(syncStatus);

            var spanCountTitle = $('<span/>', {
                class: 'sync-title',
                text: 'Anzahl nicht synchronisierter Records: ',
            }).appendTo(pNotSyncedCount);

            var spanCount = $('<span/>', {
                text: notSynced,
            }).appendTo(pNotSyncedCount);

            if (notSynced > 0) {
                spanCountTitle.addClass('error');
                spanCount.addClass('error');
            }
        });

    },

    render: function () {
        var that = this;
        ActionHelper.CurrentView = this;
        require(["lib/text!js/view/html/SyncStatusView.html"], function (template) {
            $(that.el).html(_.template(template)());

            $('#title').text('Synchronisationsübersicht')

            that.buildView();

            $('#SyncStatusView').trigger('create');
        });
        return this;
    }
});