﻿/**
* Generischer View zum Erzeugen einer Listenansicht
*
* @class ListView.js
*/
window.ListView = Backbone.View.extend({
    events: {
        "click .checkBox": "check",
        "change #search": "searchfunction",
        "keyup #search": "filter",
        "click #lnkmarkAll": "markall",
        "click #lnkSort": "sortList",
    },

    isSubView: false,
    hiddenFilter: null,
    
    replaceLanguageStrings: function (htmlTemplate) {
        htmlTemplate.find('#search').attr('placeholder', Utils.getText('LISTE_DURCHSUCHEN'));

        var fs = htmlTemplate.find('#search').params;
        var lable = $('<a/>', {
            class: 'ui-btn-right ui-btn ui-btn-inline ui-mini ui-corner-all ui-btn-icon-right ui-icon-gear',
            text: "Datei: ",
            for: 'attachment-content',
        }).appendTo(fs);

    },

    ListViewObj: {
        params: {
            ListViewTypeParam01: 'list', //Tablestyle
            genericView: null,
            entity: null,
            refModel: null,
            refModelId: null,
            renderContainer: null,
            createCheckbox: false,
            Style: null,
            CssClass: null,
        },
        ListElements: [],

        reset: function () {
            this.ListElements = [];
            this.params = {
                ListViewTypeParam01: 'list', //Tablestyle
                genericView: null,
                entity: null,
                refModel: null,
                refModelId: null,
                renderContainer: null,
                createCheckbox: false,
                Style: null,
                CssClass: null,
            }

            this.style
        }
    },

    markall: function () {

        var link = $(this.el).find("#lnkmarkAll");
        var parent = $(link).parent();

        var markAll = $(parent).find('.ui-icon-check');

        if (markAll.length > 0) {
            //MARK ALL LOGIC
            var e = { currentTarget: $(this.el).find('#search')[0] };
            this.filter(e, false)

            $(link).html("Alle Markierungen entfernen");
            $(parent).find('.ui-icon-check').removeClass('ui-icon-check').addClass('ui-icon-delete');
            $(this.el).find('#listView .checkBox').each(function () {
                if ($(this).is(":visible") == true) {
                    $(this).removeClass('ui-icon-none').addClass('ui-icon-check');
                }
            });
        } else {
            //UNMARK ALL LOGIC
            $(link).html("Alle markieren");
            $(parent).find('.ui-icon-delete').removeClass('ui-icon-delete').addClass('ui-icon-check');
            $(this.el).find('#listView .checkBox').each(function () {
                $(this).removeClass('ui-icon-check').addClass('ui-icon-none');
            });
        }
    },

    sort: function () {
        this.renderView(listElems, paging);
        $(this.ListViewObj.params.renderContainer).listview("refresh");
    },

    filter: function (e, paging) {
        if (paging == undefined) {
            paging = true;
        }

        if (this.options.view.get('Param01') === 'online' || this.options.view.get('Param01') === 'OnlineNoMainFilterChange') { 
            return; 
        }

        var listElems = [];
        listElems = jQuery.extend(true, [], this.ListViewObj.ListElements);
        var searchtext = $(e.currentTarget).val().toLowerCase();
        var searchVals = searchtext.match(/\S+/g); //Split " " no empty arrays
        if (searchVals != null && listElems.length > 0) {
            //And Search recursive: search for "sr fries" searches for all entries wich contain sr and fries. 
            var filtered = [];
            var filter = function (i, n) {
                if (n >= searchVals.length) {
                    return;
                }

                if (i >= listElems.length) {
                    filter(0, n + 1); return;
                };

                var listElem = listElems[i];
                var contains = false;
                _.each(listElem.properties, function (prop) {
                    if (prop.value.toLowerCase().indexOf(searchVals[n]) != -1) {
                        contains = true;
                    }
                });

                if (!contains) {
                    listElems.splice(i, 1);
                    i -= 1;
                }

                filter(i + 1, n);
            }
            filter(0, 0);
        }

        this.renderView(listElems, paging);
        $(this.ListViewObj.params.renderContainer).listview("refresh");

        if (e != undefined) {
            this.saveFilter(e);
        }
    },

    loading: function (showOrHide) {
        setTimeout(function () {
            if (showOrHide) {
                $('#loader_page').show();
                $.mobile.loading('show', {
                    text: 'Suche',
                    textVisible: true,
                    theme: 'a',
                });
            }
            else {
                $('#loader_page').hide();
                $.mobile.loading('hide');
            }
        }, 1);
    },
    
    getSettings: function () {
        var saved = localStorage.getItem(this.options.view.attributes.ViewId);

        var settings = Utils.isJSON(saved);
        if (settings == false) {
            settings = {
                filter: '',
                sort: [], //
                customfilter: '',
                /*
                example sort: [
                   { propId: someId, sort: 'asc', hierarchy: 1 }, hierarchy 1 means sort by this prop first
                   { propId: someId, sort: 'desc', hierarchy: 3},
                ]
                */
            }
            this.saveSettings(settings);
        }

        return settings;
    },

    saveSettings: function (settings) {
        localStorage.setItem(this.options.view.attributes.ViewId, JSON.stringify(settings));
    },

    saveSort: function (sortObj) {
        var settings = this.getSettings();
        settings.sort = sortObj;
        this.saveSettings(settings);
    },

    saveFilter: function (e, text) {
        var settings = this.getSettings();
        var filter = '';
        if (e != undefined && e != null) {
            filter = $(e.currentTarget).val();
        }
        else if (text != undefined) {
            filter = text;
        }
        settings.filter = filter;
        this.saveSettings(settings);
    },

    searchfunction: function (e) {

        if (!(this.options.view.attributes.Param01 == "online" || this.options.view.attributes.Param01 == "OnlineNoMainFilterChange")) { return; }

        $('#loader_page').show();
        $.mobile.loading('show', {
            text: 'Suche',
            textVisible: true,
            theme: 'a',
        });
        var ParentObjectID = ActionHelper.CurrentView.options.modelId;
        var searchtxt = '';
        if (this.options.view.attributes.Param01 == "OnlineNoMainFilterChange" && this.hiddenFilter != null) {
            searchtxt += this.hiddenFilter;

            if (searchtxt != null && searchtxt.length > 0)
                searchtxt += ' '
        }
        var that = this;

        if (e != undefined) {
            searchtxt += e.target.value;
        }


        var customfilter = that.getSettings().customfilter;
        if (customfilter == undefined || customfilter == ''){
            customfilter = '';
        }

        SyncController.searchData(searchtxt, this.options.view.id, this.options.view.attributes.EntityId, function () {
            $('#loader_page').hide();
            $.mobile.loading('hide');
            that.buildView(true);
            $(that.ListViewObj.params.renderContainer).listview("refresh");
        }, ParentObjectID, customfilter);
    },

    exportButton: function () {
        window.alert('Bald verfügbar!');
    },
    addButton: function () {
        window.location = 'index.html#Insert/' + this.model.attributes.EntityId + '/' + this.options.refobj + '/' + this.options.refent;
    },
    check: function (e) {
        var element = $(e.currentTarget);
        var icon = element[0];
        if (element.hasClass("ui-icon-check")) {
            element.addClass('ui-icon-none');
            element.removeClass('ui-icon-check');
        } else {
            element.removeClass('ui-icon-none');
            element.addClass('ui-icon-check');
        }
    },

    buildView: function (reload) {
        if (reload == undefined)
            reload = false;
        this.ListViewObj.reset();
        if (!this.isSubView) {
            this.buildListViewObj();
            if (!reload)
                this.buildActions();
        } else {
            this.buildListViewObj();
        }
        this.renderView(this.ListViewObj.ListElements);
    },

    prepareBuildViewParams: function () {
        var params = this.ListViewObj.params;
        params.ListViewTypeParam01 = this.options.view.attributes.Param01;
        params.entity = window.genericEntities.get(this.options.view.get('EntityId'));
        params.refModelId = this.options.refModelId;
        if (params.refModelId != "null") {
            params.refModel = _.first(_.filter(window.genericData.models, function (model) {
                return model.get('ObjectId') == params.refModelId;
            }));
        }

        params.renderContainer = $(this.el).find('#listView');
        this.ListViewObj.params = params;
    },


    renderListElement: function (ListObj) {
        var createCheckbox = this.ListViewObj.params.createCheckbox;
        var style = this.ListViewObj.ListViewType;
        var id = ListObj.id;
        var ListElement = ListViewBuilderHelper.createListElement(style, id);

        var a = $(ListElement).find('a.list_item');
        if (ListObj.Style != undefined && ListObj.Style != ''){
            a.attr('style', ListObj.Style);
        }

        Utils.addParams(a, id, ListObj.refModelId);

        var isBosoMedicus = false
        if (!!GlobalSettings.IsBoso()) {
            isBosoMedicus = !!_.find(ListObj.properties, function (property) {
                return property.sortPos === -1 && property.value === 'boso medicus system';
            });
        }

        _.each(ListObj.properties, function (property) {
            var cssStyle = !!GlobalSettings.IsBoso() ? property.PropertyStyleField : property.CssClass

            if (!!GlobalSettings.IsBoso() && ActionHelper.CurrentView.options.view.get('EntityId') === Boso.BOSO_GEREATE_ENTITY_ID) {
                property.removeLabels = property.sortPos === 1;

                var userIndexPropertyId = '4b43d09d-ac9c-4336-b740-715af69e2df3';
                if (isBosoMedicus && property.id === userIndexPropertyId) {
                    property.IsVisible = false;
                }
            }

            if (!property.IsVisible) { 
                return;
            }

            if (property.sortPos == -1) {
                //Thumbnail
                ListViewBuilderHelper.createThumbnailPosMinusOneElement(style, ListElement, property.value, cssStyle);
            } else if (property.sortPos == 0) {
                //Status right
                ListViewBuilderHelper.createWrapperPosZeroElement(style, ListElement, property.value, cssStyle);
            } else if (property.sortPos == -2) {
                //Title center
                ListViewBuilderHelper.createWrapperPosMinusTwoElement(style, ListElement, property.value, cssStyle);
            } else {
                ListViewBuilderHelper.createWrapperContent(style, ListElement, property.label, property.value, property.removeLabels, cssStyle);
            }
        });

        if (createCheckbox) {
            !!GlobalSettings.IsBoso() ? ListViewBuilderHelper.createWrapperCheckBoxBoso(style, ListElement, ListObj.objectId) : ListViewBuilderHelper.createWrapperCheckBox(style, ListElement, ListObj.objectId);
        }

        return ListElement;

    },

    renderListHeader: function (label) {
        var currentView = ActionHelper.CurrentView;
        var currentViewId = ActionHelper.CurrentView.options.view.get('ViewId');
        var currentEntityId = ActionHelper.CurrentView.options.view.get('EntityId');

        var fields = _.filter(window.genericViewFields.models, function (field) {
            return field.get('ViewId') === currentViewId
                && field.get('VisibleJS') !== "false"
        })

        fields = _.sortBy(fields, function (field) {
            return field.get('SortPosition');
        });

        _.each(fields, function (field) {
            var fieldProperty = _.find(window.genericProperties.models, function (property) {
                return property.get('PropertyId') === field.get('PropertyId');
            });
            ListViewBuilderHelper.ListViewTableStyle.createHeaderTags(fieldProperty.get('PropertyName'));
        });

        this.ListViewObj.params.createCheckbox = (currentView.options.view.get('SelectableFieldCount') == null || currentView.options.view.get('SelectableFieldCount') > 0)
        if (this.ListViewObj.params.createCheckbox) {
            ListViewBuilderHelper.ListViewTableStyle.createHeaderTags('chkBox');

            var renderContainer = $(this.ListViewObj.params.renderContainer);
            setTimeout(function () {
                var chkBoxHeaderCell = renderContainer.find('#th_chkBox');
                if (chkBoxHeaderCell.is(':empty')) {
                    ListViewBuilderHelper.ListViewTableStyle.createBosoWeightScaleButton().appendTo(chkBoxHeaderCell);
                }
            }, 300);
        }
    },

    renderView: function (ListElements, paging) {
        if (paging == undefined) {
            paging = true;
        }
        
        if (this.ListViewObj.params.entity.get('EntityId') === Boso.USER_ENTITYID) {
            paging = false;
        }

        var renderContainer = this.ListViewObj.params.renderContainer
        /* empty */
        $(renderContainer).empty();

        /* SONSTIGES */
        if (this.options.refent != 'null' && this.options.refent != null) {
            var entity = window.genericEntities.get(this.options.refent);
            $(this.el).find('#okButton').text('Zurück zu ' + entity.attributes.EntityName);
        }

        /* LISTE AUFBAUEN */
        ListViewBuilderHelper.init(this.ListViewObj.params.ListViewTypeParam01, renderContainer);

        if (ListElements.length === 0) {
            $(renderContainer).css('display', 'none');
        }

        if (GlobalSettings.IsBoso() && ActionHelper.CurrentView.options.view.get('EntityId') === Boso.MESSUREMENT_ENTITYID) {
            $(renderContainer).css('display', '');
            this.renderListHeader();
        }

        /* Lazy Loading */
        /* Es werden um Render Zeit zu Sparen nur eine gewisse Anzahl an ListElementen gerendert */
        /* Die ListElemente können aber je nach Anzahl der ViewFields eine verschiedene Höhe haben,
        deswegen muss im Vorraus bestimmt werden wieviele ListElemente vorgeladen werden sollen um die Bildschirmhöhe
        zu füllen. Der Wert wird in der Varialben PageSize gespeichert.
        Um es auszurechnen nehmen wir eine Konstante die angibt wie hoch ein ViewField ungefähr ist, anhand dieser Anzahl 
        kann die höhe eines Listelements grob bestimmt werden */

        var listlength = ListElements.length;

        var viewFieldSize = CONSTS.LISTVIEW_FIELDSIZE;

        if (this.ListViewObj.params.ListViewTypeParam01 == 'tablestyle') { //Tablestyle
            viewFieldSize = viewFieldSize / 1.6;
        }

        if (listlength > 0) {
            var listElementHeight = _.filter(ListElements[0].properties, function (p) {
                return p.IsVisible
            }).length * viewFieldSize;
        }
        var pageSize = parseInt($(document).height() / listElementHeight) + 2;
        if (!paging){
            pageSize = listlength;
        }

        var startEndlessScroll = listlength > pageSize ? pageSize : listlength;

        if (listlength != 0) {
            for (var i = 0; i < startEndlessScroll; i++) {
                this.renderListElement(ListElements[i]);
            }
            $(renderContainer).css('display', '');
        }

        ListViewBuilderHelper.render(renderContainer);

        if (!this.isSubView) {
            if (!!window.scrollListener) {
                window.clearInterval(window.scrollListener);
                $(window).off("scrollstop resize");
                $(window).unbind('scrollstop resize');
            }
            if (listlength > pageSize) {
                /*loader*/
                ListViewBuilderHelper.createLoader();

                /*endless scroll*/
                var that = this;
                window.scrollListener = function () {
                    if (ActionHelper.CurrentView != that) { return; }
                    $(window).off('scrollstop resize').on('scrollstop resize', function () { //unbinds itself every time it fires
                        if ($(window).scrollTop() >= $(document).height() - $(window).height() - 200) {
                            var difftoEnd = listlength - startEndlessScroll;

                            if (difftoEnd > 0) {
                                var add = difftoEnd > pageSize ? pageSize : difftoEnd;
                                var elemsHTML = '';
                                for (var i = startEndlessScroll; i < startEndlessScroll + add; i++) {
                                    if (that.ListViewObj.params.ListViewTypeParam01 != 'tablestyle') {
                                        elemsHTML += (that.renderListElement(ListElements[i]))[0].outerHTML;
                                    } else {
                                        that.renderListElement(ListElements[i]);
                                    }
                                }
                                startEndlessScroll += add;
                                if (that.ListViewObj.params.ListViewTypeParam01 == 'tablestyle')
                                    ListViewBuilderHelper.render(renderContainer);
                                else {
                                    $(renderContainer).append(elemsHTML);
                                    $(renderContainer).find('#listviewloader').appendTo(renderContainer);
                                }

                                $(renderContainer).listview();
                                $(renderContainer).trigger('create');
                                $(renderContainer).listview('refresh');
                            }
                            else {
                                $(renderContainer).find('#listviewloader').remove();
                            }
                            if (that.ListViewObj.params.ListViewTypeParam01 == 'tablestyle')
                                $(renderContainer).css('display', '');
                        }
                    });
                }
                window.scrollListener();
            }
        }
    },

    sortList: function () {
        ActionHelper.sortListPopUp();
    },
    buildListViewObj: function () {
        var currentView = this;

        this.prepareBuildViewParams();
        var params = this.ListViewObj.params;

        //SORT LIST
        var genericData;
        if (currentView.options.view.get('Param01') == "online" || currentView.options.view.get('Param01') == "OnlineNoMainFilterChange") {
            genericData = SyncController.TmpData;
            SyncController.TmpData = null;
            $(this.el).find('#lnkSort').remove();
        } else {
            if (this.isSubView)
                $(this.el).find('#lnkSort').remove();
            var genericData = _.filter(window.genericData.models, function (model) {
                return model.get('EntityId') == currentView.options.view.get('EntityId');
            });
            genericData = GenericHelper.filterGenericData(this.options.view, genericData);
            genericData = ActionHelper.sortForView(genericData, currentView.options.view);
        }

        var genericFieldsSorted = GenericHelper.getViewFields(this.options.view);

        if (params.refModel != null) {
            var genericProperties = _.filter(window.genericProperties.models, function (property) {
                return (property.get('EntityId') == params.entity.get('EntityId') && property.get('LookupEntityId') == params.refModel.get('EntityId'));
            });

            var genericProperty = _.first(genericProperties);
            if (genericProperty != undefined) {
                var customColName = genericProperty.get('GenericColumnName');

                genericData = _.filter(genericData, function (data) {
                    return data.get(customColName) == params.refModel.get('ObjectId');
                });
            }
        }

        genericData = _.filter(genericData, function (data) {
            return data.get('IsActive') == true;
        });

        if (genericData != null && genericData != undefined && genericData.length > 0) {
            this.ListViewObj.params.createCheckbox = (currentView.options.view.get('SelectableFieldCount') == null || currentView.options.view.get('SelectableFieldCount') > 0);
            this.ListViewObj.ListViewType = params.ListViewTypeParam01;

            _.each(genericData, function (model) {
                var refModelId = "null";
                if (params.refModel != null) {
                    refModelId = params.refModel.get('ObjectId');
                }

                var ListElement = {
                    id: model.get('DataId'),
                    refModelId: refModelId,
                    objectId: model.get('ObjectId'),
                    properties: [],
                    Style: ''
                }

                currentView.ListViewObj.ListElements.push(ListElement);

                _.each(genericFieldsSorted, function (field) {

                    var BOSO_fieldId_ConnectGoogleInsertView = '5433d09d-ac9c-4c5c-bba6-71689f676880'
                    var BOSO_fieldId_ConnectGoogleEditView = '14e7d09d-ac9c-4b35-a979-d1689f662d75'
                    var BOSO_fieldId_ConnectGoogleListView = '16aed09d-ac9c-493d-a9f4-71689f5d2067'
                    
            
                    if (!!field && !!field.id &&
                        (field.id === BOSO_fieldId_ConnectGoogleInsertView 
                        || field.id === BOSO_fieldId_ConnectGoogleEditView
                        || field.id === BOSO_fieldId_ConnectGoogleListView )) 
                    {
                        return;
                    }
            

                    var genericProperty = window.genericProperties.get(field.get('PropertyId'));
                    var displayValue = undefined;
                    var displayData = null;

                    var sortPos = field.get('SortPosition');
                    if (params.ListViewTypeParam01 == "online" || params.ListViewTypeParam01 == "OnlineNoMainFilterChange") {
                        displayValue = model.get(genericProperty.get('GenericColumnName') + '_DisplayValue');
                        displayData = new window.GenericDataRawElement();
                        displayData.id = model.get(genericProperty.get('GenericColumnName'));
                    }
                    else {
                        displayValue = model.get(genericProperty.get('GenericColumnName'));
                        var lookUpEntity = _.first(_.filter(window.genericEntities.models, function (entity) {
                            return entity.get('EntityId') == genericProperty.get('LookupEntityId');
                        }));
                    }

                    //wenn es eine lookupentität ist und kein displayproperty im feld definiert ist.
                    if (lookUpEntity != null && displayValue != "" && displayValue != null) {
                        var dispPropText = null;
                        var displayProperty = window.genericProperties.get(lookUpEntity.attributes.DisplayPropertyId);
                        
                        if (displayProperty != null){
                            displayData = _.first(window.genericData.where({ EntityId: displayProperty.get('EntityId'), ObjectId: displayValue }));
                        }

                        if (displayData != null) {
                            dispPropText = ViewBuilder.getDisplayValue(displayData, displayProperty, field);

                            if (dispPropText != null && dispPropText != '') {
                                displayValue = dispPropText;
                            } else {
                                displayValue = displayData.get(displayProperty.get('GenericColumnName'));
                            }
                        } else {
                            if (displayProperty == null)
                                displayValue = 'Lookupproperty not found:' + lookUpEntity.get('DisplayPropertyId');
                            else
                                displayValue = "NOT FOUND";
                        }
                    } else if (field.get('DisplayProperty1') != null && field.get('DisplayProperty1') != CONSTS.GUID_DEFAULT) {
                        var dispPropText = '';
                        var dispText1 = null;
                        var dispData = null;

                        var dispProp1 = window.genericProperties.get(field.get('DisplayProperty1'));

                        if (params.ListViewTypeParam01 == "online" || params.ListViewTypeParam01 == "OnlineNoMainFilterChange") {
                            if (displayValue != null) {
                                dispText1 = displayValue;
                                dispData = null;
                            }
                        }
                        else {
                            if (dispProp1 != null && dispProp1.get('EntityId') == model.get('EntityId')) {
                                dispText1 = model.get(dispProp1.get('GenericColumnName'));
                                dispData = window.genericData.get(dispText1);
                            }
                        }

                        if (dispData != null) {
                            var tmpText = GenericHelper.getDisplayValue(dispData);
                            if (!!tmpText) {
                                dispText1 = tmpText;
                            }
                        }
                        if (dispText1 != null && dispText1 != undefined && dispText1 != '') {
                            dispPropText = GenericHelper.getDisplayText(dispText1, dispProp1.get('DataType'), dispProp1.get('Param_01'));
                        }

                        var dispText2 = null;
                        var dispData = null;
                        if (field.get('DisplayProperty2') != null && field.get('DisplayProperty2') != CONSTS.GUID_DEFAULT) {
                            var dispProp2 = window.genericProperties.get(field.get('DisplayProperty2'));

                            if (params.ListViewTypeParam01 == "online" || params.ListViewTypeParam01 == "OnlineNoMainFilterChange") {
                                if (dispProp2 != null) {
                                    dispText2 = null;
                                    dispData = null;
                                }
                            }
                            else {
                                if (dispProp2 != null && dispProp2.get('EntityId') == model.get('EntityId')) {
                                    dispText2 = model.get(dispProp2.get('GenericColumnName'));
                                    dispData = window.genericData.get(dispText2);
                                }
                            }

                            if (dispData != null) {
                                var tmpText = GenericHelper.getDisplayValue(dispData);
                                if (tmpText != null && tmpText != '') {
                                    dispText2 = tmpText;
                                }
                            }
                            if (dispText2 != null && dispText2 != undefined && dispText2 != '') {
                                if (dispPropText != null && dispPropText != '') {
                                    dispPropText += ', ';
                                }
                                dispPropText += GenericHelper.getDisplayText(dispText2, dispProp2.attributes.DataType, dispProp2.attributes.Param_01);
                            }
                        }
                        var dispText3 = null;
                        var dispData = null;
                        if (field.get('DisplayProperty3') != null && field.get('DisplayProperty3') != CONSTS.GUID_DEFAULT) {
                            var dispProp3 = window.genericProperties.get(field.get('DisplayProperty3'));

                            if (params.ListViewTypeParam01 == "online" || params.ListViewTypeParam01 == "OnlineNoMainFilterChange") {
                                if (dispProp3 != null) {
                                    dispText3 = null;
                                    dispData = null;
                                }
                            }
                            else {
                                if (dispProp3 != null && dispProp3.get('EntityId') == model.get('EntityId')) {
                                    dispText3 = model.get(dispProp3.get('GenericColumnName'));
                                    dispData = window.genericData.get(dispText3);
                                }
                            }

                            if (dispData != null) {
                                var tmpText = GenericHelper.getDisplayValue(dispData);
                                if (tmpText != null && tmpText != '') {
                                    dispText3 = tmpText;
                                }
                            }
                            if (dispText3 != null && dispText3 != undefined && dispText3 != '') {
                                if (dispPropText != null && dispPropText != '') {
                                    dispPropText += ', ';
                                }
                                dispPropText += GenericHelper.getDisplayText(dispText3, dispProp3.attributes.DataType, dispProp3.attributes.Param_01);
                            }
                        }
                        if (dispPropText != null && dispPropText != '') {
                            displayValue = dispPropText;
                        }
                    } else {
                        displayValue = GenericHelper.getDisplayText(displayValue, genericProperty.attributes.DataType, genericProperty.attributes.Param_01);
                    }

                    if (displayValue == null || displayValue == 'null') {
                        displayValue = '';
                    }

                    var PropertyStyleField = null;
                    if (!!field.get('PropertyStyleField')) {
                        var PropertyPropertyStyleField = window.genericProperties.get(field.get('PropertyStyleField').toLowerCase());
                        PropertyStyleField = model.get(PropertyPropertyStyleField.get('GenericColumnName')).replace(" ", "_");
                    }

                    var lableText = '';
                    //if (sortPos != 0) {
                    lableText = genericProperty.get('PropertyName');
                    if (!!field.get('ViewFieldLabel')) {
                        lableText = field.get('ViewFieldLabel');
                    }
                    //}

                    var isVisible = true;
                    if (field.get('VisibleJS') != null && field.get('VisibleJS') != "") {
                        if (eval(field.get('VisibleJS')) == false) {
                            isVisible = false;
                        }
                    }

                    if (currentView.options.view.get("RemoveLabesFromListView")) {
                        lableText = '';
                    }

                    var removeLabels = false
                    if (!Utils.isNull(currentView.options.view)) {
                        removeLabels = currentView.options.view.get("RemoveLabesFromListView");
                    }

                    var CssClass = '';
                    if (field.get('CssClass') != null && field.get('CssClass') != '' && displayData != null) {
                        CssClass = field.get('CssClass');

                        var dataid = '';
                        var value = '';

                        if (genericProperty.get('LookupEntityId') != null
                            && genericProperty.get('LookupEntityId') != ''
                            && genericProperty.get('LookupEntityId') != CONSTS.GUID_DEFAULT) {
                            if (displayData != null && displayData.id != null) {
                                dataid = 'id_' + displayData.id;
                            }
                        }

                        if (displayValue != null) {
                            value = displayValue.replace(/\.|\,|\(|\)|\{|\}|\[|\]|\ /g, '_'); //Entfernt alle Sonderzeichen
                        }

                        CssClass = CssClass.replace('##dataid##', dataid);
                        CssClass = CssClass.replace('##value##', value);
                        CssClass = CssClass.replace(/\./g, ' ');

                    }

                    if (!ListElement.Style) {
                        var check = '';
                        if (field.attributes.PropertyId == '6ec5a177-fe01-4744-9201-614f662a3278') //Prototyp
                            check = 'AMS.checkWiedervorlagedatum(field, displayValue)'

                        if (eval(check)) {
                            ListElement.Style = 'background-color: red;text-shadow:none';
                        }

                    }

                    var listObj = {
                        id: field.attributes.PropertyId,
                        label: lableText,
                        value: displayValue,
                        sortPos: sortPos,
                        PropertyStyleField: PropertyStyleField,
                        IsVisible: isVisible,
                        removeLabels: removeLabels,
                        CssClass: CssClass,
                    };

                    ListElement.properties.push(listObj);

                });
            }, this);
        }
    },

    buildActions: function () {
        var view = this.options.view;

        if (view.get('SelectableFieldCount') != null && view.get('SelectableFieldCount') == 0) {
            var link = $("#lnkmarkAll");
            if (link != undefined) {
                //link.remove();
                var parent = $(link).parent();
                if (parent != undefined) {
                    parent.remove();
                }
            }
        }
        else {
            if (this.ListViewObj.params.ListViewTypeParam01 == 'tablestyle' && !this.isSubView) {
                ListViewBuilderHelper.ListViewTableStyle.createSelectAllCheckBox();
            }
        }

        //GENERIC ACTIONS
        var viewActions = GenericHelper.getViewActions(this.options.view);
        _.each(viewActions, function (viewAction) {
            ActionHelper.prepareAction(view, viewAction);
        });

        //ONLOAD ACTIONS
        var onloadActions = _.filter(viewActions, function (action) {
            return action.get('ActionType') == 'onload';
        });

        _.each(onloadActions, function (onloadAction) {
            var Command = 'ActionHelper.' + onloadAction.get('CommandName') + ActionHelper.prepareParams(onloadAction);
            eval(Command);
        });
    },

    render: function (eventName) {
        this.delegateEvents();
        var that = this;
        //SyncController.TmpData = null;
        if (!this.isSubView) {
            ActionHelper.CurrentView = this;
            ActionHelper.CurrentView.Type = this.options.view.get('ViewType');
        }


        require(["lib/text!js/view/generic/html/ListView.html"], function (template) {
            $(that.el).html(_.template(template)());
            that.replaceLanguageStrings($(that.el));
            that.buildView();

            $('#backButton').css('display', '');

            if (that.isSubView) {
                $(that.el.parentNode).trigger('create');
                $(that.el).find('#search').parent().css('display', 'none');
            } else {
                that.styleMainContainer();

                $('#List').trigger('create');
                //Search input clearable href fix
                $('.ui-input-clear.ui-btn.ui-icon-delete').removeAttr('href').click(function () {
                    $('#search').trigger('keyup');
                });

                var filter = '';
                if (ActionHelper.NavFilter != null && ActionHelper.NavFilter != '') {
                    that.saveFilter(null, ActionHelper.NavFilter);
                    filter = ActionHelper.NavFilter;
                    that.hiddenFilter = filter;
                    ActionHelper.NavFilter = null;

                } else {
                    var filter = that.getSettings().filter;
                }
                that.hiddenFilter = filter;

                if (!Utils.isNull(filter) || !Utils.isNull(that.getSettings().customfilter)) {
                    var filterInput = $(that.el).find('#search');

                    if ((that.options.view.get('Param01') == "online" || that.options.view.get('Param01') == "OnlineNoMainFilterChange")) {
                        $(filterInput).trigger("change");
                        if (that.options.view.get('Param01') == "online")
                        { $(filterInput).val(filter); }
                    } else {
                        $(filterInput).val(filter);
                        $(filterInput).trigger("keyup");
                    }
                }
                else {
                    if ((that.options.view.get('Param01') == "online" || that.options.view.get('Param01') == "OnlineNoMainFilterChange")) {
                        SyncController.TmpData = undefined;

                        that.buildView(true);

                        if (that.options.view.attributes.StartSearchImmediately != undefined && that.options.view.attributes.StartSearchImmediately) {
                            that.searchfunction();
                        }
                    }
                }

                if (that.options.view.attributes.Param01 == "tablestyle") {
                    var el = $(that.el).find('#listheader');
                    el.remove();
                    $('#search').css('display', 'none');
                }

                $(that).off();
            }

            ActionHelper.globalOnload();
        });

        return this;
    },

    styleMainContainer: function () {
        if (!GlobalSettings.IsBoso()) {
            return;
        }

        var DEVICES_ENTITY_ID = '6c276d9a-c8a3-11cd-b0ef-6b705d508d0e';
        var USER_ENTITY_ID = '612d526a-a19c-d78a-f5b7-e96e002999cb';

        // remove searchbar and sorting button
        if (!!this.ListViewObj && !!this.ListViewObj.params && this.ListViewObj.params.entity) {
            if (this.ListViewObj.params.entity.id === DEVICES_ENTITY_ID || this.ListViewObj.params.entity.id === USER_ENTITY_ID) {
                $(this.el).find('#settingsBar').css('display', 'none');
            }
        }

        if (this.ListViewObj.params.entity.id === USER_ENTITY_ID) {
            var activeUser = _.find(window.genericData.models, function (model) {
                return model.get('EntityId') === USER_ENTITY_ID &&
                    model.get('Custom_005') === 1 &&
                    model.get('IsActive') === true
            });

            if (!!activeUser) {
                var selectorActiveItem = '.list_item[id=\'' + activeUser.get('DataId') + '\']';
                $(selectorActiveItem).parent().addClass('item_active');
            }
        }

        var isTableStyle = $('.listViewTableStyle').length > 0;
        if (isTableStyle && $('#settingsBar').length > 0) {
            $('#settingsBar').css('display', 'none');
        }

        var mainContainerByChildElements = $('#main_container:has(#pagecontent):has(#List:not(:empty))');
        mainContainerByChildElements.addClass('main_container_listView');

        $('#main_container').unbind('DOMSubtreeModified').bind('DOMSubtreeModified', function () {
            mainContainerByChildElements = $('#main_container:has(#pagecontent):has(#List:not(:empty))');
            var mainContainerByClass = $('.main_container_listView');

            if (mainContainerByChildElements.length === 0) {
                $('#main_container').removeClass('main_container_listView');
                $('#main_container').unbind('DOMSubtreeModified');
                $('#deleteContext').popup('destroy');
            }

            if (mainContainerByClass.length === 0) {
                mainContainerByChildElements.addClass('main_container_listView');
            }
        });
    }
});