﻿/**
* Generischer View welche das Menu erzeugt
*
* @class ModuleView.js
*/
window.ModuleView = Backbone.View.extend({
    events: {
        "click .moduleButton": "moduleButton",
        "click #activate": "activate",
    },

    activate: function () {
        $('#loader_page').show();
        $.mobile.loading('show', {
            text: 'Aktiviere Gerät',
            textVisible: true,
            theme: 'a',
        });

        var email = $('#Login_Email').val();
        var password = $('#Login_Password').val();

        SyncController.validateDeviceInitial(email, password);
    },

    moduleButton: function (e) {
        var parameter = $(e.currentTarget).attr("id");
        window.app.listView(parameter);
    },


    buildView: function (that) {

        var listView = $(that.el).find('#moduleListView');

        var modules = _.filter(window.genericData.models, function (data) {
            return data.attributes.EntityId.toString() == CONSTS.MODULE_ID;
        });

        var modulesSorted = _.sortBy(modules, function (sortModule) {
            return parseInt(sortModule.attributes.Custom_002);
        });

        var ID_TEXTBLOCKS = '5cce50a5-6b65-cac1-7812-b55ace9484a4';
        _.each(modulesSorted, function (module) {

            var moduleEntities = _.filter(window.genericEntities.models, function (entity) {
                if (GlobalSettings.IsBoso()) {          
                    return entity.attributes.ModuleId == module.attributes.ObjectId && entity.attributes.EntityId != ID_TEXTBLOCKS;
                }
                else{
                    return entity.attributes.ModuleId == module.attributes.ObjectId;
                }            
            });

            if (moduleEntities.length > 0) {

                var moduleTopLevelViews = [];

                _.each(moduleEntities, function (entity) {
                    var topLevelViews = _.filter(window.genericViews.models, function (view) {
                        return (view.attributes.EntityId == entity.attributes.EntityId && view.attributes.IsTopLevel == true);
                    });
                    _.each(topLevelViews, function (view) {
                        moduleTopLevelViews.push(view);
                    })
                });

                if (moduleTopLevelViews.length > 0) {
                    var li_divider = $('<li/>', {
                        text: module.attributes.Custom_001
                    }).appendTo(listView);
                    li_divider.attr('data-role', 'list-divider');
                    li_divider.attr('data-theme', 'b');

                    var topLevelViews = _.sortBy(moduleTopLevelViews, function (view) {
                        return view.attributes.MenuSortNumber;
                    });

                    _.each(topLevelViews, function (view) {

                        var li = $('<li/>', {}).appendTo(listView);

                        var a = $('<a/>', {
                            href: '/navigate/' + view.attributes.ViewId + '/' + null + '/' + null,
                        }).appendTo(li);

                        if (view.attributes.IconCode != undefined && view.attributes.IconCode != null && view.attributes.IconCode != '') {
                            var img = $('<img/>', {
                                alt: view.attributes.ViewName,
                                class: 'iconcustom',
                                src: 'images/icons/' + view.attributes.IconCode + '.png',
                            }).appendTo(a);
                        }
                        else {
                            var pufferimg = $('<img/>', {
                                class: 'iconcustom',
                                src: 'images/icons/einstellungen.png',
                                style: 'opacity:0; filter: alpha(opacity=0);'
                            }).appendTo(a);
                        }

                        if (view.attributes.ViewType == 'List') {

                            var refObjects = _.filter(window.genericData.models, function (data) {
                                return (data.attributes.EntityId == view.attributes.EntityId && data.attributes.IsActive == true);
                            });

                            var refObjects = GenericHelper.filterGenericData(view, refObjects);

                            var count = refObjects.length;

                            var span = $('<span/>', {
                                text: count,
                                class: 'ui-li-count'
                            }).appendTo(a);

                        }

                        var h1 = $('<h1/>', {
                            text: view.attributes.ViewName
                        }).appendTo(a);

                    });
                    topLevelViews = null;
                }
                moduleTopLevelViews = null;
            }
        });

        if (GlobalSettings.IsBoso() && navigator.notification) {
            //CHECK FOR USER SETTINGS
            if (UserSettings.HasUserSettings()) {
                var modelId = '1ec29fec-1a8c-4a5c-88c9-c99f5559c58f';
                var viewId = 'fe9b40c6-4325-5230-662c-8864f482dd36';
                var li = $('<li/>', {}).appendTo(listView);

                var a = $('<a/>', {
                    href: '/navigate/' + viewId + '/' + modelId + '/null',
                }).appendTo(li);

                var img = $('<img/>', {
                    alt: 'Settings',
                    class: 'iconcustom',
                    src: 'images/icons/einstellungen.png',
                }).appendTo(a);

                var h1 = $('<h1/>', {
                    text: Utils.getText('EINSTELLUNGEN'),
                }).appendTo(a);

            }

            //ADD BOSO INFO VIEW
            var li = $('<li/>', {}).appendTo(listView);

            var a = $('<a/>', {
                href: '/bosoinfos',
            }).appendTo(li);

            var img = $('<img/>', {
                alt: 'Information',
                class: 'iconcustom',
                src: 'images/icons/info.png',
            }).appendTo(a);

            var h1 = $('<h1/>', {
                text: Utils.getText('INFORMATION'),
            }).appendTo(a);

            //REMOVE ALL LIST DEVIDERS
            $("li[data-role='list-divider']").each(function () {
                $(this).remove();
            });
        } else {
            var li_divider = $('<li/>', {
                text: "Allgemein"
            }).appendTo(listView);
            li_divider.attr('data-role', 'list-divider');
            li_divider.attr('data-theme', 'b');

            var li = $('<li/>', {}).appendTo(listView);

            var a = $('<a/>', {
                href: '/configuration',
            }).appendTo(li);

            var img = $('<img/>', {
                alt: 'Einstellungen',
                class: 'iconcustom',
                src: 'images/icons/einstellungen.png',
            }).appendTo(a);

            var h1 = $('<h1/>', {
                text: "Einstellungen",
            }).appendTo(a);

            var li = $('<li/>', {}).appendTo(listView);

            var a = $('<a/>', {
                href: '/infos',
            }).appendTo(li);

            var img = $('<img/>', {
                alt: 'info',
                class: 'iconcustom',
                src: 'images/icons/info.png',
            }).appendTo(a);

            var h1 = $('<h1/>', {
                text: "Info",
            }).appendTo(a); 
        }


        this.changeMainColor();
    },


    changeMainColor: function () {
        var mainColor = GlobalSettings.MainColor();
        $('.iconcustom').css('background-color', mainColor);
        $('.iconcustom').css('border-color', mainColor);
        $('li[data-role="list-divider"]').css('background-color', mainColor);
        $('li[data-role="list-divider"]').css('border-color', mainColor);
    },

    render: function (eventName) {
        var that = this;
        //alert(localStorage.getItem("moduleBundle"));
        ActionHelper.CurrentView = this;
        ActionHelper.Models = null;
        ActionHelper.CurrentView.Type = 'Module';
        require(["lib/text!js/view/generic/html/ModuleView.html"], function (template) {
            $(that.el).html(_.template(template)());


            $('#title').text(Utils.getText('WILLKOMMEN'));

            if (window.currentDevice.PhoneId == null) {
                $(that.el).find('#init_info').show();
                var customer = localStorage.getItem('customer');
                if (customer == 'lms') {
                    $(that.el).find('#loedige-title').show();
                    $(that.el).find('#mf-title').hide();
                }
                $(that.el).find('#moduleViewContent').hide();
                /*if (window.devicePlatform != null) {
                    $(that.el).find('#app_info').html(window.devicePlatform);
                }*/

            } else {
                that.buildView(that);
            }

            if (localStorage.getItem("moduleBundle") != null && localStorage.getItem("moduleBundle") != '') {
                var regLink = $(that.el).find('#registration_link');
                regLink.attr('href', 'https://mfapp.mobile-function.com/Webapp/index.html#register/bundle/' + localStorage.getItem("moduleBundle"));
            } else {
                var regLink = $(that.el).find('#registration_link');
                regLink.attr('href', 'https://mfapp.mobile-function.com/Webapp/index.html#register');
            }

            $('#ModuleView').trigger('create');
            ActionHelper.globalOnload();
        });

        this.delegateEvents();
        return this;
    },
});