﻿
/**
* Implementiert alle Methoden welche speziell für den Kunden sind
*
* @class Boso.js
*/
Boso = {
    VERSION: '1.3.8',

    getPlatform: function () {
        if(!device) {
            return '';
        }

        return (device.platform ? device.platform : '').toLocaleLowerCase();
    },

    BOSO_DEVICENAMES: {
        BOSO_WEIGHT_NAME: "bosogramm",
        BOSO_BLODDPRESSURE_NAME: "boso medicus"
    },

    USER_ENTITYID: '612d526a-a19c-d78a-f5b7-e96e002999cb',
    BOSO_GEREATE_ENTITY_ID: '6c276d9a-c8a3-11cd-b0ef-6b705d508d0e',
    MESSUREMENT_ENTITYID: '7c2f883a-d243-785f-b4d9-a6c705661022',
    MEASUREMENTS_VIEWID: 'ae1bdcc7-ef9c-277c-2db2-fd2efc8b1790',

    GUID_EMPTY: '00000000-0000-0000-0000-000000000000',

    CHART_IDS: {
        SysDia: 'chart_SysDia',
        Pulse: 'chart_Puls',
        KgBMI: 'chart_kgBMI'
    },

    MailBaseUrl: 'https://mfapp.mobile-function.com',
    SEND_MAIL: '/action/api/common/sendmail',
    SEND_MAIL_OLD: '/action/api/emails/SendEmail',

    IsUsedForReporting: false,

    SCREEN_ORIENTATIONS: {
        portrait: 0,
        landscape: 1
    },

    screenOrientation: null,

    onSaveTimeout:null,

    statisticLoaderTimeout: null,
    graphicsLoaderTimeout: null,
    measurementsLoaderTimeout: null,

    weightScaleCommunicationActive: false,
    weightScaleCommunicationTimeout: null,

    UserNrSampleUser: '1-M',
    MEDMOBILE_ENDPOINT: 'https://www.medmobile.at/mmdatareceiver/mmreceiver',
    MM_TRANSMISSION_ACTIVATION_STRING: 'mm',

    SCITIM_ENDPOINT_PROD: 'https://scitimsecure.production.ondemand.services',
    SCITIM_ENDPOINT_STAGING: 'https://scitimsecure.staging.ondemand.services',

    CASPROZDRAVI_ENDPOINT_PROD: 'https://casprozdravi.cz/index.php?r=ping%2Fupload-pressure',

    MEASUREMENT_TRANSMISSION_TARGETS: {
        MEDMOBILE: "Custom_010",
        GOOGLEFIT: "Custom_012",
        APPLEHEALTH: "Custom_013",
        SCITIM: "Custom_015",
        CASPROZDRAVI : "Custom_016"
    },

    pairingStartTime: null,
    readBloodPressureStartTime: null,

    REPORT_FILE_FOLDERNAME: 'BOSO_DATA',
    SCITIM_ACTIVATION_PERIOD_IN_WEEKS: 8,
    CAS_PRO_ZDRAVI_ACTIVATION_PERIOD_IN_WEEKS: 8,
    SCITIM_ACTIVATION_PASSWORD: 'SciTIM',


    isBosoExampleUser: function(model) {
        var isBosoExampleUser = !!model && model.get("EntityId") === Boso.USER_ENTITYID && model.get("Custom_001") === '1-M';
        return isBosoExampleUser;
    },

    getFileDirectory: function () {
        var directory = '';
        if (Boso.getPlatform() === 'android') {
            directory = '"Internal Storage"/BOSO_DATA/';
        }
        return directory;
    },

    createMedMobileXMLString: function (params) {
        if (!params.datetime) {
            console.log('Error: Creating medMobileXMLString failed, because no datetime was handed over!');
            return;
        }

        if (!params.currentTimeZone) {
            console.log('Error: Creating medMobileXMLString failed, because no current timezone was handed over!');
            return;
        }

        if (!params.pairedTimezone) {
            console.log('Error: Creating medMobileXMLString failed, because no pairedTimezone was handed over!');
            return;
        }

        params.systolic = !!params.systolic ? params.systolic : '';
        params.diastolic = !!params.diastolic ? params.diastolic : '';
        params.pulse = !!params.pulse ? params.pulse : '';

        var xmlString = '<?xml version="1.0" standalone=\'yes\'?>';
        xmlString += '<DataRecord>';
        xmlString += '<Version>';
        xmlString += '<Hardware>' + Boso.VERSION + '</Hardware>';
        xmlString += '</Version>';
        xmlString += '<SendingDevice>';
        xmlString += '<Model>BOSO</Model>';
        xmlString += '<Serial>' + params.bloodPressureDeviceSN + '</Serial>';
        xmlString += '<PairedTimezone>' + params.pairedTimezone + '</PairedTimezone>'
        xmlString += '</SendingDevice>';
        xmlString += '<RecordInformation>';
        xmlString += '<Measured>';
        xmlString += '<Year>' + params.datetime.year() + '</Year>';
        xmlString += '<Month>' + (params.datetime.month() + 1) + '</Month>';
        xmlString += '<Day>' + params.datetime.date() + '</Day>';
        xmlString += '<Hour>' + params.datetime.hour() + '</Hour>';
        xmlString += '<Minute>' + params.datetime.minute() + '</Minute>';
        xmlString += '<Second>' + params.datetime.second() + '</Second>';
        xmlString += '<Timezone>' + params.currentTimeZone + '</Timezone>';
        xmlString += '</Measured>';
        xmlString += '</RecordInformation>';
        xmlString += '<DataBloodPressure>';
        xmlString += '<Systolic>' + params.systolic + '</Systolic>';
        xmlString += '<Diastolic>' + params.diastolic + '</Diastolic>';
        xmlString += '<Pulse>' + params.pulse + '</Pulse>';
        xmlString += '</DataBloodPressure>';
        xmlString += '</DataRecord>';

        return xmlString;
    },

    deactivateSciTimOnPeriodOver: function() {
        var deactivateSciTimConnection = function() {
            user.set('Custom_013', null);
            user.set('Custom_012', false);
            window.genericData.add(user);
            user.saveItem();
        }
        
        var user = Boso.getCurrentUser();
        if (!user) {
            return;
        }

        var isConnected = user.get('Custom_012');
        if(!isConnected) {
            return;
        }

        //deactivate if activationdate is not set, is invalid or is older than 8 weeeks.
        var sciTimConnectedOn = user.get('Custom_013');
        if (!sciTimConnectedOn) {
            deactivateSciTimConnection();
            return;
        }

        var momentConnectedOn = moment(sciTimConnectedOn);
        if (!momentConnectedOn || !momentConnectedOn.isValid()) {
            deactivateSciTimConnection();
            return;
        }

        if (moment() >= momentConnectedOn.add('week', Boso.SCITIM_ACTIVATION_PERIOD_IN_WEEKS)) {
            deactivateSciTimConnection();
            return;
        }
    },

    deactivateCasProZdraviOnPeriodOver: function() {
        var deactivateCasProZdraviConnection = function() {
            user.set('Custom_014', false);
            user.set('Custom_015', null);
            window.genericData.add(user);
            user.saveItem();
        }
        
        var user = Boso.getCurrentUser();
        if (!user) {
            return;
        }

        var isConnected = user.get('Custom_014');
        if(!isConnected) {
            return;
        }

        //deactivate if activationdate is not set, is invalid or is older than 8 weeeks.
        var casProZdraviConnectedOn = user.get('Custom_015');
        if (!casProZdraviConnectedOn) {
            deactivateCasProZdraviConnection();
            return;
        }

        var momentConnectedOn = moment(casProZdraviConnectedOn);
        if (!momentConnectedOn || !momentConnectedOn.isValid()) {
            deactivateCasProZdraviConnection();
            return;
        }

        if (moment() >= momentConnectedOn.add('week', Boso.CAS_PRO_ZDRAVI_ACTIVATION_PERIOD_IN_WEEKS)) {
            deactivateCasProZdraviConnection();
            return;
        }
    },

    requestSciTimPassword: function(onAction) {
        var returnValue = false;
        var txtSciTimPw = null;

        var fillPopupWithContent = function(sciTimPwContext) {
            if (!sciTimPwContext) {
                return;
            }

            if(!sciTimPwContext) {
                if (_.isFunction(onAction)) {
                    onAction();
                }
                return;     
            }
            var divHeaderContent = $('<div />', {
                id: 'sciTimPwHeader',
                'data-role': 'header',
                'data-theme': 'c',
            }).appendTo(sciTimPwContext);
    
            $('<h1 />', {
                text: 'SciTIM '
            }).appendTo(divHeaderContent);
    
            var divMainContent = $('<div />', {
                class: 'ui-content',
            }).appendTo(sciTimPwContext);
    
            var fallbackLanguage = 'en'
            var language = !!window.languageShort ? window.languageShort.toLowerCase() : fallbackLanguage;

            var sciTimInfoLink = 'https://www.boso.de/de/home/der-neue-digitale-gesundheitscoach-boso-medicus-system.html';
            if (language !== 'de') {
                sciTimInfoLink = 'https://www.boso.de/en/home/duerfen-wir-vorstellen-der-digitale-gesundheitscoach-boso-medicus-system.html';
            }

            var sciTimLink = $('<a />' , {
                text: Utils.getText('WAS_IST_SCITIM'),
                onclick: "window.open('" + sciTimInfoLink + "', '_system')"
            }).appendTo(divMainContent);
            sciTimLink.css("display", "block");
            sciTimLink.css("padding-bottom", "6px");

            $('<label />', {
                for: 'sciTimPwField',
                text: Utils.getText('PASSWORD')
            }).appendTo(divMainContent);
    
            txtSciTimPw = $('<input />', {
                type: 'password',
                id: 'sciTimPwField',
                'data-theme': 'c'
            }).appendTo(divMainContent);
    
            var btnControlGroup = $('<div />', {
                'data-role': 'controlgroup', 
                'data-type': 'horizontal',
                align: 'right'
            }).appendTo(divMainContent);
            btnControlGroup.css('padding-top', '6px')

            var okButton = $('<a />', {
                id: 'sciTimPwOK',
                class: 'ui-btn ui-corner-all ui-shadow ui-btn-inline ui-btn-b',
                'data-role': 'button',
                'data-theme': 'a',
                text: Utils.getText('OK')
            }).appendTo(btnControlGroup);
            okButton.off('click').on('click', function() {
                returnValue = txtSciTimPw.val() === Boso.SCITIM_ACTIVATION_PASSWORD;
                setTimeout(function() {
                    $('#sciTimPasswordRequestPopup').popup('close');

                    if (!returnValue) {
                        alert(Utils.getText('PASSWORD_ENTRY_INVALID'));
                    }
                }, 300);
                
                var focusedElement = $(':focus');
                focusedElement.blur();
            });

            var cancelButton = $('<a />', {
                id: 'sciTimPwCancel',
                class: 'ui-btn ui-corner-all ui-shadow ui-btn-inline ui-btn-b ui-last-child',
                'data-role': 'button',
                'data-theme': 'a',
                text: Utils.getText('ABBRECHEN')
            }).appendTo(btnControlGroup);
            cancelButton.off('click').on('click', function() {
                returnValue = false;
                setTimeout(function() {
                    $('#sciTimPasswordRequestPopup').popup('close');
                }, 300);

                var focusedElement = $(':focus');
                focusedElement.blur();
            });
        }
        
        var contextElement = $('#sciTimPasswordRequestPopup');
        contextElement.empty();
        fillPopupWithContent(contextElement);
        contextElement.trigger('create');

        contextElement.popup({
            afterclose: function( event, ui ) {
                if (_.isFunction(onAction)) {
                    onAction(returnValue);
                }
            },

            afteropen: function( event, ui ) {
                var pwField = $('#sciTimPwField');
                if (pwField && pwField.length > 0) {
                    pwField.focus();
                }
            }
        });

        $('#sciTimPasswordRequestPopup').popup('open', { positionTo: 'origin' });
        var repositionSciTimPwPopup = function() {
            setTimeout(function() {
                $('#sciTimPasswordRequestPopup').popup('reposition', { positionTo: 'origin' });
            }, 500);
        }

        $(window).off('resize', repositionSciTimPwPopup).on('resize', repositionSciTimPwPopup);
    },

    requestCasProZdraviId: function(onAction) {
        var returnValue = false;
        var txtCasProZdraviId = null;

        var fillPopupWithContent = function(casProZdraviContext) {
            var saveCasProZdraviId = function(casProZdraviId) {
                var user = Boso.getCurrentUser();
                user.set('Custom_016', casProZdraviId);
                window.genericData.add(user);
                user.saveItem();
            }

            if (!casProZdraviContext) {
                return;
            }

            if(!casProZdraviContext) {
                if (_.isFunction(onAction)) {
                    onAction();
                }
                return;     
            }
            var divHeaderContent = $('<div />', {
                id: 'casProZdraviIdHeader',
                'data-role': 'header',
                'data-theme': 'c',
            }).appendTo(casProZdraviContext);
    
            $('<h1 />', {
                text: 'Čas pro zdraví '
            }).appendTo(divHeaderContent);
    
            var divMainContent = $('<div />', {
                class: 'ui-content',
            }).appendTo(casProZdraviContext);
    
            // var fallbackLanguage = 'en'
            // var language = !!window.languageShort ? window.languageShort.toLowerCase() : fallbackLanguage;

            // var sciTimInfoLink = 'https://www.boso.de/de/home/der-neue-digitale-gesundheitscoach-boso-medicus-system.html';
            // if (language !== 'de') {
            //     sciTimInfoLink = 'https://www.boso.de/en/home/duerfen-wir-vorstellen-der-digitale-gesundheitscoach-boso-medicus-system.html';
            // }

            // var sciTimLink = $('<a />' , {
            //     text: Utils.getText('WAS_IST_SCITIM'),
            //     onclick: "window.open('" + sciTimInfoLink + "', '_system')"
            // }).appendTo(divMainContent);
            // sciTimLink.css("display", "block");
            // sciTimLink.css("padding-bottom", "6px");

            $('<label />', {
                for: 'casProZdraviField',
                text: Utils.getText('BOSO_CAS_PRO_ZDRAVI_ID')
            }).appendTo(divMainContent);
    
            txtCasProZdraviId = $('<input />', {
                type: 'text',
                id: 'casProZdraviField',
                'data-theme': 'c'
            }).appendTo(divMainContent);
    
            var btnControlGroup = $('<div />', {
                'data-role': 'controlgroup', 
                'data-type': 'horizontal',
                align: 'right'
            }).appendTo(divMainContent);
            btnControlGroup.css('padding-top', '6px')

            var okButton = $('<a />', {
                id: 'casProZdraviIdOK',
                class: 'ui-btn ui-corner-all ui-shadow ui-btn-inline ui-btn-b',
                'data-role': 'button',
                'data-theme': 'a',
                text: Utils.getText('OK')
            }).appendTo(btnControlGroup);
            okButton.off('click').on('click', function() {
                returnValue = !!txtCasProZdraviId.val();
                setTimeout(function() {
                    $('#casProZdraviRequestPopup').popup('close');

                    if (!returnValue) {
                        alert(Utils.getText('BOSO_CAS_PRO_ZDRAVI_ID_EINGEBEN'));
                    } else {
                        saveCasProZdraviId(txtCasProZdraviId.val());
                    }
                }, 300);
                
                var focusedElement = $(':focus');
                focusedElement.blur();
            });

            var cancelButton = $('<a />', {
                id: 'casProZdraviIdCancel',
                class: 'ui-btn ui-corner-all ui-shadow ui-btn-inline ui-btn-b ui-last-child',
                'data-role': 'button',
                'data-theme': 'a',
                text: Utils.getText('ABBRECHEN')
            }).appendTo(btnControlGroup);
            cancelButton.off('click').on('click', function() {
                returnValue = false;
                setTimeout(function() {
                    $('#casProZdraviRequestPopup').popup('close');
                }, 300);

                var focusedElement = $(':focus');
                focusedElement.blur();
            });
        }
        
        var contextElement = $('#casProZdraviRequestPopup');
        contextElement.empty();
        fillPopupWithContent(contextElement);
        contextElement.trigger('create');

        contextElement.popup({
            afterclose: function( event, ui ) {
                if (_.isFunction(onAction)) {
                    onAction(returnValue);
                }
            },

            afteropen: function( event, ui ) {
                var pwField = $('#casProZdraviField');
                if (pwField && pwField.length > 0) {
                    pwField.focus();
                }
            }
        });

        $('#casProZdraviRequestPopup').popup('open', { positionTo: 'origin' });
        var repositionCasProZdraviIdPopup = function() {
            setTimeout(function() {
                $('#casProZdraviRequestPopup').popup('reposition', { positionTo: 'origin' });
            }, 500);
        }

        $(window).off('resize', repositionCasProZdraviIdPopup).on('resize', repositionCasProZdraviIdPopup);
    },

    sendXMLRequest: function (dataRecord, measurementId, fnGetResult) {
        if (!dataRecord) {
            console.log('Error: no dataRecord was handed over to the sendXMLRequest-function!');
            return;
        }

        if (!measurementId) {
            console.log('Error: no measurementId was handed over to the sendXMLRequest-function!');
            return;
        }

        if (!fnGetResult || !_.isFunction(fnGetResult)) {
            console.log('Error: no callback was handed over to the sendXMLRequest-function!');
            return;
        }

        var successString = 'saved';
        var READYSTATES = {
            UNSENT: 0,
            OPENED: 1,
            HEADERS_RECEIVED: 2,
            LOADING: 3,
            DONE: 4
        }

        var http = new XMLHttpRequest();
        var url = Boso.MEDMOBILE_ENDPOINT;
        //var params = "lorem=ipsum&name=binny";
        http.open("POST", url, true);

        //Send the proper header information along with the request
        http.setRequestHeader("Content-type", "application/xml");

        http.onreadystatechange = function () {
            if (http.readyState === READYSTATES.DONE) {
                if (http.status === 200) {
                    if (http.response === successString) {
                        fnGetResult(true, measurementId);
                    } else {
                        fnGetResult(false, measurementId);
                    }
                } else {
                    fnGetResult(false, measurementId);
                }
            }
        }

        http.send(dataRecord);
    },

    markAsTransmittedToMedMobile: function (measurementId) {
        var measurement = _.find(window.genericData.models, function (model) {
            return model.get('DataId') === measurementId
        });

        measurement.set('Custom_010', moment().format(CONSTS.DATETIME_PERSIST_FORMAT));
        measurement.saveItem();
        window.genericData.add(measurement);
    },

    markMeasurementAsTransmitted: function (genericColumnName, measurementId) {
        var measurement = _.find(window.genericData.models, function (model) {
            return model.get('DataId') === measurementId
        });

        measurement.set(genericColumnName, moment().format(CONSTS.DATETIME_PERSIST_FORMAT));
        measurement.saveItem();
        window.genericData.add(measurement);
    },

    unmarkMeasurementAsTransmitted: function(genericColumnName, measurementId) {
        var measurement = _.find(window.genericData.models, function (model) {
            return model.get('DataId') === measurementId
        });

        measurement.set(genericColumnName, null);
        measurement.saveItem();
        window.genericData.add(measurement);
    },

    displayTelefonField: function () {
        var currentUser = Boso.getCurrentUser();
        var userNr = currentUser.get('Custom_001');
        if (userNr === Boso.UserNrSampleUser) {
            return false;
        }

        return true;
    },

    getCurrentIanaTimeZone: function (callback) {
        navigator.globalization.getDatePattern(function (result) {
            if (!result || !result.iana_timezone) {
                var errorMessage = Utils.getText('LESE_ZEITZONE_FEHLGESCHLAGEN');
                var logMessage = 'Error: The current timezone could not be determined!' + JSON.stringify(result);
                Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'readTimezone: readError');
                return;
            }

            if (_.isFunction(callback)) {
                callback(result.iana_timezone);
            }
        }, function (error) {
            var errorMessage = Utils.getText('LESE_ZEITZONE_FEHLGESCHLAGEN');
            var logMessage = 'Error: The current timezone could not be determined!' + JSON.stringify(error);

            Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'readTimezone: readError');
            return;
        });
    },

    showMMResendDialog: function (serialNr, pairedTimezone) {
        var confirmMessage = Utils.getText('TRANSMISSION_FAILED');
        showConfirmMessage(confirmMessage, function (buttonIndex) {
            if (buttonIndex !== 2) {
                return;
            }

            Boso.sendToMedMobile(serialNr, pairedTimezone);
        }, '', Utils.getText('ABBRECHEN').concat(',', Utils.getText('WIEDERHOLEN')));
    },

    sendToGoogleFitOrAppleHealth: function(newGenericDataUserModel, onGoogleFitTransmissionReady, onlyGetAuthorization) {
        showAppleGoogleResendDialog = function () {
            var confirmMessage = '';
            if(Boso.getPlatform() === 'android') {
                confirmMessage = Utils.getText('TRANSMISSION_FAILED_GOOGLE_FIT');
            }

            if(Boso.getPlatform() === 'ios') {
                confirmMessage = Utils.getText('TRANSMISSION_FAILED_APPLE_HEALTH');
            }
            
            showConfirmMessage(confirmMessage, function (buttonIndex) {
                if (buttonIndex !== 2) {
                    return;
                }
                
                sendToGoogleFitOrAppleHealth(newGenericDataUserModel, onGoogleFitTransmissionReady);
            }, '', Utils.getText('ABBRECHEN').concat(',', Utils.getText('WIEDERHOLEN')));
        }
        
        var genericColNameSentTo = null;
        if(Boso.getPlatform() === 'android') {
            genericColNameSentTo = Boso.MEASUREMENT_TRANSMISSION_TARGETS.GOOGLEFIT;
        }

        if (Boso.getPlatform() === 'ios') {
            genericColNameSentTo = Boso.MEASUREMENT_TRANSMISSION_TARGETS.APPLEHEALTH
        }

        if (!newGenericDataUserModel) {
            if(_.isFunction(onGoogleFitTransmissionReady)) {
                onGoogleFitTransmissionReady();
            };
            return;
        }

        var userNr = newGenericDataUserModel.get('Custom_001');
        if (userNr === Boso.UserNrSampleUser) {
            if(_.isFunction(onGoogleFitTransmissionReady)) {
                onGoogleFitTransmissionReady();
            }
            return;
        }

        var isForwardingActive = 0;
        if (Boso.getPlatform() === 'android') {
            isForwardingActive = +newGenericDataUserModel.get('Custom_010');
        }

        if (Boso.getPlatform() === 'ios') {
            isForwardingActive = +newGenericDataUserModel.get('Custom_011');
        }

        if (!isForwardingActive) {    
            if(_.isFunction(onGoogleFitTransmissionReady)) {
                onGoogleFitTransmissionReady();
            };
            return;
        }

        if(Boso.getPlatform() === 'android' && !navigator.onLine) {
            if(_.isFunction(onGoogleFitTransmissionReady)) {
                onGoogleFitTransmissionReady();
            }
            showAppleGoogleResendDialog();
            return;
        }

        //Alle Messwerte (Blutdruck und Puls)auslesen, die nicht noch nicht übertragen sind.
        var bloodPressureAndHeartRateMeasurementes = _.filter(window.genericData.models, function (model) {
            return model.get('EntityId') === Boso.MESSUREMENT_ENTITYID
                && model.get('IsActive') === true
                && model.get('Custom_009') === newGenericDataUserModel.get('DataId') //BOSO_Benutzer
                && !model.get('Custom_006')
                && !model.get(genericColNameSentTo) //if property "sentToGoogleFitOn not set"
        });

        //Alle Messwerte (Gewicht)auslesen, die nicht noch nicht übertragen sind.
        var weightMeasurements = _.filter(window.genericData.models, function (model) {
            return model.get('EntityId') === Boso.MESSUREMENT_ENTITYID
                && model.get('IsActive') === true
                && model.get('Custom_009') === newGenericDataUserModel.get('DataId') //BOSO_Benutzer
                && !!model.get('Custom_006')
                && !model.get(genericColNameSentTo) //if property "sentToGoogleFitOn not set"
        });

        var requiredAccessRights = [
            {
                write : ['weight', 'heart_rate', 'blood_pressure']  // Write only permission
            }
        ];

        var onTransmissionError = function(error) {
            console.log('error: ', error);
            if(_.isFunction(onGoogleFitTransmissionReady)) {
                onGoogleFitTransmissionReady();
            };
            
            if (Boso.onSaveTimeout) {
                Boso.onSaveTimeout.clear();
            }
            showAppleGoogleResendDialog();
        }
        navigator.health.isAvailable(checkAuthorization, function(error) {
            onTransmissionError(error)
        });

        function checkAuthorization(isGoogleFitAvailable) {
            if(!isGoogleFitAvailable) {
                console.log('Google Fit is not installed on this phone!');
                if(_.isFunction(onGoogleFitTransmissionReady)) {
                    onGoogleFitTransmissionReady();
                };
                return;
            }

            navigator.health.isAuthorized(requiredAccessRights, function(isAuthorized) {
                if(!isAuthorized) {   
                    getAuthorization();
                    return;
                }
                
                if(!onlyGetAuthorization) {
                    transmitValues();
                }
            }, function(error) {
                onTransmissionError(error)
            });
        }

        function getAuthorization() {
            navigator.health.requestAuthorization(requiredAccessRights, function(isAuthorized) {
                if (!isAuthorized) {
                    console.log('write authorization for google-fit denied');
                    if(_.isFunction(onGoogleFitTransmissionReady)) {
                        onGoogleFitTransmissionReady();
                    };
                    return;
                }

                if(!onlyGetAuthorization) {
                    transmitValues();
                }
            }, function(error) {
                onTransmissionError(error)
            });
        }

        function transmitValues() {
            transmitWeightValues(function() {
                transmitHeartRateValues(function() {
                    transmitBloodPressureValues(function() {
                        if(Boso.getPlatform() === 'android') {
                            alert(Utils.getText('TRANSMISSION_SUCCESS_GOOGLE_FIT'));
                        }
                        
                        if (Boso.getPlatform() === 'ios') {
                            alert(Utils.getText('TRANSMISSION_SUCCESS_APPLE_HEALTH'));
                        }

                        if (Boso.onSaveTimeout) {
                            Boso.onSaveTimeout.clear();
                        }

                        if(_.isFunction(onGoogleFitTransmissionReady)) {
                            onGoogleFitTransmissionReady();
                        };
                    });
                });
            });
        }

        var weightMeasurementsIndex = 0;
        function transmitWeightValues(onWeightTransmissionReady) {
            if(!weightMeasurements ||  weightMeasurementsIndex === weightMeasurements.length) {
                if(_.isFunction(onWeightTransmissionReady)) {
                    onWeightTransmissionReady();
                }
                return;
            }

            var weightTransmissionObject = {
                startDate:  moment(weightMeasurements[weightMeasurementsIndex].get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT).toDate(),
                endDate: moment(weightMeasurements[weightMeasurementsIndex].get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT).toDate(),
                dataType: 'weight',
                value: weightMeasurements[weightMeasurementsIndex].get('Custom_006'),
                sourceName: 'boso'
            }

            navigator.health.store(weightTransmissionObject, function() {
                var measurementId = weightMeasurements[weightMeasurementsIndex].get('DataId');

                Boso.markMeasurementAsTransmitted(genericColNameSentTo, measurementId);

                weightMeasurementsIndex++;
                transmitWeightValues(onWeightTransmissionReady);
            }, function(error) {
                console.log('error: ', error);
                weightMeasurementsIndex++;
                transmitWeightValues(onWeightTransmissionReady);
            });
        }

        var bloodPressureMeasurementsIndex = 0;
        function transmitBloodPressureValues(onBloodPressureTransmissionReady) {
            if(!bloodPressureAndHeartRateMeasurementes ||  bloodPressureMeasurementsIndex === bloodPressureAndHeartRateMeasurementes.length) {
                if (_.isFunction(onBloodPressureTransmissionReady)) {
                    onBloodPressureTransmissionReady();
                }
                return;
            }

            var systolicValue = bloodPressureAndHeartRateMeasurementes[bloodPressureMeasurementsIndex].get('Custom_002');
            var diastolicValue = bloodPressureAndHeartRateMeasurementes[bloodPressureMeasurementsIndex].get('Custom_003');
            var bloodPressureValuesTransmissionObject = {
                startDate:  moment(bloodPressureAndHeartRateMeasurementes[bloodPressureMeasurementsIndex].get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT).toDate(),
                endDate: moment(bloodPressureAndHeartRateMeasurementes[bloodPressureMeasurementsIndex].get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT).toDate(),
                dataType: 'blood_pressure',
                value: { systolic: systolicValue, diastolic: diastolicValue },
                sourceName: 'boso',
            }

            navigator.health.store(bloodPressureValuesTransmissionObject, function() {
                var measurementId = bloodPressureAndHeartRateMeasurementes[bloodPressureMeasurementsIndex].get('DataId');
                Boso.markMeasurementAsTransmitted(genericColNameSentTo, measurementId);

                bloodPressureMeasurementsIndex++;
                transmitBloodPressureValues(onBloodPressureTransmissionReady);
            }, function(error) {
                console.log('error: ', error);

                var sentToGoogleCommitDate = bloodPressureAndHeartRateMeasurementes[bloodPressureMeasurementsIndex].get(genericColNameSentTo);
                if (sentToGoogleCommitDate) {
                    var measurementId = bloodPressureAndHeartRateMeasurementes[bloodPressureMeasurementsIndex].get('DataId');
                    Boso.unmarkMeasurementAsTransmitted(genericColNameSentTo, measurementId);
                }

                bloodPressureMeasurementsIndex++;
                transmitBloodPressureValues(onBloodPressureTransmissionReady);
            });
        }

        var heartRateMeasurementsIndex = 0;
        function transmitHeartRateValues(onHeartRateTransmissionReady) {
            if(!bloodPressureAndHeartRateMeasurementes ||  heartRateMeasurementsIndex === bloodPressureAndHeartRateMeasurementes.length) {
                if (_.isFunction(onHeartRateTransmissionReady)) {
                    onHeartRateTransmissionReady();
                }
                return;
            }

            var heartRateValuesTransmissionObject = {
                startDate:  moment(bloodPressureAndHeartRateMeasurementes[heartRateMeasurementsIndex].get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT).toDate(),
                endDate: moment(bloodPressureAndHeartRateMeasurementes[heartRateMeasurementsIndex].get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT).toDate(),
                dataType: 'heart_rate',
                value: bloodPressureAndHeartRateMeasurementes[heartRateMeasurementsIndex].get('Custom_004'),
                sourceName: 'boso'
            }

            navigator.health.store(heartRateValuesTransmissionObject, function() {
                var measurementId = bloodPressureAndHeartRateMeasurementes[heartRateMeasurementsIndex].get('DataId');

                Boso.markMeasurementAsTransmitted(genericColNameSentTo, measurementId);

                heartRateMeasurementsIndex++;
                transmitHeartRateValues(onHeartRateTransmissionReady);
            }, function(error) {
                console.log('error: ', error);

                var sentToGoogleCommitDate = bloodPressureAndHeartRateMeasurementes[heartRateMeasurementsIndex].get(genericColNameSentTo);
                if (sentToGoogleCommitDate) {
                    var measurementId = bloodPressureAndHeartRateMeasurementes[heartRateMeasurementsIndex].get('DataId');
                    Boso.unmarkMeasurementAsTransmitted(genericColNameSentTo, measurementId);
                }

                heartRateMeasurementsIndex++;
                transmitHeartRateValues(onHeartRateTransmissionReady);
            });
        }
    },

    sendToSciTIM: function(newGenericDataUserModel, onSciTimTransmissionReady) {
        var SCITIM_DATE_FORMAT = 'YYYY-MM-DD';
        var SCITIM_TIME_FORMAT = 'HH:mm:ss';
        
        Boso.deactivateSciTimOnPeriodOver()

        if(!navigator.onLine) {
            if(_.isFunction(onSciTimTransmissionReady)) {
                onSciTimTransmissionReady();
            }
            return;
        }
        
        if (!newGenericDataUserModel) {
            if(_.isFunction(onSciTimTransmissionReady)) {
                onSciTimTransmissionReady();
            };
            return;
        }

        var userNr = newGenericDataUserModel.get('Custom_001');
        if (userNr === Boso.UserNrSampleUser) {
            if(_.isFunction(onSciTimTransmissionReady)) {
                onSciTimTransmissionReady();
            };
            return;
        }

        var isForwardingActive = +newGenericDataUserModel.get('Custom_012');
        if (!isForwardingActive) {
            if(_.isFunction(onSciTimTransmissionReady)) {
                onSciTimTransmissionReady();
            };
            return;
        }

        //Alle Messwerte (Blutdruck und Puls)auslesen, die nicht noch nicht übertragen sind.
        var bloodPressureAndHeartRateMeasurementes = _.filter(window.genericData.models, function (model) {
            return model.get('EntityId') === Boso.MESSUREMENT_ENTITYID
                && model.get('IsActive') === true
                && model.get('Custom_009') === newGenericDataUserModel.get('DataId') //BOSO_Benutzer
                && !model.get('Custom_006')
                && !model.get('Custom_015') //if property "sentToSciTIMOn" not set
        });

        if (!bloodPressureAndHeartRateMeasurementes || bloodPressureAndHeartRateMeasurementes.length === 0) {
            if(_.isFunction(onSciTimTransmissionReady)) {
                onSciTimTransmissionReady();
            }

            return;
        }

        var bloodPressureDevice = _.find(window.genericData.models, function (deviceModel) {
            return (
                deviceModel.get('EntityId') === Boso.BOSO_GEREATE_ENTITY_ID
                && deviceModel.get('IsActive') === true
                && deviceModel.get('Custom_003') === 'boso medicus system'
            );
        });

        var serialNr = bloodPressureDevice.get('Custom_005');
        var csvFileContent = [];
        var separator = ',';
        bloodPressureAndHeartRateMeasurementes.forEach(function(measurement) {
            var params = {
                senderId: 'boso',
                recordID: 'RR',
                deviceType: 'BP1',
                deviceId: serialNr,
                createDate: moment(measurement.get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT).format(SCITIM_DATE_FORMAT),
                createTime: moment(measurement.get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT).format(SCITIM_TIME_FORMAT),
                unitBloodPressure: 'mmHg',
                systolic: measurement.get('Custom_002'),
                diastolic: measurement.get('Custom_003'),
                unitHeartRate: 'bpm',
                heartRate: measurement.get('Custom_004'),
                sendDate: moment().format(SCITIM_DATE_FORMAT),
                sendTime: moment().format(SCITIM_TIME_FORMAT),
                counter: 0,
                flag: 'N'
            }

            var csvRow = params.senderId + separator + params.recordID + separator + params.deviceType + separator + params.deviceId + separator;
            csvRow += params.createDate + separator + params.createTime + separator + params.unitBloodPressure + separator
            csvRow += params.systolic + separator + params.diastolic + separator + params.unitHeartRate + separator + params.heartRate + separator
            csvRow += params.sendDate + separator + params.sendTime + separator + params.counter + separator + params.flag

            csvFileContent.push(csvRow);
        });

        var createDateCsvFile = new Date();
        var createdYear = createDateCsvFile.getFullYear();
        var createdMonth = createDateCsvFile.getMonth() + 1;
        var createdDay = createDateCsvFile.getDate();
        var createdHours = createDateCsvFile.getHours();
        var createdMinutes = createDateCsvFile.getMinutes();
        var createdMilliseconds = createDateCsvFile.getMilliseconds();

        var recordsSent = 0;
        var successCounter = 0;
        var sendTries = 0;
        var maxSendTries = 3;

        showSciTIMResendDialog = function () {
            var confirmMessage = Utils.getText('TRANSMISSION_FAILED_SCITIM');
            showConfirmMessage(confirmMessage, function (buttonIndex) {
                if (buttonIndex === 2) {
                    restartTransmission();
                    return;
                }
                
                finishTransmission();    
            }, '', Utils.getText('ABBRECHEN').concat(',', Utils.getText('WIEDERHOLEN')));
        }

        function finishTransmission() {
            alert(successCounter + ' ' + Utils.getText('TRANSMISSION_SUCCESS_SCITIM'));
            if(_.isFunction(onSciTimTransmissionReady)) {
                onSciTimTransmissionReady();
            }
        }

        function restartTransmission() {
            recordsSent = 0;
            successCounter = 0;
            sendTries = 0;
            sendAllRecordsAsCsv();
        }

        function sendAllRecordsAsCsv() {
            if(recordsSent >= bloodPressureAndHeartRateMeasurementes.length) {
                if(!!successCounter && successCounter === recordsSent) {
                    alert(successCounter + ' ' + Utils.getText('TRANSMISSION_SUCCESS_SCITIM'));
                    
                    if(_.isFunction(onSciTimTransmissionReady)) {
                        onSciTimTransmissionReady();
                    } 
                    return;
                }

                if((!!successCounter || successCounter === 0) && successCounter < recordsSent) 
                {
                    showSciTIMResendDialog();
                }

                return;
            }

            var READYSTATES = {
                UNSENT: 0,
                OPENED: 1,
                HEADERS_RECEIVED: 2,
                LOADING: 3,
                DONE: 4
            }

            // testsystem zugangsdaten
            // var url = Boso.SCITIM_ENDPOINT_STAGING;
            // var username = "https_boso_tst";
            // var password = "aib9Ca2zootheesookachao3";

            
            // produktivsystem zugangsdaten
            var url = Boso.SCITIM_ENDPOINT_PROD;
            var username = "https_boso_pro";
            var password = "Chai3caNgeeph0ze1shooGhe5chei9Je";

            var http = new XMLHttpRequest();
            http.open('POST', url, true, username, password);
            http.setRequestHeader('Authorization', "Basic " + btoa(username + ':' + password));
            http.setRequestHeader('Cache-Control', 'no-cache');

            var row = csvFileContent[recordsSent]
            var blob = new Blob([row], { type: 'text/csv;charset=utf-8;' });

            http.onreadystatechange = function () {
                if (http.readyState === READYSTATES.DONE) {
                    var success = http.status === 200 && !!http.response;
                    console.log('successfully sent to sciTIM: ', success);

                    if (success) {
                        var measurementId = bloodPressureAndHeartRateMeasurementes[recordsSent].get('DataId');
                        Boso.markMeasurementAsTransmitted(Boso.MEASUREMENT_TRANSMISSION_TARGETS.SCITIM, measurementId);
                        successCounter++;
                    }

                    sendTries++;
                    if(!success && (http.status === 0 || http.status === 502) && sendTries < maxSendTries) {
                        sendAllRecordsAsCsv();
                    } else {
                        sendTries = 0;
                        recordsSent++;
                        sendAllRecordsAsCsv();
                    }
                }
            }
            http.send(blob);
        }

        sendAllRecordsAsCsv();
    },

    sendToCasProZdravi: function(newGenericDataUserModel, onCasProZdraviTransmissionReady) {
        var CASPROZDRAVI_DATE_FORMAT = 'YYYY-MM-DD';
        var CASPROZDRAVI_TIME_FORMAT = 'HH:mm:ss';
        
        Boso.deactivateCasProZdraviOnPeriodOver();

        if(!navigator.onLine) {
            if(_.isFunction(onCasProZdraviTransmissionReady)) {
                onCasProZdraviTransmissionReady();
            }
            return;
        }
        
        if (!newGenericDataUserModel) {
            if(_.isFunction(onCasProZdraviTransmissionReady)) {
                onCasProZdraviTransmissionReady();
            };
            return;
        }

        var userNr = newGenericDataUserModel.get('Custom_001');
        if (userNr === Boso.UserNrSampleUser) {
            if(_.isFunction(onCasProZdraviTransmissionReady)) {
                onCasProZdraviTransmissionReady();
            };
            return;
        }

        var isForwardingActive = +newGenericDataUserModel.get('Custom_014');
        if (!isForwardingActive) {
            if(_.isFunction(onCasProZdraviTransmissionReady)) {
                onCasProZdraviTransmissionReady();
            };
            return;
        }

        //Alle Messwerte (Blutdruck und Puls)auslesen, die nicht noch nicht übertragen sind.
        var bloodPressureAndHeartRateMeasurementes = _.filter(window.genericData.models, function (model) {
            return model.get('EntityId') === Boso.MESSUREMENT_ENTITYID
                && model.get('IsActive') === true
                && model.get('Custom_009') === newGenericDataUserModel.get('DataId') //BOSO_Benutzer
                && !model.get('Custom_006')
                && !model.get('Custom_016') //if property "sentToCasProZdraviOn" not set
        });

        if (!bloodPressureAndHeartRateMeasurementes || bloodPressureAndHeartRateMeasurementes.length === 0) {
            if(_.isFunction(onCasProZdraviTransmissionReady)) {
                onCasProZdraviTransmissionReady();
            }

            return;
        }

        var bloodPressureDevice = _.find(window.genericData.models, function (deviceModel) {
            return (
                deviceModel.get('EntityId') === Boso.BOSO_GEREATE_ENTITY_ID
                && deviceModel.get('IsActive') === true
                && deviceModel.get('Custom_003') === 'boso medicus system'
            );
        });

        var serialNr = bloodPressureDevice.get('Custom_005');
        var csvFileContent = [];
        var separator = ',';
        bloodPressureAndHeartRateMeasurementes.forEach(function(measurement) {
            var params = {
                senderId: newGenericDataUserModel.get('Custom_016'),
                recordID: 'RR',
                deviceType: 'BP1',
                deviceId: serialNr,
                createDate: moment(measurement.get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT).format(CASPROZDRAVI_DATE_FORMAT),
                createTime: moment(measurement.get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT).format(CASPROZDRAVI_TIME_FORMAT),
                unitBloodPressure: 'mmHg',
                systolic: measurement.get('Custom_002'),
                diastolic: measurement.get('Custom_003'),
                unitHeartRate: 'bpm',
                heartRate: measurement.get('Custom_004'),
                sendDate: moment().format(CASPROZDRAVI_DATE_FORMAT),
                sendTime: moment().format(CASPROZDRAVI_TIME_FORMAT),
                counter: 0,
                flag: 'N'
            }

            var csvRow = params.senderId + separator + params.recordID + separator + params.deviceType + separator + params.deviceId + separator;
            csvRow += params.createDate + separator + params.createTime + separator + params.unitBloodPressure + separator
            csvRow += params.systolic + separator + params.diastolic + separator + params.unitHeartRate + separator + params.heartRate + separator
            csvRow += params.sendDate + separator + params.sendTime + separator + params.counter + separator + params.flag

            csvFileContent.push(csvRow);
        });

        var createDateCsvFile = new Date();
        var createdYear = createDateCsvFile.getFullYear();
        var createdMonth = createDateCsvFile.getMonth() + 1;
        var createdDay = createDateCsvFile.getDate();
        var createdHours = createDateCsvFile.getHours();
        var createdMinutes = createDateCsvFile.getMinutes();
        var createdMilliseconds = createDateCsvFile.getMilliseconds();

        var recordsSent = 0;
        var successCounter = 0;
        var sendTries = 0;
        var maxSendTries = 3;

        showCasProZdraviResendDialog = function () {
            var confirmMessage = Utils.getText('TRANSMISSION_FAILED_CAS_PRO_ZDRAVI');
            showConfirmMessage(confirmMessage, function (buttonIndex) {
                if (buttonIndex === 2) {
                    restartTransmission();
                    return;
                }
                
                finishTransmission();    
            }, '', Utils.getText('ABBRECHEN').concat(',', Utils.getText('WIEDERHOLEN')));
        }

        function finishTransmission() {
            alert(successCounter + ' ' + Utils.getText('TRANSMISSION_SUCCESS_CAS_PRO_ZDRAVI'));
            if(_.isFunction(onCasProZdraviTransmissionReady)) {
                onCasProZdraviTransmissionReady();
            }
        }

        function restartTransmission() {
            recordsSent = 0;
            successCounter = 0;
            sendTries = 0;
            sendAllRecordsAsCsv();
        }

        function sendAllRecordsAsCsv() {
            if(recordsSent >= bloodPressureAndHeartRateMeasurementes.length) {
                if(!!successCounter && successCounter === recordsSent) {
                    alert(successCounter + ' ' + Utils.getText('TRANSMISSION_SUCCESS_CAS_PRO_ZDRAVI'));
                    
                    if(_.isFunction(onCasProZdraviTransmissionReady)) {
                        onCasProZdraviTransmissionReady();
                    } 
                    return;
                }

                if((!!successCounter || successCounter === 0) && successCounter < recordsSent) 
                {
                    showCasProZdraviResendDialog();
                }

                return;
            }

            var READYSTATES = {
                UNSENT: 0,
                OPENED: 1,
                HEADERS_RECEIVED: 2,
                LOADING: 3,
                DONE: 4
            }

            // testsystem zugangsdaten
            // var url = Boso.SCITIM_ENDPOINT_STAGING;
            // var username = "https_boso_tst";
            // var password = "aib9Ca2zootheesookachao3";

            // produktivsystem zugangsdaten
            var url = Boso.CASPROZDRAVI_ENDPOINT_PROD;
            var username = "https_boso_pro";
            var password = "Chai3caNgeeph0ze1shooGhe5chei9Je";

            var http = new XMLHttpRequest();
            http.open('POST', url, true, username, password);
            // http.setRequestHeader('Authorization', "Basic " + btoa(username + ':' + password));
            http.setRequestHeader('Cache-Control', 'no-cache');

            var row = csvFileContent[recordsSent]
            var blob = new Blob([row], { type: 'text/csv;charset=utf-8;' });

            http.onreadystatechange = function () {
                if (http.readyState === READYSTATES.DONE) {
                    var success = http.status === 200 && !!http.response;
                    console.log('successfully sent to Cas pro zdravi: ', success);

                    if (success) {
                        var measurementId = bloodPressureAndHeartRateMeasurementes[recordsSent].get('DataId');
                        Boso.markMeasurementAsTransmitted(Boso.MEASUREMENT_TRANSMISSION_TARGETS.CASPROZDRAVI, measurementId);
                        successCounter++;
                    }

                    sendTries++;
                    if(!success && (http.status === 0 || http.status === 502) && sendTries < maxSendTries) {
                        sendAllRecordsAsCsv();
                    } else {
                        sendTries = 0;
                        recordsSent++;
                        sendAllRecordsAsCsv();
                    }
                }
            }
            http.send(blob);
        }

        sendAllRecordsAsCsv();
    },

    sendToMedMobile: function (serialNr, pairedTimezone) {
        var currentUser = Boso.getCurrentUser();

        var phoneString = currentUser.get('Custom_008');
        if (!phoneString || phoneString.toLowerCase().trim() !== Boso.MM_TRANSMISSION_ACTIVATION_STRING) {
            console.log('Transmission to medmobile is disabled!');
            return
        }

        var userNr = currentUser.get('Custom_001');
        console.log('Benutzernummer: ', userNr);
        if (userNr === Boso.UserNrSampleUser) {
            console.log('SampleUser-data will not be transmitted to medmobile');
            return;
        }

        //Alle Messwerte auslesen, die nicht noch nicht übertragen sind.
        var measurements = _.filter(window.genericData.models, function (model) {
            return model.get('EntityId') === Boso.MESSUREMENT_ENTITYID
                && model.get('IsActive') === true
                && model.get('Custom_009') === currentUser.get('DataId')
                && !model.get('Custom_006')
                && !model.get('Custom_010')
        });

        if (!measurements || measurements.length === 0) {
            console.log('There are no measurements to transmit to medmobile.');
            return;
        }

        var ianaTimeZone = '';
        var successCounter = 0;
        var measurementIndex = 0;
        var medMobileResult = function (successful, measurementId) {
            measurementIndex++;
            if (!!successful && successCounter + 1 < measurements.length) {
                successCounter++;
                Boso.markAsTransmittedToMedMobile(measurementId);
                sendData(ianaTimeZone);
            } else if (!!successful && successCounter + 1 === measurements.length) {
                successCounter++;
                Boso.markAsTransmittedToMedMobile(measurementId);
                alert(successCounter + ' ' + Utils.getText('TRANSMISSION_SUCCESS'));
            } else if (!successful) {
                if (successCounter  > 0) {
                    alert(successCounter + ' ' + Utils.getText('TRANSMISSION_SUCCESS'));
                }
                Boso.showMMResendDialog(serialNr, pairedTimezone);
            }
        }

        var sendData = function (currentIanaTimeZone) {
            ianaTimeZone = currentIanaTimeZone;
            console.log('Transmit ' + measurements.length + ' measurements to medmobile.');
            if (measurements.length > 0 && !!measurements[measurementIndex]) {
                var mmParams = {
                    systolic: measurements[measurementIndex].get('Custom_002'),
                    diastolic: measurements[measurementIndex].get('Custom_003'),
                    pulse: measurements[measurementIndex].get('Custom_004'),
                    datetime: moment(measurements[measurementIndex].get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT),
                    bloodPressureDeviceSN: serialNr,
                    currentTimeZone: currentIanaTimeZone,
                    pairedTimezone: pairedTimezone
                }

                var mMDataRecord = Boso.createMedMobileXMLString(mmParams);
                Boso.sendXMLRequest(mMDataRecord, measurements[measurementIndex].get('DataId'), medMobileResult);
            }
        }

        Boso.getCurrentIanaTimeZone(sendData);
    },

    showStatisticLoader: function () {
        if (!Boso.IsUsedForReporting) {
            if (!Boso.statisticLoaderTimeout || (!!Boso.statisticLoaderTimeout && Boso.statisticLoaderTimeout.cleared)) {
                Boso.statisticLoaderTimeout = new Utils.ExplicitTimeout(function () {
                    hideloader();
                }, 30000, 'statisticLoaderTimeout');
                Boso.statisticLoaderTimeout.restart();
                showLoader(Utils.getText('STATISTIK_AKTUALISIEREN'));
            }
        }
    },

    hideStatisticLoader: function () {
        if (!!Boso.statisticLoaderTimeout && !Boso.statisticLoaderTimeout.cleared && !Boso.IsUsedForReporting) {
            Boso.statisticLoaderTimeout.clear();
            Boso.statisticLoaderTimeout = null;
            hideloader();
        }
    },

    showGraphicsLoader: function () {
        if (!Boso.IsUsedForReporting) {
            if (!Boso.graphicsLoaderTimeout || (!!Boso.graphicsLoaderTimeout && Boso.graphicsLoaderTimeout.cleared)) {
                Boso.graphicsLoaderTimeout = new Utils.ExplicitTimeout(function () {
                    hideloader();
                }, 30000);
                Boso.graphicsLoaderTimeout.restart();

                setTimeout(function () {
                    showLoader(Utils.getText('GRAFIK_LADEN'));
                }, 0);
            }
        }
    },

    hideGraphicsLoader: function () {
        if (!!Boso.graphicsLoaderTimeout && !Boso.graphicsLoaderTimeout.cleared && !Boso.IsUsedForReporting) {
            Boso.graphicsLoaderTimeout.clear();
            Boso.graphicsLoaderTimeout = null;

            setTimeout(function () {
                hideloader();
            }, 0)
        }
    },

    checkItem: function (element) {
        if (!element) {
            return;
        }

        var activeChkBox = $('.ui-icon-check');
        if (!activeChkBox || activeChkBox.length === 0) {
            return
        }
        activeChkBox.addClass('ui-icon-none');
        activeChkBox.removeClass('ui-icon-check');

        var clickedChkBox = $('[id=' + element.id + '].checkBox');
        if (!clickedChkBox || clickedChkBox.length === 0) {
            return;
        }
        clickedChkBox.removeClass('ui-icon-none');
        clickedChkBox.addClass('ui-icon-check');
    },

    onSaveActions: function (model) {
        switch (model.get('EntityId')) {
            case Boso.USER_ENTITYID:
                Boso.onSaveUser(model);
                break;
            case Boso.MESSUREMENT_ENTITYID:
                Boso.onSaveWeightMeasurement(model);
                break;
            default:
                break;
        }
    },

    onSaveUser: function (newUserModel) {
        if(!newUserModel) {
            return;
        }
    },

    disconnectFromGoogleFit: function(newUserModel, onDisconnected) {
        if(Boso.getPlatform() !== 'android') {
            if (_.isFunction(onDisconnected)) {
                onDisconnected();
            }
            return;
        }

        if(!newUserModel) {
            if(_.isFunction(onDisconnected)) {
                onDisconnected()
            }
            return;
        }

        var datatypes = [
            {
                write : ['weight', 'heart_rate', 'blood_pressure']  // Write only permission
            }
        ];

        var disconnect = function() {
            navigator.health.disconnect(function(success) {
                console.log('successfully disconnected: ', success);
                if(_.isFunction(onDisconnected)) {
                    onDisconnected()
                }
            }, function(error) {
                console.log('error disconnecting from google fit: ', error);
                if(_.isFunction(onDisconnected)) {
                    onDisconnected()
                }
            });
        }
        
        navigator.health.isAuthorized(datatypes, function(success) {
            disconnect();
        }, function(error) {
            disconnect();
        });

        
    },

    onSaveWeightMeasurement: function(genericDataMeasurementModel) {
        setTimeout(function() {
            if (!genericDataMeasurementModel.get('Custom_006')) {
                return;
            }
    
            var currentUser = Boso.getCurrentUser();

            Boso.sendToGoogleFitOrAppleHealth(currentUser);
        }, 1000);
    },

    beforeSaveActions: function (model) {
        switch (model.get('EntityId')) {
            case Boso.USER_ENTITYID:
                // beforeSaveUser(model);
                break;
            default:
                break;
        }
    },

    beforeSaveUser: function (genericDataUserModel) {
        var activeUserModel = Boso.getCurrentUser();
        if(!activeUserModel) {
            return;
        }

        if (!!ActionHelper && !!ActionHelper.CurrentView && ActionHelper.CurrentView.Type === 'Edit') {
            var googleFitConnectionActivated = +genericDataUserModel.get('Custom_010');
            var appleHealthConnectionActivated = +genericDataUserModel.get('Custom_011');
            var sciTimConnectionActivated = +genericDataUserModel.get('Custom_012');
            if (!googleFitConnectionActivated) {
                Boso.disconnectFromGoogleFit(genericDataUserModel, function() {})
            } 

            // if (googleFitConnectionActivated ||appleHealthConnectionActivated || sciTimConnectionActivated) {
            if (googleFitConnectionActivated) {  
                Boso.sendToGoogleFitOrAppleHealth(genericDataUserModel, undefined, true);
            }
        }
    },

    activateNewUser: function () {
        if (!ActionHelper.IsValid) {
            return;
        }

        var users = _.filter(window.genericData.models, function (model) {
            return model.get('EntityId') === Boso.USER_ENTITYID
        });

        var oldActiveUser = null;
        var deactivateActiveUser = function () {
            oldActiveUser = _.find(users, function (user) {
                return user.get('Custom_005') === 1
            });

            if (!!oldActiveUser) {
                oldActiveUser.set('Custom_005', 0);
                window.genericData.add(oldActiveUser);
                oldActiveUser.saveItem();
            }
        }

        var newUser = null;
        var activateNewUser = function () {
            users = _.filter(users, function (user) {
                return user.get('IsActive') === true
            });

            newUser = _.last(users);
            newUser.set("Custom_005", 1);
            window.genericData.add(newUser);
            newUser.saveItem();
        }

        deactivateActiveUser();
        activateNewUser();

        setTimeout(function() {
            Boso.disconnectFromGoogleFit(oldActiveUser, function() {
                setTimeout(function() {
                    Boso.sendToGoogleFitOrAppleHealth(newUser, undefined, true);
                }, 500);
            }); 
        }, 500);
    },

    deleteUsersMeasurements: function (userId, callback) {
        var userMeasurementData = _.filter(window.genericData.models, function (model) {
            return model.get('EntityId') === Boso.MESSUREMENT_ENTITYID
                && model.get('Custom_009') === userId
                && model.get('IsActive') === true
        });

        _.each(userMeasurementData, function (measurement) {
            measurement.PersistToLocalStorageImmediately = false;
            window.genericData.remove(measurement);
            measurement.destroy();
        });

        window.persistGenericData();

        if (_.isFunction(callback)) {
            callback();
        }
    },

    deleteUser: function () {
        var users = _.filter(window.genericData.models, function (model) {
            return model.get('EntityId') === Boso.USER_ENTITYID
                && model.get('IsActive') === true
        });

        if (_.size(users) <= 1) {
            alert(Utils.getText('BENUTZER_NICHT_LOESCHBAR_KEINER_MARKIERT'));
            return;
        }

        var confirmMessage = Utils.getText('BENUTZER_LOESCHEN_BESTAETIGEN');
        showConfirmMessage(confirmMessage, function (buttonIndex) {
            if (buttonIndex !== 1) {
                return;
            }

            var checkedElements = $(ActionHelper.CurrentView.el).find('#listView .checkBox');

            var idsDeletedUsers = [];
            _.each(checkedElements, function (element) {
                if ($(element).hasClass("ui-icon-check") || $(element.parentNode.firstChild).hasClass("ui-checkbox-on")) {
                    idsDeletedUsers.push(element.id);

                    var user = window.genericData.get(element.id);
                    user.set("IsActive", false);
                    user.set("SyncTime", CONSTS.DATE_DEFAULT);
                    window.genericData.add(user);
                    user.saveItem();

                    //user nicht mit remove() aus window.genericData entfernen, da sonst die Benutzernummer von gelöschten Benutzern wiederverwendet werden kann
                    Boso.activateNewUser(user);

                    var element = $(element.parentNode.firstChild);
                    var checked = element.hasClass("ui-checkbox-on")
                    !!checked ? $(element).closest('tr').remove() : $(element.parentNode).remove();
                }
            });

            if (idsDeletedUsers.length > 0) {
                setTimeout(function () {
                    showLoader(Utils.getText('MESSWERTE_LOESCHEN'));
                }, 0);

                setTimeout(function () {
                    delMeasurementsStartTime = moment();
                    _.each(idsDeletedUsers, function (userId) {
                        Boso.deleteUsersMeasurements(userId, hideloader);
                    });
                }, 1000);

                //REFRESH CURRENT VIEW
                var view = ActionHelper.CurrentView.options.view;
                var createViewString = 'new window.' + ActionHelper.CurrentView.options.view.attributes.ViewType + 'View({ view: view, modelId: null, refModelId: null });'
                var page = eval(createViewString);
                var viewTypeId = '#' + ActionHelper.CurrentView.options.view.attributes.ViewType;

                $(viewTypeId).empty();
                $(viewTypeId).append($(page.el));
                page.render();
                $.mobile.changePage(viewTypeId, { changeHash: false });
            }
        }, Utils.getText('WIRKLICH_LOESCHEN'), Utils.getText('LOESCHEN_ABBRECHEN'));
    },

    BenutzerAktivieren: function () {
        ActionHelper.Models = new Array();
        $(ActionHelper.CurrentView.el).find('#listView .checkBox').each(function () {
            if ($(this).hasClass("ui-icon-check")) {
                ActionHelper.Models.push(this.id);
            }
        });

        if (ActionHelper.Models == null || ActionHelper.Models.length != 1) {
            showMessage(Utils.getText('BENUTZER_AUSGEWAEHLT_MIN_1'));
        }
        else {
            var recordsRemoveAktiv = _.filter(window.genericData.models, function (model) {
                return model.get('EntityId') === Boso.USER_ENTITYID &&
                    model.get('Custom_005') === 1 &&
                    model.id !== ActionHelper.Models[0] &&
                    model.get('IsActive') === true
            });
            
            var recordAktiv = window.genericData.get(ActionHelper.Models[0]);
            recordAktiv.set("Custom_005", 1);
            recordAktiv.saveItem();
            window.genericData.add(recordAktiv);

            if (!!recordsRemoveAktiv && recordsRemoveAktiv.length > 0) {
                var recordRemoveAktiv = _.first(recordsRemoveAktiv);
                recordRemoveAktiv.set("Custom_005", 0);
                recordRemoveAktiv.saveItem();
                window.genericData.add(recordAktiv);
            }

            if (recordsRemoveAktiv && recordsRemoveAktiv.length > 0) {
                setTimeout(function() {
                    Boso.disconnectFromGoogleFit(recordsRemoveAktiv, function() {
                        setTimeout(function() {
                            Boso.sendToGoogleFitOrAppleHealth(recordAktiv, undefined, true);
                        }, 500);
                    }); 
                }, 500);
            } else {
                Boso.sendToGoogleFitOrAppleHealth(recordAktiv, undefined, true);
            }

            //REFRESH CURRENT VIEW
            var view = ActionHelper.CurrentView.options.view;
            var createViewString = 'new window.' + ActionHelper.CurrentView.options.view.attributes.ViewType + 'View({ view: view, modelId: null, refModelId: null });'
            var page = eval(createViewString);
            var viewTypeId = '#' + ActionHelper.CurrentView.options.view.attributes.ViewType;

            $(viewTypeId).empty();
            $(viewTypeId).append($(page.el));
            page.render();
            $.mobile.changePage(viewTypeId, { changeHash: false });
        }
    },

    checkActiveUser: function () {
        setTimeout(function () {
            var users = _.filter(window.genericData.models, function (model) {
                return model.get('EntityId') === Boso.USER_ENTITYID
                    && model.get('IsActive') === true
            });

            if (!users || users.length === 0) {
                return;
            }

            var activeUser = _.find(users, function (user) {
                return user.get('Custom_005') === 1
            });

            if (!activeUser || activeUser.length === 0) {
                return;
            }

            for (i = 0; i < users.length; i++) {
                var box = $('[id=' + users[i].get('DataId') + '].checkBox');

                if (!!box && box.length > 0) {
                    box.off('click').on('click', function () {
                        var activeChkBox = $('.ui-icon-check');
                        if (!activeChkBox || activeChkBox.length === 0) {
                            return;
                        }
                        activeChkBox.addClass('ui-icon-none');
                        activeChkBox.removeClass('ui-icon-check');

                        setTimeout(function () {
                            Boso.BenutzerAktivieren();
                        }, 0);
                    });
                }
            }
            
            var chkBox = $('[id=' + activeUser.get('DataId') + '].checkBox');
            if (!chkBox.hasClass("ui-icon-check")) {
                chkBox.removeClass('ui-icon-none');
                chkBox.addClass('ui-icon-check');
            }

            if (chkBox && chkBox.offset() && (chkBox.offset().top || chkBox.offset().top === 0)) {
                $('#pagecontent').animate({scrollTop: chkBox.offset().top - 80}, 0);
            }
        }, 0);
    },

    GetNextBenutzerNummer: function () {
        var ret = "";

        var recordsAktiv = _.filter(window.genericData.models, function (model) {
            return model.get('EntityId') === Boso.USER_ENTITYID
        });

        if (recordsAktiv != null && recordsAktiv.length > 0) {
            ret = (recordsAktiv.length + 1).toString();
        }
        else {
            ret = "1";
        }

        return ret + "-M";
    },

    isSampleUser: function () {
        var currentUser = Boso.getCurrentUser();
        if(!currentUser) {
            return false;
        }

        var userNr = currentUser.get('Custom_001');
        return userNr === Boso.UserNrSampleUser;
    },

    getCurrentUser: function () {
        var activeUser = _.find(window.genericData.models, function (model) {
            return model.get('EntityId') === Boso.USER_ENTITYID
                && model.get('IsActive') === true
                && model.get('Custom_005') === 1
        });
        return !!activeUser ? activeUser : null;
    },

    preSetMeasurementsUser: function() {
        var fieldIdUser = '63a23987-0c1a-43cc-95b4-41663edfd865';
        var model = Boso.getCurrentUser();
        $('#' + fieldIdUser).val(model.get('DataId'));
    },

    roundWeightBeforeSave: function() {
        var fieldIdKg = '49e03987-0c1a-4716-a912-f1663534d4ca'

        var currentValue = $('#' + fieldIdKg).val();
        if (currentValue) {
            console.log('currentValue: ', currentValue);
            $('#' + fieldIdKg).val(Utils.round(currentValue, 1));
            console.log('newFieldValue: ', $('#' + fieldIdKg).val());
        }
    },

    deleteDevices: function () {
        var device;
        var confirmMessage = Utils.getText('MARKIERTE_EINTRAEGE_ENTFERNEN');
        var removeDevice = function (device, deviceDomElement) {
            device.PersistToLocalStorageImmediately = true;
            window.genericData.remove(device);
            device.destroy();

            //Eintrag aus DOM entfernen
            var element = $(deviceDomElement.parentNode.firstChild);
            var checked = element.hasClass("ui-checkbox-on")
            !!checked ? $(this).closest('tr').remove() : $(deviceDomElement.parentNode).remove();
        }

        var deleteDeviceOnAndroid = function (device, deviceDomElement) {
            var deviceAdress = device.get('Custom_001');
            BluetoothHelper.unbond(deviceAdress, function (result) {
                if (!!result) {
                    removeDevice(device, deviceDomElement);
                }
            }, function (error) {
                if (!!error) {
                    console.log('Error during device-unbonding: ', error)
                    removeDevice(device, deviceDomElement);
                    alert(Utils.getText('GERAET_ENTKOPPELN'));
                }
            });
        }

        showConfirmMessage(confirmMessage, function (buttonIndex) {
            if (buttonIndex !== 1) {
                return;
            }

            $(ActionHelper.CurrentView.el).find('#listView .checkBox').each(function () {
                if ($(this).hasClass("ui-icon-check") || $(this.parentNode.firstChild).hasClass("ui-checkbox-on")) {
                    var deviceDomElement = this;
                    device = window.genericData.get(this.id);

                    if (window.device.platform === 'iOS') {
                        removeDevice(device, deviceDomElement);
                        alert(Utils.getText('GERAET_ENTKOPPELN'));
                    } else {
                        deleteDeviceOnAndroid(device, deviceDomElement);
                    }
                }
            });

            $(document).trigger('scrollstop');
            $(document).trigger('create');
        }, Utils.getText('WIRKLICH_LOESCHEN'), Utils.getText('LOESCHEN_ABBRECHEN'));
    },

    startBluetoothBackgroundWorker: function () {
        var devicesArePaired = _.some(window.genericData.models, function (deviceModel) {
            return deviceModel.get('EntityId') === Boso.BOSO_GEREATE_ENTITY_ID && deviceModel.get('IsActive') === true
        });

        if (devicesArePaired && !Boso.BluetoothDeviceControl.isPairing) {
            Boso.BluetoothDeviceControl.worker.restart();
        }
    },

    KoppleGeraet: function () {
        try {
            Boso.BluetoothDeviceControl.pairDevice();
        } catch (ex) {
            var logMessage = !!ex ? ex.toString() : null;
            Boso.BluetoothDeviceControl.processError(null, logMessage);
        }
    },

    BluetoothDeviceControl: {
        lastConnectionAddress: null,
        isPairing: false,

        worker: new Utils.ExplicitTimeout(function () {
            Boso.BluetoothDeviceControl.findAdvertisingDevices();
        }, 2000, 'BLE backgroundworker'),

        closeCurrentConnection: function (address, success, error) {
            var currentAddress = address;
            var fnSuccess = function (result) {
                if (_.isFunction(success)) {
                    success();
                }
                lastConnectionAddress = null;
                console.log('Connection closed --> by closeCurrentConnection(): ', JSON.stringify(result, null, 4));
            }

            if (!currentAddress) {
                currentAddress = Boso.BluetoothDeviceControl.lastConnectionAddress;
            }

            if (currentAddress) {
                if (window.device.platform === 'iOS') {
                    BluetoothHelper.disconnect(currentAddress);
                }
                BluetoothHelper.close(currentAddress, fnSuccess, error);
            }
        },

        BTProcessTimeout: new Utils.ExplicitTimeout(function () {
            var errorMessage = Utils.getText('GERAET_VERBINDUNG_ABGEBROCHEN');
            Boso.BluetoothDeviceControl.processError(errorMessage, errorMessage, 'BTProcessTimeout');
        }, 30000, 'BTProcessTimeout'),

        processError: function (errorMessage, logMessage, callingMethodName) {
            hideloader();

            if (errorMessage) {
                alert(errorMessage);
            }

            if (logMessage) {
                console.log('Fehler in: ', callingMethodName, logMessage);
            }

            Boso.BluetoothDeviceControl.closeCurrentConnection();
            Boso.BluetoothDeviceControl.BTProcessTimeout.clear();
            Boso.BluetoothDeviceControl.isPairing = false;
            Boso.startBluetoothBackgroundWorker();
        },

        //pair boso-devices with the app
        pairDevice: function () {
            var scanTries = 0;
            var maxScanTries = 10;
            var scanDuration = 1000;

            var pairingTries = 0;
            var maxPairingTries = 10;

            var newDevice = {
                address: null,
                name: null,
                displayValue: null
            }

            //as soon as the device is paired with the phone and settings are set, store it in the app.
            var storePairedDevice = function (userIndex, serialNumber, pairingTimezone) {
                //Data zur DB hinzufügen
                console.log('Neues Gerät zur Datenbank hinzufügen');
                var newModel = null;
                newModel = new window.GenericDataRawElement();
                var itemId = window.generateGuid();
                newModel.set('DataId', itemId);
                newModel.set('ObjectId', itemId);
                newModel.id = itemId;
                newModel.set({ 'id': itemId });
                newModel.set('EntityId', Boso.BOSO_GEREATE_ENTITY_ID);
                newModel.set('UserId', window.currentUser.UserId);
                newModel.set('MandatorId', window.currentUser.MandatorId);
                newModel.set('OrgUnitId', window.currentUser.OrgUnitId);
                newModel.set('Custom_001', newDevice.address);
                newModel.set('Custom_002', newDevice.name);
                newModel.set('Custom_003', newDevice.displayValue);
                newModel.set('Custom_004', userIndex);
                newModel.set('Custom_005', serialNumber);
                newModel.set('Custom_006', pairingTimezone);
                newModel.set('IsActive', true);
                newModel.set('IsNew', true);
                newModel.set('EntryTime', moment().toDate());
                newModel.set('SyncTime', CONSTS.DATE_DEFAULT);

                newModel.save();
                window.genericData.add(newModel);
                newModel.save();

                console.log('USERINDEX: ', userIndex);
                console.log('ADD DEVICE: ', JSON.stringify(newModel, null, 4));

                //Verbindung schließen.
                Boso.BluetoothDeviceControl.closeCurrentConnection(newDevice.address);
                hideloader();
                Boso.BluetoothDeviceControl.BTProcessTimeout.clear();
                Boso.BluetoothDeviceControl.isPairing = false;
                Boso.startBluetoothBackgroundWorker();
                alert(Utils.getText('GERAET_HINZUGEFUEGT'));

                //REFRESH CURRENT VIEW
                var view = ActionHelper.CurrentView.options.view;
                var createViewString = 'new window.' + ActionHelper.CurrentView.options.view.attributes.ViewType + 'View({ view: view, modelId: null, refModelId: null });'
                var page = eval(createViewString);
                var viewTypeId = '#' + ActionHelper.CurrentView.options.view.attributes.ViewType;

                $(viewTypeId).empty();
                $(viewTypeId).append($(page.el));
                page.render();
                $.mobile.changePage(viewTypeId, { changeHash: false });
            }

            //read the chosen weightScale userId and pass it for storing.
            var getDeviceUser = function (onWriteResult) {
                var errorMessage = Utils.getText('VERBINDUNG_ABGEBROCHEN');

                var params = {
                    "address": newDevice.address,
                    "service": BluetoothHelper.SERVICE_IDS.weightScale,
                    "characteristic": BluetoothHelper.CHARACTERISTIC_IDS.weightMeasurement,
                }

                if (newDevice.name.indexOf(Boso.BOSO_DEVICENAMES.BOSO_BLODDPRESSURE_NAME) > -1) {
                    params.service = BluetoothHelper.SERVICE_IDS.bloodPressure;
                    params.characteristic = BluetoothHelper.CHARACTERISTIC_IDS.bloodPressureMeasurement;
                }

                BluetoothHelper.subscribe(function (result) {
                    if (result && result.status === BluetoothHelper.SUBSCRIPTION_STATES.subscribed) {
                        console.log('Subscribed')
                    }

                    if (result && result.status === BluetoothHelper.SUBSCRIPTION_STATES.subscribedResult) {
                        if (result.value) {
                            var userIndex = bluetoothle.encodedStringToBytes(result.value);
                            userIndex = userIndex[userIndex.length - 1];

                            //store userIndex as the number that is shown on the weightScale display
                            if (!!userIndex || userIndex === 0) {
                                userIndex = userIndex + 1;
                            }

                            console.log('IndicationValue: ', bluetoothle.encodedStringToBytes(result.value))
                            console.log('USERINDEX: ', userIndex);
                            // storePairedDevice(userIndex, undefined);
                            if (_.isFunction(onWriteResult)) {
                                onWriteResult(userIndex);
                            }
                        }
                    }
                }, function (error) {
                    var logMessage = 'SUBSCRIBE-ERROR: ' + JSON.stringify(error, null, 4);
                    Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'pairDevice: getDeviceUser:');
                }, params);
            }

            var readSerialNumber = function () {
                var params = {
                    'address': newDevice.address,
                    'service': BluetoothHelper.SERVICE_IDS.deviceInformation,
                    'characteristic': BluetoothHelper.CHARACTERISTIC_IDS.serialNumber
                }

                BluetoothHelper.read(function (result) {
                    if (!!result && !!result.value) {
                        var serialNumber = bluetoothle.bytesToString(bluetoothle.encodedStringToBytes(result.value));

                        if (newDevice.name.indexOf(Boso.BOSO_DEVICENAMES.BOSO_WEIGHT_NAME) > -1) {
                            getDeviceUser(function (userIndex) {
                                Boso.getCurrentIanaTimeZone(function (timezone) {
                                    storePairedDevice(userIndex, serialNumber, timezone);
                                });
                            });
                        } else if (newDevice.name.indexOf(Boso.BOSO_DEVICENAMES.BOSO_BLODDPRESSURE_NAME) > -1) {
                            Boso.getCurrentIanaTimeZone(function (timezone) {
                                storePairedDevice(undefined, serialNumber, timezone);
                            });
                        }
                    }
                }, function (error) {
                    var errorMessage = Utils.getText('LESE_SERIENNUMMER_FEHLGESCHLAGEN');
                    var logMessage = 'READ SN ERROR: ' + JSON.stringify(error, null, 4);
                    Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'readSN: readError');
                }, params)
            }

            //writeDateTime to device
            setDateTime = function (onWriteSuccess, deviceName, address) {
                showLoader(Utils.getText('GERAETEEINSTELLUNGEN'));
                var date = new Date();
                var years = date.getFullYear();
                var endian = Math.floor(years / 256);

                var tmp = new Uint8Array(7);
                tmp[0] = years - (endian * 256);
                tmp[1] = endian;
                tmp[2] = date.getMonth() + 1;
                tmp[3] = date.getDate();
                tmp[4] = date.getHours();
                tmp[5] = date.getMinutes();
                tmp[6] = date.getSeconds();

                var value = bluetoothle.bytesToEncodedString(tmp);
                console.log('Set DateTime: ' + value + " > " + tmp[0] + "," + tmp[1] + "," + tmp[2] + "," + tmp[3] + "," + tmp[4] + "," + tmp[5] + "," + tmp[6]);
                console.log('DateTime as encoded string: ', value);

                var serviceUuid = '';
                var characteristicUuid = '';

                if (deviceName.indexOf(Boso.BOSO_DEVICENAMES.BOSO_WEIGHT_NAME) > -1) {
                    serviceUuid = BluetoothHelper.SERVICE_IDS.weightScale;
                    characteristicUuid = BluetoothHelper.CHARACTERISTIC_IDS.dateTime;
                } else if (deviceName.indexOf(Boso.BOSO_DEVICENAMES.BOSO_BLODDPRESSURE_NAME) > -1) {
                    serviceUuid = BluetoothHelper.SERVICE_IDS.bloodPressure;
                    characteristicUuid = BluetoothHelper.CHARACTERISTIC_IDS.dateTime;
                }

                var type = window.device.platform === 'iOS' ? 'response' : 'noResponse';
                var params = {
                    "address": address,
                    "service": serviceUuid,
                    "characteristic": characteristicUuid,
                    "type": type,
                    "value": value
                };

                BluetoothHelper.write(function (writeResult) {
                    if (writeResult && writeResult.status && writeResult.status === BluetoothHelper.WRITE_STATES.written && _.isFunction(onWriteSuccess)) {
                        console.log('DATETIME WRITTEN: ', JSON.stringify(writeResult, null, 4));
                        onWriteSuccess();
                    }
                }, function (error) {
                    var errorMessage = Utils.getText('GERAET_NICHT_HINZUGEFUEGT');
                    var logMessage = !!error ? 'DateTime schreiben fehlgeschlagen: ' + JSON.stringify(error, null, 4) : null;
                    Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'pairDevice: setDateTime:');
                }, params);
            }

            var onDiscoverySuccess = function (discoveryResult) {
                console.log('DISCOVERY SUCCEEDED: ', JSON.stringify(discoveryResult, null, 4));
                if (discoveryResult && discoveryResult.address && discoveryResult.status && discoveryResult.status === BluetoothHelper.DISCOVER_STATES.discovered) {
                    // if (newDevice.name.indexOf(Boso.BOSO_DEVICENAMES.BOSO_WEIGHT_NAME) > -1) {
                    //     setDateTime(getDeviceUser, newDevice.name, newDevice.address);
                    // } else if (newDevice.name.indexOf(Boso.BOSO_DEVICENAMES.BOSO_BLODDPRESSURE_NAME) > -1) {
                    //     setDateTime(readSerialNumber, newDevice.name, newDevice.address);
                    // }

                    setDateTime(readSerialNumber, newDevice.name, newDevice.address);
                }
            }

            var onPaired = function () {
                showLoader(Utils.getText('GERAET_VERBUNDEN'));
                Boso.BluetoothDeviceControl.BTProcessTimeout.restart();

                var discoveryTries = 0;
                var discover = function () {
                    discoveryTries++;
                    BluetoothHelper.discover(newDevice.address, onDiscoverySuccess, function (error) {
                        if (discoveryTries < 3) {
                            discover();
                        } else {
                            var errorMessage = Utils.getText('GERAET_VERBINDUNG_ABGEBROCHEN');
                            Boso.BluetoothDeviceControl.processError(errorMessage, 'DISCOVERY-ERROR: Connection lost while pairing: ' + JSON.stringify(error, null, 4), 'pairDevice: discover:');
                        }
                    });
                }
                discover();
            }

            var onPairingFail = function () {
                pairingTimeout.clear();
                Boso.BluetoothDeviceControl.processError(Utils.getText('ERROR_NOT_PAIRED'));
            }

            var pairDeviceClassic = function () {
                pairingTries++;
                console.log('PairingTries: ' + pairingTries);

                var retrieveSuccess = function (isPaired) {
                    if (isPaired) {
                        if (window.device.platform !== 'iOS') {
                            var endTime = moment();
                            var diffSeconds = endTime.diff(Boso.pairingStartTime, 'milliseconds');

                            var base = 3000;
                            var factor = Utils.ceil((diffSeconds / base), 0);
                            var timeToWait = (base * factor - diffSeconds);

                            setTimeout(function () {
                                onPaired();
                            }, timeToWait);
                        } else {
                            onPaired();
                        }
                    } else if (pairingTries < maxPairingTries) {
                        console.log('PAIRING TIMEOUT RESTART: ', 'PairingTries:', pairingTries, 'MaxPairingTries:', maxPairingTries);
                        pairingTimeout.restart();
                    } else {
                        console.log('Pairing failed');
                        onPairingFail();
                    }
                }

                BluetoothHelper.isPaired(Boso.BluetoothDeviceControl.lastConnectionAddress, retrieveSuccess, function (error) {
                    var errorMessage = Utils.getText('ERROR_NOT_PAIRED');
                    var logMessage = Utils.getText('ERROR_NOT_PAIRED') + ': ' + JSON.stringify(error, null, 4);
                    Boso.BluetoothDeviceControl.processError(errorMessage, errorMessage, 'pairDevice: isPaired:');
                });
            }

            var onConnectionResult = function (connectionResult) {
                if (connectionResult && connectionResult.status && connectionResult.status === BluetoothHelper.CONNECTION_STATES.connected && connectionResult.address) {

                    Boso.BluetoothDeviceControl.BTProcessTimeout.restart();
                    Boso.BluetoothDeviceControl.lastConnectionAddress = connectionResult.address;

                    Boso.pairingStartTime = moment();
                    pairDeviceClassic();
                }

                if (connectionResult && connectionResult.status && connectionResult.status === BluetoothHelper.CONNECTION_STATES.disconnected) {
                    console.log('pairDevice: Connection Lost')
                    BluetoothHelper.reconnect(connectionResult.address, function (result) {
                        console.log('RECONNECT SUCCESS:', JSON.stringify(result, null, 4));
                    }, function (error) {
                        console.log('RECONNECT FAILED:', JSON.stringify(error, null, 4));
                        var errorMessage = Utils.getText('GERAET_VERBINDUNG_ABGEBROCHEN');
                        var logMessage = 'Connection lost while pairing: ' + JSON.stringify(connectionResult, null, 4);
                        Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'pairDevice: onConnectionResult:');
                    });
                }
            }
            var pairingTimeout = new Utils.ExplicitTimeout(pairDeviceClassic, 1000, 'pairingTimeout');

            var connectionTries = 0;
            var onConnectionError = function (error) {
                connectionTries++;
                console.log('PAIRING: CONNECTION ERROR: ', connectionTries);
                if (connectionTries < 2) {
                    BluetoothHelper.reconnect(newDevice.address, onConnectionResult, onConnectionError);
                } else {
                    var errorMessage = Utils.getText("VERBINDUNG_ABGEBROCHEN");
                    var logMessage = 'PAIRING: ConnectionError: ' + JSON.stringify(error, null, 4);
                    Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'pairdDevice: onConnectionError');
                }
            }

            //scan for new boso devices in pairing mode.
            //if new device has an address-property and is not already paired try to connect.
            var onScanSuccess = function (devicelist) {
                scanTries++;
                console.log('SCAN-TRIES: ', scanTries);

                //remove allready paired devices and devices without an address from devicelist
                var newBosoDevices = _.filter(devicelist, function (newDevice) {
                    return (
                        !!newDevice.address
                        && !_.some(window.genericData.models, function (model) {
                            return (
                                model.get('EntityId') === Boso.BOSO_GEREATE_ENTITY_ID
                                && model.get('IsActive') == true
                                && model.get('Custom_001') === newDevice.address
                            )
                        })
                    )
                });

                var handleFoundDevices = function (devicelist) {
                    if (!newBosoDevices || newBosoDevices.length === 0) {
                        alert(Utils.getText('KEINE_GERAETE'));
                        hideloader();
                        Boso.BluetoothDeviceControl.BTProcessTimeout.clear();
                        Boso.BluetoothDeviceControl.isPairing = false;
                        Boso.startBluetoothBackgroundWorker();
                        return;
                    }

                    var connectDevice = _.first(devicelist);
                    if (connectDevice.name.indexOf(Boso.BOSO_DEVICENAMES.BOSO_WEIGHT_NAME) > -1) {
                        newDevice.displayValue = "bosogramm system";
                    } else if (connectDevice.name.indexOf(Boso.BOSO_DEVICENAMES.BOSO_BLODDPRESSURE_NAME) > -1) {
                        newDevice.displayValue = "boso medicus system";
                    }
                    newDevice.address = connectDevice.address;
                    newDevice.name = connectDevice.name;

                    var msgConnectDevice = Utils.getText('GERAET_VERBINDEN');
                    msgConnectDevice = msgConnectDevice.replace('{Geraet}', newDevice.displayValue);
                    showLoader(msgConnectDevice);

                    Boso.BluetoothDeviceControl.BTProcessTimeout.restart();
                    BluetoothHelper.quirkConnect(newDevice.address, onConnectionResult, onConnectionError);
                }

                Boso.BluetoothDeviceControl.BTProcessTimeout.restart();
                var scanAgain = (!newBosoDevices || newBosoDevices.length === 0) && scanTries < maxScanTries;
                scanAgain ? BluetoothHelper.startScan(scanParams) : handleFoundDevices(devicelist);
            }

            var onScanError = function (error) {
                Boso.BluetoothDeviceControl.processError(null, 'PAIRING: ScanError: ' + JSON.stringify(error, null, 4), 'pairDevice: onScanError:');
            }

            //parameters for scan function.
            var scanParams = {
                scanSuccess: onScanSuccess,
                scanError: onScanError,
                scanDuration: scanDuration,
                deviceFilter: {
                    authorizedDevices: Object.keys(Boso.BOSO_DEVICENAMES).map(function (key) { return Boso.BOSO_DEVICENAMES[key] }), //Array with devicenames
                    services: [
                        BluetoothHelper.SERVICE_IDS.bloodPressure,
                        BluetoothHelper.SERVICE_IDS.heartRate,
                        BluetoothHelper.SERVICE_IDS.weightScale,
                        BluetoothHelper.SERVICE_IDS.customService
                    ]
                }
            }

            var onInitSuccess = function () {
                showLoader(Utils.getText('GERAETE_SUCHEN'));
                Boso.BluetoothDeviceControl.BTProcessTimeout.restart();
                BluetoothHelper.startScan(scanParams);
            }

            Boso.BluetoothDeviceControl.BTProcessTimeout.restart();
            showLoader(Utils.getText('INIT_BLUETOOTH'));
            var init = function () {
                BluetoothHelper.initialize(onInitSuccess, function (error) {
                    var errorMessage = !!error ? 'BluetoothError: ' + error.toString() : 'BluetoothError: ';
                    Boso.BluetoothDeviceControl.processError(errorMessage, errorMessage, 'pairDevice: initError:');
                });
            }

            Boso.BluetoothDeviceControl.isPairing = true;
            Boso.BluetoothDeviceControl.worker.clear();

            BluetoothHelper.isScanning(function (result) {
                if (result && result.isScanning) {
                    console.log(JSON.stringify(result, null, 4));

                    BluetoothHelper.stopScan(init, init);
                } else if (result && !result.isScanning) {
                    init();
                }
            });
        },

        // search for advertising devices that are paired with the app.
        // take the first if there are more than one in the list.
        findAdvertisingDevices: function () {
            try {
                if (!Boso.getCurrentUser()) {
                    console.log('no user available: Search for advertising devices canceled!');
                    return;
                }

                console.log('search for advertising devices');
                var onScanSuccess = function (devicelist) {
                    console.log('search advertising devices scan success', JSON.stringify(devicelist, null, 4));
                    //only connect to sending device if it's allready paired
                    var communicationDevice = _.find(window.genericData.models, function (deviceModel) {
                        return (
                            deviceModel.get('EntityId') === Boso.BOSO_GEREATE_ENTITY_ID
                            && deviceModel.get('IsActive') === true
                            && _.some(devicelist, function (advertisingDevice) {
                                return advertisingDevice.address === deviceModel.get('Custom_001')
                            })
                        )
                    });

                    if (communicationDevice && communicationDevice.get('Custom_002')) {
                        console.log('Advertising and paired boso device found: ', JSON.stringify(communicationDevice, null, 4));
                        var deviceName = communicationDevice.get('Custom_002');
                        if (deviceName.indexOf(Boso.BOSO_DEVICENAMES.BOSO_BLODDPRESSURE_NAME) > -1) {
                            Boso.BluetoothDeviceControl.readBloodPressure(communicationDevice);
                        } else if (deviceName.indexOf(Boso.BOSO_DEVICENAMES.BOSO_WEIGHT_NAME) > -1) {
                            Boso.BluetoothDeviceControl.readWeight(communicationDevice);
                        }
                    } else {
                        if (!communicationDevice) {
                            console.log('Es wurde kein Gerät gefunden!');
                        } else if (!communicationDevice.get('Custom_002')) {
                            console.log('Gefundenes Gerät enthält keine Gerätebezeichnung!');
                        }

                        Boso.startBluetoothBackgroundWorker();
                    }
                }

                var onScanError = function (error) {
                    Boso.BluetoothDeviceControl.processError(null, 'SEARCH AVERTISING DEVICES: ScanError: ' + JSON.stringify(error, null, 4), 'findAdvertisingDevices: scanError:');
                }

                var initSuccess = function () {
                    console.log('search for advertising Devices: BLE Initialized');
                    var authorizedDevices = Object.keys(Boso.BOSO_DEVICENAMES).map(function (key) { return Boso.BOSO_DEVICENAMES[key] }); //Array with devicenames

                    var test = null;
                    var multiuserMode = UserSettings.GetWeightScaleMode() === '1';
                    if (multiuserMode && !Boso.weightScaleCommunicationActive) {
                        var index = authorizedDevices.indexOf(Boso.BOSO_DEVICENAMES.BOSO_WEIGHT_NAME);
                        if (index > -1) {
                            test = authorizedDevices.splice(index, 1);
                        }
                    }
                    console.log('authorizedDevices', JSON.stringify(authorizedDevices, null, 4));

                    //parameters for scan function.
                    var scanParams = {
                        scanSuccess: onScanSuccess,
                        scanError: onScanError,
                        scanDuration: 2000,
                        deviceFilter: {
                            authorizedDevices: authorizedDevices,
                            services: [
                                BluetoothHelper.SERVICE_IDS.bloodPressure,
                                BluetoothHelper.SERVICE_IDS.heartRate,
                                BluetoothHelper.SERVICE_IDS.weightScale,
                                BluetoothHelper.SERVICE_IDS.customService
                            ]
                        }
                    }

                    BluetoothHelper.startScan(scanParams);
                }

                BluetoothHelper.initialize(initSuccess, function (error) {
                    var errorMessage = !!error ? 'BluetoothError: ' + error.toString() : 'BluetoothError: ';
                    Boso.BluetoothDeviceControl.processError(Boso.BluetoothDeviceControl.processError(errorMessage, errorMessage, 'findAdvertisingDevices: initError:'));
                });
            } catch (ex) {
                var logMessage = !!ex ? ex.toString() : null;
                Boso.BluetoothDeviceControl.processError(null, logMessage, 'findAdvertisingDevices: catchBlock:');
            }
        },

        //	  				  //
        // READ BLOODPRESSURE //
        // 					  //
        readBloodPressure: function (deviceModel) {
            console.log('TRY TO READ BLOODPRESSURE');

            if (!deviceModel || !deviceModel.get('Custom_001')) {
                var errorMessage = 'Zum gekoppelten Gerät ist keine Geräteadresse hinterlegt. Bitte führen Sie die Kopplung erneut durch!';
                Boso.BluetoothDeviceControl.processError(errorMessage, errorMessage, 'readBloodPressure: nodeviceModel:');
                return;
            }

            var resultlist = [];
            var onUnsubscribeSuccess = function (unsubscribeResult) {
                Boso.getCurrentIanaTimeZone(function (timezone) {
                    Boso.BluetoothDeviceControl.closeCurrentConnection(deviceModel.get('Custom_001'));
                    hideloader();
                    Boso.BluetoothDeviceControl.BTProcessTimeout.clear();

                    resultlist.sort(function (a, b) {
                        return moment(b.timestamp, CONSTS.DATETIME_PERSIST_FORMAT) - moment(a.timestamp, CONSTS.DATETIME_PERSIST_FORMAT);
                    });

                    //Nur neusten wert abfragen?
                    if (UserSettings.GetOnlyLastValue()) {
                        resultlist = [_.first(resultlist)];
                    }

                    console.log('resultlist after usersettings sorting: ', JSON.stringify(resultlist, null, 4));
                    //Zur DB hinzufügen.
                    var userID = Boso.getCurrentUser().get('DataId');
                    _.each(resultlist, function (data) {
                        var newModel = null;
                        newModel = new window.GenericDataRawElement();
                        var itemId = window.generateGuid();
                        newModel.set('DataId', itemId);
                        newModel.set('ObjectId', itemId);
                        newModel.id = itemId;
                        newModel.set({ 'id': itemId });

                        newModel.set('EntityId', Boso.MESSUREMENT_ENTITYID);

                        newModel.set('UserId', window.currentUser.UserId);
                        newModel.set('MandatorId', window.currentUser.MandatorId);
                        newModel.set('OrgUnitId', window.currentUser.OrgUnitId);

                        newModel.set('Custom_001', data.timestamp);
                        newModel.set('Custom_002', data.SYS);
                        newModel.set('Custom_003', data.DIA);
                        newModel.set('Custom_004', data.PUL);
                        newModel.set('Custom_005', data.ARR);
                        newModel.set('Custom_008', data.ABP);
                        newModel.set('Custom_007', deviceModel.get('Custom_003'));
                        newModel.set('Custom_009', userID);
                        newModel.set('Custom_011', timezone);

                        newModel.set('IsActive', true);
                        newModel.set('IsNew', true);
                        newModel.set('EntryTime', moment().toDate());
                        newModel.set('SyncTime', CONSTS.DATE_DEFAULT);

                        if (data.SYS != 2047) {
                            newModel.save();
                            window.genericData.add(newModel);
                            newModel.save();
                        }
                    });

                    var measureCounter = 0;
                    _.each(resultlist, function (data) {
                        measureCounter = (data.SYS != 2047 ? (measureCounter + 1) : measureCounter);
                    });

                    var text = Utils.getText('ANZAHL_MESSWERTE');
                    text = text.replace('{Anzahl}', measureCounter);

                    _.each(resultlist, function (data) {
                        text += "SYS: " + (data.SYS != 2047 ? (data.SYS + ' mmHg') : 'Err');
                        text += '\n';
                        text += "DIA: " + (data.DIA !== 2047 ? (data.DIA + ' mmHg') : 'Err');
                        text += '\n';
                        text += "PUL: " + (data.PUL !== 2047 ? (data.PUL + ' 1/min') : 'Err');
                        text += '\n';
                        text += "ARR: " + (data.ARR == 1 ? Utils.getText('JA') : Utils.getText('NEIN'));
                        text += '\n';
                        text += Utils.getText('UHRZEIT') + moment.getDisplayValue(data.timestamp);
                        text += "\n\n";
                    });
                    alert(text);

                    var currentUser = Boso.getCurrentUser();
                    var phoneString = currentUser.get('Custom_008');
                    if (!!phoneString && phoneString.toLowerCase().trim() === Boso.MM_TRANSMISSION_ACTIVATION_STRING) {
                        console.log('Device serial number:', deviceModel.get('Custom_005'));
                        console.log('Timezone while pairing is:', deviceModel.get('Custom_006'));
                        Boso.sendToMedMobile(deviceModel.get('Custom_005'), deviceModel.get('Custom_006'));
                    } else {
                        console.log('Transmission to medmobile is disabled!');
                    }

                    Boso.sendToGoogleFitOrAppleHealth(Boso.getCurrentUser(), function() {
                        Boso.sendToSciTIM(Boso.getCurrentUser(), function() {
                            Boso.sendToCasProZdravi(Boso.getCurrentUser(), function() {
                                Boso.startBluetoothBackgroundWorker();

                                //REFRESH CURRENT VIEW
                                ActionHelper.doNavigate(null, Boso.MEASUREMENTS_VIEWID);
                            });
                        });
                    });
                });
            }

            var unsubscribeTimer = null;
            var unsubscribeParams = {
                "address": deviceModel.get('Custom_001'),
                "service": BluetoothHelper.SERVICE_IDS.bloodPressure,
                "characteristic": BluetoothHelper.CHARACTERISTIC_IDS.bloodPressureMeasurement
            }
            var onSubscribeSuccess = function (subscribeResult) {
                if (subscribeResult && subscribeResult.status === BluetoothHelper.SUBSCRIPTION_STATES.subscribed) {
                    Boso.BluetoothDeviceControl.BTProcessTimeout.restart();

                    unsubscribeTimer = new Utils.ExplicitTimeout(function () {
                        BluetoothHelper.unsubscribe(onUnsubscribeSuccess, function (error) {
                            var logMessage = 'SUBSCRIPTION-ERROR: ' + JSON.stringify(error, null, 4);
                            Boso.BluetoothDeviceControl.processError(null, logMessage, 'readBloodPressure: unsubscribeError');
                        }, unsubscribeParams, 'unsubscribe timer read bloodPressure');
                    }, 1000);

                    console.log("Subscription started");
                }

                if (subscribeResult && subscribeResult.status === BluetoothHelper.SUBSCRIPTION_STATES.subscribedResult) {
                    Boso.BluetoothDeviceControl.BTProcessTimeout.restart();
                    unsubscribeTimer.restart();
                    console.log("Subscription data received\n" + JSON.stringify(subscribeResult, null, 4));
                    var resultval = subscribeResult.value;

                    var bytesToInt = function (byte1, byte2) {
                        return byte1 + (byte2 * 256);
                    };

                    var i = 0;
                    var bytes = bluetoothle.encodedStringToBytes(resultval);

                    var configBin = bytes[i++].toString(2).split("").reverse().join(""); // => bytes[0]
                    var unitFlagKPa = configBin.substr(0, 1) === '1';
                    var timeStampPresent = configBin.substr(1, 1) === '1';
                    var pulsRatePresent = configBin.substr(2, 1) === '1';
                    var userIDPresent = configBin.substr(3, 1) === '1';
                    var measurementPresent = configBin.substr(4, 1) === '1';

                    if (unitFlagKPa) {
                        alert(Utils.getText('GERAET_MMHG'));
                    }

                    var SYS = bytesToInt(bytes[i++], bytes[i++]);   // bytes[1], bytes[2]
                    var DIA = bytesToInt(bytes[i++], bytes[i++]);   // bytes[3], bytes[4]
                    var ABP = bytesToInt(bytes[i++], bytes[i++]);   // bytes[5], bytes[6]

                    var timestamp = moment().format(CONSTS.DATETIME_PERSIST_FORMAT);
                    if (timeStampPresent) {
                        var year = bytesToInt(bytes[i++], bytes[i++]);          // bytes[7], bytes[8]
                        var month = (bytes[i] < 10 ? "0" : "") + bytes[i++];    // bytes[9]
                        var day = (bytes[i] < 10 ? "0" : "") + bytes[i++];      // bytes[10]
                        var hour = (bytes[i] < 10 ? "0" : "") + bytes[i++];     // bytes[11]
                        var minute = (bytes[i] < 10 ? "0" : "") + bytes[i++];   // bytes[12]
                        var second = (bytes[i] < 10 ? "0" : "") + bytes[i++];   // bytes[13]

                        //DATETIME_PERSIST_FORMAT: 'YYYY-MM-DD HH:mm:ss',
                        timestamp = year + "-" + month + "-" + day + " " + hour + ":" + minute + ":" + second;
                    }
                    var PUL = pulsRatePresent ? bytesToInt(bytes[i++], bytes[i++]) : 0; // bytes[7], bytes[8] ODER bytes[14], bytes[15]

                    if (userIDPresent) { i++; }

                    if (measurementPresent) {
                        var messurement = bytesToInt(bytes[i++], bytes[i++]);
                        var measurementBin = messurement.toString(2).split("").reverse().join("");
                        var ARR = parseInt(measurementBin[2]);
                    } else {
                        var ARR = 0;
                    }

                    var retObj = {
                        SYS: SYS,
                        DIA: DIA,
                        ABP: ABP,
                        PUL: PUL,
                        ARR: ARR,
                        timestamp: timestamp
                    };

                    resultlist.push(retObj);
                }
            }

            var onDiscoverSuccess = function (discoverResult) {
                if (discoverResult && discoverResult.status === BluetoothHelper.DISCOVER_STATES.discovered) {
                    console.log('DISCOVER-SUCCESS: ', JSON.stringify(discoverResult, null, 4));

                    showLoader(Utils.getText('DATEN_LESEN'));
                    Boso.BluetoothDeviceControl.BTProcessTimeout.restart();

                    var params = {
                        'address': deviceModel.get('Custom_001'),
                        'service': BluetoothHelper.SERVICE_IDS.bloodPressure,
                        'characteristic': BluetoothHelper.CHARACTERISTIC_IDS.bloodPressureMeasurement
                    }

                    console.log('SUBSCRIBE-PARAMS: ', JSON.stringify(params, null, 4));

                    BluetoothHelper.subscribe(onSubscribeSuccess, function (error) {
                        if (window.device.platform !== 'iOS') {
                            var errorMessage = Utils.getText('LESE_DATEN_FEHLGESCHLAGEN');
                            var logMessage = 'SUBSCRIPTION-ERROR: ' + JSON.stringify(error, null, 4);
                            Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'readBloodPressure: subscribeError');
                        }
                    }, params);
                }
            }

            var onConnectionResult = function (connectionResult) {
                if (connectionResult && connectionResult.status && connectionResult.status === BluetoothHelper.CONNECTION_STATES.connected) {
                    setTimeout(function () {
                        Boso.BluetoothDeviceControl.BTProcessTimeout.restart();
                        BluetoothHelper.discover(deviceModel.get('Custom_001'), onDiscoverSuccess, function (error) {
                            Boso.BluetoothDeviceControl.processError(null, 'DISCOVERY-ERROR: ' + JSON.stringify(error, null, 4), 'readBloodPressure: discoveryError');
                        });
                    }, 2000);
                }

                if (connectionResult && connectionResult.status && connectionResult.status === BluetoothHelper.CONNECTION_STATES.disconnected) {
                    if (window.device.platform !== 'iOS') {
                        var errorMessage = Utils.getText('GERAET_VERBINDUNG_ABGEBROCHEN');
                        var logMessage = 'Connection lost while reading bloodPressure: ' + JSON.stringify(connectionResult, null, 4);
                        Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'onConnectionResult: disconnected');
                    }
                }
            }

            var connectionTries = 0;
            var onConnectionError = function (error) {
                connectionTries++;

                console.log('READ BLOODPRESSURE: CONNECTION ERROR: ', connectionTries);
                if (connectionTries < 2) {
                    BluetoothHelper.reconnect(deviceModel.get('Custom_001'), onConnectionResult, onConnectionError);
                } else {
                    var errorMessage = Utils.getText('VERBINDUNG_ABGEBROCHEN');
                    var logMessage = 'READ BLOODPRESSURE: ConnectionError: ' + JSON.stringify(error, null, 4);
                    Boso.BluetoothDeviceControl.processError(errorMessage, logMessage);
                }
            }

            var msgConnectDevice = Utils.getText('GERAET_VERBINDEN');
            if (deviceModel && deviceModel.get('Custom_003')) {
                msgConnectDevice = msgConnectDevice.replace('{Geraet}', deviceModel.get('Custom_003'));
            }
            showLoader(msgConnectDevice);

            Boso.BluetoothDeviceControl.BTProcessTimeout.restart();
            Boso.BluetoothDeviceControl.lastConnectionAddress = deviceModel.get('Custom_001');

            BluetoothHelper.connect(deviceModel.get('Custom_001'), onConnectionResult, onConnectionError);
        },

        //	  		   //
        // READ WEIGHT //
        // 			   //
        readWeight: function (model) {
            if (!model || !model.get('Custom_001')) {
                var errorMessage = 'Zum gekoppelten Gerät ist keine Geräteadresse hinterlegt. Bitte führen Sie die Kopplung erneut durch!';
                Boso.BluetoothDeviceControl.processError(errorMessage, errorMessage, 'readWeight: noModel');
                return;
            }

            var resultlist = [];
            var onUnsubscribeSuccess = function (unsubscribeResult) {
                hideloader();
                Boso.BluetoothDeviceControl.closeCurrentConnection(model.get('Custom_001'));
                Boso.BluetoothDeviceControl.BTProcessTimeout.clear();

                if (_.size(resultlist) < 1) {
                    var errorMessage = Utils.getText('LESE_DATEN_FEHLGESCHLAGEN');
                    var logMessage = 'SUBSCRIPTION-ERROR: Leeres ResultArray übergeben';
                    Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'readWeight: noDataReceived');
                    return;
                }

                if (_.first(resultlist).weight === '0.0') {
                    var errorMessage = Utils.getText('PAIRING_BEREITS_VORHANDEN');
                    var logMessage = 'MESSGERAET IST IM PAIRING MODUS';
                    Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'readWeight: weight scale tried to pair not to send a measurement');
                    return;
                }

                resultlist.sort(function (a, b) {
                    return moment(b.timestamp, CONSTS.DATETIME_PERSIST_FORMAT) - moment(a.timestamp, CONSTS.DATETIME_PERSIST_FORMAT);
                });

                //Nur neusten Wert abfragen?
                if (UserSettings.GetOnlyLastValue()) {
                    resultlist = [_.first(resultlist)];
                }

                console.log('READ WEIGHT: resultlist: ', JSON.stringify(resultlist, null, 4));

                // Zur DB hinzufügen.
                var userID = Boso.getCurrentUser().get('ObjectId');
                _.each(resultlist, function (data) {
                    var newModel = null;
                    newModel = new window.GenericDataRawElement();
                    var itemId = window.generateGuid();
                    newModel.set('DataId', itemId);
                    newModel.set('ObjectId', itemId);
                    newModel.id = itemId;
                    newModel.set({ 'id': itemId });

                    newModel.set('EntityId', Boso.MESSUREMENT_ENTITYID);

                    newModel.set('UserId', window.currentUser.UserId);
                    newModel.set('MandatorId', window.currentUser.MandatorId);
                    newModel.set('OrgUnitId', window.currentUser.OrgUnitId);

                    newModel.set('Custom_001', data.timestamp);
                    newModel.set('Custom_006', Utils.round(data.weight, 1));
                    newModel.set('Custom_007', model.get('Custom_003'));

                    //SET USER
                    newModel.set('Custom_009', userID);

                    newModel.set('IsActive', true);
                    newModel.set('IsNew', true);
                    newModel.set('EntryTime', moment().toDate());
                    newModel.set('SyncTime', CONSTS.DATE_DEFAULT);

                    newModel.save();
                    window.genericData.add(newModel);
                    newModel.save();
                });

                var text = Utils.getText('ANZAHL_MESSWERTE');
                text = text.replace('{Anzahl}', resultlist.length);

                _.each(resultlist, function (data) {
                    text += Utils.getText('MESSWERT') + data.weight + ' kg';
                    text += '\n';
                    text += Utils.getText('UHRZEIT') + moment.getDisplayValue(data.timestamp);
                    text += "\n\n";
                });
                alert(text);

                setTimeout(function () {
                    if (Boso.weightScaleCommunicationActive) {
                        Boso.toggleWeightScaleCommunication();
                    }

                    Boso.startBluetoothBackgroundWorker();
                    //REFRESH CURRENT VIEW
                    ActionHelper.doNavigate(null, Boso.MEASUREMENTS_VIEWID);
                }, 300);
            }

            var unsubscribeTimer = null;
            var unsubscribeParams = {
                "address": model.get('Custom_001'),
                "service": BluetoothHelper.SERVICE_IDS.weightScale,
                "characteristic": BluetoothHelper.CHARACTERISTIC_IDS.weightMeasurement
            }
            var onSubscribeSuccess = function (subscribeResult) {
                if (subscribeResult && subscribeResult.status === BluetoothHelper.SUBSCRIPTION_STATES.subscribed) {
                    Boso.BluetoothDeviceControl.BTProcessTimeout.restart();

                    unsubscribeTimer = new Utils.ExplicitTimeout(function () {
                        BluetoothHelper.unsubscribe(onUnsubscribeSuccess, function (error) {
                            var logMessage = 'SUBSCRIPTION-ERROR: ' + JSON.stringify(error, null, 4);
                            Boso.BluetoothDeviceControl.processError(null, logMessage, 'readWeight: unsubscribeError');
                        }, unsubscribeParams, 'unsubscribeTimer readWeight');
                    }, 1000);

                    console.log("Subscription started");
                }

                if (subscribeResult.status === BluetoothHelper.SUBSCRIPTION_STATES.subscribedResult) {
                    Boso.BluetoothDeviceControl.BTProcessTimeout.restart();
                    unsubscribeTimer.restart();
                    console.log("Subscription data received\n" + JSON.stringify(subscribeResult, null, 4));

                    var resultval = subscribeResult.value;

                    var bytesToInt = function (byte1, byte2) {
                        return byte1 + (byte2 * 256);
                    };
                    var i = 0;
                    var bytes = bluetoothle.encodedStringToBytes(resultval);

                    var configBin = bytes[i++].toString(2).split("").reverse().join(""); // => bytes[0]
                    var unitsLb = configBin.substr(0, 1) === '1';
                    var timeStampPresent = configBin.substr(1, 1) === '1';
                    var userIDPresent = configBin.substr(2, 1) === '1'; // NOT USED
                    var bmiPresent = configBin.substr(3, 1) === '1';    // NOT USED

                    var weightInt = bytesToInt(bytes[i++], bytes[i++]); // bytes[1], bytes[2]
                    if (unitsLb) {
                        alert(Utils.getText('GERAET_KG'));
                        return;
                    } else {
                        var weightDec = parseFloat(weightInt * 0.005).toFixed(1);   // kg
                    }

                    var timestamp = moment().format(CONSTS.DATETIME_PERSIST_FORMAT);

                    if (timeStampPresent) {
                        var year = bytesToInt(bytes[i++], bytes[i++]);          // bytes[3], bytes[4]
                        var month = (bytes[i] < 10 ? "0" : "") + bytes[i++];    // bytes[5]
                        var day = (bytes[i] < 10 ? "0" : "") + bytes[i++];      // bytes[6]
                        var hour = (bytes[i] < 10 ? "0" : "") + bytes[i++];     // bytes[7]
                        var minute = (bytes[i] < 10 ? "0" : "") + bytes[i++];   // bytes[8]
                        var second = (bytes[i] < 10 ? "0" : "") + bytes[i++];   // bytes[9]

                        //DATETIME_PERSIST_FORMAT: 'YYYY-MM-DD HH:mm:ss',
                        timestamp = year + "-" + month + "-" + day + " " + hour + ":" + minute + ":" + second;
                    }

                    var retObj = {
                        weight: weightDec,
                        timestamp: timestamp
                    }

                    resultlist.push(retObj);
                }
            }

            var onWriteUserIdSuccess = function (writeResult) {
                var params = {
                    'address': model.get('Custom_001'),
                    'service': BluetoothHelper.SERVICE_IDS.weightScale,
                    'characteristic': BluetoothHelper.CHARACTERISTIC_IDS.weightMeasurement
                }

                showLoader(Utils.getText('DATEN_LESEN'));
                Boso.BluetoothDeviceControl.BTProcessTimeout.restart();
                console.log('USER ID WRITTEN');

                setTimeout(function () {
                    BluetoothHelper.subscribe(onSubscribeSuccess, function (error) {
                        var errorMessage = Utils.getText('LESE_DATEN_FEHLGESCHLAGEN');
                        var logMessage = 'SUBSCRIPTION-ERROR: ' + JSON.stringify(error, null, 4);
                        Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'readWeight: subscribeError');
                    }, params);
                }, 1000);
            }

            var onDiscoverSuccess = function (discoverResult) {
                if (discoverResult && discoverResult.address && discoverResult.status === BluetoothHelper.DISCOVER_STATES.discovered) {
                    console.log('DISCOVER-SUCCESS: ', JSON.stringify(discoverResult, null, 4));

                    var weightScaleDevice = _.find(window.genericData.models, function (model) {
                        return (
                            model.get('EntityId') === Boso.BOSO_GEREATE_ENTITY_ID &&
                            model.get('IsActive') === true &&
                            model.get('Custom_001') === discoverResult.address
                        );
                    });

                    console.log('writing userId to WeightScale: USERID: ', ~~weightScaleDevice.get('Custom_004'));

                    var userIndex = ~~weightScaleDevice.get('Custom_004');
                    if (!!userIndex) {
                        userIndex = userIndex - 1;
                    }
                    var tmp = new Uint8Array(4);
                    tmp[0] = 3;
                    tmp[1] = 1;
                    tmp[2] = 22;
                    tmp[3] = userIndex;
                    var value = bluetoothle.bytesToEncodedString(tmp);

                    var params = {
                        'address': model.get('Custom_001'),
                        'service': BluetoothHelper.SERVICE_IDS.customService,
                        'characteristic': BluetoothHelper.CHARACTERISTIC_IDS.customCharacteristic,
                        'value': value
                    }

                    Boso.BluetoothDeviceControl.BTProcessTimeout.restart();
                    var writeTries = 0;
                    var writeUserId = function () {
                        BluetoothHelper.write(onWriteUserIdSuccess, function (error) {
                            writeTries++;
                            if (writeTries < 2) {
                                writeUserId();
                            } else {
                                var logMessage = 'WRITE-ERROR: ' + JSON.stringify(error, null, 4);
                                var errorMessage = Utils.getText("BENUTZER_ID_NICHT_UEBERTRAGEN");
                                Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'readWeight: write userId error');
                            }
                        }, params);
                    }

                    writeUserId();
                }
            }

            var onConnectionResult = function (connectionResult) {
                if (connectionResult && connectionResult.status && connectionResult.status === BluetoothHelper.CONNECTION_STATES.connected) {
                    console.log('connected');
                    Boso.BluetoothDeviceControl.BTProcessTimeout.restart();

                    setTimeout(function () {
                        BluetoothHelper.discover(model.get('Custom_001'), onDiscoverSuccess, function (error) {
                            Boso.BluetoothDeviceControl.processError(null, 'DISCOVERY-ERROR: ' + JSON.stringify(error, null, 4), 'readWeight: discoveryError');
                        });
                    }, 2000);
                }

                if (connectionResult && connectionResult.status && connectionResult.status === BluetoothHelper.CONNECTION_STATES.disconnected) {
                    BluetoothHelper.reconnect(connectionResult.address, function (result) {
                        console.log('RECONNECT SUCCEEDED: ', JSON.stringify(result, null, 4));
                    }, function (error) {
                        var errorMessage = Utils.getText('GERAET_VERBINDUNG_ABGEBROCHEN');
                        var logMessage = 'Connection lost while reading weight: ' + JSON.stringify(connectionResult, null, 4);
                        Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'readWeight: onConnectionResult: disconnected' + JSON.stringify(error, null, 4));
                    });
                }
            }

            var connectionTries = 0;
            var onConnectionError = function (error) {
                connectionTries++;
                console.log('READ WEIGHT: CONNECTION ERROR: ', connectionTries);
                if (connectionTries < 2) {
                    BluetoothHelper.reconnect(model.get('Custom_001'), onConnectionResult, onConnectionError);
                } else {
                    var errorMessage = Utils.getText('VERBINDUNG_ABGEBROCHEN');
                    var logMessage = 'READ WEIGHT: ConnectionError: ' + JSON.stringify(error, null, 4);
                    Boso.BluetoothDeviceControl.processError(errorMessage, logMessage, 'readWeight: all connection attampts failed');
                }
            }

            var msgConnectDevice = Utils.getText('GERAET_VERBINDEN');
            if (model && model.get('Custom_003')) {
                msgConnectDevice = msgConnectDevice.replace('{Geraet}', model.get('Custom_003'));
            }
            showLoader(msgConnectDevice);

            Boso.BluetoothDeviceControl.BTProcessTimeout.restart();
            Boso.BluetoothDeviceControl.lastConnectionAddress = model.get('Custom_001');

            BluetoothHelper.connect(model.get('Custom_001'), onConnectionResult, onConnectionError);
        }
    },

    getGenericDataBloodGraphics: function (page, DateFrom, DateTo) {
        var userID = Boso.getCurrentUser().attributes.ObjectId;

        var genericDataFiltered = _.filter(window.genericData.models, function (model) {
            return (
                model.attributes.EntityId == Boso.MESSUREMENT_ENTITYID &&
                model.attributes.IsActive == true &&
                (model.attributes.Custom_002 != undefined && model.attributes.Custom_002 != null
                    && model.attributes.Custom_002 > 0)
                && model.attributes.Custom_009 == userID
            );
        });

        //Nach Datum sortieren
        genericDataFiltered = genericDataFiltered.sort(function (x, y) {
            var date1 = moment.getMilliSecondsFromPersistFormat(x.attributes['Custom_001']);
            var date2 = moment.getMilliSecondsFromPersistFormat(y.attributes['Custom_001']);
            return date1 - date2;
        })

        if (!!DateFrom && !!DateTo) {
            var from = moment.getMilliSecondsFromDateTimeFormat(DateFrom);
            var to = moment.getMilliSecondsFromDateTimeFormat(DateTo);

            genericDataFiltered = _.filter(genericDataFiltered, function (data) {
                var groessermin = (moment.getMilliSecondsFromPersistFormat(data.attributes["Custom_001"]) >= from);
                var kleinermax = (moment.getMilliSecondsFromPersistFormat(data.attributes["Custom_001"]) <= to);
                return (groessermin && kleinermax)
            });
        }


        var displayWidth = window.innerWidth;
        var numberOfValues = !!displayWidth && displayWidth <= 500 ? 10 : 15;

        //Blätter Funktion nach Eintragzahl
        var upperBound = genericDataFiltered.length - ((page - 1) * numberOfValues);
        var lowerBound = upperBound - numberOfValues;

        if (lowerBound < 0) {
            lowerBound = 0;
        }

        Boso.prevValuesEnabledBP = upperBound >= numberOfValues && !!page;
        if (!!page) {
            genericDataFiltered = genericDataFiltered.slice(lowerBound, upperBound);
        }

        return genericDataFiltered;
    },

    getGenericDataWeightGraphics: function (page, DateFrom, DateTo) {
        var userID = Boso.getCurrentUser().attributes.ObjectId;

        var genericDataFiltered = _.filter(window.genericData.models, function (model) {
            return (
                model.attributes.EntityId == Boso.MESSUREMENT_ENTITYID &&
                model.attributes.IsActive == true &&
                (model.attributes.Custom_006 != undefined && model.attributes.Custom_006 != null
                    && model.attributes.Custom_006 > 0)
                && model.attributes.Custom_009 == userID
            );
        });

        //Nach Datum sortieren
        genericDataFiltered = genericDataFiltered.sort(function (x, y) {
            var date1 = moment.getMilliSecondsFromPersistFormat(x.attributes['Custom_001']);
            var date2 = moment.getMilliSecondsFromPersistFormat(y.attributes['Custom_001']);
            return date1 - date2;
        });

        if (!Utils.isNull(DateFrom) && !Utils.isNull(DateTo)) {
            var from = moment.getMilliSecondsFromDateTimeFormat(DateFrom);
            var to = moment.getMilliSecondsFromDateTimeFormat(DateTo);

            genericDataFiltered = _.filter(genericDataFiltered, function (data) {
                var groessermin = (moment.getMilliSecondsFromPersistFormat(data.attributes["Custom_001"]) >= from);
                var kleinermax = (moment.getMilliSecondsFromPersistFormat(data.attributes["Custom_001"]) <= to);
                return (groessermin && kleinermax)
            });
        }


        var displayWidth = window.innerWidth;
        var numberOfValues = !!displayWidth && displayWidth <= 500 ? 10 : 15;

        //Blätter Funktion nach Eintragzahl
        var upperBound = genericDataFiltered.length - ((page - 1) * numberOfValues);
        var lowerBound = upperBound - numberOfValues;

        if (lowerBound < 0) {
            lowerBound = 0;
        }

        Boso.prevValuesEnabledBMI = upperBound >= numberOfValues && !!page;
        if (!!page) {
            genericDataFiltered = genericDataFiltered.slice(lowerBound, upperBound);
        }

        return genericDataFiltered;
    },

    createFieldsGraphics: function () {
        var fieldContainer = $('#CustomViewContent');
        fieldContainer.empty();

        var container = $('<div/>', {
            style: 'text-align: center; display: none;'
        }).appendTo(fieldContainer)
        $(container).attr('data-role', 'controlgroup');
        $(container).attr('data-type', 'horizontal');

        var hiddenField = $('<input/>', {
            type: 'hidden',
            style: 'display: none',
            id: 'hiddenPageValBMI',
            value: 1
        }).appendTo(container);

        var hiddenField = $('<input/>', {
            type: 'hidden',
            style: 'display: none',
            id: 'hiddenPageValBP',
            value: 1
        }).appendTo(container);

        var refreshfunction = function () {
            var renderfunction = ActionHelper.CurrentView.options.view.attributes.CustomRoute;
            if (renderfunction != undefined && renderfunction != "") {
                renderfunction = renderfunction.substr(0, renderfunction.indexOf('(') + 1) + 'false' + ')';
                eval(renderfunction);
            }
        };

        $('<div/>', {
            id: 'chart_content',
            style: 'visibility:visible; width:103%'
        }).appendTo(fieldContainer);
    },

    nextValuesEnabledBMI: false,
    prevValuesEnabledBMI: false,

    nextValuesEnabledBP: false,
    prevValuesEnabledBP: false,

    getPrevGraphValues: function (callback) {

        function closeLoader() {
            $.mobile.loading('hide');
            $('#loader_page').hide();
        }

        if ($('#chart_kgBMI').is(":visible")) {
            if (Boso.prevValuesEnabledBMI) {
                showLoader(Utils.getText('GRAFIK_LADEN'), 30000);
                setTimeout(function () {
                    Boso.nextValuesEnabledBMI = true;
                    var val = parseInt($('#hiddenPageValBMI').val()) + 1;
                    $('#hiddenPageValBMI').val(val);
                    Boso.OpenWeightGraphic(null, null, closeLoader);
                }, 100);
            }
        }
        else {
            if (Boso.prevValuesEnabledBP) {
                showLoader(Utils.getText('GRAFIK_LADEN'), 30000);
                setTimeout(function () {
                    Boso.nextValuesEnabledBP = true;
                    var val = parseInt($('#hiddenPageValBP').val()) + 1;
                    $('#hiddenPageValBP').val(val);
                    Boso.OpenBloodPressureGraphic(null, null, closeLoader);
                }, 100);
            }
        }
    },

    getNextGraphValues: function () {
        function closeLoader() {
            $.mobile.loading('hide');
            $('#loader_page').hide();
        }

        if ($('#chart_kgBMI').is(":visible")) {
            if (Boso.nextValuesEnabledBMI) {
                showLoader(Utils.getText('GRAFIK_LADEN'), 30000);
                setTimeout(function () {
                    Boso.prevValuesEnabledBMI = true;
                    if (val == 1) {
                        Boso.nextValuesEnabledBMI = false;
                    }
                    var val = parseInt($('#hiddenPageValBMI').val()) - 1;
                    $('#hiddenPageValBMI').val(val);
                    Boso.OpenWeightGraphic(null, null, closeLoader());
                }, 100);
            }
        } else {
            if (Boso.nextValuesEnabledBP) {
                showLoader(Utils.getText('GRAFIK_LADEN'), 30000);
                setTimeout(function () {
                    Boso.prevValuesEnabledBP = true;
                    if (val == 1) {
                        Boso.nextValuesEnabledBP = false;
                    }
                    var val = parseInt($('#hiddenPageValBP').val()) - 1;
                    $('#hiddenPageValBP').val(val);
                    Boso.OpenBloodPressureGraphic(null, null, closeLoader());
                }, 100);
            }
        }
    },

    createFieldsStatistic: function (callback) {
        var measurementVals = Boso.openStatPrepareMeasurementData();

        var fieldContainer = $('#CustomViewContent');
        fieldContainer.empty();

        //Felder Initialisieren
        var fs = $('<fieldset/>', {
            id: 'FieldSetFilter'
        }).appendTo(fieldContainer);

        fs.attr('data-role', 'fieldcontain');
        fs.attr('fieldtype', "date");

        var lablevon = $('<label/>', {
            class: 'select',
            text: Utils.getText('ZEIT_VON') + ": ",
            for: 'inputvon',
        }).appendTo(fs);

        var inputvon = $('<input/>', {
            id: 'inputvon',
            name: 'inputvon',
        }).appendTo(fs);

        inputvon.off('change').on('change', function (el) {
            if (!!inputvon.val()) {
                localStorage.setItem('BOSO_STAT_START', $('#inputvon').val());
            }

            if (!!inputbis && !!inputbis.val() && !!inputvon.val()) {
                var start = moment(inputvon.val(), FormatHelper.get_DATE_FORMAT());
                var end = moment(inputbis.val(), FormatHelper.get_DATE_FORMAT());

                if (start.isAfter(end)) {
                    inputbis.scroller('setDate', moment(inputvon.val(), FormatHelper.get_DATE_FORMAT()).toDate(), true)
                }
            }
        });

        var lablebis = $('<label/>', {
            class: 'select',
            text: Utils.getText('ZEIT_BIS') + ": ",
            for: 'inputbis',
        }).appendTo(fs);

        var inputbis = $('<input/>', {
            id: 'inputbis',
            name: 'inputbis',
        }).appendTo(fs);

        inputbis.off('change').on('change', function (el) {
            if (!!inputbis.val()) {
                localStorage.setItem('BOSO_STAT_END', $('#inputbis').val());
            }

            if (!!inputvon && !!inputvon.val() && !!inputbis.val()) {
                var start = moment(inputvon.val(), FormatHelper.get_DATE_FORMAT());
                var end = moment(inputbis.val(), FormatHelper.get_DATE_FORMAT());

                if (start.isAfter(end)) {
                    inputvon.scroller('setDate', moment(inputbis.val(), FormatHelper.get_DATE_FORMAT()).toDate(), true)
                }
            }
        });

        var btn = $('<input/>', {
            type: 'button',
            value: Utils.getText('STATISTIK_AKTUALISIEREN'),
            id: 'refreshButton',
        }).appendTo(fieldContainer);

        var refreshfunction = function () {
            Boso.OpenStatistic();
        };

        $(btn).click(refreshfunction);

        inputbis.attr('class', 'date');
        inputbis.scroller({
            preset: 'date',
            theme: 'default',
            display: 'modal',
            mode: 'scroller',
            dateOrder: FormatHelper.get_SCROLLER_DATEORDER(),
            dateFormat: FormatHelper.get_DATE_FORMAT().toLowerCase(),
            monthNames: FormatHelper.get_MONTH_NAMES(),//['Januar', 'Februar', 'März', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember'],
            monthNamesShort: FormatHelper.get_MONTH_NAMES_SHORT(),//['Jan', 'Feb', 'Mär', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez'],
            monthText: Utils.getText('MONAT'),
            dayText: Utils.getText('TAG'),
            yearText: Utils.getText('JAHR'),
            nowText: Utils.getText('JETZT'),
            cancelText: Utils.getText('ABBRECHEN'),
            setText: Utils.getText('OK'),
            maxDate: new Date(),
            defaultValue: '',
        });
        var dateLast = _.size(measurementVals) > 0 ? measurementVals[measurementVals.length - 1].attributes['Custom_001'] : moment();
        dateLast = moment(dateLast, CONSTS.DATE_PERSIST_FORMAT).toDate();
        inputbis.scroller('setDate', dateLast, true);

        inputvon.attr('class', 'date');
        inputvon.scroller({
            preset: 'date',
            theme: 'default',
            display: 'modal',
            mode: 'scroller',
            dateOrder: FormatHelper.get_SCROLLER_DATEORDER(),
            dateFormat: FormatHelper.get_DATE_FORMAT().toLowerCase(),
            monthNames: FormatHelper.get_MONTH_NAMES(),//['Januar', 'Februar', 'März', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember'],
            monthNamesShort: FormatHelper.get_MONTH_NAMES_SHORT(),//['Jan', 'Feb', 'Mär', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez'],
            monthText: Utils.getText('MONAT'),
            dayText: Utils.getText('TAG'),
            yearText: Utils.getText('JAHR'),
            nowText: Utils.getText('JETZT'),
            cancelText: Utils.getText('ABBRECHEN'),
            setText: Utils.getText('OK'),
            maxDate: new Date(),
            defaultValue: '',
        });

        var dateFirst = _.size(measurementVals) > 0 ? measurementVals[0].attributes['Custom_001'] : moment();
        dateFirst = moment(dateFirst, CONSTS.DATE_PERSIST_FORMAT).toDate();
        inputvon.scroller('setDate', dateFirst, true);

        $('<div/>', {
            id: 'chart_content'
        }).appendTo(fs);

        setTimeout(function () {
            if (!Utils.isNull($('#inputvon').val())) {
                localStorage.setItem('BOSO_STAT_START', $('#inputvon').val());
            }

            if (!Utils.isNull($('#inputbis').val())) {
                localStorage.setItem('BOSO_STAT_END', $('#inputbis').val());
            }
        }, 0);
    },

    GetReportFileDescription: function (fileType) {
        var patient = Boso.getCurrentUser();
        var patientNo = patient.get('Custom_001');

        var fileDescription = {
            fileName: 'medicus_' + patientNo,
            fileType: fileType,
            folderName: Boso.REPORT_FILE_FOLDERNAME,
            fileSystem: LocalFileSystem.TEMPORARY,
            isBinary: false
        }

        return fileDescription;
    },

    SavePdfReport: function (fileDescription, onSaved) {
        if (!fileDescription) {
            console.log('SavePdfReport:', 'no filedescription!');
            return;
        }

        Boso.createPdfReport(function (patientNr, PDFContent) {
            FileHelper.saveFile(PDFContent, fileDescription.fileName, fileDescription.fileType, fileDescription.folderName, fileDescription.fileSystem, fileDescription.isBinary, onSaved);
        }, function (error, onError) {
            if (_.isFunction(onError)) {
                onError();
            }
            console.log('error:', error);
        });
    },

    SaveXmlReport: function (fileDescription, onSaved) {
        if (!fileDescription) {
            console.log('SaveXmlReport:', 'no filedescription!');
            return;
        }

        var xmlContent = Boso.createXMLReportContent();
        if (xmlContent == null || xmlContent == '') {
            alert(Utils.getText('XML_NICHT_ERSTELLT'));
            hideloader();
            return;
        }

        FileHelper.saveFile(xmlContent, fileDescription.fileName, fileDescription.fileType, fileDescription.folderName, fileDescription.fileSystem, fileDescription.isBinary, onSaved);
    },

    OpenXmlReport: function () {
        var xmlRepTimeout = null;
        var showXmlRepLoader = function () {
            if (!xmlRepTimeout || (!!xmlRepTimeout && xmlRepTimeout.cleared)) {
                xmlRepTimeout = new Utils.ExplicitTimeout(function () {
                    hideloader();
                }, 30000, 'openXmlReportTimeout');
                xmlRepTimeout.restart();
                showLoader(Utils.getText('BERICHTE_WERDEN_GESPEICHERT'));
            }
        }

        showXmlRepLoader();

        var fileDescription = Boso.GetReportFileDescription('xml');
        var onSaved = function (content, fileName, shortFileName) {
            FileHelper.openFile(fileDescription.fileSystem, fileDescription.folderName, fileDescription.fileName, fileDescription.fileType);
            hideloader();
        }

        setTimeout(function () {
            Boso.SaveXmlReport(fileDescription, onSaved);
        }, 300);
    },

    ShareReports: function () {
        var xmlRepTimeout = null;
        var showXmlRepLoader = function () {
            if (!xmlRepTimeout || (!!xmlRepTimeout && xmlRepTimeout.cleared)) {
                xmlRepTimeout = new Utils.ExplicitTimeout(function () {
                    hideloader();
                }, 30000, 'openXmlReportTimeout');
                xmlRepTimeout.restart();

                var msg = Utils.getText('BERICHTE_WERDEN_ERZEUGT');
                if (Boso.getPlatform() === 'android') {
                    msg = Utils.getText('BERICHTE_WERDEN_GESPEICHERT');
                    msg = msg.replace('{Verzeichnis}', Boso.getFileDirectory());
                }
                showLoader(msg);
            }
        }
        showXmlRepLoader();
        var shareFiles = function (fileNames) {
            if (!window.plugins || !window.plugins.socialsharing) {
                console.error("plugin \"cordova-plugin-x-socialsharing\" is not available!");
                return;
            }

            var options = {
                // message: msg,
                // subject: Utils.getText('BERICHT'), // fi. for email
                files: fileNames // an array of filenames either locally or remotely
                // chooserTitle: 'Pick an app' // Android only, you can override the default share sheet title
            }

            var onSuccess = function (result) {
                Boso.CleanUpReportingLeftover();
                hideloader();
                console.log("Share completed? " + result.completed); // On Android apps mostly return false even while it's true
                console.log("Shared to app: " + result.app); // On Android result.app is currently empty. On iOS it's empty when sharing is cancelled (result.completed=false)
            }

            var onError = function (msg) {
                Boso.CleanUpReportingLeftover();
                hideloader();
                console.log("Sharing failed with message: " + msg);
            }

            window.plugins.socialsharing.shareWithOptions(options, onSuccess, onError);
        }

        var xmlFileDescription = Boso.GetReportFileDescription('xml');
        var pdfFileDescription = Boso.GetReportFileDescription('pdf');
        var fileNames = [];
        var onSavedXml = function (content, fileName, shortFileName) {
            var onSavedPdf = function (content, fileName, shortFileName) {
                var gotXmlFileEntry = function (xmlFileEntry) {
                    fileNames.push(xmlFileEntry.toURL());
                    var gotPdfFileEntry = function (pdfFileEntry) {
                        fileNames.push(pdfFileEntry.toURL());
                        shareFiles(fileNames);
                        xmlRepTimeout.clear();
                        xmlRepTimeout = null;
                    }
                    FileHelper.getFileEntry(pdfFileDescription.fileSystem, pdfFileDescription.folderName, pdfFileDescription.fileName + '.' + pdfFileDescription.fileType, gotPdfFileEntry);
                }
                FileHelper.getFileEntry(xmlFileDescription.fileSystem, xmlFileDescription.folderName, xmlFileDescription.fileName + '.' + xmlFileDescription.fileType, gotXmlFileEntry);
            }
            Boso.SavePdfReport(pdfFileDescription, onSavedPdf);
        }
        Boso.SaveXmlReport(xmlFileDescription, onSavedXml);
    },

    SendXmlReport: function () {
        var xmlRepTimeout = null;
        var showXmlRepLoader = function () {
            if (!xmlRepTimeout || (!!xmlRepTimeout && xmlRepTimeout.cleared)) {
                xmlRepTimeout = new Utils.ExplicitTimeout(function () {
                    hideloader();
                }, 30000, 'xmlReportTimeout');
                xmlRepTimeout.restart();
                showLoader(Utils.getText('XML_WIRD_VERSENDET'));
            }
        }

        var hideXmlRepLoader = function () {
            if (!!xmlRepTimeout && !xmlRepTimeout.cleared) {
                xmlRepTimeout.clear();
                xmlRepTimeout = null;
                hideloader();
            }
        }

        var sendReport = function () {
            var to = $('#5b8717b4-386f-601d-e320-a5246551bb91').val();
            if (to == null || to == '') {
                alert(Utils.getText('EMPFAENGER_WAEHLEN'));
                $.mobile.loading('hide');
                $('#loader_page').hide();
                return;
            }

            var invalidMailFormat = Utils.getText('EMAIL_UNGUELTIG');
            var mailPattern = /[A-Za-z0-9]([A-Za-z0-9-_]\.?)*@[A-Za-z]([A-Za-z0-9-_]\.?)+\.[A-Za-z]+/;
            var fieldIdTo = '5b8717b4-386f-601d-e320-a5246551bb91';
            var valid = false;

            valid = ValidationHelper.validateBy_RegExpValidator(fieldIdTo, mailPattern, invalidMailFormat);

            var cc = $('#21e3c6a5-261e-e739-2a97-69adc284b9b3').val();
            var fieldIdCC = '21e3c6a5-261e-e739-2a97-69adc284b9b3';

            if (!Utils.isNull(cc)) {
                valid = (valid && ValidationHelper.validateBy_RegExpValidator(fieldIdCC, mailPattern, invalidMailFormat));
            }

            if (!valid) {
                alert(Utils.getText('EMAIL_UNGUELTIG'));
                $.mobile.loading('hide');
                $('#loader_page').hide();
                return;
            }

            var XMLContent = Boso.createXMLReportContent();
            if (XMLContent == null || XMLContent == '') {
                alert(Utils.getText('XML_NICHT_ERSTELLT'));
                $.mobile.loading('hide');
                $('#loader_page').hide();
                return;
            }

            var patient = Boso.getCurrentUser();
            var patientNo = patient.attributes.Custom_001;
            var body = $('#422de166-d008-d172-6437-5341a7800fe7').val();

            if (!Utils.isNull(to)) {
                ActionHelper.UpsertAutoComplete('GRUPPE1_EMAIL', to, true);
            }

            if (!Utils.isNull(cc)) {
                ActionHelper.UpsertAutoComplete('GRUPPE1_EMAIL', cc, true);
            }

            MailObject = {
                Token: CryptoJSHelper.getEncryptedString(window.currentUser.UserId),
                To: to,
                CC: cc,
                Subject: Utils.getText('BERICHT'),
                Body: Utils.getText('BOSO_APP_DATEN_XML') + body,
                FileContent: XMLContent,
                FileName: "medicus_" + patientNo + '.xml',
                MandatorID: window.currentUser.MandatorId,
                CurrentUserID: window.currentUser.UserId
            };

            $.ajax({
                url: Boso.MailBaseUrl + Boso.SEND_MAIL,
                data: JSON.stringify(MailObject),
                type: 'post',
                contentType: "application/json; charset=utf-8",
                timeout: CONSTS.TIMEOUT_SUPPORT_DATA,
                success: function (AjaxReturnObject) {
                    if (AjaxReturnObject.ReturnType === RETURN_TYPE.SUCCESS || AjaxReturnObject.ReturnType === RETURN_TYPE.INFO || AjaxReturnObject.ReturnType === RETURN_TYPE.WARNING) {
                        if (AjaxReturnObject.ReturnType === RETURN_TYPE.WARNING) {
                            alert(AjaxReturnObject.ReturnMsg, null, Utils.getText('ERFOLG'), Utils.getText('OK'));
                        }
                    }
                    alert(Utils.getText('XML_BERICHT_VERSANDT'));
                    hideloader();
                    Boso.SendPdfReport();
                },
                error: function (response) {
                    alert(JSON.stringify(response));
                    alert(Utils.getText('BERICHT_NICHT_VERSANDT'));
                    hideloader();
                }
            });
        }

        showXmlRepLoader();
        setTimeout(function () {
            sendReport();
        }, 100);
    },

    OpenPdfReport: function () {
        var pdfRepTimeout = null;
        var showPdfRepLoader = function () {
            if (!pdfRepTimeout || (!!pdfRepTimeout && pdfRepTimeout.cleared)) {
                pdfRepTimeout = new Utils.ExplicitTimeout(function () {
                    hideloader();
                    Boso.CleanUpReportingLeftover();
                }, 30000, 'openPdfReportTimeout');
                pdfRepTimeout.restart();

                var msg = Utils.getText('BERICHTE_WERDEN_ERZEUGT');
                if (Boso.getPlatform() === 'android') {
                    msg = Utils.getText('BERICHTE_WERDEN_GESPEICHERT');
                    msg = msg.replace('{Verzeichnis}', Boso.getFileDirectory());
                }
                showLoader(msg);
            }
        }
        showPdfRepLoader();

        setTimeout(function () {
            var onXmlSaved = function (content, fileName, shortFileName) {
                var fileDescription = Boso.GetReportFileDescription('pdf');
                Boso.SavePdfReport(fileDescription, function () {
                    hideloader();
                    FileHelper.openFile(fileDescription.fileSystem, fileDescription.folderName, fileDescription.fileName, fileDescription.fileType);
                    Boso.CleanUpReportingLeftover();
                    pdfRepTimeout.clear();
                    pdfRepTimeout = null;
                });
            }

            var fileDescription = Boso.GetReportFileDescription('xml');
            Boso.SaveXmlReport(fileDescription, onXmlSaved);
        }, 300);
    },

    SendPdfReport: function () {
        var msgInvalidMailFormat = Utils.getText('EMAIL_UNGUELTIG');
        var fieldIdTo = '5b8717b4-386f-601d-e320-a5246551bb91';
        var fieldIdCc = '21e3c6a5-261e-e739-2a97-69adc284b9b3';
        var to = $('#' + fieldIdTo).val();
        if (!to) {
            alert(Utils.getText('EMPFAENGER_WAEHLEN'));
            return;
        }

        var mailPattern = /[A-Za-z0-9]([A-Za-z0-9-_]\.?)*@[A-Za-z]([A-Za-z0-9-_]\.?)+\.[A-Za-z]+/;
        var valid = ValidationHelper.validateBy_RegExpValidator(fieldIdTo, mailPattern, msgInvalidMailFormat);

        var cc = $('#' + fieldIdCc).val();
        if (!!cc) {
            valid = valid && ValidationHelper.validateBy_RegExpValidator(fieldIdCc, mailPattern, msgInvalidMailFormat);
        }

        if (!valid) {
            hideloader();
            alert(msgInvalidMailFormat);
            return;
        }

        ActionHelper.UpsertAutoComplete('GRUPPE1_EMAIL', to, true);
        ActionHelper.UpsertAutoComplete('GRUPPE1_EMAIL', cc, true);

        showLoader(Utils.getText('PDF_WIRD_VERSENDET'));
        window.setTimeout(function () {
            Boso.createPdfReport(function (patientNo, PDFContent) {
                Boso.sendPdfAjaxCall(patientNo, PDFContent, to, cc);
            }, function (error) {
                console.log('error:', error);
            });
        }, 100);
    },

    createDetailDataTable: function () {
        var columns = ["", Utils.getText('SYSTOLE'), Utils.getText('DIASTOLE'), Utils.getText('PULS'), Utils.getText('ARRHYTHMIE'), Utils.getText('KILOGRAMM')];
        var rows = [];

        var dateFrom = $('#inputvon').val();
        dateFrom = !!Boso.IsUsedForReporting && !dateFrom ? localStorage.getItem('BOSO_STAT_START') : dateFrom;
        dateFrom = !dateFrom ? moment([0001, 00, 01]).format(FormatHelper.get_DATE_FORMAT()) : dateFrom;

        var dateUntil = $('#inputbis').val();
        dateUntil = !!Boso.IsUsedForReporting && !dateUntil ? moment(localStorage.getItem('BOSO_STAT_END'), FormatHelper.get_DATE_FORMAT()) : dateUntil;
        dateUntil = !dateUntil ? moment([9999, 11, 31]) : moment(dateUntil, FormatHelper.get_DATE_FORMAT()).add('day', 1);
        dateUntil = dateUntil.format(FormatHelper.get_DATE_FORMAT());

        var userID = Boso.getCurrentUser().get('DataId');
        var measurements = _.filter(window.genericData.models, function (model) {
            return (
                model.get('EntityId') === Boso.MESSUREMENT_ENTITYID &&
                model.get('IsActive') == true &&
                model.get('Custom_009') === userID &&
                moment.getMilliSecondsFromPersistFormat(model.get('Custom_001')) >= moment.getMilliSecondsFromDateFormat(dateFrom) &&
                moment.getMilliSecondsFromPersistFormat(model.get('Custom_001')) <= moment.getMilliSecondsFromDateFormat(dateUntil)
            );
        });

        if (!measurements || measurements.length == 0) {
            return;
        }

        //Nach Datum Sortieren
        measurementsSorted = _.sortBy(measurements, function (measurement) {
            return moment(measurement.get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT)
        }).reverse();

        for (var i = 0; i < measurementsSorted.length; i++) {
            var measurement = measurementsSorted[i];
            var date = moment(measurement.get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT).format(FormatHelper.getPattern('DATETIME_FORMAT'));

            var arr = measurement.get('Custom_005') === 1 ? Utils.getText('JA') : '-';
            rows.push([date, measurement.get('Custom_002'), measurement.get('Custom_003'), measurement.get('Custom_004'), arr, measurement.get('Custom_006')]);
        }

        var result = {
            columns: columns,
            rows: rows
        }

        return result;
    },

    createPdfReport: function (successCallback, errorCallback) {
        var defaultFontSizeInPx = 16;

        var currentBodyFontSize = !!$('body') && !!$('body').css('font-size') ? $('body').css('font-size') : '16px';
        var currentBodyFontSizeNumber = currentBodyFontSize.match(/[\d]+\.?[\d]*/)[0];

        var fontScale = currentBodyFontSizeNumber / defaultFontSizeInPx;

        var fontS1 = Math.round(48 / fontScale) + 'px';
        var fontS2 = Math.round(32 / fontScale) + 'px';
        var fontS3 = Math.round(28 / fontScale) + 'px';
        var fontS4 = Math.round(28 / Math.pow(fontScale, 2)) + 'px';
        var fontS5 = 22 + 'px';

        var tableWidthInPerc = 70;
        var letterSpacing = 1 * Math.pow(fontScale, 7) + 'px';
        var rptStartTime = moment();

        Boso.IsUsedForReporting = true;
        var setObjectsToNull_ChartContainer = function () {
            bodyContainer = null;
            invalidMailFormat = null;
            mailPattern = null;
            DateFrom = null;
            DateTo = null;
            container = null;
            headerContainer = null;
            logo = null;
            userContainer = null;
            patientPreName = null;
            patientBirthday = null;
            patientHeight = null;
            p = null;
            statisticContainer = null;
            h2 = null;
            chartContainer = null;
            container_statistics = null;
            container_graphics = null;
        }

        var bodyContainer = $('body');
        bodyContainer.addClass('reportBody');

        if (Utils.isNull(localStorage.getItem('BOSO_STAT_START')) || Utils.isNull(localStorage.getItem('BOSO_STAT_END'))) {
            var measurements = Boso.openStatPrepareMeasurementData();

            if (Utils.isNull(measurements)) {
                hideloader();
                return;
            }

            Boso.bosoGraphs.resetStatDate();
        }

        var DateFrom = localStorage.getItem('BOSO_STAT_START');
        var DateTo = moment(localStorage.getItem('BOSO_STAT_END'), FormatHelper.getPattern('DATE_FORMAT')).add(1, 'days').subtract(1, 'milliseconds').format(FormatHelper.getPattern('DATETIME_FORMAT'));

        //Export statistics on the first page
        var container = $('<div/>', {
            id: 'pdf_export_statistics',
            style: 'background-color: white; visibility: hidden;'
        }).appendTo(bodyContainer);
        container.addClass('pdfReportContainer');
        container.css('letter-spacing', letterSpacing);

        var headerContainer = $('<div/>', {
            id: 'header_container',
            style: 'background-color: white;'
        }).appendTo(container);
        headerContainer.css('border-bottom', 'solid 1px black');
        headerContainer.css('margin', '10px');

        var logo = $('<img/>', {
            id: 'logo',
        }).appendTo(headerContainer);
        logo.attr('src', 'images/brands/boso.png');
        logo.css('height', '150px');
        logo.css('width', '300px');

        var userContainer = $('<div/>', {
            id: 'user_container',
            style: 'background-color: white;'
        }).appendTo(container);
        userContainer.css('margin-top', '30px');
        userContainer.css('margin-left', '30px');

        var patient = Boso.getCurrentUser();
        var patientNo = patient.get('Custom_001');
        var patientName = patient.get('Custom_002');
        var patientPreName = patient.get('Custom_006');
        var patientBirthday = patient.get('Custom_003');
        var patientHeight = patient.get('Custom_004');

        console.log('TEST:', 'patientNo', patientNo, 'patientName', patientName, 'patientPreName', patientPreName, 'patientBirthday', patientBirthday, 'patientHeight', patientHeight);


        var t1 = Utils.getText('PATIENT') + patientName

        if(!!patientPreName) {
            t1 += ', ' + patientPreName;
        }

        t1 = t1.replace(/\s/g, '\u00A0').replace(/-/g, '\u2012');
        var p = $('<p/>', {
            text: t1,
            style: 'font-size:' + fontS3
        }).appendTo(userContainer);

        var t2 = Utils.getText('PATIENT_NR') + patientNo;
        t2 = t2.replace(/\s/g, '\u00A0').replace(/-/g, '\u2012');
        var p = $('<p/>', {
            text: t2,
            style: 'font-size:' + fontS3
        }).appendTo(userContainer);

        if(!!patientBirthday) {
            var t3 = Utils.getText('GEBURTSTAG') + ': ' + moment.getDisplayValue(patientBirthday, FormatHelper.get_DATE_FORMAT());
            t3 = t3.replace(/\s/g, '\u00A0').replace(/-/g, '\u2012');
            var p = $('<p/>', {
                text: t3,
                style: 'font-size:' + fontS3
            }).appendTo(userContainer);
        }

        if(!!patientHeight) {
            var t4 = Utils.getText('GROESSE') + patientHeight + ' cm';
            t4 = t4.replace(/\s/g, '\u00A0').replace(/-/g, '\u2012');
            var p = $('<p/>', {
                text: t4,
                style: 'font-size:' + fontS3
            }).appendTo(userContainer);
        }

        var statisticContainer = $('<div/>', {
            id: 'stat_container',
            style: 'background-color: white;'
        }).appendTo(container);
        statisticContainer.css('margin-top', '30px');
        statisticContainer.css('margin-left', '30px');

        var t5 = Utils.getText('STATISTIK') + ': ' + '(' + DateFrom + ' - ' + moment(DateTo, FormatHelper.getPattern('DATETIME_FORMAT')).format(FormatHelper.getPattern('DATE_FORMAT')) + ')';
        t5 = t5.replace(/\s/g, '\u00A0').replace(/-/g, '\u2012');
        var h2 = $('<h2/>', {
            text: t5,
            style: 'font-size:' + fontS1
        }).appendTo(statisticContainer);

        Boso.OpenStatistic(false, 'stat_container');
        $('#stat_container').css('font-size', fontS2);
        $('#stat_container').css('width', tableWidthInPerc + '%');

        Boso.bosoGraphs.Graphs.labelSize = fontS5;
        Boso.bosoGraphs.Graphs.legendFontSize = fontS4;

        var container_graphics = $('<div/>', {
            id: 'pdf_export_graphics',
            style: 'background-color: white; visibility:hidden;'
        }).appendTo(bodyContainer);
        container_graphics.addClass('pdfReportContainer');

        var chartContainer = $('<div/>', {
            id: 'chart_container',
            style: 'background-color: white;'
        }).appendTo(container_graphics);

        var h2 = $('<h2/>', {
            text: Utils.getText('GRAFIKEN') + ':',
            style: 'font-size:' + fontS1
        }).appendTo(chartContainer);

        var chartContent = $('<div/>', {
            id: 'chart_content',
            style: 'background-color: white;'
        }).appendTo(chartContainer);
        chartContainer.css('margin', '30px');

        Boso.OpenGraphic(false, DateFrom, DateTo, function () {
            var container_statistics = $('#pdf_export_statistics');
            var container_graphics = $('#pdf_export_graphics');
            var pdf = new jsPDF('p', 'pt', 'a4');

            container_statistics.css('visibility', 'visible');
            pdf.addHTML(container_statistics, function () {
                container_statistics.css('visibility', 'hidden');

                pdf.addPage();
                var options = {
                    h: 200
                };

                container_graphics.css('visibility', 'visible');
                $('#' + Boso.CHART_IDS.SysDia).show();
                $('#' + Boso.CHART_IDS.Pulse).show();
                $('#' + Boso.CHART_IDS.KgBMI).show();
                pdf.addHTML(container_graphics, options, function () {
                    container_graphics.css('visibility', 'hidden');
                    $('#' + Boso.CHART_IDS.SysDia).hide();
                    $('#' + Boso.CHART_IDS.Pulse).hide();
                    $('#' + Boso.CHART_IDS.KgBMI).hide();

                    pdf.addPage();

                    var tableData = Boso.createDetailDataTable();

                    var tWidth = (pdf.internal.pageSize.width / 100) * tableWidthInPerc;
                    pdf.autoTable(tableData.columns, tableData.rows, {
                        theme: 'striped',
                        headerStyles: {
                            fillColor: [255, 255, 255],
                            textColor: [50, 50, 50]
                        },
                        tableWidth: tWidth,
                        margin: { left: 15 },
                        beforePageContent: function (data) {
                            pdf.setFont('helvetica');
                            pdf.setFontType("bold");
                            pdf.text(Utils.getText("MESSWERTE"), 15, 30);
                        }
                    });

                    if (_.isFunction(successCallback)) {
                        console.log('pdfContent erstellen')
                        var PDFContent = pdf.output('blob');
                        successCallback(patientNo, PDFContent);
                    }
                    setObjectsToNull_ChartContainer();
                });
            });
        });
    },

    CleanUpReportingLeftover: function () {
        $('#pdf_export_statistics').remove();
        $('#pdf_export_graphics').remove();
        $('#container_statistics').remove();
        $('body').removeClass('reportBody');
        Boso.IsUsedForReporting = false;
        Boso.bosoGraphs.Graphs.labelSize = '12pt';
        Boso.bosoGraphs.Graphs.legendFontSize = '16pt';
    },

    sendPdfAjaxCall: function (patientNo, PDFContent, to, cc) {
        var body = $('#422de166-d008-d172-6437-5341a7800fe7').val();
        MailObject = {
            Token: CryptoJSHelper.getEncryptedString(window.currentUser.UserId),
            To: to,
            CC: cc,
            Subject: Utils.getText('BERICHT'),
            Body: Utils.getText('BOSO_APP_DATEN_PDF') + body,
            FileName: "medicus_" + patientNo + '.pdf',
            FileContent: PDFContent,
            MandatorID: window.currentUser.MandatorId,
            CurrentUserID: window.currentUser.UserId
        };

        $.ajax({
            url: Boso.MailBaseUrl + Boso.SEND_MAIL,
            data: JSON.stringify(MailObject),
            type: 'post',
            contentType: "application/json; charset=utf-8",
            timeout: CONSTS.TIMEOUT_SUPPORT_DATA,
            success: function (AjaxReturnObject) {
                if (AjaxReturnObject.ReturnType === RETURN_TYPE.SUCCESS || AjaxReturnObject.ReturnType === RETURN_TYPE.INFO || AjaxReturnObject.ReturnType === RETURN_TYPE.WARNING) {
                    if (AjaxReturnObject.ReturnType === RETURN_TYPE.WARNING) {
                        alert(AjaxReturnObject.ReturnMsg, null, 'Erfolg', 'Ok');
                    }
                }
                hideloader();
                Boso.CleanUpReportingLeftover();
                alert(Utils.getText('PDF_BERICHT_VERSANDT'));
                options = null;
                MailObject = null;
            },
            error: function (response) {
                alert(Utils.getText('BERICHT_NICHT_VERSANDT'));
                hideloader();
                Boso.CleanUpReportingLeftover();
                options = null;
                MailObject = null;
            }
        });
    },


    ScrollGraphicToCenter: function (graphicContainer) {
        if (!!graphicContainer && !!graphicContainer[0] && !Boso.IsUsedForReporting) {
            $('html,body').animate({ scrollTop: graphicContainer.offset().top - ($(window).height() - graphicContainer.outerHeight(true)) / 2 }, 1000);
        }
    },

    StyleMainContainerForChartsView: function () {
        var mainContainerByChildElements = $('#main_container:has(#pagecontent):has(#Custom):has(div):has(#CustomViewContent)');
        mainContainerByChildElements.addClass('main_container_graphicsView');

        $('#main_container').unbind('DOMSubtreeModified').bind('DOMSubtreeModified', function () {
            mainContainerByChildElements = $('#main_container:has(#pagecontent):has(#Custom):has(div):has(#CustomViewContent)');
            var mainContainerByClass = $('.main_container_graphicsView');

            if (mainContainerByChildElements.length === 0) {
                $('#main_container').removeClass('main_container_graphicsView');
                $('#main_container').removeClass('fullscreenMainContainer');
                $('#main_container').unbind('DOMSubtreeModified');
            }

            if (mainContainerByClass.length === 0) {
                mainContainerByChildElements.addClass('main_container_graphicsView');
            }
        });
    },

    determinScreenOrientation: function () {
        var orientation = window.innerHeight > window.innerWidth ? Boso.SCREEN_ORIENTATIONS.portrait : Boso.SCREEN_ORIENTATIONS.landscape;
        return orientation;
    },

    OpenGraphic: function (createfields, DateFrom, DateTo, callback) {
        if (!Boso.IsUsedForReporting) {
            Boso.showGraphicsLoader();
            this.StyleMainContainerForChartsView();
        }

        setTimeout(function () {
            Boso.OpenGraphic1(createfields, DateFrom, DateTo, callback);
        }, 200);
    },

    OpenGraphic1: function (createfields, DateFrom, DateTo, callback) {
        var getOrientation = function () {
            Boso.screenOrientation = Boso.determinScreenOrientation();
        }
        getOrientation();
        $(window).off('resize', getOrientation).on('resize', getOrientation)

        //Felder erstellen:
        if (createfields == undefined || createfields) {
            Boso.createFieldsGraphics();
        }

        if (!DateFrom && !!localStorage.getItem('BOSO_STAT_START')) {
            DateFrom = moment(localStorage.getItem('BOSO_STAT_START'), FormatHelper.getPattern('DATE_FORMAT'));
        }

        if (!DateTo && !!localStorage.getItem('BOSO_STAT_END')) {
            DateTo = moment(localStorage.getItem('BOSO_STAT_END'), FormatHelper.getPattern('DATE_FORMAT')).add(1, 'days');
        }

        DateFrom = moment(DateFrom, FormatHelper.getPattern('DATE_FORMAT')).format(FormatHelper.getPattern('DATETIME_FORMAT'));

        Boso.OpenBloodPressureGraphic(DateFrom, DateTo, function () {
            Boso.OpenWeightGraphic(DateFrom, DateTo, function () {

                if ((createfields === undefined || createfields === true) && $('#btSysDia').length === 0) {
                    var container = $('#chart_content');
                    var fs = $('<div/>', {}).prependTo(container);
                    fs.attr('class', 'ui-content decision_graphs')
                    fs.attr('data-type', 'horizontal');
                    fs.attr('fieldtype', 'decision');

                    var input_sysdia = $('<button/>', {
                        id: 'btSysDia',
                        type: 'button',
                        class: "ui-btn btn_custom btn_sys_dia_inactive ui-btn-inline",
                        value: 'true',
                    }).appendTo(fs);

                    $(input_sysdia).removeClass('btn_sys_dia_inactive');
                    $(input_sysdia).addClass('btn_sys_dia_active');

                    var input_puls = $('<button/>', {
                        id: 'btPuls',
                        type: 'button',
                        class: "ui-btn btn_custom btn_pulse_inactive ui-btn-inline",
                        value: 'true',
                    }).appendTo(fs);

                    var input_bmi = $('<button/>', {
                        id: 'btKgBmi',
                        type: 'button',
                        class: "ui-btn btn_custom btn_kg_inactive ui-btn-inline",
                        value: 'true',
                    }).appendTo(fs);

                    //Events
                    $(input_sysdia).on('click', function () {
                        $('#' + Boso.CHART_IDS.SysDia).show();
                        $('#' + Boso.CHART_IDS.Pulse).hide();
                        $('#' + Boso.CHART_IDS.KgBMI).hide();

                        $(input_sysdia).addClass('btn_sys_dia_active');
                        $(input_puls).addClass('btn_pulse_inactive');
                        $(input_bmi).addClass('btn_kg_inactive');

                        $(input_sysdia).removeClass('btn_sys_dia_inactive');
                        $(input_puls).removeClass('btn_pulse_active');
                        $(input_bmi).removeClass('btn_kg_active');

                        this.blur();
                        $('#sysDiaPluseGraphInfo').length > 0 ? $('#boso_graph_info').css({ 'display': 'block' }) : $('#boso_graph_info').css({ 'display': 'none' });
                    });

                    $(input_puls).on('click', function () {
                        $('#' + Boso.CHART_IDS.SysDia).hide();
                        $('#' + Boso.CHART_IDS.Pulse).show();
                        $('#' + Boso.CHART_IDS.KgBMI).hide();

                        $(input_puls).addClass('btn_pulse_active');
                        $(input_sysdia).addClass('btn_sys_dia_inactive');
                        $(input_bmi).addClass('btn_kg_inactive');

                        $(input_puls).removeClass('btn_pulse_inactive');
                        $(input_sysdia).removeClass('btn_sys_dia_active');
                        $(input_bmi).removeClass('btn_kg_active');

                        this.blur();
                        $('#sysDiaPluseGraphInfo').length > 0 ? $('#boso_graph_info').css({ 'display': 'block' }) : $('#boso_graph_info').css({ 'display': 'none' });
                    });


                    $(input_bmi).on('click', function () {
                        $('#chart_SysDia').hide();
                        $('#chart_Puls').hide();
                        $('#chart_kgBMI').show();

                        $(input_bmi).addClass('btn_kg_active');
                        $(input_puls).addClass('btn_pulse_inactive');
                        $(input_sysdia).addClass('btn_sys_dia_inactive');

                        $(input_bmi).removeClass('btn_kg_inactive');
                        $(input_puls).removeClass('btn_pulse_active');
                        $(input_sysdia).removeClass('btn_sys_dia_active');

                        this.blur();
                        $('#weightGraphInfo').length > 0 ? $('#boso_graph_info').css({ 'display': 'block' }) : $('#boso_graph_info').css({ 'display': 'none' });
                    });

                    Boso.bosoGraphs.Graphs.displaySingleGraphic();

                    $(document).ready(function () {
                        $(document).off('click', '.boso_chart:visible .jqplot-event-canvas').on('click', '.boso_chart:visible .jqplot-event-canvas', function () {
                            if (Boso.bosoGraphs.Graphs.isFullScreen) {
                                $('#main_container').removeClass('fullscreenMainContainer');
                                $('.boso_chart').removeClass('fullscreen');
                            }
                            else {
                                $('#main_container').addClass('fullscreenMainContainer');
                                $('.boso_chart').addClass('fullscreen');
                            }

                            Boso.bosoGraphs.Graphs.isFullScreen = _.size($('.fullscreen')) > 0;
                            $(document).trigger('resize');
                        });
                    });

                    Boso.hideGraphicsLoader();
                }

                if (_.isFunction(callback)) {
                    callback();
                }
            });
        });
    },

    OpenWeightGraphic: function (DateFrom, DateTo, callback) {
        var page = $('#hiddenPageValBMI').val();
        if (Utils.isNull(page)) {
            page = 1;
        }

        var genericDataFiltered = this.getGenericDataWeightGraphics(null, DateFrom, DateTo);

        if (genericDataFiltered.length < 2) {
            var bosoGraphInfo = $('#boso_graph_info');
            if (bosoGraphInfo.length === 0) {
                var customViewContent = $('#CustomViewContent');
                var div = $('<div/>', {
                    id: 'boso_graph_info',
                    css: { 'display': 'block' }
                }).appendTo(customViewContent);
            }

            var weightGraphInfo = $('#weightGraphInfo');
            if (weightGraphInfo.length === 0) {
                var div = $('<p/>', {
                    id: 'weightGraphInfo',
                    style: 'text-align: center;',
                    text: Utils.getText('GEWICHT_WENIG_DATEN')
                }).appendTo($('#boso_graph_info'));

                var graphInfo = $('#boso_graph_info');
                graphInfo.css('display', 'block');
            }

            if (_.isFunction(callback)) {
                callback();
            }
            return;
        }

        //DATEN AUFBEREITEN
        var lables = [];
        var gcns = [];
        var gncProps = _.filter(genericProperties.models, function (prop) {
            return prop.attributes.EntityId == Boso.MESSUREMENT_ENTITYID
        });
        _.each(gncProps, function (prop) {
            lables.push(prop.attributes.PropertyName);
            gcns.push(prop.attributes.GenericColumnName);
        });

        var koerpergroesse = Boso.getCurrentUser().attributes.Custom_004 / 100;

        var dataChartKiloBMI = [[Utils.getText('DATUM'), Utils.getText('KILOGRAMM'), Utils.getText('BODY_MASS_INDEX')]];
        var dataChartKiloBMIAxis = ["x1", "yaxis", "y2axis"]; //x - y zuordnung
        _.each(genericDataFiltered, function (model) {
            var date = moment.getDisplayValue(model.attributes['Custom_001'], 'YYYY/MM/DD HH:mm:ss').toString();
            var kg = 0;
            var BMI = 0;
            for (var i = 0; i < gcns.length; i++) {
                var val = model.attributes[gcns[i]];

                if (lables[i] == Utils.getText('KILOGRAMM')) {
                    if (!val) { val = 0; }
                    kg = parseFloat(val);
                    BMI = kg / Math.pow(koerpergroesse, 2)
                }
            }

            if (kg != 0) {
                dataChartKiloBMI.push([date, kg, BMI]);
            }

        });

        var seriesColors = ["#3276B1", "#3276B1"];
        //RENDERN
        var currentCallback = callback;
        Boso.bosoGraphs.Graphs.drawChart(dataChartKiloBMI, dataChartKiloBMIAxis, seriesColors, Boso.CHART_IDS.KgBMI, function () {
            if (currentCallback != null) {
                currentCallback();
            }
        });


    },

    OpenBloodPressureGraphic: function (DateFrom, DateTo, callback) {
        var page = $('#hiddenPageValBP').val();
        if (!page) {
            page = 1;
        }

        var genericDataFiltered = this.getGenericDataBloodGraphics(null, DateFrom, DateTo);
        if (genericDataFiltered.length < 2) {
            var bosoGraphInfo = $('#boso_graph_info');
            if (bosoGraphInfo.length === 0) {
                var customViewContent = $('#CustomViewContent');
                var div = $('<div/>', {
                    id: 'boso_graph_info',
                    css: { 'display': 'none' }
                }).appendTo(customViewContent);
            }

            var sysDiaPluseGraphInfo = $('#sysDiaPluseGraphInfo');
            if (sysDiaPluseGraphInfo.length === 0) {
                var div = $('<p/>', {
                    id: 'sysDiaPluseGraphInfo',
                    style: 'text-align: center;',
                    text: Utils.getText('BLUTDRUCK_PULS_WENIG_DATEN')
                }).appendTo($('#boso_graph_info'));
            }

            if (callback != null) {
                callback();
            }
            return;
        }

        //DATEN AUFBEREITEN
        var lables = [];
        var gcns = [];

        var gncProps = _.filter(genericProperties.models, function (prop) {
            return prop.attributes.EntityId == Boso.MESSUREMENT_ENTITYID; //ActionHelper.CurrentView.options.view.attributes.EntityId;
        });
        _.each(gncProps, function (prop) {
            lables.push(prop.attributes.PropertyName);
            gcns.push(prop.attributes.GenericColumnName);
        });

        var koerpergroesse = Boso.getCurrentUser().attributes.Custom_004;

        var defaultFontSizeInPx = 16;
        var currentBodyFontSize = !!$('body') && !!$('body').css('font-size') ? $('body').css('font-size') : '16px';
        var currentBodyFontSizeNumber = currentBodyFontSize.match(/[\d]+\.?[\d]*/)[0];
        var scale = currentBodyFontSizeNumber / defaultFontSizeInPx;
        var margin = Math.pow(scale, 7) + 'px';

        var einheit = '\u00A0<span style="font-size:80%;margin-left:' + margin + ';">(mmHg)</span>'
        var lbSys = Utils.getText('SYSTOLE').replace(/\s/g, '\u00A0').replace(/-/g, '\u2012') + einheit;
        var lbDia = Utils.getText('DIASTOLE').replace(/\s/g, '\u00A0').replace(/-/g, '\u2012') + einheit
        var dataChartSysDia = [[Utils.getText('DATUM'), lbSys, lbDia]];
        var dataChartSysDiaAxis = ["x1", "yaxis", "yaxis"];
        var seriesColors = ["#dc332e", "#3276B1"] //x1 ignorieren

        var einheitPul = '\u00A0<span style="font-size:80%;margin-left:' + margin + ';">(1/min)</span>'
        var lbPul = Utils.getText('PULS').replace(/\s/g, '\u00A0').replace(/-/g, '\u2012') + einheitPul
        var dataChartPul = [[Utils.getText('DATUM'), lbPul]];
        _.each(genericDataFiltered, function (model) {
            var date = moment.getDisplayValue(model.attributes['Custom_001'], 'YYYY/MM/DD HH:mm:ss').toString();
            var SYS = 0;
            var DIA = 0;
            var PUL = 0;
            for (var i = 0; i < gcns.length; i++) {
                var val = model.attributes[gcns[i]];

                if (lables[i] == Utils.getText('DIASTOLE')) {
                    if (val == undefined) { val = 0; }
                    DIA = parseInt(val);
                }
                else if (lables[i] == Utils.getText('SYSTOLE')) {
                    if (val == undefined) { val = 0; }
                    SYS = parseInt(val);
                }
                else if (lables[i] == Utils.getText('PULS')) {
                    if (val == undefined) { val = 0; }
                    PUL = parseInt(val);
                }

            }
            if (SYS != 0 || DIA != 0) {
                dataChartSysDia.push([date, SYS, DIA]);
            }
            if (PUL != 0) {
                dataChartPul.push([date, PUL]);
            }
        });

        Boso.bosoGraphs.Graphs.drawChart(dataChartSysDia, dataChartSysDiaAxis, seriesColors, Boso.CHART_IDS.SysDia, function () {
            Boso.bosoGraphs.Graphs.drawChart(dataChartPul, ['xaxis', 'yaxis'], ['#dc332e'], Boso.CHART_IDS.Pulse, function () {
                if (_.isFunction(callback)) {
                    callback();
                }
            });
        });
    },

    openStatPrepareContainer: function (createFields, containerId, callback) {
        //Felder erstellen:
        if (createFields) {
            Boso.createFieldsStatistic();
        }

        //Statistik initialisieren
        Boso.bosoGraphs.Statistics.init(containerId);

        if (!!callback && _.isFunction(callback)) {
            callback();
        }
    },

    openStatGetMeasurementProps: function (callback) {
        var measurementProps = {
            labels: [Utils.getText('SYSTOLE'), Utils.getText('DIASTOLE'), Utils.getText('PULS'), Utils.getText('KILOGRAMM'), Utils.getText('BODY_MASS_INDEX')],
            genericColNames: []
        }

        var gncProps = _.filter(genericProperties.models, function (prop) {
            return prop.attributes.EntityId == Boso.MESSUREMENT_ENTITYID;
        });

        var filterlist = [Utils.getText('ARRHYTHMIE'), Utils.getText('MESSGERAET'), Utils.getText('MESSDATUM'), Utils.getText('BENUTZER'), Utils.getText('ABP')];

        _.each(gncProps, function (prop) {
            var propName = prop.get('PropertyName');
            if (filterlist.indexOf(propName) == -1) {
                measurementProps.genericColNames[measurementProps.labels.indexOf(propName)] = prop.get('GenericColumnName');
            }
        });

        if (!!callback && _.isFunction(callback)) {
            callback(measurementProps);
        }

        return measurementProps;
    },

    openStatGetTimeFilters: function (callback) {
        var dayNightIntervall = UserSettings.GetDayNightIntervall();
        var intervallMorning = !!dayNightIntervall && dayNightIntervall.Start ? dayNightIntervall.Start : '00:00:00';
        var intervallAfternoon = !!dayNightIntervall && !!dayNightIntervall.End ? dayNightIntervall.End : '12:00:00';

        var totalStart = '00:00:00';
        var totalEnd = '24:00:00';
        var time_format = FormatHelper.get_TIME_FORMAT();
        totalEnd = moment(totalEnd, time_format);
        totalEnd.subtract('seconds', 1);
        totalEnd = totalEnd.format(time_format, 'de');

        var startMorning = intervallMorning;
        var endMorning = moment(intervallAfternoon, time_format);
        endMorning.subtract('seconds', 1);
        endMorning = endMorning.format(time_format, 'de');

        var startAfternoon = intervallAfternoon;
        var endAfternoon = moment(startMorning, time_format);
        endAfternoon.subtract('seconds', 1);
        endAfternoon = endAfternoon.format(time_format, 'de');

        var timeFilters = [
            [totalStart, totalEnd],
            [startMorning, endMorning],
            [startAfternoon, endAfternoon],
        ]

        if (!!callback && _.isFunction(callback)) {
            callback(timeFilters);
        }
        return timeFilters;
    },

    openStatPrepareMeasurementData: function (callback) {
        var userID = Boso.getCurrentUser().get('DataId');

        var dateFrom = $('#inputvon').val();
        dateFrom = !!Boso.IsUsedForReporting && !dateFrom ? localStorage.getItem('BOSO_STAT_START') : dateFrom;
        dateFrom = !dateFrom ? moment([0001, 00, 01]).format(FormatHelper.get_DATE_FORMAT()) : dateFrom;

        var dateUntil = $('#inputbis').val();
        dateUntil = !!Boso.IsUsedForReporting && !dateUntil ? moment(localStorage.getItem('BOSO_STAT_END'), FormatHelper.get_DATE_FORMAT()) : dateUntil;
        dateUntil = !dateUntil ? moment([9999, 11, 31]) : moment(dateUntil, FormatHelper.get_DATE_FORMAT()).add('day', 1);
        dateUntil = dateUntil.format(FormatHelper.get_DATE_FORMAT());

        var measurements = _.filter(window.genericData.models, function (model) {
            return (
                model.get('EntityId') === Boso.MESSUREMENT_ENTITYID &&
                model.get('IsActive') == true &&
                model.get('Custom_009') === userID &&
                moment.getMilliSecondsFromPersistFormat(model.get('Custom_001')) >= moment.getMilliSecondsFromDateFormat(dateFrom) &&
                moment.getMilliSecondsFromPersistFormat(model.get('Custom_001')) <= moment.getMilliSecondsFromDateFormat(dateUntil)
            );
        });

        if (measurements.length == 0) {
            Boso.hideStatisticLoader();
            return;
        }

        //Nach Datum Sortieren
        measurementsSorted = _.sortBy(measurements, function (measurement) {
            return moment(measurement.get('Custom_001'), CONSTS.DATETIME_PERSIST_FORMAT)
        });

        if (!!callback && _.isFunction(callback)) {
            callback(measurementsSorted);
        }

        return measurementsSorted;
    },

    openStatFilterMeasurementsByDayTime: function (timespan, measurements, index, callback) {
        var filteredMeasurements = null;

        if (!!timespan) {
            var from = timespan[0];
            var until = timespan[1];

            if (from <= until) {
                filteredMeasurements = _.filter(measurements, function (measurement) {
                    var time = measurement.attributes["Custom_001"].split(' ')[1];
                    var greaterFrom = moment.getTimeGreaterThanOrEqual(time, from);
                    var lesserUntil = !moment.getTimeGreaterThanOrEqual(time, until);
                    return (greaterFrom && lesserUntil);
                });
            }
            else {
                filteredMeasurements = _.filter(measurements, function (measurement) {
                    var time = measurement.attributes["Custom_001"].split(' ')[1];
                    var greaterFrom = moment.getTimeGreaterThanOrEqual(time, from);
                    var lesserUntil = !moment.getTimeGreaterThanOrEqual(time, until);
                    return (greaterFrom || lesserUntil);
                });
            }
        }

        if (!!callback && _.isFunction(callback) && !!timespan) {
            callback(filteredMeasurements, timespan, index);
        }
        return filteredMeasurements;
    },

    openStatPrepareTable: function (timespan, measurements, measurementProps, dayIntervallThird, callback) {
        if (!measurements) {
            return;
        }

        var tableElements = {
            dataTable: [],
            title: '',
            subtitle: ''
        }

        tableElements.dataTable.push(measurementProps.labels);

        _.each(measurements, function (measurement) {
            var row = [];
            var BMI = null;

            var addKgBmi = function (val, i) {
                var kg = !!val ? parseFloat(val) : null;
                BMI = !!kg ? (kg / Math.pow(bodySize, 2)) : null;
                row[i] = kg;
            }

            var bodySize = Boso.getCurrentUser().get('Custom_004') / 100;
            for (var i = 0; i < measurementProps.genericColNames.length; i++) {
                var val = measurement.attributes[measurementProps.genericColNames[i]];

                if (measurementProps.labels[i] === Utils.getText('KILOGRAMM')) {
                    addKgBmi(val, i);
                }
                else if (measurementProps.labels[i] === Utils.getText('MESSDATUM')) {
                    var date = moment.getDisplayValue(val);
                    row[i] = date;
                }
                else {
                    val = !!val ? parseFloat(val) : null;
                    row[i] = val;
                }
            }

            row[4] = BMI;
            tableElements.dataTable.push(row);
        });

        var arrythmen = _.filter(measurements, function (measurement) {
            return measurement.get('Custom_005') == 1;
        }).length;

        tableElements.title = "";
        tableElements.subtitle = "";

        if (dayIntervallThird == 0) {
            var time = timespan.toString().split(",");
            tableElements.title = Utils.getText('GESAMT') + " 00:00 - 23:59";
            tableElements.subtitle = Utils.getText('MESSWERTE') + ': ' + (tableElements.dataTable.length - 1) + '<br/>' + Utils.getText('ARRHYTHMIEN ERKANNT') + ': ' + arrythmen;
            tableElements.title = tableElements.title.replace(/\s/g, '\u00A0').replace(/-/g, '\u2012');
            tableElements.subtitle = tableElements.subtitle.replace(/\s/g, '\u00A0').replace(/-/g, '\u2012');
        }
        else if (dayIntervallThird == 1) {
            var time = timespan.toString().split(",");
            tableElements.title = Utils.getText('VORMITTAG') + ': ' + time[0].substring(0, 5) + ' - ' + time[1].substring(0, 5);
            tableElements.subtitle = Utils.getText('MESSWERTE') + ': ' + (tableElements.dataTable.length - 1) + '<br/>' + Utils.getText('ARRHYTHMIEN ERKANNT') + ': ' + arrythmen;
            tableElements.title = tableElements.title.replace(/\s/g, '\u00A0').replace(/-/g, '\u2012');
            tableElements.subtitle = tableElements.subtitle.replace(/\s/g, '\u00A0').replace(/-/g, '\u2012');
        }
        else if (dayIntervallThird == 2) {
            var time = timespan.toString().split(",");
            tableElements.title = Utils.getText('NACHMITTAG') + ': ' + time[0].substring(0, 5) + ' - ' + time[1].substring(0, 5);
            tableElements.subtitle = Utils.getText('MESSWERTE') + ': ' + (tableElements.dataTable.length - 1) + '<br/>' + Utils.getText('ARRHYTHMIEN ERKANNT') + ': ' + arrythmen;
            tableElements.title = tableElements.title.replace(/\s/g, '\u00A0').replace(/-/g, '\u2012');
            tableElements.subtitle = tableElements.subtitle.replace(/\s/g, '\u00A0').replace(/-/g, '\u2012');
        }

        if (!!callback && _.isFunction(callback)) {
            callback(tableElements, dayIntervallThird);
        }
        return tableElements;
    },

    OpenStatistic: function (createFields, containerId, dateFrom, dateTo) {
        if (!Boso.IsUsedForReporting) {
            Boso.OpenStatisticAsync(createFields, containerId, dateFrom, dateTo);
            return;
        }

        //Statistik-Aufbau Synchron --> Für den Bericht.
        var dateStart = moment();

        if (!Boso.statisticLoaderTimeout || (!!Boso.statisticLoaderTimeout && Boso.statisticLoaderTimeout.cleared)) {
            Boso.showStatisticLoader();
        }

        Boso.openStatPrepareContainer(createFields, containerId);
        var measurementProps = Boso.openStatGetMeasurementProps();
        var timeBorders = Boso.openStatGetTimeFilters();
        var measurements = Boso.openStatPrepareMeasurementData();

        _.each(timeBorders, function (timespan, i) {
            var measuremnetsFiltered = Boso.openStatFilterMeasurementsByDayTime(timespan, measurements);
            if (measuremnetsFiltered) {
                var tableElements = Boso.openStatPrepareTable(timespan, measuremnetsFiltered, measurementProps, i);
                Boso.bosoGraphs.Statistics.create(tableElements.dataTable, tableElements.title, tableElements.subtitle);
            }
        });

        $(document).trigger('create');
        Boso.hideStatisticLoader();
        console.log(moment().diff(dateStart, 'seconds'));
    },

    OpenStatisticAsync: function (createFields, containerId, dateFrom, dateTo) {
        var dateStart = moment();
        var _timeBorders = null;
        var _measurementProps = null;

        if (!Boso.statisticLoaderTimeout || (!!Boso.statisticLoaderTimeout && Boso.statisticLoaderTimeout.cleared)) {
            Boso.showStatisticLoader();
        }

        var onContainerPrepared = function () {
            setTimeout(function () {
                Boso.openStatGetMeasurementProps(onMeasurementPropsResult)
            }, 10);
        }

        var onMeasurementPropsResult = function (measurementProps) {
            setTimeout(function () {
                _measurementProps = measurementProps
                Boso.openStatGetTimeFilters(onTimeFiltersResult)
            }, 10);
        }

        var onTimeFiltersResult = function (timeBorders) {
            setTimeout(function () {
                _timeBorders = timeBorders;
                Boso.openStatPrepareMeasurementData(onMeasurementDataPrepared);
            }, 10);
        }

        var onMeasurementDataPrepared = function (measurements) {
            setTimeout(function () {
                _.each(_timeBorders, function (timespan, i) {
                    Boso.openStatFilterMeasurementsByDayTime(timespan, measurements, i, onFilteredMeasurementsResult);
                });
            }, 10);
        }

        var onFilteredMeasurementsResult = function (measurementsFiltered, timespan, index) {
            setTimeout(function () {
                if (!!measurementsFiltered) {
                    Boso.openStatPrepareTable(timespan, measurementsFiltered, _measurementProps, index, onTablePrepared);
                }
            }, 10);
        }

        var onTablePrepared = function (tableElements, timeFiltersCount) {
            Boso.bosoGraphs.Statistics.create(tableElements.dataTable, tableElements.title, tableElements.subtitle);

            if (timeFiltersCount == _timeBorders.length - 1) {
                $(document).trigger('create');
                Boso.hideStatisticLoader();
                console.log('Dauer Statistikaufbau: ' + moment().diff(dateStart, 'seconds'));
            }
        }

        Boso.openStatPrepareContainer(createFields, containerId, onContainerPrepared);
    },

    bosoGraphs: {
        filterColumn: function (index, dataTable) {
            var returndata = [];
            _.each(dataTable, function (row) {
                returndata.push(row[index]);
            });
            return returndata;
        },

        Graphs: {
            isFullScreen: false,
            labelSize: '12pt',
            legendFontSize: '16pt',
            currentPlots: [],

            drawChart: function (data, axis, seriesColors, chartId, callback) {
                // Enable plugins like highlighter and cursor by default.
                // Otherwise, must specify show: true option for those plugins.
                $.jqplot.config.enablePlugins = true;
                this.isFullScreen = _.size($('.fullscreen')) > 0;

                this.renderGraph(data, axis, seriesColors, chartId, function () {
                    _.isFunction(callback) ? callback() : hideloader();
                });

                var onResize = function () {
                    if (!!chartId && !Boso.IsUsedForReporting) {
                        Boso.bosoGraphs.Graphs.renderGraph(data, axis, seriesColors, chartId);
                    }
                }

                $(window).off('resize', $('#' + chartId), onResize).on('resize', $('#' + chartId), onResize);
            },

            renderGraph: function (data, axis, seriesColors, chartId, callback) {
                var chart = $('#' + chartId);

                var isVisible = !!chart && chart.length > 0 && chart.is(':visible');

                this.prepareContainer(chartId);
                var chartOptions = this.getChartOptions(this.prepareLegend(data), seriesColors, data, axis, chartId);
                var preparedData = this.prepareData(data);

                if (this.currentPlots[chartId]) {
                    this.currentPlots[chartId].destroy();
                }

                var plot = $.jqplot(chartId, preparedData, chartOptions);
                this.currentPlots[chartId] = plot;

                !!isVisible ? $('#' + chartId).show() : $('#' + chartId).hide();

                if (_.isFunction(callback)) {
                    callback();
                }
            },

            prepareContainer: function (chartId) {
                var container = $('#chart_content');
                var bounds = this.calcContainerBounds(chartId);

                //empty old
                var renderdiv = $('#' + chartId);
                if (!!container && renderdiv.length === 0) {
                    renderdiv = $('<div/>', {
                        id: chartId,
                        class: 'boso_chart',
                    });

                    renderdiv.appendTo(container);
                } else {
                    $(renderdiv).unbind();
                    $(renderdiv).empty();
                }

                renderdiv.show();
                renderdiv.css({
                    'width': !!bounds.width ? bounds.width : width,
                    'height': !!bounds.height ? bounds.height : '92%',
                    'margin-top': !!Boso.IsUsedForReporting ? '50px' : 'auto'
                });
            },

            calcContainerBounds: function (chartId) {
                var footerHeight = !!$('#footer').height() ? $('#footer').height() : 56;
                var headerHeight = $('#header').outerHeight();
                var chartButtons = $(window).width() > 760 ? 122 : 92
                var whitespace = 10;
                var offset = footerHeight + headerHeight + chartButtons + whitespace;

                var minHeight = 0;
                var chartHeight = ($(window).innerHeight() - offset > minHeight ? $(window).innerHeight() - offset : minHeight) + 'px';
                var width = Boso.screenOrientation === Boso.SCREEN_ORIENTATIONS.portrait ? '97%' : '92%';
                width = chartId === Boso.CHART_IDS.KgBMI ? '97%' : width;

                if (Boso.IsUsedForReporting) {
                    width = chartId === Boso.CHART_IDS.KgBMI ? '96%' : '92%';
                    chartHeight = '650px';
                }

                return { height: chartHeight, width: width }
            },

            prepareLegend: function (data) {
                var legendlabels = [];
                for (var i = 1; i < data[0].length; i++) {
                    title += (data[0])[i].replace(/<.*>/, '');
                    legendlabels.push((data[0])[i]);
                    if (i < data[0].length - 1) { title += " / "; }
                }

                return legendlabels;
            },

            prepareData: function (data) {
                var preparedData = [];

                //deep copy data array then remove the fields that are not required
                var line1 = $.extend(true, [], data);
                line1.shift();
                if (!!line1[0] && line1[0].length == 3) {
                    for (var i = 0; i < line1.length; i++) {
                        line1[i].pop();
                    }
                }
                preparedData.push(line1);

                var line2 = [];
                if (!!data && data[0] && data[0][2]) {
                    line2 = $.extend(true, [], data);
                    line2.shift();

                    for (var i = 0; i < line2.length; i++) {
                        line2[i].splice(1, 1);
                    }
                    preparedData.push(line2);
                }

                return preparedData;
            },

            getChartOptions: function (legendlabels, seriesColors, data, axes, chartId) {
                var options = { axes: {} };
                var tickRepresentation = this.defineTickRepresentation(data);

                var lineWidth = Boso.IsUsedForReporting ? 3 : 1;
                var displayMarkerOptions = {
                    style: 'filledCircle',
                    size: 2,
                };
                var reportingMarkerOptions = {
                    style: 'filledCircle',
                    size: 9,
                };

                var xAxis_tickAngle = Boso.screenOrientation === Boso.SCREEN_ORIENTATIONS.portrait ? -65 : 0;

                options.legend = {
                    renderer: jQuery.jqplot.EnhancedLegendRenderer,
                    show: Boso.bosoGraphs.Graphs.isFullScreen || Boso.IsUsedForReporting,
                    placement: 'outside',
                    rendererOptions: {
                        numberRows: '1',
                        numberColumns: '3',
                        seriesToggle: false,
                    },
                    location: 'n',
                    yoffset: 10,
                    fontSize: Boso.bosoGraphs.Graphs.legendFontSize,
                    rowSpacing: '60px',
                    labels: legendlabels,
                    escapeHtml: false
                };

                options.axes.xaxis = {
                    renderer: $.jqplot.DateAxisRenderer,
                    rendererOptions: {
                        tickRenderer: $.jqplot.CanvasAxisTickRenderer
                    },
                    tickOptions: {
                        fontSize: Boso.bosoGraphs.Graphs.labelSize,
                        fontFamily: 'Arial',
                        angle: xAxis_tickAngle,
                        formatString: tickRepresentation.format
                    },
                    tickInterval: tickRepresentation.interval,
                    min: tickRepresentation.min,
                    max: tickRepresentation.max
                };

                options.axes.yaxis = {
                    renderer: $.jqplot.LinearAxisRenderer,
                    rendererOptions: {
                        tickRenderer: $.jqplot.CanvasAxisTickRenderer
                    },
                    tickOptions: {
                        fontSize: Boso.bosoGraphs.Graphs.labelSize,
                        fontFamily: 'Arial',
                        angle: 30,
                        formatString: ''
                    }
                };


                options.seriesColors = !!seriesColors ? seriesColors : null;
                options.series = [];
                _.each(axes, function (axis) {
                    if (axis === 'yaxis') {
                        options.series.push({
                            lineWidth: lineWidth,
                            yaxis: 'yaxis',
                            markerOptions: Boso.IsUsedForReporting ? reportingMarkerOptions : displayMarkerOptions,
                        });
                    };
                });

                if (_.contains(axes, 'y2axis')) {
                    options.axes.y2axis = {};
                    options.renderer = $.jqplot.LinearAxisRenderer,
                        options.axes.y2axis.rendererOptions = {
                            tickRenderer: $.jqplot.CanvasAxisTickRenderer,
                        };
                    options.axes.y2axis.tickOptions = {
                        fontSize: Boso.bosoGraphs.Graphs.labelSize,
                        fontFamily: 'Arial',
                        angle: -30,
                        mark: 'outside',
                        showMark: true,
                        showGridline: false,
                    };

                    options.series.push({
                        lineWidth: lineWidth,
                        yaxis: 'y2axis',
                        highlighter: { show: true },
                        markerOptions: Boso.IsUsedForReporting ? reportingMarkerOptions : displayMarkerOptions,
                    });
                    options = this.unifyYAxes(options, data, axes);
                }
                return options;
            },

            unifyYAxes: function (options, data, axes) {
                var min = {};
                var max = {};

                //get min and max values of each axe
                for (var i = 1; i < 3; i++) {
                    var chartValues = Boso.bosoGraphs.filterColumn(i, data)
                    chartValues.shift();

                    min[axes[i]] = chartValues.min();
                    max[axes[i]] = chartValues.max();
                }

                options.axes['yaxis'].min = Math.floor(min['yaxis'] - 1);
                options.axes['yaxis'].max = Math.ceil(max['yaxis'] + 1);

                options.axes['y2axis'].min = min['y2axis'] * (Math.floor((min['yaxis'] - 1)) / min['yaxis']);
                options.axes['y2axis'].max = max['y2axis'] * (Math.ceil((max['yaxis'] + 1)) / max['yaxis']);

                var chartHeight = $('.boso_chart').height();
                var maxTicks = chartHeight / 500 * 12;

                //set tick interval
                var tickIntervalYAxis = (options.axes.yaxis.max - options.axes.yaxis.min) / maxTicks;
                options.axes.yaxis.tickInterval = Math.ceil(tickIntervalYAxis);

                var diff = options.axes.y2axis.max - options.axes.y2axis.min;
                var tickIntervalY2Axis = diff / maxTicks;
                tickIntervalY2Axis = diff >= 2 ? 1 : Utils.ceil(tickIntervalY2Axis, 1);
                options.axes.y2axis.tickInterval = tickIntervalY2Axis;


                //Workaround für Display Bug oberste Zeile.
                options.axes.yaxis.tickOptions.formatter = function (format, value) {
                    return (value > Math.ceil(max['yaxis'] + 1) ? '' : Utils.floor(value, 1));
                };

                options.axes.y2axis.tickOptions.formatter = function (format, value) {
                    return (value > max['y2axis'] * (Math.ceil((max['yaxis'] + 1)) / max['yaxis'])) ? '' : Utils.floor(value, 1);
                };

                return options
            },

            defineTickRepresentation: function (data) {
                var tickRepresentation = {
                    interval: '',
                    format: '',
                    min: {},
                    max: {}
                };

                tickRepresentation.min = moment(localStorage.getItem('BOSO_STAT_START'), FormatHelper.getPattern('DATE_FORMAT')).toDate();
                tickRepresentation.max = moment(localStorage.getItem('BOSO_STAT_END'), FormatHelper.getPattern('DATE_FORMAT')).add(1, 'days').toDate();
                if (!moment(tickRepresentation.min).isValid() || !moment(tickRepresentation.max).isValid()) {
                    var dates = Boso.bosoGraphs.filterColumn(0, data)
                    dates.shift();
                    for (var i = 0; i < dates.length; i++) {
                        dates[i] = moment(dates[i], CONSTS.DATETIME_PERSIST_FORMAT);
                    }
                    tickRepresentation.min = dates.reduce(function (a, b) { return a < b ? a : b; }).toDate();
                    tickRepresentation.max = dates.reduce(function (a, b) { return a > b ? a : b; }).toDate();
                }

                var diffInSeconds = moment(tickRepresentation.max).diff(tickRepresentation.min, 'seconds')

                var oneDay = 86400;
                if (diffInSeconds <= oneDay) {
                    var h = Boso.screenOrientation === Boso.SCREEN_ORIENTATIONS.portrait ? 2 : 6;
                    tickRepresentation.interval = h + ' hours';
                    tickRepresentation.format = '%d.%m %H:%M';
                }
                else if (diffInSeconds <= oneDay * 10) {
                    tickRepresentation.interval = '1 day';
                    tickRepresentation.format = '%d.%m.%y';
                }
                else if (diffInSeconds <= oneDay * 60) {
                    tickRepresentation.interval = '1 week';
                    tickRepresentation.format = '%d.%m.%y';
                }
                else if (diffInSeconds <= oneDay * 365) {
                    tickRepresentation.interval = '1 month';
                    tickRepresentation.format = '%d.%m.%y';
                }
                else {
                    tickRepresentation.interval = '3 months';
                    tickRepresentation.format = '%d.%m.%y';
                }

                return tickRepresentation;
            },

            displaySingleGraphic: function () {
                var isAvailableBloodpressureChart = !!$('#' + Boso.CHART_IDS.SysDia) && !!$('#' + Boso.CHART_IDS.SysDia).length > 0;
                var isAvailableWeightChart = !!$('#' + Boso.CHART_IDS.KgBMI) && !!$('#' + Boso.CHART_IDS.KgBMI).length > 0;

                if (isAvailableBloodpressureChart) {
                    window.setTimeout(function () {
                        $('#btSysDia').click();
                    }, 10);
                }
                else if (isAvailableWeightChart) {
                    window.setTimeout(function () {
                        $('#btKgBmi').click();
                    }, 10);
                }
                else {
                    $('#boso_graph_info').css({ 'display': 'block' });
                }
            }
        },

        resetStatDate: function () {
            var genericDataFiltered = Boso.openStatPrepareMeasurementData();
            var dateFirst = !!genericDataFiltered ? genericDataFiltered[0].get('Custom_001') : moment();
            var dateLatest = !!genericDataFiltered ? genericDataFiltered[genericDataFiltered.length - 1].get('Custom_001') : moment();

            dateFirst = moment.getIndivudualFormat(dateFirst, CONSTS.DATETIME_PERSIST_FORMAT, FormatHelper.getPattern('DATE_FORMAT'));
            dateLatest = moment.getIndivudualFormat(dateLatest, CONSTS.DATETIME_PERSIST_FORMAT, FormatHelper.getPattern('DATE_FORMAT'));

            localStorage.setItem('BOSO_STAT_START', dateFirst);
            localStorage.setItem('BOSO_STAT_END', dateLatest);
        },

        Statistics: {
            Table: null,

            clear: function () {
                var container = $('#statisticContent');
                container.empty();
            },
            create: function (dataTable, Title, Undertitle) {
                var container = $('#statisticContent');
                $('<h2/>', {}).appendTo(container).html(Title);
                $('<h4/>', {}).appendTo(container).html(Undertitle);

                var Table = $('<table/>', {
                    class: 'listViewTableStyle'
                }).appendTo(container);
                var ListViewTableHeaderRow = $('<tr/>', {
                    class: 'listViewTableStyleHeaderRow'
                }).appendTo(Table);

                $('<th>', {}).appendTo(ListViewTableHeaderRow);
                $('<th>', {}).appendTo(ListViewTableHeaderRow).html(Utils.getText('MIN'));
                $('<th>', {}).appendTo(ListViewTableHeaderRow).html(Utils.getText('MITTEL'));
                $('<th>', {}).appendTo(ListViewTableHeaderRow).html(Utils.getText('MAX'));
                this.Table = Table;



                for (var i = 0; i < dataTable[0].length; i++) {
                    var data = Boso.bosoGraphs.filterColumn(i, dataTable);
                    var header = data[0];
                    data.remove(header);
                    data.remove(null);

                    var row = $('<tr/>', {}).appendTo(this.Table);
                    $('<th/>', {}).appendTo(row).html(header);

                    this.create_min(data, row);
                    this.create_average(data, row);
                    this.create_max(data, row);
                }
            },

            init: function (containerId) {
                this.clear();
                var fs = $('#CustomViewContent');
                if (containerId || containerId === 0) {
                    fs = $('#' + containerId);
                }
                var container = $('<div/>', {
                    id: 'statisticContent'
                }).appendTo(fs);
            },


            create_min: function (dataFiltered, row) {
                var min = dataFiltered.min();
                if (min != '-' && min.toString().indexOf('.') > -1) {
                    min = min.toFixed(1);
                }
                var td = $('<td/>', {}).appendTo(row);
                $(td).html(min);

            },

            create_max: function (dataFiltered, row) {
                var max = dataFiltered.max();
                if (max != '-' && max.toString().indexOf('.') > -1) {
                    max = max.toFixed(1);
                }
                var td = $('<td/>', {}).appendTo(row);
                $(td).html(max);
            },

            create_average: function (dataFiltered, row) {
                var average = dataFiltered.average();
                if (average != '-' && average.toString().indexOf('.') > -1) {
                    average = average.toFixed(1);
                }

                var td = $('<td/>', {}).appendTo(row);
                $(td).html(average);
            },
        }
    },

    filterListByCurrentUser: function () {
        var userID = Boso.getCurrentUser().get('Custom_001');
        ActionHelper.CurrentView.ListViewObj.ListElements = _.filter(ActionHelper.CurrentView.ListViewObj.ListElements, function (Listelem) {
            var isCurrentUser = true;
            _.each(Listelem.properties, function (property) {
                if (property.label == Utils.getText('BENUTZER')) {
                    if (property.value != userID) {
                        isCurrentUser = false;
                    }
                }
            });

            return isCurrentUser;
        });
    },

    createXMLReportContent: function () {
        var patient = Boso.getCurrentUser();
        var patientNo = patient.get('Custom_001');
        var patientName = patient.get('Custom_002');
        var patientPreName = patient.get('Custom_006');
        var patientGender = 'F';
        if (patient.get('Custom_009') == 1) patientGender = 'M';
        var patientBirthday = moment.getIndivudualFormat(patient.get('Custom_003'), CONSTS.DATE_PERSIST_FORMAT, 'YYYYMMDD');
        var patientHeight = patient.get('Custom_004');
        var patientAddress = patient.get('Custom_007');
        var dayNightIntervallStart = UserSettings.GetDayNightIntervall().Start;
        var dayNightIntervallEnd = UserSettings.GetDayNightIntervall().End

        if (dayNightIntervallStart) {
            dayNightIntervallStart = moment.getIndivudualFormat(dayNightIntervallStart, CONSTS.TIME_PERSIST_FORMAT, 'HHmm');
        } else {
            dayNightIntervallStart = '';
        }

        if (dayNightIntervallEnd) {
            dayNightIntervallEnd = moment.getIndivudualFormat(dayNightIntervallEnd, CONSTS.TIME_PERSIST_FORMAT, 'HHmm');
        } else {
            dayNightIntervallEnd = '';
        }

        var uhrZeit;
        var startMorning;
        var endMorning;
        var nightFlag = 0;

        if (patientName == null || patientName == 'undefined') { patientName = ''; }
        if (patientPreName == null || patientPreName == 'undefined') { patientPreName = ''; }
        if (patient.attributes.Custom_009 == null || patient.attributes.Custom_009 == 'undefined') { patientGender = ''; }
        if (patientBirthday == null || patientBirthday == 'undefined') { patientBirthday = ''; }
        if (patientHeight == null || patientHeight == 'undefined') { patientHeight = ''; }
        if (patientAddress == null || patientAddress == 'undefined') { patientAddress = ''; }

        var start = '';
        var end = '';

        var weightData = Boso.getGenericDataWeightGraphics();
        var bloodPresuerData = Boso.getGenericDataBloodGraphics();

        if (!localStorage.getItem('BOSO_STAT_START') || !localStorage.getItem('BOSO_STAT_END')) {
            var genericDataFiltered = _.filter(window.genericData.models, function (model) {
                return (
                    model.get('EntityId') === Boso.MESSUREMENT_ENTITYID &&
                    model.get('IsActive') == true &&
                    model.get('Custom_009') === userID
                );
            });

            if (Utils.isNull(genericDataFiltered)) {
                return;
            }
            Boso.bosoGraphs.resetStatDate();
        }

        var fromDateString = localStorage.getItem('BOSO_STAT_START');
        var toDateString = localStorage.getItem('BOSO_STAT_END');

        if (Utils.isNull(fromDateString) || Utils.isNull(toDateString)) {
            alert(Utils.getText('DATEN_ZEITRAUM_KEINE'));
            return '';
        }

        var from = moment.getMilliSecondsFromDateTimeFormat(fromDateString);
        var to = moment.getMilliSecondsFromDateTimeFormat(toDateString) + (1000 * 60 * 60 * 24);

        var weightDataFiltered = _.filter(weightData, function (data) {
            var groessermin = (moment.getMilliSecondsFromPersistFormat(data.attributes["Custom_001"]) >= from);
            var kleinermax = (moment.getMilliSecondsFromPersistFormat(data.attributes["Custom_001"]) <= to);
            return (groessermin && kleinermax)
        });

        weightData = null;

        var bloodPresuerDataFiltered = _.filter(bloodPresuerData, function (data) {
            var dateTimeVal = moment.getMilliSecondsFromPersistFormat(data.attributes["Custom_001"]);
            var groessermin = (dateTimeVal >= from);
            var kleinermax = (dateTimeVal <= to);
            return (groessermin && kleinermax)
        });
        bloodPresuerData = null;

        if (weightDataFiltered.length + bloodPresuerDataFiltered.length <= 0) {

            alert(Utils.getText('DATEN_ZEITRAUM_KEINE'));
            return '';
        }

        //XML HEAD
        var XMLString = '<?xml version="1.0" encoding="windows-1252" ?>';
        XMLString += '<boso-medicus>';

        //Patient Information
        XMLString += '<patient id="1">';
        XMLString += '<pa_no>' + patientNo + '</pa_no>';
        XMLString += '<pa_name>' + patientName + '</pa_name>';
        XMLString += '<pa_prename>' + patientPreName + '</pa_prename>';
        XMLString += '<pa_gender>' + patientGender + '</pa_gender>';
        XMLString += '<pa_birthday>' + patientBirthday + '</pa_birthday>';
        XMLString += '<pa_size>' + patientHeight + '</pa_size>';
        XMLString += '<pa_weight>0</pa_weight>';
        XMLString += '<pa_street></pa_street>';
        XMLString += '<pa_zip></pa_zip>';
        XMLString += '<pa_city>' + patientAddress + '</pa_city>';

        //Period
        XMLString += '<period id="1">';
        XMLString += '<pa_id>1</pa_id>';
        XMLString += '<start_date>##start##</start_date>';
        XMLString += '<end_date>##end##</end_date>';
        XMLString += '<device>S</device>'
        XMLString += '<daystart>' + dayNightIntervallStart + '</daystart>'
        XMLString += '<nightstart>' + dayNightIntervallEnd + '</nightstart>'

        //Measurement
        var i = 1;
        _.each(weightDataFiltered, function (data) {
            if (data.attributes.Custom_001 < start || start == "") {
                start = data.attributes.Custom_001
            }

            if (data.attributes.Custom_001 > end) {
                end = data.attributes.Custom_001
            }

            XMLString += '<si-measure id="' + i + '">'
            XMLString += '<sys>0</sys>';
            XMLString += '<dia>0</dia>';
            XMLString += '<pulse>0</pulse>';
            XMLString += '<map>0</map>';
            XMLString += '<ihb>0</ihb>';
            XMLString += '<exclude>0</exclude>';
            XMLString += '<date_time>' + moment.getIndivudualFormat(data.attributes.Custom_001, CONSTS.DATETIME_PERSIST_FORMAT, 'YYYYMMDDHHmm') + '</date_time>';

            uhrZeit = data.attributes.Custom_001.split(' ')[1];


            if (!Utils.isNull(uhrZeit) && !Utils.isNull(dayNightIntervallStart) && !Utils.isNull(dayNightIntervallEnd)) {
                var timeformat = FormatHelper.get_TIME_FORMAT()
                startMorning = moment(dayNightIntervallStart, timeformat);
                startMorning = startMorning.format(timeformat, 'de');
                endMorning = moment(dayNightIntervallEnd, timeformat);
                endMorning.subtract('seconds', 1);
                endMorning = endMorning.format(timeformat, 'de');

                if ((uhrZeit >= startMorning) && uhrZeit <= endMorning) {
                    nightFlag = 0;
                }
                else {
                    nightFlag = 1;
                }
            }
            XMLString += '<nightflag> ' + nightFlag + ' </nightflag>';
            XMLString += '<weight>' + data.attributes.Custom_006 + '</weight>';
            XMLString += '</si-measure>'

            i++;
        });

        _.each(bloodPresuerDataFiltered, function (data) {

            if (data.attributes.Custom_001 < start || start == "") {
                start = data.attributes.Custom_001
            }

            if (data.attributes.Custom_001 > end) {
                end = data.attributes.Custom_001
            }

            XMLString += '<si-measure id="' + i + '">'
            XMLString += '<sys>' + data.attributes.Custom_002 + '</sys>';
            XMLString += '<dia>' + data.attributes.Custom_003 + '</dia>';
            XMLString += '<pulse>' + data.attributes.Custom_004 + '</pulse>';
            XMLString += '<map>' + Utils.round((parseInt(data.attributes.Custom_003) + 1 / 3 * (data.attributes.Custom_002 - data.attributes.Custom_003)), 0) + '</map>';
            XMLString += '<ihb>' + data.attributes.Custom_005 + '</ihb>';
            XMLString += '<exclude>0</exclude>';
            XMLString += '<ismanually>0</ismanually>';
            XMLString += '<date_time>' + moment.getIndivudualFormat(data.attributes.Custom_001, CONSTS.DATETIME_PERSIST_FORMAT, 'YYYYMMDDHHmm') + '</date_time>';

            uhrZeit = data.attributes.Custom_001.split(' ')[1];
            if (!Utils.isNull(uhrZeit) && !Utils.isNull(dayNightIntervallStart) && !Utils.isNull(dayNightIntervallEnd)) {
                var timeformat = FormatHelper.get_TIME_FORMAT()
                startMorning = moment(dayNightIntervallStart, timeformat);
                startMorning = startMorning.format(timeformat, 'de');
                endMorning = moment(dayNightIntervallEnd, timeformat);
                endMorning.subtract('seconds', 1);
                endMorning = endMorning.format(timeformat, 'de');

                if (uhrZeit >= startMorning && uhrZeit <= endMorning) {
                    nightFlag = 0;
                }
                else {
                    nightFlag = 1;
                }
            }
            XMLString += '<nightflag> ' + nightFlag + ' </nightflag>';
            XMLString += '<weight> 00.00 </weight>';
            XMLString += '</si-measure>'

            i++;
        });

        XMLString += '</period>'
        XMLString += '</patient>';
        XMLString += '</boso-medicus>';

        XMLString = XMLString.replace('##start##', moment.getIndivudualFormat(start, CONSTS.DATETIME_PERSIST_FORMAT, 'YYYYMMDDHHmm'));
        XMLString = XMLString.replace('##end##', moment.getIndivudualFormat(end, CONSTS.DATETIME_PERSIST_FORMAT, 'YYYYMMDDHHmm'));

        return XMLString;
    },

    getFirstMeasurementValueDate: function () {
        var userID = Boso.getCurrentUser().attributes.ObjectId;

        var FirstDate =
            _.first(
                _.sortBy(
                    _.filter(
                        window.genericData.models, function (model) {
                            return model.attributes.EntityId == Boso.MESSUREMENT_ENTITYID && model.attributes.IsActive == true && model.attributes.Custom_009 == userID
                        }), function (sortModel) {
                            return sortModel.attributes.Custom_001;
                        }));
        if (FirstDate == null || FirstDate.attributes.Custom_001 == null) {
            return "";
        }
        var dateValue = moment(FirstDate.attributes.Custom_001).format(CONSTS.DATE_PERSIST_FORMAT, 'de');
        dateValue = moment(dateValue, CONSTS.DATE_PERSIST_FORMAT).format(FormatHelper.get_DATE_FORMAT(), 'de');
        return dateValue;
    },

    getLastMeasurementValueDate: function () {
        var userID = Boso.getCurrentUser().attributes.ObjectId;

        var LastDate =
            _.last(
                _.sortBy(
                    _.filter(
                        window.genericData.models, function (model) {
                            return model.attributes.EntityId == Boso.MESSUREMENT_ENTITYID && model.attributes.IsActive == true && model.attributes.Custom_009 == userID
                        }), function (sortModel) {
                            return sortModel.attributes.Custom_001;
                        }));

        if (LastDate == null || LastDate.attributes.Custom_001 == null) {
            return "";
        }
        var dateValue = moment(LastDate.attributes.Custom_001).format(CONSTS.DATE_PERSIST_FORMAT, 'de');
        dateValue = moment(dateValue, CONSTS.DATE_PERSIST_FORMAT).format(FormatHelper.get_DATE_FORMAT(), 'de');
        return dateValue;
    },

    createDeleteItemsMenu: function () {
        var navbar = ActionHelper.findNavBar(null, 'List');
        var menuButton = navbar.find('#deleteItemsContextButton');
        if (menuButton.length == 0) {
            var li = $('<li/>', {});
            navbar.append(li);

            var button = $('<a/>', {
                id: 'deleteItemsContextButton',
                text: Utils.getText('LOESCHEN')
            }).appendTo(li);

            button.attr('data-icon', 'bars');
            button.click(function () { Boso.openDeleteItemsMenu() });
        }

        $('#deleteContext-popup')
        Boso.addPopupContent();
    },

    addPopupContent: function () {
        var delitionContext = $('#deleteContext');
        delitionContext.empty();

        var divDeleteSelected = $('<div />', {
            id: 'divDeleteSelected'
        }).appendTo(delitionContext);
        divDeleteSelected.css('display', 'none');

        var btDeleteSelected = $('<button />', {
            id: 'btDeleteSelected',
            class: 'bt_small',
            text: Utils.getText('MESSWERTE_AUSGEWAEHLT')
        }).appendTo(divDeleteSelected);
        btDeleteSelected.click(function () {
            setTimeout(function () {
                Boso.deleteSelectedItems();
            }, 100);
        });
        btDeleteSelected.focus(function () {
            this.blur();
        });

        var btDeleteAll = $('<button />', {
            id: 'btDeleteAll',
            class: 'bt_small',
            text: Utils.getText('MESSWERTE_ALLE')
        }).appendTo(delitionContext);
        btDeleteAll.click(function () {
            setTimeout(function () {
                Boso.deleteAllItems();
            }, 100);
        });
        btDeleteAll.focus(function () {
            this.blur();
        });

        var btDeleteOneMonth = $('<button />', {
            id: 'btDeleteOneMonth',
            class: 'bt_small',
            text: Utils.getText('MESSWERTE_AELTER_1_MONAT')
        }).appendTo(delitionContext);
        btDeleteOneMonth.click(function () {
            setTimeout(function () {
                Boso.deleteOlderItems(1);
            }, 100);
        });
        btDeleteOneMonth.focus(function () {
            this.blur();
        });

        var btDeleteThreeMonths = $('<button />', {
            id: 'btDeleteThreeMonths',
            class: 'bt_small',
            text: Utils.getText('MESSWERTE_AELTER_3_MONATE')
        }).appendTo(delitionContext);
        btDeleteThreeMonths.click(function () {
            setTimeout(function () {
                Boso.deleteOlderItems(3);
            }, 100);
        });
        btDeleteThreeMonths.focus(function () {
            this.blur();
        });

        var btDeleteSixMonths = $('<button />', {
            id: 'btDeleteSixMonths',
            class: 'bt_small',
            text: Utils.getText('MESSWERTE_AELTER_6_MONATE')
        }).appendTo(delitionContext);
        btDeleteSixMonths.click(function () {
            setTimeout(function () {
                Boso.deleteOlderItems(6);
            }, 100);
        });
        btDeleteSixMonths.focus(function () {
            this.blur();
        })

        var btDeleteTwelveMonths = $('<button />', {
            id: 'btDeleteTwelveMonths',
            class: 'bt_small',
            text: Utils.getText('MESSWERTE_AELTER_12_MONATE')
        }).appendTo(delitionContext);
        btDeleteTwelveMonths.click(function () {
            setTimeout(function () {
                Boso.deleteOlderItems(12);
            }, 100);
        });
        btDeleteTwelveMonths.focus(function () {
            this.blur();
        })

        var br = $('<p />', {
            height: '4px'
        }).appendTo(delitionContext);

        var btClosesDeletePopup = $('<button />', {
            class: 'bt_small',
            text: Utils.getText('ABBRECHEN')
        }).appendTo(delitionContext);
        btClosesDeletePopup.click(function () {
            $('#deleteContext').popup("close");
        });

        var removeOverflow = function (event, ui) {
            $('body').css('overflow', 'hidden');
        }

        var addOverflow = function (event, ui) {
            $('body').css('overflow', 'auto');
        }

        var repositionPopup = function (event, ui) {
            setTimeout(function () {
                delitionContext.popup('reposition', { positionTo: 'origin' })
            }, 0);
        }

        $(document)
            .off('popupafteropen', '#deleteContext', removeOverflow)
            .on('popupafteropen', '#deleteContext', removeOverflow)
            .off('popupafterclose', '#deleteContext', addOverflow)
            .on('popupafterclose', '#deleteContext', addOverflow)
            .off('popupafteropen', '#deleteContext', repositionPopup)
            .on('popupafteropen', '#deleteContext', repositionPopup);

        $('#deleteContext').trigger('create');
    },

    openDeleteItemsMenu: function () {
        var hasItems = !!$('.checkBox').length;
        if (!hasItems) { return; }

        $('#deleteContext').popup("open");
        
        var hasSelectedItems = $('.checkBox').hasClass("ui-icon-check")
        !!hasSelectedItems ? $('#divDeleteSelected').css('display', 'block') : $('#divDeleteSelected').css('display', 'none');
    },

    measurementsExist: function(userId) {
        if(!userId) {
            return false;
        }

        var userMeasurementData = _.find(window.genericData.models, function (model) {
            return model.get('EntityId') === Boso.MESSUREMENT_ENTITYID
                && model.get('Custom_009') === userId
                && model.get('IsActive') === true
        });

        return !!userMeasurementData;
    },

    deleteAllItems: function() {
        var userID = Boso.getCurrentUser().get('DataId');
        if(!Boso.measurementsExist(userID)) {
            return;
        }

        var confirmMessage = Utils.getText('EINTRAEGE_LOESCHEN_ZEITRAUM_BEST');
        showConfirmMessage(confirmMessage, function (buttonIndex) {
            if (buttonIndex === 1) {
                $('#deleteContext').popup("close");
                showLoader(Utils.getText('MESSWERTE_LOESCHEN'));
                Boso.deleteUsersMeasurements(userID, function () {
                    var currentView = ActionHelper.CurrentView;
                    var viewId = Boso.GUID_EMPTY;
                    if (!!currentView && !!currentView.options && !!currentView.options.view) {
                        viewId = currentView.options.view.get('ViewId');
                    }

                    if (ActionHelper.IsValid) {
                        ActionHelper.navigate(null, viewId, null);
                    }

                    hideloader();
                });
            }
        }, Utils.getText('WIRKLICH_LOESCHEN'), Utils.getText('LOESCHEN_ABBRECHEN'));
    },

    deleteOlderItems: function (toDelete) {
        if (!toDelete) {
            return;
        }

        var confirmMessage = Utils.getText('EINTRAEGE_LOESCHEN_ZEITRAUM_BEST');
        showConfirmMessage(confirmMessage, function (buttonIndex) {
            if (!!toDelete && toDelete !== 'selected' && buttonIndex === 1) {
                var date = moment.getMilliSecondsFromDateFormat(moment().subtract('months', toDelete))

                var userId = Boso.getCurrentUser().get('ObjectId');
                var measurementsToDelete = _.filter(window.genericData.models, function (model) {
                    var d = moment.getMilliSecondsFromPersistFormat(model.get('Custom_001'))
                    return (
                        model.get('EntityId') === Boso.MESSUREMENT_ENTITYID &&
                        model.get('IsActive') == true &&
                        model.get('Custom_009') === userId &&
                        d < date
                    );
                });

                $('#deleteContext').popup("close");
                showLoader(Utils.getText('MESSWERTE_LOESCHEN'));

                setTimeout(function () {
                    _.each(measurementsToDelete, function (measurement) {
                        measurement.PersistToLocalStorageImmediately = false;
                        window.genericData.remove(measurement);
                        measurement.destroy();
                    });

                    window.persistGenericData();
                    var currentView = ActionHelper.CurrentView;
                    var viewId = Boso.GUID_EMPTY;
                    if (!!currentView && !!currentView.options && !!currentView.options.view) {
                        viewId = currentView.options.view.get('ViewId');
                    }

                    if (ActionHelper.IsValid) {
                        ActionHelper.navigate(null, viewId, null);
                    }

                    hideloader();
                }, 100);
            }
        }, Utils.getText('WIRKLICH_LOESCHEN'), Utils.getText('LOESCHEN_ABBRECHEN'));
    },

    deleteSelectedItems: function () {
        var measurement;
        var confirmMessage = Utils.getText('EINTRAEGE_LOESCHEN_ZEITRAUM_BEST');
        showConfirmMessage(confirmMessage, function (buttonIndex) {
            if (buttonIndex !== 1) {
                return;
            }

            $(ActionHelper.CurrentView.el).find('#listView .checkBox').each(function () {
                var isChecked = $(this).hasClass("ui-icon-check");
                if (isChecked) {
                    measurement = window.genericData.get(this.id);
                    measurement.PersistToLocalStorageImmediately = false;
                    window.genericData.remove(measurement);
                    measurement.destroy();
    
                    //Eintrag aus DOM entfernen
                    var element = $(this.parentNode.firstChild);
                    var checked = element.hasClass("ui-icon-check")
                    !!checked ? $(this).closest('tr').remove() : $(this.parentNode).remove();  
                }
            });

            window.persistGenericData();

            $('#deleteContext').popup("close");
            $(document).trigger('scrollstop');
            $(document).trigger('create');
        }, Utils.getText('WIRKLICH_LOESCHEN'), Utils.getText('LOESCHEN_ABBRECHEN'));
    },

    openSettings: function (model) {
        Boso.buildSettingsMenu();
    },

    buildSettingsMenu: function () {
        var fieldContainer = $('#CustomViewContent');
        fieldContainer.empty();

        var genericSettingFields = _.filter(window.genericViewFields.models, function (field) {
            return field.get('ViewId') === ActionHelper.CurrentView.options.view.id &&
                field.get('IsActive') === true
        });

        var settingsView = ActionHelper.CurrentView;

        Boso.buildGeneralSettginsSection(settingsView, genericSettingFields, fieldContainer);
        Boso.buildBloodPressureDeviceSection(genericSettingFields, fieldContainer);
        Boso.buildWeightScaleDeviceSection(genericSettingFields, fieldContainer);
        Boso.buildDataExportImportSection(fieldContainer);
    },

    /// GENERAL SETTINGS SECTION
    buildGeneralSettginsSection: function (settingsView, confSettingFields, rootContainer) {
        var settings = settingsView.model;
        var FIELDID_DATEMORNING = '49776856-ab5d-c233-f7ea-25b3ee72101d';
        var FIELDID_DATEAFTERNOON = 'dbcd0a70-9b8b-6d31-4028-210845784607';

        var genericDateMorningField = _.find(confSettingFields, function (settingsField) {
            return settingsField.get('FieldId') === FIELDID_DATEMORNING;
        });

        var genericDateAfternoonField = _.find(confSettingFields, function (settingsField) {
            return settingsField.get('FieldId') === FIELDID_DATEAFTERNOON;
        });

        var dateMorningProperty = _.find(genericProperties.models, function (property) {
            return property.get('PropertyId') === genericDateMorningField.get('PropertyId');
        });

        var dateAfternoonProperty = _.find(genericProperties.models, function (property) {
            return property.get('PropertyId') === genericDateAfternoonField.get('PropertyId');
        });

        var container = $('<div />', {
            'style': 'border: solid 1px lightgrey; margin-top:4px; padding-left: 8px; padding-right: 8px;'
        }).appendTo(rootContainer);

        var generalSettingsLabel = $('<span />', {
            'class': 'settingsHeading'
        }).appendTo(container);
        generalSettingsLabel.text(Utils.getText('ALLGEMEINE_EINSTELLUNGEN'));

        var generalSettingsField = $('<div />', {
            'class': 'ui-field-contain'
        }).appendTo(container);

        var dateMorningField = $('<div />', {
            'style': 'display: table; width: 100%; vertical-align: middle;'
        }).appendTo(generalSettingsField);

        var dateMorningFieldLabel = $('<label />', {
            'style': 'width: 230px; float:left; margin: 0; margin-right:10px; line-height:15px; padding-top:12px'
        }).appendTo(dateMorningField);
        dateMorningFieldLabel.text(dateMorningProperty.get('PropertyName'));
        Boso.createSettingsTime(dateMorningProperty, genericDateMorningField, settings.get('Custom_004')).appendTo(dateMorningField);

        var dateAfternoonField = $('<div />', {
            'style': 'display: table; width: 100%; vertical-align: middle;'
        }).appendTo(generalSettingsField);

        var dateAfternoonFieldLabel = $('<label />', {
            'style': 'width: 230px; float:left; margin: 0; margin-right:10px; line-height:15px; padding-top:12px'
        }).appendTo(dateAfternoonField);
        dateAfternoonFieldLabel.text(dateAfternoonProperty.get('PropertyName'));
        Boso.createSettingsTime(dateAfternoonProperty, genericDateAfternoonField, settings.get('Custom_005')).appendTo(dateAfternoonField);
    },

    createSettingsTime: function (property, field, time) {
        var dateControl = $('<fieldset />', {
            'data-role': 'controlgroup',
            'data-type': 'horizontal',
            'display': 'block'
        });

        var input = $('<input />', {
            id: field.get('FieldId'),
            name: property.get('GenericColumnName'),
            type: 'text'
        }).appendTo(dateControl);
        input.attr('class', 'date');

        var timestep = property.get('Scale');
        if (timestep <= 0) { timestep = 1; }

        input.scroller({
            preset: 'time',
            theme: 'android-holo',
            display: 'modal',
            mode: 'scroller',
            timeFormat: 'HH:ii',
            timeWheels: 'HHii',
            hourText: Utils.getText('STUNDEN'),
            minuteText: Utils.getText('MINUTEN'),
            //Intervall
            stepMinute: timestep,
            monthNames: FormatHelper.get_MONTH_NAMES(),//['Januar', 'Februar', 'März', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember'],
            monthNamesShort: FormatHelper.get_MONTH_NAMES_SHORT(),//['Jan', 'Feb', 'Mär', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez'],
            monthText: Utils.getText('MONAT'),
            dayText: Utils.getText('TAG'),
            yearText: Utils.getText('JAHR'),
            nowText: Utils.getText('JETZT'),
            cancelText: Utils.getText('BEENDEN'),
            setText: Utils.getText('OK'),
        });

        var measurementTakeoverField = $('')

        var time = !!time ? time.split(':') : [''];
        if (time.length === 3) {
            time.pop();
        }
        time = time.join(':');
        input.val(time);

        return dateControl;
    },

    /// WEIGHT SCALE DEVICE SETTINGS SECTION
    buildWeightScaleDeviceSection: function (genericSettingFields, rootContainer) {
        var FIELDID_WEIGHTSCALE_USERMODE = 'ee56d09d-ac9c-4161-9e04-5158f3964651';

        var genericFieldUserMode = _.find(genericSettingFields, function (settingsField) {
            return settingsField.get('FieldId') === FIELDID_WEIGHTSCALE_USERMODE;
        });

        var genericPropertyUserMode = _.find(genericProperties.models, function (property) {
            return property.get('PropertyId') === genericFieldUserMode.get('PropertyId');
        });

        var container = $('<div />', {
            'class': 'settingsMedicusSystemContainer'
        }).appendTo(rootContainer);

        var weightScaleDeviceTitle = $('<span />', {
            'class': 'settingsHeading settingsHeadingPadded'
        }).appendTo(container);
        weightScaleDeviceTitle.text('bosogramm system');

        var fieldcontain = $('<div />', {
            'class': 'ui-field-contain'
        }).appendTo(container);

        var userModeLabel = $('<label />', {
            'class': 'settingsSectionLabel'
        }).appendTo(fieldcontain);
        userModeLabel.text(genericPropertyUserMode.get('PropertyName'));

        Boso.buildUserModeDecisionControl(genericPropertyUserMode.get('Param_01'), genericFieldUserMode, fieldcontain);
    },

    buildUserModeDecisionControl: function (options, field, elementAppendTo) {
        var decisionControl = $('<fieldset />', {
            'data-role': 'controlgroup',
            'data-type': 'horizontal',
            'fieldtype': 'decision',
            'style': 'width:auto',
            'data-mini': 'true'
        }).appendTo(elementAppendTo);

        var settings = ActionHelper.CurrentView.model;
        var noYes = !!settings ? settings.get('Custom_006') : undefined;

        var userModeSingleuser = $('<input />', {
            type: 'radio', name: 'userMode', id: 'radioFalse' + field.get('FieldId'), value: '0'
        }).appendTo(decisionControl);
        var userModeSingleuserLabel = $('<label for="radioFalse' + field.get('FieldId') + '"> ' + options.split('|')[0] + ' </label>').appendTo(decisionControl);
        if (noYes === '0' || !noYes) {
            userModeSingleuser.prop('checked', 'checked');
        }

        var userModeMultiuser = $('<input />', {
            type: 'radio', name: 'userMode', id: 'radioTrue' + field.get('FieldId'), value: '1'
        }).appendTo(decisionControl);
        var userModeMultiuserLabel = $('<label for="radioTrue' + field.get('FieldId') + '"> ' + options.split('|')[1] + ' </label>').appendTo(decisionControl);
        if (noYes === '1') {
            userModeMultiuser.prop('checked', 'checked');
        }

        var label = $('label[for="' + $(decisionControl).find('input:checked').attr('id') + '"]');
        if (label.length > 0) {
            label.addClass('ui-btn-active');
        }

        var rbuserModeSingleuser = $('#radioFalse' + field.get('FieldId'));
        rbuserModeSingleuser.on('click', function () {
            $(this).closest('fieldset').find('.ui-btn-active').removeClass('ui-btn-active');
            $('label[for="' + rbuserModeSingleuser.attr('id') + '"]').addClass('ui-btn-active');
        });

        var rbuserModeMultiuser = $('#radioTrue' + field.get('FieldId'));
        rbuserModeMultiuser.on('click', function () {
            $(this).closest('fieldset').find('.ui-btn-active').removeClass('ui-btn-active');
            $('label[for="' + rbuserModeMultiuser.attr('id') + '"]').addClass('ui-btn-active');
        });
    },

    /// BLOOD PRESSURE DEVICE SETTINGS SECTION
    buildBloodPressureDeviceSection: function (genericSettingFields, rootContainer) {
        var FIELDID_MEASUREMENT_TAKEOVER = 'd8488d00-d1f9-ac20-09e3-b54afe2eccb0';
        var genericFieldMeasurementTakeover = _.find(genericSettingFields, function (settingsField) {
            return settingsField.get('FieldId') === FIELDID_MEASUREMENT_TAKEOVER;
        });

        var genericPropertyMeasurementTakeover = _.find(genericProperties.models, function (property) {
            return property.get('PropertyId') === genericFieldMeasurementTakeover.get('PropertyId');
        });

        var container = $('<div />', {
            'style': 'border: solid 1px lightgrey; margin-top:4px; padding-left: 8px; padding-right: 8px;'
        }).appendTo(rootContainer);

        var bloodPressureDeviceTitle = $('<span />', {
            'class': 'settingsHeading settingsHeadingPadded'
        }).appendTo(container);
        bloodPressureDeviceTitle.text('boso medicus system');

        var fieldcontain = $('<div />', {
            'class': 'ui-field-contain'
        }).appendTo(container);

        var measurementTakeoverLabel = $('<label />', {
            'class': 'settingsSectionLabel'
        }).appendTo(fieldcontain);
        measurementTakeoverLabel.text(genericPropertyMeasurementTakeover.get('PropertyName'));

        Boso.buildMeasurementTakeoverDecisionControl(genericPropertyMeasurementTakeover.get('Param_01'), genericFieldMeasurementTakeover, fieldcontain);
    },

    buildMeasurementTakeoverDecisionControl: function (options, field, elementAppendTo) {
        var decisionControl = $('<fieldset />', {
            'data-role': 'controlgroup',
            'data-type': 'horizontal',
            'fieldtype': 'decision',
            'style': 'width:auto',
            'data-mini': 'true'
        }).appendTo(elementAppendTo);

        var settings = ActionHelper.CurrentView.model;

        var noYes = !!settings ? settings.get('Custom_003') : undefined;

        var measuredValueTakeoverLast = $('<input />', {
            type: 'radio', name: 'measuredValueTakeover', id: 'radioFalse' + field.get('FieldId'), value: '0'
        }).appendTo(decisionControl);
        var measuredValueTakeoverLastLabel = $('<label for="radioFalse' + field.get('FieldId') + '"> ' + options.split('|')[0] + ' </label>')
            .appendTo(decisionControl);

        if (noYes === '0') {
            measuredValueTakeoverLast.prop('checked', 'checked');
        }

        var measuredValueTakeoverAll = $('<input />', {
            type: 'radio', name: 'measuredValueTakeover', id: 'radioTrue' + field.get('FieldId'), value: '1'
        }).appendTo(decisionControl);
        var measuredValueTakeoverAllLabel = $('<label for="radioTrue' + field.get('FieldId') + '"> ' + options.split('|')[1] + ' </label>')
            .appendTo(decisionControl);

        if (noYes === '1' || !noYes) {
            measuredValueTakeoverAll.prop('checked', 'checked');
        }

        var label = $('label[for="' + $(decisionControl).find('input:checked').attr('id') + '"]');
        if (label.length > 0) {
            label.addClass('ui-btn-active');
        }

        var rbMeasuredValueTakeoverLast = $('#radioFalse' + field.get('FieldId'));
        rbMeasuredValueTakeoverLast.on('click', function () {
            $(this).closest('fieldset').find('.ui-btn-active').removeClass('ui-btn-active');
            $('label[for="' + rbMeasuredValueTakeoverLast.attr('id') + '"]').addClass('ui-btn-active');
        });

        var rbMeasuredValueTakeoverAll = $('#radioTrue' + field.get('FieldId'));
        rbMeasuredValueTakeoverAll.on('click', function () {
            $(this).closest('fieldset').find('.ui-btn-active').removeClass('ui-btn-active');
            $('label[for="' + rbMeasuredValueTakeoverAll.attr('id') + '"]').addClass('ui-btn-active');
        });
    },



    buildDataExportImportSection: function (rootContainer) {
        var container = $('<div />', {
            'style': 'border: solid 1px lightgrey; margin-top:4px; padding-left: 8px; padding-right: 8px;'
        }).appendTo(rootContainer);
        
        var fieldcontain = $('<div />', {
            'class': 'ui-field-contain'
        }).appendTo(container);


        var btExportData = $('<button />', {
            id: 'btExportData',
            class: 'bt_small',
            text: Utils.getText('EXPORT_DATA')
        }).appendTo(fieldcontain);
        btExportData.click(function () {
            setTimeout(function () {
                Boso.ExportGenericData();
            }, 100);
        });
        btExportData.focus(function () {
            this.blur();
        });

        var btImportData = $('<button />', {
            id: 'btImportData',
            class: 'bt_small',
            text: Utils.getText('IMPORT_DATA')
        }).appendTo(fieldcontain);
        
        btImportData.click(function () {
            setTimeout(function () {
                Boso.OnImportClick();
            }, 100);
        });

        btImportData.focus(function () {
            this.blur();
        });
    },


    toggleWeightScaleCommunication: function () {
        var switchState = function () {
            var btn = $('#bosoWeightButton');
            Boso.weightScaleCommunicationActive = !Boso.weightScaleCommunicationActive;
            if (Boso.weightScaleCommunicationActive) {
                btn.addClass('bosoWeightButton_active');
                btn.removeClass('bosoWeightButton_inactive');
            } else {
                btn.addClass('bosoWeightButton_inactive');
                btn.removeClass('bosoWeightButton_active');
            }
        }

        if (!Boso.weightScaleCommunicationTimeout) {
            Boso.weightScaleCommunicationTimeout = new Utils.ExplicitTimeout(function () {
                switchState();
            }, 300000);
        }

        switchState();
        if (Boso.weightScaleCommunicationActive) {
            Boso.weightScaleCommunicationTimeout.restart();
        }

        if (!Boso.weightScaleCommunicationActive) {
            Boso.weightScaleCommunicationTimeout.clear();
        }
    },

    fillReportViewDateFilter: function () {
        var fieldIdFrom = '9900d09d-ac9c-4ba2-97ff-d1619a6569f5';
        var fieldIdUntil = 'd5c8d09d-ac9c-4ed9-ae27-21619a6616c9';

        var fromValue = localStorage.getItem('BOSO_STAT_START');
        var untilValue = localStorage.getItem('BOSO_STAT_END')

        var fieldFrom = $('#' + fieldIdFrom).val(fromValue);
        fieldFrom.scroller('setDate', moment(fromValue, FormatHelper.get_DATE_FORMAT()).toDate(), true);

        var fieldUntil = $('#' + fieldIdUntil).val(untilValue);
        fieldUntil.scroller('setDate', moment(untilValue, FormatHelper.get_DATE_FORMAT()).toDate(), true);

        var correct = function () {
            if (!!fieldFrom && !!fieldUntil) {
                var start = moment(fieldFrom.scroller('getDate'));
                var end = moment(fieldUntil.scroller('getDate'));

                if (start.isAfter(end)) {
                    fieldUntil.scroller('setDate', start.toDate(), true)
                }
            }
        }

        fieldFrom.off('change').on('change', function (el) {
            if (!!fieldFrom.val()) {
                var date = fieldFrom.scroller('getDate');

                localStorage.setItem('BOSO_STAT_START', moment(date).format(FormatHelper.getPattern('DATE_FORMAT')));
            }
            correct();
        });

        fieldUntil.off('change').on('change', function (el) {
            if (!!fieldUntil.val()) {
                var date = fieldUntil.scroller('getDate');
                localStorage.setItem('BOSO_STAT_END', moment(date).format(FormatHelper.getPattern('DATE_FORMAT')));
            }

            correct();
        });
    },

    ExportGenericData: function() 
    {
        var msg = Utils.getText('EXPORT_DATA');
        if (Boso.getPlatform() === 'android') {
            msg = Utils.getText('SAVE_EXPORT_DATA_LOADER');
            msg = msg.replace('{Verzeichnis}', Boso.getFileDirectory());
        }
        showLoader(msg);
        var shareFiles = function (fileNames) {
            if (!window.plugins || !window.plugins.socialsharing) {
                console.error("plugin \"cordova-plugin-x-socialsharing\" is not available!");
                return;
            }

            var options = {
                // message: msg,
                // subject: Utils.getText('BERICHT'), // fi. for email
                files: fileNames // an array of filenames either locally or remotely
                // chooserTitle: 'Pick an app' // Android only, you can override the default share sheet title
            }

            var onSuccess = function (result) {
                hideloader();
            }

            var onError = function (msg) {
                hideloader();
                console.log("Sharing failed with message: " + msg);
            }

            window.plugins.socialsharing.shareWithOptions(options, onSuccess, onError);
        }
        
        var gotFileEntry = function (fileEntry) {
            var fileNames = [];
            fileNames.push(fileEntry.toURL());
            shareFiles(fileNames);
        }

        var currentDate = moment().format(CONSTS.DATETIME_PERSIST_FORMAT).replace(/:/g, '-').replace(' ', '-');
        var fileDescription = {
            fileName: 'boso_' + currentDate + '_DataExport',
            fileType: 'json',
            folderName: Boso.REPORT_FILE_FOLDERNAME,
            fileSystem: LocalFileSystem.TEMPORARY,
            isBinary: false
        }

        var onEndSave = function(fileContent, fileName, shortfileName) {
            FileHelper.getFileEntry(fileDescription.fileSystem, fileDescription.folderName, fileDescription.fileName + '.' + fileDescription.fileType, gotFileEntry);
        }

        var GenericDataAsjson = localStorage.getItem('GenericData');
        if (!GenericDataAsjson) {
            onEndSave();
            return;
        }

        var Hash = CryptoJS.SHA512(GenericDataAsjson);
        var jsonToSave = '{"GenericData": ' + GenericDataAsjson + ', "Hash": "' + Hash + '"}';
        FileHelper.saveFile(jsonToSave, fileDescription.fileName, fileDescription.fileType, fileDescription.folderName, fileDescription.fileSystem, fileDescription.isBinary, onEndSave);
    },

    OnImportClick: function () {
        var confirmMessage = Utils.getText('IMPORT_FILE_DATA_DELETION_WARNING');
        showConfirmMessage(confirmMessage, function (buttonIndex) {
            if (buttonIndex === 2) {
                return;
            }
            
            Boso.ImportGenericData();
        }, '', Utils.getText('JA').concat(',', Utils.getText('NEIN')));
    },

    ImportGenericData: function () {
        showLoader(Utils.getText('WIRD_GELADEN'));

        var filePromise = FileHelper.pickFile();
        filePromise
            .then(function(fileInfo){ DoImport(fileInfo) })
            .finally(hideloader());
    
        function DoImport(fileInfo) {
            try {
                var dataAsString = new TextDecoder("utf-8").decode(fileInfo.data);
                var dataAsJson = JSON.parse(dataAsString);
                
                if (!dataAsJson || !dataAsJson.GenericData) {
                    alert(Utils.getText('IMPORT_FILE_DATA_NOT_VALID'));
                    return;
                }

                var stringifiedGenericData = JSON.stringify(dataAsJson.GenericData); 
                var hash = CryptoJS.SHA512(stringifiedGenericData);
                if (!hash || !dataAsJson.Hash || hash.toString() !== dataAsJson.Hash) {
                    alert(Utils.getText('IMPORT_FILE_DATA_NOT_VALID'));
                    return;
                }

                localStorage.removeItem('GenericData');
                localStorage.setItem('GenericData', stringifiedGenericData);

                if (window.navigator.notification) {
                    doInit();
                }
            } catch (error) {
                alert(Utils.getText('IMPORT_FILE_DATA_NOT_VALID'));
                return;
            }
            return;
        }
    }
};